Method: Array#pg_array

Defined in:
lib/sequel/extensions/pg_array.rb

#pg_array(type = nil) ⇒ Object

Return a PGArray proxy to the receiver, using a specific database type if given. This is mostly useful as a short cut for creating PGArray objects that didn’t come from the database.



600
601
602
# File 'lib/sequel/extensions/pg_array.rb', line 600

def pg_array(type=nil)
  Sequel::Postgres::PGArray.new(self, type)
end