Method: Sequel::Dataset#graph

Defined in:
lib/sequel/dataset/graph.rb

#graph(dataset, join_conditions = nil, options = OPTS, &block) ⇒ Object

Similar to Dataset#join_table, but uses unambiguous aliases for selected columns and keeps metadata about the aliases for use in other methods.

Arguments:

dataset

Can be a symbol (specifying a table), another dataset, or an SQL::Identifier, SQL::QualifiedIdentifier, or SQL::AliasedExpression.

join_conditions

Any condition(s) allowed by join_table.

block

A block that is passed to join_table.

Options:

:from_self_alias

The alias to use when the receiver is not a graphed dataset but it contains multiple FROM tables or a JOIN. In this case, the receiver is wrapped in a from_self before graphing, and this option determines the alias to use.

:implicit_qualifier

The qualifier of implicit conditions, see #join_table.

:join_only

Only join the tables, do not change the selected columns.

:join_type

The type of join to use (passed to join_table). Defaults to :left_outer.

:qualify

The type of qualification to do, see #join_table.

:select

An array of columns to select. When not used, selects all columns in the given dataset. When set to false, selects no columns and is like simply joining the tables, though graph keeps some metadata about the join that makes it important to use graph instead of join_table.

:table_alias

The alias to use for the table. If not specified, doesn’t alias the table. You will get an error if the alias (or table) name is used more than once.



53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
# File 'lib/sequel/dataset/graph.rb', line 53

def graph(dataset, join_conditions = nil, options = OPTS, &block)
  # Allow the use of a dataset or symbol as the first argument
  # Find the table name/dataset based on the argument
  table_alias = options[:table_alias]
  table = dataset
  create_dataset = true

  case dataset
  when Symbol
    # let alias be the same as the table name (sans any optional schema)
    # unless alias explicitly given in the symbol using ___ notation and symbol splitting is enabled
    table_alias ||= split_symbol(table).compact.last
  when Dataset
    if dataset.simple_select_all?
      table = dataset.opts[:from].first
      table_alias ||= table
    else
      table_alias ||= dataset_alias((@opts[:num_dataset_sources] || 0)+1)
    end
    create_dataset = false
  when SQL::Identifier
    table_alias ||= table.value
  when SQL::QualifiedIdentifier
    table_alias ||= split_qualifiers(table).last
  when SQL::AliasedExpression
    return graph(table.expression, join_conditions, {:table_alias=>table.alias}.merge!(options), &block)
  else
    raise Error, "The dataset argument should be a symbol or dataset"
  end
  table_alias = table_alias.to_sym

  if create_dataset
    dataset = db.from(table)
  end

  # Raise Sequel::Error with explanation that the table alias has been used
  raise_alias_error = lambda do
    raise(Error, "this #{options[:table_alias] ? 'alias' : 'table'} has already been been used, please specify " \
      "#{options[:table_alias] ? 'a different alias' : 'an alias via the :table_alias option'}") 
  end

  # Only allow table aliases that haven't been used
  raise_alias_error.call if @opts[:graph] && @opts[:graph][:table_aliases] && @opts[:graph][:table_aliases].include?(table_alias)
  
  table_alias_qualifier = qualifier_from_alias_symbol(table_alias, table)
  implicit_qualifier = options[:implicit_qualifier]
  joined_dataset = joined_dataset?
  ds = self
  graph = opts[:graph]

  if !graph && (select = @opts[:select]) && !select.empty?
    select_columns = nil

    unless !joined_dataset && select.length == 1 && (select[0].is_a?(SQL::ColumnAll))
      force_from_self = false
      select_columns = select.map do |sel|
        unless col = _hash_key_symbol(sel)
          force_from_self = true
          break
        end

        [sel, col]
      end

      select_columns = nil if force_from_self
    end
  end

  # Use a from_self if this is already a joined table (or from_self specifically disabled for graphs)
  if (@opts[:graph_from_self] != false && !graph && (joined_dataset || force_from_self))
    from_selfed = true
    implicit_qualifier = options[:from_self_alias] || first_source
    ds = ds.from_self(:alias=>implicit_qualifier)
  end
  
  # Join the table early in order to avoid cloning the dataset twice
  ds = ds.join_table(options[:join_type] || :left_outer, table, join_conditions, :table_alias=>table_alias_qualifier, :implicit_qualifier=>implicit_qualifier, :qualify=>options[:qualify], &block)

  return ds if options[:join_only]

  opts = ds.opts

  # Whether to include the table in the result set
  add_table = options[:select] == false ? false : true

  if graph
    graph = graph.dup
    select = opts[:select].dup
    [:column_aliases, :table_aliases, :column_alias_num].each{|k| graph[k] = graph[k].dup}
  else
    # Setup the initial graph data structure if it doesn't exist
    qualifier = ds.first_source_alias
    master = alias_symbol(qualifier)
    raise_alias_error.call if master == table_alias

    # Master hash storing all .graph related information
    graph = {}

    # Associates column aliases back to tables and columns
    column_aliases = graph[:column_aliases] = {}

    # Associates table alias (the master is never aliased)
    table_aliases = graph[:table_aliases] = {master=>self}

    # Keep track of the alias numbers used
    ca_num = graph[:column_alias_num] = Hash.new(0)

    select = if select_columns
      select_columns.map do |sel, column|
        column_aliases[column] = [master, column]
        if from_selfed
          # Initial dataset was wrapped in subselect, selected all
          # columns in the subselect, qualified by the subselect alias.
          Sequel.qualify(qualifier, Sequel.identifier(column))
        else
          # Initial dataset not wrapped in subslect, just make
          # sure columns are qualified in some way.
          qualified_expression(sel, qualifier)
        end
      end
    else
      columns.map do |column|
        column_aliases[column] = [master, column]
        SQL::QualifiedIdentifier.new(qualifier, column)
      end
    end
  end

  # Add the table alias to the list of aliases
  # Even if it isn't been used in the result set,
  # we add a key for it with a nil value so we can check if it
  # is used more than once
  table_aliases = graph[:table_aliases]
  table_aliases[table_alias] = add_table ? dataset : nil

  # Add the columns to the selection unless we are ignoring them
  if add_table
    column_aliases = graph[:column_aliases]
    ca_num = graph[:column_alias_num]
    # Which columns to add to the result set
    cols = options[:select] || dataset.columns
    # If the column hasn't been used yet, don't alias it.
    # If it has been used, try table_column.
    # If that has been used, try table_column_N 
    # using the next value of N that we know hasn't been
    # used
    cols.each do |column|
      col_alias, identifier = if column_aliases[column]
        column_alias = :"#{table_alias}_#{column}"
        if column_aliases[column_alias]
          column_alias_num = ca_num[column_alias]
          column_alias = :"#{column_alias}_#{column_alias_num}" 
          ca_num[column_alias] += 1
        end
        [column_alias, SQL::AliasedExpression.new(SQL::QualifiedIdentifier.new(table_alias_qualifier, column), column_alias)]
      else
        ident = SQL::QualifiedIdentifier.new(table_alias_qualifier, column)
        [column, ident]
      end
      column_aliases[col_alias] = [table_alias, column].freeze
      select.push(identifier)
    end
  end
  [:column_aliases, :table_aliases, :column_alias_num].each{|k| graph[k].freeze}
  ds = ds.clone(:graph=>graph.freeze)
  ds.select(*select)
end