Method: Sequel::Dataset#hash
- Defined in:
- lib/sequel/dataset/misc.rb
#hash ⇒ Object
Define a hash value such that datasets with the same class, DB, and opts, will have the same hash value.
146 147 148 |
# File 'lib/sequel/dataset/misc.rb', line 146 def hash [self.class, db, opts].hash end |