Method: Sequel::Dataset#limit

Defined in:
lib/sequel/dataset/query.rb

#limit(l, o = (no_offset = true; nil)) ⇒ Object

If given an integer, the dataset will contain only the first l results. If given a range, it will contain only those at offsets within that range. If a second argument is given, it is used as an offset. To use an offset without a limit, pass nil as the first argument.

DB[:items].limit(10) # SELECT * FROM items LIMIT 10
DB[:items].limit(10, 20) # SELECT * FROM items LIMIT 10 OFFSET 20
DB[:items].limit(10...20) # SELECT * FROM items LIMIT 10 OFFSET 10
DB[:items].limit(10..20) # SELECT * FROM items LIMIT 11 OFFSET 10
DB[:items].limit(nil, 20) # SELECT * FROM items OFFSET 20


659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
# File 'lib/sequel/dataset/query.rb', line 659

def limit(l, o = (no_offset = true; nil))
  return from_self.limit(l, o) if @opts[:sql]

  if l.is_a?(Range)
    no_offset = false
    o = l.first
    l = l.last - l.first + (l.exclude_end? ? 0 : 1)
  end
  l = l.to_i if l.is_a?(String) && !l.is_a?(LiteralString)
  if l.is_a?(Integer)
    raise(Error, 'Limits must be greater than or equal to 1') unless l >= 1
  end

  ds = clone(:limit=>l)
  ds = ds.offset(o) unless no_offset
  ds
end