Method: Sequel::Dataset#requires_sql_standard_datetimes?

Defined in:
lib/sequel/dataset/features.rb

#requires_sql_standard_datetimes?Boolean

Whether the dataset requires SQL standard datetimes. False by default, as most allow strings with ISO 8601 format. Only for backwards compatibility, no longer used internally, do not use in new code.

Returns:

  • (Boolean)


33
34
35
36
# File 'lib/sequel/dataset/features.rb', line 33

def requires_sql_standard_datetimes?
  # SEQUEL6: Remove
  false
end