Method: Sequel::Deprecation.backtrace_filter

Defined in:
lib/sequel/deprecated.rb

.backtrace_filterObject

How to filter backtraces. false does not include backtraces, true includes full backtraces, an Integer includes that number of backtrace lines, and a proc is called with the backtrace line and line number to select the backtrace lines to include. The default is 10 backtrace lines.



30
31
32
# File 'lib/sequel/deprecated.rb', line 30

def backtrace_filter
  @backtrace_filter
end