Method: Sequel::Inflections.clear

Defined in:
lib/sequel/model/inflections.rb

.clear(scope = :all) ⇒ Object

Clears the loaded inflections within a given scope (default is :all). Give the scope as a symbol of the inflection type, the options are: :plurals, :singulars, :uncountables

Examples:

clear :all
clear :plurals


47
48
49
50
51
52
53
54
# File 'lib/sequel/model/inflections.rb', line 47

def self.clear(scope = :all)
  case scope
  when :all
    @plurals, @singulars, @uncountables = [], [], []
  else
    instance_variable_set("@#{scope}", [])
  end
end