Method: Sequel::Inflections.plurals
- Defined in:
- lib/sequel/model/inflections.rb
.plurals ⇒ Object (readonly)
Array of two element arrays, first containing a regex, and the second containing a substitution pattern, used for plurization.
32 33 34 |
# File 'lib/sequel/model/inflections.rb', line 32 def plurals @plurals end |