Method: Sequel::Model::Associations::AssociationReflection#remove_should_check_existing?

Defined in:
lib/sequel/model/associations.rb

#remove_should_check_existing?Boolean

Whether to check that an object to be disassociated is already associated to this object, false by default.

Returns:

  • (Boolean)


554
555
556
# File 'lib/sequel/model/associations.rb', line 554

def remove_should_check_existing?
  false
end