Method: Sequel::Model::Associations::ClassMethods#cache_associations

Defined in:
lib/sequel/model/associations.rb

#cache_associationsObject

Whether association metadata should be cached in the association reflection. If not cached, it will be computed on demand. In general you only want to set this to false when using code reloading. When using code reloading, setting this will make sure that if an associated class is removed or modified, this class will not have a reference to the previous class.



1611
1612
1613
# File 'lib/sequel/model/associations.rb', line 1611

def cache_associations
  @cache_associations
end