Method: Sequel::Model::Associations::ClassMethods#default_association_type_options

Defined in:
lib/sequel/model/associations.rb

#default_association_type_optionsObject

The default options to use for all associations of a given type. This is a hash keyed by association type symbol. If there is a value for the association type symbol key, the resulting hash will be merged into the association reflection hash for all association reflections of that type.



1620
1621
1622
# File 'lib/sequel/model/associations.rb', line 1620

def default_association_type_options
  @default_association_type_options
end