Method: Sequel::Model::Associations::DatasetMethods#association_join

Defined in:
lib/sequel/model/associations.rb

#association_join(*associations) ⇒ Object

Adds one or more INNER JOINs to the existing dataset using the keys and conditions specified by the given association(s). Take the same arguments as eager_graph, and operates similarly, but only adds the joins as opposed to making the other changes (such as adding selected columns and setting up eager loading).

The following methods also exist for specifying a different type of JOIN:

association_full_join

FULL JOIN

association_inner_join

INNER JOIN

association_left_join

LEFT JOIN

association_right_join

RIGHT JOIN

Examples:

# For each album, association_join load the artist
Album.association_join(:artist).all
# SELECT *
# FROM albums
# INNER JOIN artists AS artist ON (artists.id = albums.artist_id)

# For each album, association_join load the artist, using a specified alias
Album.association_join(Sequel[:artist].as(:a)).all
# SELECT *
# FROM albums
# INNER JOIN artists AS a ON (a.id = albums.artist_id)

# For each album, association_join load the artist and genre
Album.association_join(:artist, :genre).all
Album.association_join(:artist).association_join(:genre).all
# SELECT *
# FROM albums
# INNER JOIN artists AS artist ON (artist.id = albums.artist_id)
# INNER JOIN genres AS genre ON (genre.id = albums.genre_id)

# For each artist, association_join load albums and tracks for each album
Artist.association_join(albums: :tracks).all
# SELECT *
# FROM artists 
# INNER JOIN albums ON (albums.artist_id = artists.id)
# INNER JOIN tracks ON (tracks.album_id = albums.id)

# For each artist, association_join load albums, tracks for each album, and genre for each track
Artist.association_join(albums: {tracks: :genre}).all
# SELECT *
# FROM artists 
# INNER JOIN albums ON (albums.artist_id = artists.id)
# INNER JOIN tracks ON (tracks.album_id = albums.id)
# INNER JOIN genres AS genre ON (genre.id = tracks.genre_id)

# For each artist, association_join load albums with year > 1990
Artist.association_join(albums: proc{|ds| ds.where{year > 1990}}).all
# SELECT *
# FROM artists 
# INNER JOIN (
#   SELECT * FROM albums WHERE (year > 1990)
# ) AS albums ON (albums.artist_id = artists.id)

# For each artist, association_join load albums and tracks 1-10 for each album
Artist.association_join(albums: {tracks: proc{|ds| ds.where(number: 1..10)}}).all
# SELECT *
# FROM artists 
# INNER JOIN albums ON (albums.artist_id = artists.id)
# INNER JOIN (
#   SELECT * FROM tracks WHERE ((number >= 1) AND (number <= 10))
# ) AS tracks ON (tracks.albums_id = albums.id)

# For each artist, association_join load albums with year > 1990, and tracks for those albums
Artist.association_join(albums: {proc{|ds| ds.where{year > 1990}}=>:tracks}).all
# SELECT *
# FROM artists 
# INNER JOIN (
#   SELECT * FROM albums WHERE (year > 1990)
# ) AS albums ON (albums.artist_id = artists.id)
# INNER JOIN tracks ON (tracks.album_id = albums.id)


3050
3051
3052
# File 'lib/sequel/model/associations.rb', line 3050

def association_join(*associations)
  association_inner_join(*associations)
end