Method: Sequel::Model::Associations::EagerGraphLoader#column_maps

Defined in:
lib/sequel/model/associations.rb

#column_mapsObject (readonly)

Hash with table alias symbol keys and subhash values mapping column_alias symbols to the symbol of the real name of the column



3758
3759
3760
# File 'lib/sequel/model/associations.rb', line 3758

def column_maps
  @column_maps
end