Method: Sequel::Model::Associations::InstanceMethods#associations

Defined in:
lib/sequel/model/associations.rb

#associationsObject

The currently cached associations. A hash with the keys being the association name symbols and the values being the associated object or nil (many_to_one), or the array of associated objects (*_to_many).



2522
2523
2524
# File 'lib/sequel/model/associations.rb', line 2522

def associations
  @associations ||= {}
end