Method: Sequel::Model::Associations::ManyToManyAssociationReflection#join_table_source

Defined in:
lib/sequel/model/associations.rb

#join_table_sourceObject

The source of the join table. This is the join table itself, unless it is aliased, in which case it is the unaliased part.



1367
1368
1369
# File 'lib/sequel/model/associations.rb', line 1367

def join_table_source
  cached_fetch(:join_table_source){split_join_table_alias[0]}
end