Method: Sequel::Model::Associations::ManyToManyAssociationReflection#right_primary_key_method

Defined in:
lib/sequel/model/associations.rb

#right_primary_key_methodObject

The method symbol or array of method symbols to call on the associated objects to get the foreign key values for the join table.



1404
1405
1406
# File 'lib/sequel/model/associations.rb', line 1404

def right_primary_key_method
  cached_fetch(:right_primary_key_method){right_primary_key}
end