Method: Sequel::Model::Associations::ManyToManyAssociationReflection#right_primary_key_methods

Defined in:
lib/sequel/model/associations.rb

#right_primary_key_methodsObject

The array of method symbols to call on the associated objects to get the foreign key values for the join table.



1410
1411
1412
# File 'lib/sequel/model/associations.rb', line 1410

def right_primary_key_methods
  cached_fetch(:right_primary_key_methods){Array(right_primary_key_method)}
end