Method: Sequel::Model::Associations::ManyToOneAssociationReflection#primary_key_method

Defined in:
lib/sequel/model/associations.rb

#primary_key_methodObject

The method symbol or array of method symbols to call on the associated object to get the value to use for the foreign keys.



927
928
929
# File 'lib/sequel/model/associations.rb', line 927

def primary_key_method
 cached_fetch(:primary_key_method){primary_key}
end