Method: Sequel::Model::Associations::ManyToOneAssociationReflection#primary_key_methods

Defined in:
lib/sequel/model/associations.rb

#primary_key_methodsObject

The array of method symbols to call on the associated object to get the value to use for the foreign keys.



933
934
935
# File 'lib/sequel/model/associations.rb', line 933

def primary_key_methods
 cached_fetch(:primary_key_methods){Array(primary_key_method)}
end