Method: Sequel::Model::Associations::ManyToOneAssociationReflection#returns_array?
- Defined in:
- lib/sequel/model/associations.rb
#returns_array? ⇒ Boolean
Whether this association returns an array of objects instead of a single object, false for a many_to_one association.
949 950 951 |
# File 'lib/sequel/model/associations.rb', line 949 def returns_array? false end |