Method: Sequel::Model::Associations::OneToManyAssociationReflection#reciprocal_array?

Defined in:
lib/sequel/model/associations.rb

#reciprocal_array?Boolean

Whether the reciprocal of this association returns an array of objects instead of a single object, false for a one_to_many association.

Returns:

  • (Boolean)


1083
1084
1085
# File 'lib/sequel/model/associations.rb', line 1083

def reciprocal_array?
  false
end