Method: Sequel::Mysql2::Database#connect

Defined in:
lib/sequel/adapters/mysql2.rb

#connect(server) ⇒ Object

Connect to the database. In addition to the usual database options, the following options have effect:

:auto_is_null

Set to true to use MySQL default behavior of having a filter for an autoincrement column equals NULL to return the last inserted row.

:charset

Same as :encoding (:encoding takes precendence)

:encoding

Set all the related character sets for this connection (connection, client, database, server, and results).

The options hash is also passed to mysql2, and can include mysql2 options such as :local_infile.



37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
# File 'lib/sequel/adapters/mysql2.rb', line 37

def connect(server)
  opts = server_opts(server)
  opts[:username] ||= opts.delete(:user)
  opts[:flags] ||= 0
  opts[:flags] |= ::Mysql2::Client::FOUND_ROWS if ::Mysql2::Client.const_defined?(:FOUND_ROWS)
  opts[:encoding] ||= opts[:charset]
  conn = ::Mysql2::Client.new(opts)
  conn.query_options.merge!(:symbolize_keys=>true, :cache_rows=>false)
    
  if NativePreparedStatements
    conn.instance_variable_set(:@sequel_default_query_options, conn.query_options.dup)
  end

  sqls = mysql_connection_setting_sqls

  # Set encoding a slightly different way after connecting,
  # in case the READ_DEFAULT_GROUP overrode the provided encoding.
  # Doesn't work across implicit reconnects, but Sequel doesn't turn on
  # that feature.
  if encoding = opts[:encoding]
    sqls.unshift("SET NAMES #{conn.escape(encoding.to_s)}")
  end

  sqls.each{|sql| log_connection_yield(sql, conn){conn.query(sql)}}

  add_prepared_statements_cache(conn)
  conn
end