Method: Sequel::Plugins::AfterInitialize::InstanceMethods#after_initialize
- Defined in:
- lib/sequel/plugins/after_initialize.rb
#after_initialize ⇒ Object
An empty after_initialize hook, so that plugins that use this can always call super to get the default behavior.
34 35 |
# File 'lib/sequel/plugins/after_initialize.rb', line 34 def after_initialize end |