Method: Sequel::Plugins::AutoValidations::ClassMethods#auto_validate_max_length_columns

Defined in:
lib/sequel/plugins/auto_validations.rb

#auto_validate_max_length_columnsObject (readonly)

The columns or sets of columns with automatic max_length validations, as an array of pairs, with the first entry being the column name and second entry being the maximum length.



145
146
147
# File 'lib/sequel/plugins/auto_validations.rb', line 145

def auto_validate_max_length_columns
  @auto_validate_max_length_columns
end