Method: Sequel::Plugins::BlacklistSecurity::ClassMethods#restricted_columns
- Defined in:
- lib/sequel/plugins/blacklist_security.rb
#restricted_columns ⇒ Object (readonly)
Which columns are specifically restricted in a call to set/update/new/etc. (default: not set). Some columns are restricted regardless of this setting, such as the primary key column and columns in Model::RESTRICTED_SETTER_METHODS.
26 27 28 |
# File 'lib/sequel/plugins/blacklist_security.rb', line 26 def restricted_columns @restricted_columns end |