Method: Sequel::Plugins::BlacklistSecurity::InstanceMethods#update_except

Defined in:
lib/sequel/plugins/blacklist_security.rb

#update_except(hash, *except) ⇒ Object

Update all values using the entries in the hash, except for the keys given in except. You should probably use update_fields instead of this method, as blacklist approaches to security are a bad idea.

artist.update_except({name: 'Jim'}, :hometown) # UPDATE artists SET name = 'Jim' WHERE (id = 1)


84
85
86
# File 'lib/sequel/plugins/blacklist_security.rb', line 84

def update_except(hash, *except)
  update_restricted(hash, ExceptionList.new(except.flatten))
end