Method: Sequel::Plugins::ColumnEncryption::Cryptor#current_key_prefix

Defined in:
lib/sequel/plugins/column_encryption.rb

#current_key_prefix(search_type) ⇒ Object

The prefix string of columns for the given search type and the first configured encryption key. Used to find values that do not use this prefix in order to perform reencryption.



449
450
451
# File 'lib/sequel/plugins/column_encryption.rb', line 449

def current_key_prefix(search_type)
  urlsafe_encode64("#{search_type.chr}\0#{@key_id.chr}")
end