Method: Sequel::Plugins::ForceEncoding.configure

Defined in:
lib/sequel/plugins/force_encoding.rb

.configure(model, encoding = nil) ⇒ Object

Set the forced_encoding based on the value given in the plugin call. Note that if a the plugin has been previously loaded, any previous forced encoding is overruled, even if no encoding is given when calling the plugin.



28
29
30
# File 'lib/sequel/plugins/force_encoding.rb', line 28

def self.configure(model, encoding=nil)
  model.forced_encoding = encoding
end