Method: Sequel::Plugins::JsonSerializer::InstanceMethods#to_json

Defined in:
lib/sequel/plugins/json_serializer.rb

#to_json(*a) ⇒ Object

Return a string in JSON format. Accepts the following options:

:except

Symbol or Array of Symbols of columns not to include in the JSON output.

:include

Symbol, Array of Symbols, or a Hash with Symbol keys and Hash values specifying associations or other non-column attributes to include in the JSON output. Using a nested hash, you can pass options to associations to affect the JSON used for associated objects.

:only

Symbol or Array of Symbols of columns to only include in the JSON output, ignoring all other columns.

:root

Qualify the JSON with the name of the object. If a string is given, use the string as the key, otherwise use an underscored version of the model’s name.



279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
# File 'lib/sequel/plugins/json_serializer.rb', line 279

def to_json(*a)
  opts = model.json_serializer_opts
  opts = Hash[opts].merge!(@json_serializer_opts) if @json_serializer_opts
  if (arg_opts = a.first).is_a?(Hash)
    opts = Hash[opts].merge!(arg_opts)
    a = []
  end

  vals = values
  cols = if only = opts[:only]
    Array(only)
  else
    vals.keys - Array(opts[:except])
  end

  h = {}

  cols.each{|c| h[c.to_s] = get_column_value(c)}
  if inc = opts[:include]
    if inc.is_a?(Hash)
      inc.each do |k, v|
        v = v.empty? ? [] : [v]
        h[k.to_s] = case objs = send(k)
        when Array
          objs.map{|obj| Literal.new(Sequel.object_to_json(obj, *v))}
        else
          Literal.new(Sequel.object_to_json(objs, *v))
        end
      end
    else
      Array(inc).each{|c| h[c.to_s] = send(c)}
    end
  end

  if root = opts[:root]
    unless root.is_a?(String)
      root = model.send(:underscore, model.send(:demodulize, model.to_s))
    end
    h = {root => h}
  end

  Sequel.object_to_json(h, *a)
end