Method: Sequel::Plugins::List::InstanceMethods#prev

Defined in:
lib/sequel/plugins/list.rb

#prev(n = 1) ⇒ Object

The model instance the given number of places below this model instance in the list, or 1 place below if no argument is given.



183
184
185
# File 'lib/sequel/plugins/list.rb', line 183

def prev(n = 1)
  self.next(n * -1)
end