Method: Sequel::Plugins::PgArrayAssociations::ManyToPgArrayAssociationReflection#remove_before_destroy?

Defined in:
lib/sequel/plugins/pg_array_associations.rb

#remove_before_destroy?Boolean

Destroying the associated object automatically removes the association, since the association is stored in the associated object.

Returns:



150
151
152
# File 'lib/sequel/plugins/pg_array_associations.rb', line 150

def remove_before_destroy?
  false
end