Method: Sequel::Plugins::Touch::ClassMethods#touched_associations

Defined in:
lib/sequel/plugins/touch.rb

#touched_associationsObject (readonly)

A hash specifying the associations to touch when instances are updated or destroyed. Keys are association name symbols and values are column name symbols.



59
60
61
# File 'lib/sequel/plugins/touch.rb', line 59

def touched_associations
  @touched_associations
end