Method: Sequel::Postgres::JSONDatabaseMethods#wrap_json_primitives

Defined in:
lib/sequel/extensions/pg_json.rb

#wrap_json_primitivesObject

Whether to wrap JSON primitives instead of using Ruby objects. Wrapping the primitives allows the primitive values to roundtrip, but it can cause problems, especially as false/null JSON values will be treated as truthy in Ruby due to the wrapping. False by default.



358
359
360
# File 'lib/sequel/extensions/pg_json.rb', line 358

def wrap_json_primitives
  @wrap_json_primitives
end