Method: Sequel::Postgres::PGRow::DatabaseMethods#row_types

Defined in:
lib/sequel/extensions/pg_row.rb

#row_typesObject (readonly)

A hash mapping row type keys (usually symbols), to option hashes. At the least, the values will contain the :parser option for the Parser instance that the type will use.



389
390
391
# File 'lib/sequel/extensions/pg_row.rb', line 389

def row_types
  @row_types
end