Method: Sequel::SQL::Function#quoted
- Defined in:
- lib/sequel/sql.rb
#quoted ⇒ Object
Return a new function where the function name will be quoted if the database supports quoted functions:
Sequel.function(:foo).quoted # "foo"()
1456 1457 1458 |
# File 'lib/sequel/sql.rb', line 1456 def quoted with_opts(:quoted=>true) end |