Method: Sequel::SQL::Function#unquoted

Defined in:
lib/sequel/sql.rb

#unquotedObject

Return a new function where the function name will not be quoted even if the database supports quoted functions:

Sequel[:foo][:bar].function.unquoted # foo.bar()


1464
1465
1466
# File 'lib/sequel/sql.rb', line 1464

def unquoted
  with_opts(:quoted=>false)
end