Method: Sequel::Schema::AlterTableGenerator#add_foreign_key

Defined in:
lib/sequel/database/schema_generator.rb

#add_foreign_key(name, table, opts = OPTS) ⇒ Object

Add a foreign key with the given name and referencing the given table. See CreateTableGenerator#column for the available options.

You can also pass an array of column names for creating composite foreign keys. In this case, it will assume the columns exist and will only add the constraint. You can provide a :name option to name the constraint.

NOTE: If you need to add a foreign key constraint to a single existing column use the composite key syntax even if it is only one column.

add_foreign_key(:artist_id, :table) # ADD COLUMN artist_id integer REFERENCES table
add_foreign_key([:name], :table) # ADD FOREIGN KEY (name) REFERENCES table

PostgreSQL specific options:

:not_valid

Set to true to add the constraint with the NOT VALID syntax. This makes it so that future inserts must respect referential integrity, but allows the constraint to be added even if existing column values reference rows that do not exist. After all the existing data has been cleaned up, validate_constraint can be used to mark the constraint as valid. Note that this option only makes sense when using an array of columns.



520
521
522
523
# File 'lib/sequel/database/schema_generator.rb', line 520

def add_foreign_key(name, table, opts = OPTS)
  return add_composite_foreign_key(name, table, opts) if name.is_a?(Array)
  add_column(name, Integer, {:table=>table}.merge!(opts))
end