Method: Sequel::SequelMethods#condition_specifier?

Defined in:
lib/sequel/core.rb

#condition_specifier?(obj) ⇒ Boolean

Returns true if the passed object could be a specifier of conditions, false otherwise. Currently, Sequel considers hashes and arrays of two element arrays as condition specifiers.

Sequel.condition_specifier?({}) # => true
Sequel.condition_specifier?([[1, 2]]) # => true
Sequel.condition_specifier?([]) # => false
Sequel.condition_specifier?([1]) # => false
Sequel.condition_specifier?(1) # => false

Returns:

  • (Boolean)


83
84
85
86
87
88
89
90
91
92
# File 'lib/sequel/core.rb', line 83

def condition_specifier?(obj)
  case obj
  when Hash
    true
  when Array
    !obj.empty? && !obj.is_a?(SQL::ValueList) && obj.all?{|i| i.is_a?(Array) && (i.length == 2)}
  else
    false
  end
end