Method: Sequel::SequelMethods#object_to_json

Defined in:
lib/sequel/core.rb

#object_to_json(obj, *args, &block) ⇒ Object

Convert given object to json and return the result. This can be overridden to use an alternative json implementation.



198
199
200
# File 'lib/sequel/core.rb', line 198

def object_to_json(obj, *args, &block)
  obj.to_json(*args, &block)
end