Method: Sequel::SequelMethods#split_symbols=

Defined in:
lib/sequel/core.rb

#split_symbols=(v) ⇒ Object

Setting this to true enables Sequel’s historical behavior of splitting symbols on double or triple underscores:

:table__column         # table.column
:column___alias        # column AS alias
:table__column___alias # table.column AS alias

It is only recommended to turn this on for backwards compatibility until such symbols have been converted to use newer Sequel APIs such as:

Sequel[:table][:column]            # table.column
Sequel[:column].as(:alias)         # column AS alias
Sequel[:table][:column].as(:alias) # table.column AS alias

Sequel::Database instances do their own caching of literalized symbols, and changing this setting does not affect those caches. It is recommended that if you want to change this setting, you do so directly after requiring Sequel, before creating any Sequel::Database instances.

Disabling symbol splitting will also disable the handling of double underscores in virtual row methods, causing such methods to yield regular identifers instead of qualified identifiers:

# Sequel.split_symbols = true
Sequel.expr{table__column}  # table.column
Sequel.expr{table[:column]} # table.column

# Sequel.split_symbols = false
Sequel.expr{table__column}  # table__column
Sequel.expr{table[:column]} # table.column


296
297
298
299
# File 'lib/sequel/core.rb', line 296

def split_symbols=(v)
  Sequel.synchronize{SPLIT_SYMBOL_CACHE.clear}
  @split_symbols = v
end