Method: Sequel::SequelMethods#typecast_timezone

Defined in:
lib/sequel/timezones.rb

#typecast_timezoneObject (readonly)

The timezone that incoming data that Sequel needs to typecast is assumed to be already in (if they don’t include an offset).



42
43
44
# File 'lib/sequel/timezones.rb', line 42

def typecast_timezone
  @typecast_timezone
end