Method: Sequel::ShardedThreadedConnectionPool#allocated
- Defined in:
- lib/sequel/connection_pool/sharded_threaded.rb
#allocated(server = :default) ⇒ Object
A hash of connections currently being used for the given server, key is the Thread, value is the connection. Nonexistent servers will return nil. Treat this as read only, do not modify the resulting object. The calling code should already have the mutex before calling this.
54 55 56 |
# File 'lib/sequel/connection_pool/sharded_threaded.rb', line 54 def allocated(server=:default) @allocated[server] end |