Method: Sequel::ToDot::DatasetMethods#to_dot

Defined in:
lib/sequel/extensions/to_dot.rb

#to_dotObject

Return a string that can be processed by the dot program (included with graphviz) in order to see a visualization of the dataset’s abstract syntax tree.



21
22
23
# File 'lib/sequel/extensions/to_dot.rb', line 21

def to_dot
  ToDot.output(self)
end