Class: Sequel::BasicObject
- Inherits:
- BasicObject
- Defined in:
- lib/sequel/sql.rb
Overview
The Sequel::BasicObject class is just like the
default BasicObject class, except that missing constants are resolved in
Object. This allows the virtual row support to work with classes
without prefixing them with ::, such as:
DB[:bonds].where{maturity_date > Time.now}
Direct Known Subclasses
Database::AsyncThreadPool::BaseProxy, Dataset::Query, MigrationAlterTableReverser, MigrationDSL, MigrationReverser, Plugins::AssociationProxies::AssociationProxy, SQL::VirtualRow
Class Method Summary collapse
-
.const_missing(name) ⇒ Object
Lookup missing constants in ::Object.