Module: Sequel::ConnectionCheckoutEventCallbacks::ShardedTimedQueue
- Defined in:
- lib/sequel/extensions/connection_checkout_event_callback.rb
Instance Attribute Summary collapse
-
#on_checkout_event ⇒ Object
The callback that is called with connection checkout events.
Instance Attribute Details
#on_checkout_event ⇒ Object
The callback that is called with connection checkout events.
110 111 112 |
# File 'lib/sequel/extensions/connection_checkout_event_callback.rb', line 110 def on_checkout_event @on_checkout_event end |