Module: Sequel::ConnectionCheckoutEventCallbacks::TimedQueue
- Defined in:
- lib/sequel/extensions/connection_checkout_event_callback.rb
Instance Attribute Summary collapse
-
#on_checkout_event ⇒ Object
The callback that is called with connection checkout events.
Instance Attribute Details
#on_checkout_event ⇒ Object
The callback that is called with connection checkout events.
84 85 86 |
# File 'lib/sequel/extensions/connection_checkout_event_callback.rb', line 84 def on_checkout_event @on_checkout_event end |