Module: Sequel::Database::ResetIdentifierMangling
- Defined in:
- lib/sequel/database/misc.rb
Overview
Module to be included in shared adapters so that when the DatabaseMethods are included in the database, the identifier mangling defaults are reset correctly.
Class Method Summary collapse
Class Method Details
.extended(obj) ⇒ Object
32 33 34 35 36 37 |
# File 'lib/sequel/database/misc.rb', line 32 def self.extended(obj) # :nocov: Sequel::Deprecation.deprecate("Sequel::Database::ResetIdentifierMangling is no longer needed and will be removed in Sequel 5. Please update your adapter.") obj.send(:reset_identifier_mangling) if obj.respond_to?(:reset_identifier_mangling) # :nocov: end |