Class: Sequel::Firebird::Dataset
- Defined in:
- lib/sequel/adapters/firebird.rb
Overview
Dataset class for Firebird datasets
Constant Summary collapse
- BOOL_TRUE =
'1'.freeze
- BOOL_FALSE =
'0'.freeze
- NULL =
LiteralString.new('NULL').freeze
- COMMA_SEPARATOR =
', '.freeze
- FIREBIRD_TIMESTAMP_FORMAT =
"TIMESTAMP '%Y-%m-%d %H:%M:%S".freeze
- SELECT_CLAUSE_ORDER =
%w'with distinct limit columns from join where group having compounds order'.freeze
Constants inherited from Dataset
Dataset::AND_SEPARATOR, Dataset::ARRAY_ACCESS_ERROR_MSG, Dataset::COLUMN_CHANGE_OPTS, Dataset::COLUMN_REF_RE1, Dataset::COLUMN_REF_RE2, Dataset::COLUMN_REF_RE3, Dataset::COUNT_FROM_SELF_OPTS, Dataset::COUNT_OF_ALL_AS_COUNT, Dataset::DATASET_ALIAS_BASE_NAME, Dataset::GET_ERROR_MSG, Dataset::IMPORT_ERROR_MSG, Dataset::INSERT_SQL_BASE, Dataset::IS_LITERALS, Dataset::IS_OPERATORS, Dataset::MAP_ERROR_MSG, Dataset::MUTATION_METHODS, Dataset::NOTIMPL_MSG, Dataset::N_ARITY_OPERATORS, Dataset::PREPARED_ARG_PLACEHOLDER, Dataset::QUALIFY_KEYS, Dataset::QUESTION_MARK, Dataset::SQL_WITH, Dataset::STOCK_COUNT_OPTS, Dataset::TWO_ARITY_OPERATORS, Dataset::WILDCARD, Dataset::WITH_SUPPORTED
Instance Attribute Summary
Attributes inherited from Dataset
#db, #identifier_input_method, #identifier_output_method, #opts, #quote_identifiers, #row_proc
Instance Method Summary collapse
-
#fetch_rows(sql, &block) ⇒ Object
Yield all rows returned by executing the given SQL and converting the types.
-
#insert(*values) ⇒ Object
Insert given values into the database.
-
#insert_returning_pk_sql(*values) ⇒ Object
Use the RETURNING clause to return the primary key of the inserted record, if it exists.
-
#insert_returning_sql(returning, *values) ⇒ Object
Use the RETURNING clause to return the columns listed in returning.
-
#insert_select(*values) ⇒ Object
Insert a record returning the record inserted.
-
#select_clause_order ⇒ Object
The order of clauses in the SELECT SQL statement.
- #select_limit_sql(sql) ⇒ Object
-
#supports_intersect_except? ⇒ Boolean
Firebird does not support INTERSECT or EXCEPT.
Methods inherited from Dataset
#<<, #[], #[]=, #add_graph_aliases, #aliased_expression_sql, #all, #and, #array_sql, #as, #avg, #call, #case_expression_sql, #cast_sql, #clone, #column_all_sql, #columns, #columns!, #complex_expression_sql, #count, def_mutation_method, #def_mutation_method, #delete, #delete_sql, #distinct, #each, #each_page, #empty?, #except, #exclude, #exists, #filter, #first, #first_source_alias, #from, #from_self, #function_sql, #get, #graph, #grep, #group, #group_and_count, #having, #import, #initialize, #insert_multiple, #insert_sql, #inspect, #intersect, #interval, #invert, #join_clause_sql, #join_on_clause_sql, #join_table, #join_using_clause_sql, #last, #limit, #literal, #map, #max, #min, #multi_insert, #multi_insert_sql, #naked, #or, #order, #order_more, #ordered_expression_sql, #paginate, #placeholder_literal_string_sql, #prepare, #print, #qualified_identifier_sql, #qualify, #qualify_to, #qualify_to_first_source, #query, #quote_identifier, #quote_identifiers?, #quote_schema_table, #quoted_identifier, #range, #requires_sql_standard_datetimes?, #reverse_order, #schema_and_table, #select, #select_all, #select_more, #select_sql, #server, #set, #set_defaults, #set_graph_aliases, #set_overrides, #single_record, #single_value, #sql, #subscript_sql, #sum, #supports_cte?, #supports_distinct_on?, #supports_intersect_except_all?, #supports_is_true?, #supports_window_functions?, #to_csv, #to_hash, #unfiltered, #ungraphed, #union, #unordered, #update, #update_sql, #where, #window_function_sql, #window_sql, #with, #with_recursive, #with_sql
Methods included from Metaprogramming
Constructor Details
This class inherits a constructor from Sequel::Dataset
Instance Method Details
#fetch_rows(sql, &block) ⇒ Object
Yield all rows returned by executing the given SQL and converting the types.
210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 |
# File 'lib/sequel/adapters/firebird.rb', line 210 def fetch_rows(sql, &block) execute(sql) do |s| begin @columns = s.fields.map{|c| output_identifier(c.name)} s.fetchall(:symbols_hash).each do |r| h = {} r.each{|k,v| h[output_identifier(k)] = v} yield h end ensure s.close end end self end |
#insert(*values) ⇒ Object
Insert given values into the database.
227 228 229 230 231 232 233 234 |
# File 'lib/sequel/adapters/firebird.rb', line 227 def insert(*values) if !@opts[:sql] clone(default_server_opts(:sql=>insert_returning_pk_sql(*values))).single_value else execute_insert(insert_sql(*values), :table=>opts[:from].first, :values=>values.size == 1 ? values.first : values) end end |
#insert_returning_pk_sql(*values) ⇒ Object
Use the RETURNING clause to return the primary key of the inserted record, if it exists
237 238 239 240 |
# File 'lib/sequel/adapters/firebird.rb', line 237 def insert_returning_pk_sql(*values) pk = db.primary_key(opts[:from].first) insert_returning_sql(pk ? Sequel::SQL::Identifier.new(pk) : NULL, *values) end |
#insert_returning_sql(returning, *values) ⇒ Object
Use the RETURNING clause to return the columns listed in returning.
243 244 245 |
# File 'lib/sequel/adapters/firebird.rb', line 243 def insert_returning_sql(returning, *values) "#{insert_sql(*values)} RETURNING #{column_list(Array(returning))}" end |
#insert_select(*values) ⇒ Object
Insert a record returning the record inserted
248 249 250 |
# File 'lib/sequel/adapters/firebird.rb', line 248 def insert_select(*values) naked.clone(default_server_opts(:sql=>insert_returning_sql(nil, *values))).single_record end |
#select_clause_order ⇒ Object
The order of clauses in the SELECT SQL statement
253 254 255 |
# File 'lib/sequel/adapters/firebird.rb', line 253 def select_clause_order SELECT_CLAUSE_ORDER end |
#select_limit_sql(sql) ⇒ Object
257 258 259 260 |
# File 'lib/sequel/adapters/firebird.rb', line 257 def select_limit_sql(sql) sql << " FIRST #{@opts[:limit]}" if @opts[:limit] sql << " SKIP #{@opts[:offset]}" if @opts[:offset] end |
#supports_intersect_except? ⇒ Boolean
Firebird does not support INTERSECT or EXCEPT
263 264 265 |
# File 'lib/sequel/adapters/firebird.rb', line 263 def supports_intersect_except? false end |