Module: Sequel::JDBC::Derby::DatabaseMethods
- Includes:
- Transactions
- Defined in:
- lib/sequel/adapters/jdbc/derby.rb
Instance Method Summary collapse
-
#cast_type_literal(type) ⇒ Object
Derby doesn’t support casting integer to varchar, only integer to char, and char(254) appears to have the widest support (with char(255) failing).
- #database_type ⇒ Object
- #freeze ⇒ Object
-
#serial_primary_key_options ⇒ Object
Derby uses an IDENTITY sequence for autoincrementing columns.
-
#supports_transactional_ddl? ⇒ Boolean
Derby supports transactional DDL statements.
-
#svn_version ⇒ Object
The SVN version of the database.
Methods included from Transactions
#supports_savepoints?, #supports_transaction_isolation_levels?
Instance Method Details
#cast_type_literal(type) ⇒ Object
Derby doesn’t support casting integer to varchar, only integer to char, and char(254) appears to have the widest support (with char(255) failing). This does add a bunch of extra spaces at the end, but those will be trimmed elsewhere.
27 28 29 |
# File 'lib/sequel/adapters/jdbc/derby.rb', line 27 def cast_type_literal(type) (type == String) ? 'CHAR(254)' : super end |
#database_type ⇒ Object
31 32 33 |
# File 'lib/sequel/adapters/jdbc/derby.rb', line 31 def database_type :derby end |
#freeze ⇒ Object
35 36 37 38 |
# File 'lib/sequel/adapters/jdbc/derby.rb', line 35 def freeze svn_version super end |
#serial_primary_key_options ⇒ Object
Derby uses an IDENTITY sequence for autoincrementing columns.
41 42 43 |
# File 'lib/sequel/adapters/jdbc/derby.rb', line 41 def {:primary_key => true, :type => Integer, :identity=>true, :start_with=>1} end |
#supports_transactional_ddl? ⇒ Boolean
Derby supports transactional DDL statements.
55 56 57 |
# File 'lib/sequel/adapters/jdbc/derby.rb', line 55 def supports_transactional_ddl? true end |
#svn_version ⇒ Object
The SVN version of the database.
46 47 48 49 50 51 52 |
# File 'lib/sequel/adapters/jdbc/derby.rb', line 46 def svn_version @svn_version ||= begin v = synchronize{|c| c..get_database_product_version} v =~ /\((\d+)\)\z/ $1.to_i end end |