Class: Sequel::JDBC::Postgres::Dataset
- Includes:
- Postgres::DatasetMethods
- Defined in:
- lib/sequel/adapters/jdbc/postgresql.rb
Constant Summary
Constants included from Postgres::DatasetMethods
Postgres::DatasetMethods::LOCK_MODES, Postgres::DatasetMethods::NULL
Instance Method Summary collapse
-
#with_fetch_size(size) ⇒ Object
Warn when calling as the fetch size is ignored by the JDBC adapter currently.
Methods included from Postgres::DatasetMethods
#analyze, #complex_expression_sql_append, #disable_insert_returning, #empty?, #explain, #for_key_share, #for_no_key_update, #for_portion_of, #for_share, #full_text_search, #insert, #insert_conflict, #insert_ignore, #insert_select, #insert_select_sql, #join_table, #lock, #merge, #merge_delete_when_not_matched_by_source, #merge_do_nothing_when_matched, #merge_do_nothing_when_not_matched, #merge_do_nothing_when_not_matched_by_source, #merge_insert, #merge_update_when_not_matched_by_source, #overriding_system_value, #overriding_user_value, #supports_cte?, #supports_cte_in_subqueries?, #supports_distinct_on?, #supports_group_cube?, #supports_group_rollup?, #supports_grouping_sets?, #supports_insert_conflict?, #supports_insert_select?, #supports_lateral_subqueries?, #supports_merge?, #supports_modifying_joins?, #supports_nowait?, #supports_regexp?, #supports_returning?, #supports_skip_locked?, #supports_timestamp_timezones?, #supports_window_clause?, #supports_window_function_frame_option?, #supports_window_functions?, #truncate, #with_ties
Instance Method Details
#with_fetch_size(size) ⇒ Object
Warn when calling as the fetch size is ignored by the JDBC adapter currently.
185 186 187 188 |
# File 'lib/sequel/adapters/jdbc/postgresql.rb', line 185 def with_fetch_size(size) warn("Sequel::JDBC::Postgres::Dataset#with_fetch_size does not currently have an effect.", :uplevel=>1) super end |