Class: Sequel::Model::Associations::OneToManyAssociationReflection
- Inherits:
-
AssociationReflection
- Object
- Hash
- AssociationReflection
- Sequel::Model::Associations::OneToManyAssociationReflection
- Defined in:
- lib/sequel/model/associations.rb
Direct Known Subclasses
Constant Summary collapse
- FINALIZE_SETTINGS =
superclass::FINALIZE_SETTINGS.merge( :qualified_primary_key=>:qualified_primary_key ).freeze
Constants inherited from AssociationReflection
AssociationReflection::ASSOCIATION_DATASET_PROC
Instance Method Summary collapse
-
#apply_eager_graph_limit_strategy(strategy, ds) ⇒ Object
Support a lateral_subquery and correlated_subquery limit strategy when using eager_graph.
-
#associated_object_keys ⇒ Object
The keys in the associated model’s table related to this association.
-
#can_have_associated_objects?(obj) ⇒ Boolean
one_to_many associations can only have associated objects if none of the :keys options have a nil value.
-
#cloneable?(ref) ⇒ Boolean
one_to_many and one_to_one associations can be clones.
-
#default_key ⇒ Object
Default foreign key name symbol for key in associated table that points to current table’s primary key.
- #finalize_settings ⇒ Object
-
#handle_silent_modification_failure? ⇒ Boolean
Handle silent failure of add/remove methods if raise_on_save_failure is false.
-
#predicate_key ⇒ Object
(also: #qualified_key)
The hash key to use for the eager loading predicate (left side of IN (1, 2, 3)).
-
#primary_key ⇒ Object
The column in the current table that the key in the associated table references.
-
#qualified_primary_key ⇒ Object
#primary_key qualified by the current table.
-
#reciprocal_array? ⇒ Boolean
Whether the reciprocal of this association returns an array of objects instead of a single object, false for a one_to_many association.
-
#remove_before_destroy? ⇒ Boolean
Destroying one_to_many associated objects automatically deletes the foreign key.
-
#remove_should_check_existing? ⇒ Boolean
The one_to_many association needs to check that an object to be removed already is associated.
-
#set_reciprocal_to_self? ⇒ Boolean
One to many associations set the reciprocal to self when loading associated records.
Methods inherited from AssociationReflection
#_add_method, #_remove_all_method, #_remove_method, #_setter_method, #add_method, #apply_dataset_changes, #apply_distinct_on_eager_limit_strategy, #apply_eager_dataset_changes, #apply_eager_limit_strategy, #apply_ruby_eager_limit_strategy, #apply_window_function_eager_limit_strategy, #assign_singular?, #associated_class, #associated_dataset, #association_dataset_for, #association_dataset_proc, #association_method, #dataset_method, #dataset_need_primary_key?, #delete_row_number_column, #eager_graph_lazy_dataset?, #eager_graph_limit_strategy, #eager_limit_strategy, #eager_load_results, #eager_loader_key, #eager_loading_use_associated_key?, #filter_by_associations_add_conditions?, #filter_by_associations_conditions_expression, #finalize, #hash, #initialize_association_cache, #inspect, #limit_and_offset, #need_associated_primary_key?, #placeholder_loader, #predicate_key_values, #predicate_keys, #qualify, #qualify_assoc, #qualify_cur, #reciprocal, #remove_all_method, #remove_method, #returns_array?, #select, #setter_method, #slice_range
Methods included from Inflections
clear, irregular, plural, singular, uncountable
Methods inherited from Hash
#&, #case, #hstore, #pg_json, #pg_jsonb, #sql_expr, #sql_negate, #sql_or, #|, #~
Instance Method Details
#apply_eager_graph_limit_strategy(strategy, ds) ⇒ Object
Support a lateral_subquery and correlated_subquery limit strategy when using eager_graph.
1094 1095 1096 1097 1098 1099 1100 1101 |
# File 'lib/sequel/model/associations.rb', line 1094 def apply_eager_graph_limit_strategy(strategy, ds) case strategy when :correlated_subquery (ds) else super end end |
#associated_object_keys ⇒ Object
The keys in the associated model’s table related to this association
1104 1105 1106 |
# File 'lib/sequel/model/associations.rb', line 1104 def associated_object_keys self[:keys] end |
#can_have_associated_objects?(obj) ⇒ Boolean
one_to_many associations can only have associated objects if none of the :keys options have a nil value.
1110 1111 1112 |
# File 'lib/sequel/model/associations.rb', line 1110 def can_have_associated_objects?(obj) !self[:primary_keys].any?{|k| obj.get_column_value(k).nil?} end |
#cloneable?(ref) ⇒ Boolean
one_to_many and one_to_one associations can be clones
1115 1116 1117 |
# File 'lib/sequel/model/associations.rb', line 1115 def cloneable?(ref) ref[:type] == :one_to_many || ref[:type] == :one_to_one end |
#default_key ⇒ Object
Default foreign key name symbol for key in associated table that points to current table’s primary key.
1121 1122 1123 |
# File 'lib/sequel/model/associations.rb', line 1121 def default_key :"#{underscore(demodulize(self[:model].name))}_id" end |
#finalize_settings ⇒ Object
1128 1129 1130 |
# File 'lib/sequel/model/associations.rb', line 1128 def finalize_settings FINALIZE_SETTINGS end |
#handle_silent_modification_failure? ⇒ Boolean
Handle silent failure of add/remove methods if raise_on_save_failure is false.
1133 1134 1135 |
# File 'lib/sequel/model/associations.rb', line 1133 def handle_silent_modification_failure? self[:raise_on_save_failure] == false end |
#predicate_key ⇒ Object Also known as: qualified_key
The hash key to use for the eager loading predicate (left side of IN (1, 2, 3))
1138 1139 1140 |
# File 'lib/sequel/model/associations.rb', line 1138 def predicate_key cached_fetch(:predicate_key){qualify_assoc(self[:key])} end |
#primary_key ⇒ Object
The column in the current table that the key in the associated table references.
1144 1145 1146 |
# File 'lib/sequel/model/associations.rb', line 1144 def primary_key self[:primary_key] end |
#qualified_primary_key ⇒ Object
#primary_key qualified by the current table
1149 1150 1151 |
# File 'lib/sequel/model/associations.rb', line 1149 def qualified_primary_key cached_fetch(:qualified_primary_key){qualify_cur(primary_key)} end |
#reciprocal_array? ⇒ Boolean
Whether the reciprocal of this association returns an array of objects instead of a single object, false for a one_to_many association.
1155 1156 1157 |
# File 'lib/sequel/model/associations.rb', line 1155 def reciprocal_array? false end |
#remove_before_destroy? ⇒ Boolean
Destroying one_to_many associated objects automatically deletes the foreign key.
1160 1161 1162 |
# File 'lib/sequel/model/associations.rb', line 1160 def remove_before_destroy? false end |
#remove_should_check_existing? ⇒ Boolean
The one_to_many association needs to check that an object to be removed already is associated.
1165 1166 1167 |
# File 'lib/sequel/model/associations.rb', line 1165 def remove_should_check_existing? true end |
#set_reciprocal_to_self? ⇒ Boolean
One to many associations set the reciprocal to self when loading associated records.
1170 1171 1172 |
# File 'lib/sequel/model/associations.rb', line 1170 def set_reciprocal_to_self? true end |