Module: Sequel::Model::ClassMethods

Included in:
Sequel::Model
Defined in:
lib/sequel/model/base.rb

Overview

Class methods for Sequel::Model that implement basic model functionality.

  • All of the following methods have class methods created that send the method to the model’s dataset: all, any?, as_hash, as_set, avg, count, cross_join, distinct, each, each_server, empty?, except, exclude, exclude_having, fetch_rows, filter, first, first!, for_update, from, from_self, full_join, full_outer_join, get, graph, grep, group, group_and_count, group_append, group_by, having, import, inner_join, insert, intersect, invert, join, join_table, last, left_join, left_outer_join, limit, lock_style, map, max, min, multi_insert, naked, natural_full_join, natural_join, natural_left_join, natural_right_join, offset, order, order_append, order_by, order_more, order_prepend, paged_each, qualify, reverse, reverse_order, right_join, right_outer_join, select, select_all, select_append, select_group, select_hash, select_hash_groups, select_map, select_more, select_order_map, select_prepend, select_set, server, single_record, single_record!, single_value, single_value!, sum, to_hash, to_hash_groups, truncate, unfiltered, ungraphed, ungrouped, union, unlimited, unordered, where, where_all, where_each, where_single_value, with, with_recursive, with_sql

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#cache_anonymous_modelsObject

Whether to cache the anonymous models created by Sequel::Model(), true by default. This is required for reloading them correctly (avoiding the superclass mismatch).



29
30
31
# File 'lib/sequel/model/base.rb', line 29

def cache_anonymous_models
  @cache_anonymous_models
end

#dataset_method_modulesObject (readonly)

Array of modules that extend this model’s dataset. Stored so that if the model’s dataset is changed, it will be extended with all of these modules.



34
35
36
# File 'lib/sequel/model/base.rb', line 34

def dataset_method_modules
  @dataset_method_modules
end

#dataset_module_classObject (readonly)

The Module subclass to use for dataset_module blocks.



37
38
39
# File 'lib/sequel/model/base.rb', line 37

def dataset_module_class
  @dataset_module_class
end

#default_set_fields_optionsObject

The default options to use for Model#set_fields. These are merged with the options given to set_fields.



41
42
43
# File 'lib/sequel/model/base.rb', line 41

def default_set_fields_options
  @default_set_fields_options
end

#fast_instance_delete_sqlObject (readonly)

SQL string fragment used for faster DELETE statement creation when deleting/destroying model instances, or nil if the optimization should not be used. For internal use only.



45
46
47
# File 'lib/sequel/model/base.rb', line 45

def fast_instance_delete_sql
  @fast_instance_delete_sql
end

#fast_pk_lookup_sqlObject (readonly)

SQL string fragment used for faster lookups by primary key, or nil if the optimization should not be used. For internal use only.



49
50
51
# File 'lib/sequel/model/base.rb', line 49

def fast_pk_lookup_sql
  @fast_pk_lookup_sql
end

#instance_datasetObject (readonly)

The dataset that instance datasets (#this) are based on. Generally a naked version of the model’s dataset limited to one row. For internal use only.



53
54
55
# File 'lib/sequel/model/base.rb', line 53

def instance_dataset
  @instance_dataset
end

#pluginsObject (readonly)

Array of plugin modules loaded by this class

Sequel::Model.plugins
# => [Sequel::Model, Sequel::Model::Associations]


59
60
61
# File 'lib/sequel/model/base.rb', line 59

def plugins
  @plugins
end

#primary_keyObject (readonly)

The primary key for the class. Sequel can determine this automatically for many databases, but not all, so you may need to set it manually. If not determined automatically, the default is :id.



64
65
66
# File 'lib/sequel/model/base.rb', line 64

def primary_key
  @primary_key
end

#raise_on_save_failureObject

Whether to raise an error instead of returning nil on a failure to save/create/save_changes/update/destroy due to a validation failure or a before_* hook returning false (default: true).



69
70
71
# File 'lib/sequel/model/base.rb', line 69

def raise_on_save_failure
  @raise_on_save_failure
end

#raise_on_typecast_failureObject

Whether to raise an error when unable to typecast data for a column (default: false). This should be set to true if you want to have model setter methods raise errors if the argument cannot be typecast properly.



74
75
76
# File 'lib/sequel/model/base.rb', line 74

def raise_on_typecast_failure
  @raise_on_typecast_failure
end

#require_modificationObject

Whether to raise an error if an UPDATE or DELETE query related to a model instance does not modify exactly 1 row. If set to false, Sequel will not check the number of rows modified (default: true).



79
80
81
# File 'lib/sequel/model/base.rb', line 79

def require_modification
  @require_modification
end

#require_valid_tableObject

If true (the default), requires that all models have valid tables, raising exceptions if creating a model without a valid table backing it. Setting this to false will allow the creation of model classes where the underlying table doesn’t exist.



85
86
87
# File 'lib/sequel/model/base.rb', line 85

def require_valid_table
  @require_valid_table
end

#shape_friendlyObject

Whether the model uses a shape friendly design (initializing all potentially used instance variables to nil).



89
90
91
# File 'lib/sequel/model/base.rb', line 89

def shape_friendly
  @shape_friendly
end

#simple_pkObject

Should be the literal primary key column name if this Model’s table has a simple primary key, or nil if the model has a compound primary key or no primary key.



93
94
95
# File 'lib/sequel/model/base.rb', line 93

def simple_pk
  @simple_pk
end

#simple_tableObject

Should be the literal table name if this Model’s dataset is a simple table (no select, order, join, etc.), or nil otherwise. This and simple_pk are used for an optimization in Model[].



97
98
99
# File 'lib/sequel/model/base.rb', line 97

def simple_table
  @simple_table
end

#strict_param_settingObject

Whether mass assigning via .create/.new/#set/#update should raise an error if an invalid key is used. A key is invalid if no setter method exists for that key or the access to the setter method is restricted (e.g. due to it being a primary key field). If set to false, silently skip any key where the setter method doesn’t exist or access to it is restricted.



104
105
106
# File 'lib/sequel/model/base.rb', line 104

def strict_param_setting
  @strict_param_setting
end

#typecast_empty_string_to_nilObject

Whether to typecast the empty string (”) to nil for columns that are not string or blob. In most cases the empty string would be the way to specify a NULL SQL value in string form (nil.to_s == ”), and an empty string would not usually be typecast correctly for other types, so the default is true.



111
112
113
# File 'lib/sequel/model/base.rb', line 111

def typecast_empty_string_to_nil
  @typecast_empty_string_to_nil
end

#typecast_on_assignmentObject

Whether to typecast attribute values on assignment (default: true). If set to false, no typecasting is done, so it will be left up to the database to typecast the value correctly.



116
117
118
# File 'lib/sequel/model/base.rb', line 116

def typecast_on_assignment
  @typecast_on_assignment
end

#use_transactionsObject

Whether to use a transaction by default when saving/deleting records (default: true). If you are sending database queries in before_* or after_* hooks, you shouldn’t change the default setting without a good reason.



121
122
123
# File 'lib/sequel/model/base.rb', line 121

def use_transactions
  @use_transactions
end

Instance Method Details

#[](*args) ⇒ Object

Returns the first record from the database matching the conditions. If a hash is given, it is used as the conditions. If another object is given, it finds the first record whose primary key(s) match the given argument(s). If no object is returned by the dataset, returns nil.

Artist[1] # SELECT * FROM artists WHERE id = 1
# => #<Artist {:id=>1, ...}>

Artist[name: 'Bob'] # SELECT * FROM artists WHERE (name = 'Bob') LIMIT 1
# => #<Artist {:name=>'Bob', ...}>


218
219
220
221
# File 'lib/sequel/model/base.rb', line 218

def [](*args)
  args = args.first if args.size <= 1
  args.is_a?(Hash) ? first(args) : (primary_key_lookup(args) unless args.nil?)
end

#call(values) ⇒ Object

Initializes a model instance as an existing record. This constructor is used by Sequel to initialize model instances when fetching records. Requires that values be a hash where all keys are symbols. It probably should not be used by external code.



227
228
229
# File 'lib/sequel/model/base.rb', line 227

def call(values)
  allocate.initialize_from_db(values)
end

#clear_setter_methods_cacheObject

Clear the setter_methods cache



232
233
234
# File 'lib/sequel/model/base.rb', line 232

def clear_setter_methods_cache
  @setter_methods = nil unless frozen?
end

#columnsObject

Returns the columns in the result set in their original order. Generally, this will use the columns determined via the database schema, but in certain cases (e.g. models that are based on a joined dataset) it will use Dataset#columns to find the columns.

Artist.columns
# => [:id, :name]


243
244
245
246
247
# File 'lib/sequel/model/base.rb', line 243

def columns
  return @columns if @columns
  return nil if frozen?
  set_columns(dataset.naked.columns)
end

#create(values = OPTS, &block) ⇒ Object

Creates instance using new with the given values and block, and saves it.

Artist.create(name: 'Bob')
# INSERT INTO artists (name) VALUES ('Bob')

Artist.create do |a|
  a.name = 'Jim'
end # INSERT INTO artists (name) VALUES ('Jim')


257
258
259
# File 'lib/sequel/model/base.rb', line 257

def create(values = OPTS, &block)
  new(values, &block).save
end

#datasetObject

Returns the dataset associated with the Model class. Raises an Error if there is no associated dataset for this class. In most cases, you don’t need to call this directly, as Model proxies many dataset methods to the underlying dataset.

Artist.dataset.all # SELECT * FROM artists


267
268
269
# File 'lib/sequel/model/base.rb', line 267

def dataset
  @dataset || raise(Error, "No dataset associated with #{self}")
end

#dataset=(ds) ⇒ Object

Alias of set_dataset



272
273
274
# File 'lib/sequel/model/base.rb', line 272

def dataset=(ds)
  set_dataset(ds)
end

#dataset_module(mod = nil, &block) ⇒ Object

Extend the dataset with a module, similar to adding a plugin with the methods defined in DatasetMethods. This is the recommended way to add methods to model datasets.

If given an argument, it should be a module, and is used to extend the underlying dataset. Otherwise an anonymous module is created, and if a block is given, it is module_evaled, allowing you do define dataset methods directly using the standard ruby def syntax. Returns the module given or the anonymous module created.

# Usage with existing module
Album.dataset_module Sequel::ColumnsIntrospection

# Usage with anonymous module
Album.dataset_module do
  def foo
    :bar
  end
end
Album.dataset.foo
# => :bar
Album.foo
# => :bar

Any anonymous modules created are actually instances of Sequel::Model::DatasetModule (a Module subclass), which allows you to call the subset method on them, which defines a dataset method that adds a filter. There are also a number of other methods with the same names as the dataset methods, which can use to define named dataset methods:

Album.dataset_module do
  where(:released, Sequel[:release_date] <= Sequel::CURRENT_DATE)
  order :by_release_date, :release_date
  select :for_select_options, :id, :name, :release_date
end
Album.released.sql
# => "SELECT * FROM artists WHERE (release_date <= CURRENT_DATE)"
Album.by_release_date.sql
# => "SELECT * FROM artists ORDER BY release_date"
Album.for_select_options.sql
# => "SELECT id, name, release_date FROM artists"
Album.released.by_release_date.for_select_options.sql
# => "SELECT id, name, release_date FROM artists WHERE (release_date <= CURRENT_DATE) ORDER BY release_date"

The following methods are supported: distinct, eager, exclude, exclude_having, grep, group, group_and_count, group_append, having, limit, offset, order, order_append, order_prepend, select, select_all, select_append, select_group, where, and server.

The advantage of using these DatasetModule methods to define your dataset methods is that they can take advantage of dataset caching to improve performance.

Any public methods in the dataset module will have class methods created that call the method on the dataset, assuming that the class method is not already defined.



331
332
333
334
335
336
337
338
339
340
341
342
# File 'lib/sequel/model/base.rb', line 331

def dataset_module(mod = nil, &block)
  if mod
    raise Error, "can't provide both argument and block to Model.dataset_module" if block
    dataset_extend(mod)
    mod
  else
    @dataset_module ||= dataset_module_class.new(self)
    @dataset_module.module_eval(&block) if block
    dataset_extend(@dataset_module)
    @dataset_module
  end
end

#dbObject

Returns the database associated with the Model class. If this model doesn’t have a database associated with it, assumes the superclass’s database, or the first object in Sequel::DATABASES. If no Sequel::Database object has been created, raises an error.

Artist.db.transaction do # BEGIN
  Artist.create(name: 'Bob')
  # INSERT INTO artists (name) VALUES ('Bob')
end # COMMIT

Raises:



354
355
356
357
358
359
# File 'lib/sequel/model/base.rb', line 354

def db
  return @db if @db
  @db = self == Model ? Sequel.synchronize{DATABASES.first} : superclass.db
  raise(Error, "No database associated with #{self}: have you called Sequel.connect or #{self}.db= ?") unless @db
  @db
end

#db=(db) ⇒ Object

Sets the database associated with the Model class. Should only be used if the Model class currently does not have a dataset defined.

This can be used directly on Sequel::Model to set the default database to be used by subclasses, or to override the database used for specific models:

Sequel::Model.db = DB1
Artist = Class.new(Sequel::Model)
Artist.db = DB2

Note that you should not use this to change the model’s database at runtime. If you have that need, you should look into Sequel’s sharding support, or consider using separate model classes per Database.

Raises:



375
376
377
378
# File 'lib/sequel/model/base.rb', line 375

def db=(db)
  raise Error, "Cannot use Sequel::Model.db= on model with existing dataset.  Use Sequel::Model.dataset= instead." if @dataset
  @db = db
end

#db_schemaObject

Returns the cached schema information if available or gets it from the database. This is a hash where keys are column symbols and values are hashes of information related to the column. See Database#schema.

Artist.db_schema
# {:id=>{:type=>:integer, :primary_key=>true, ...},
#  :name=>{:type=>:string, :primary_key=>false, ...}}


388
389
390
391
392
# File 'lib/sequel/model/base.rb', line 388

def db_schema
  return @db_schema if @db_schema
  return nil if frozen?
  @db_schema = get_db_schema
end

#def_column_alias(meth, column) ⇒ Object

Create a column alias, where the column methods have one name, but the underlying storage uses a different name.



396
397
398
399
400
401
402
# File 'lib/sequel/model/base.rb', line 396

def def_column_alias(meth, column)
  clear_setter_methods_cache
  overridable_methods_module.module_eval do
    define_method(meth){self[column]}
    define_method("#{meth}="){|v| self[column] = v}
  end
end

#def_Model(mod) ⇒ Object

Define a Model method on the given module that calls the Model method on the receiver. This is how the Sequel::Model() method is defined, and allows you to define Model() methods on other modules, making it easier to have custom model settings for all models under a namespace. Example:

module Foo
  Model = Class.new(Sequel::Model)
  Model.def_Model(self)
  DB = Model.db = Sequel.connect(ENV['FOO_DATABASE_URL'])
  Model.plugin :prepared_statements

  class Bar < Model
    # Uses Foo::DB[:bars]
  end

  class Baz < Model(:my_baz)
    # Uses Foo::DB[:my_baz]
  end
end


143
144
145
146
147
148
# File 'lib/sequel/model/base.rb', line 143

def def_Model(mod)
  model = self
  mod.define_singleton_method(:Model) do |source|
    model.Model(source)
  end
end

#find(*args, &block) ⇒ Object

Finds a single record according to the supplied filter. You are encouraged to use Model[] or Model.first instead of this method.

Artist.find(name: 'Bob')
# SELECT * FROM artists WHERE (name = 'Bob') LIMIT 1

Artist.find{name > 'M'}
# SELECT * FROM artists WHERE (name > 'M') LIMIT 1


412
413
414
# File 'lib/sequel/model/base.rb', line 412

def find(*args, &block)
  first(*args, &block)
end

#find_or_create(cond, &block) ⇒ Object

Like find but invokes create with given conditions when record does not exist. Unlike find in that the block used in this method is not passed to find, but instead is passed to create only if find does not return an object.

Artist.find_or_create(name: 'Bob')
# SELECT * FROM artists WHERE (name = 'Bob') LIMIT 1
# INSERT INTO artists (name) VALUES ('Bob')

Artist.find_or_create(name: 'Jim'){|a| a.hometown = 'Sactown'}
# SELECT * FROM artists WHERE (name = 'Jim') LIMIT 1
# INSERT INTO artists (name, hometown) VALUES ('Jim', 'Sactown')


428
429
430
# File 'lib/sequel/model/base.rb', line 428

def find_or_create(cond, &block)
  find(cond) || create(cond, &block)
end

#freezeObject

Freeze a model class, disallowing any further changes to it.



433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
# File 'lib/sequel/model/base.rb', line 433

def freeze
  return self if frozen?
  dataset_module.freeze
  overridable_methods_module.freeze

  if @dataset
    db_schema.freeze.each_value(&:freeze)
    columns.freeze
    setter_methods.freeze
  else
    @setter_methods = [].freeze
  end

  @dataset_method_modules.freeze
  @default_set_fields_options.freeze
  @plugins.freeze

  super
end

#has_dataset?Boolean

Whether the model has a dataset. True for most model classes, but can be false if the model class is an abstract model class designed for subclassing, such as Sequel::Model itself.

Returns:

  • (Boolean)


456
457
458
# File 'lib/sequel/model/base.rb', line 456

def has_dataset?
  !@dataset.nil?
end

#implicit_table_nameObject

Returns the implicit table name for the model class, which is the demodulized, underscored, pluralized name of the class.

Artist.implicit_table_name # => :artists
Foo::ArtistAlias.implicit_table_name # => :artist_aliases


472
473
474
# File 'lib/sequel/model/base.rb', line 472

def implicit_table_name
  pluralize(underscore(demodulize(name))).to_sym
end

#include(*mods) ⇒ Object

Clear the setter_methods cache when a module is included, as it may contain setter methods.



462
463
464
465
# File 'lib/sequel/model/base.rb', line 462

def include(*mods)
  clear_setter_methods_cache
  super
end

#load(values) ⇒ Object

Calls #call with the values hash.



477
478
479
# File 'lib/sequel/model/base.rb', line 477

def load(values)
  call(values)
end

#Model(source) ⇒ Object

Lets you create a Model subclass with its dataset already set. source should be an instance of one of the following classes:

Database

Sets the database for this model to source. Generally only useful when subclassing directly from the returned class, where the name of the subclass sets the table name (which is combined with the Database in source to create the dataset to use)

Dataset

Sets the dataset for this model to source.

other

Sets the table name for this model to source. The class will use the default database for model classes in order to create the dataset.

The purpose of this method is to set the dataset/database automatically for a model class, if the table name doesn’t match the default table name that Sequel would use.

When creating subclasses of Sequel::Model itself, this method is usually called on Sequel itself, using Sequel::Model(:something).

# Using a symbol
class Comment < Sequel::Model(:something)
  table_name # => :something
end

# Using a dataset
class Comment < Sequel::Model(DB1[:something])
  dataset # => DB1[:something]
end

# Using a database
class Comment < Sequel::Model(DB1)
  dataset # => DB1[:comments]
end


185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
# File 'lib/sequel/model/base.rb', line 185

def Model(source)
  if cache_anonymous_models
    cache = Sequel.synchronize{@Model_cache ||= {}}
    if klass = Sequel.synchronize{cache[source]}
      return klass
    end
  end

  klass = Sequel.set_temp_name(Class.new(self)){"Sequel::_Model(#{source.inspect})"}

  if source.is_a?(::Sequel::Database)
    klass.db = source
  else
    klass.set_dataset(source)
  end

  if cache_anonymous_models
    Sequel.synchronize{cache[source] = klass}
  end

  klass
end

#no_primary_keyObject

Mark the model as not having a primary key. Not having a primary key can cause issues, among which is that you won’t be able to update records.

Artist.primary_key # => :id
Artist.no_primary_key
Artist.primary_key # => nil


487
488
489
490
# File 'lib/sequel/model/base.rb', line 487

def no_primary_key
  clear_setter_methods_cache
  self.simple_pk = @primary_key = nil
end

#plugin(plugin, *args, &block) ⇒ Object

Loads a plugin for use with the model class, passing optional arguments to the plugin. If the plugin is a module, load it directly. Otherwise, require the plugin from sequel/plugins/##plugin and then attempt to load the module using a the camelized plugin name under Sequel::Plugins.



496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
# File 'lib/sequel/model/base.rb', line 496

def plugin(plugin, *args, &block)
  m = plugin.is_a?(Module) ? plugin : plugin_module(plugin)

  if !m.respond_to?(:apply) && !m.respond_to?(:configure) && (!args.empty? || block)
    Deprecation.deprecate("Plugin #{plugin} accepts no arguments or block, and passing arguments/block to it", "Remove arguments and block when loading the plugin")
  end

  unless @plugins.include?(m)
    @plugins << m
    m.apply(self, *args, &block) if m.respond_to?(:apply)
    if m.const_defined?(:ClassMethods, false)
      class_methods = m::ClassMethods
      extend(class_methods)
      if class_methods.private_method_defined?(:each_model_instance_variable)
        def_initialize_nil_instance_variables
      end
    end
    include(m::InstanceMethods) if m.const_defined?(:InstanceMethods, false)
    if m.const_defined?(:DatasetMethods, false)
      dataset_extend(m::DatasetMethods, :create_class_methods=>false)
    end
  end

  m.configure(self, *args, &block) if m.respond_to?(:configure)
end

#primary_key_hash(value) ⇒ Object

Returns primary key attribute hash. If using a composite primary key value such be an array with values for each primary key in the correct order. For a standard primary key, value should be an object with a compatible type for the key. If the model does not have a primary key, raises an Error.

Artist.primary_key_hash(1) # => {:id=>1}
Artist.primary_key_hash([1, 2]) # => {:id1=>1, :id2=>2}


533
534
535
536
537
538
539
540
541
542
543
544
# File 'lib/sequel/model/base.rb', line 533

def primary_key_hash(value)
  case key = @primary_key
  when Symbol
    {key => value}
  when Array
    hash = {}
    key.zip(Array(value)){|k,v| hash[k] = v}
    hash
  else
    raise(Error, "#{self} does not have a primary key")
  end
end

#qualified_primary_key(qualifier = table_name) ⇒ Object

Return a qualified identifier or array of qualified identifiers for the model’s primary key. Uses the given qualifier if provided, or the table_name otherwise. If the model does not have a primary key, raises an Error.

Artist.order(Artist.qualified_primary_key)
# SELECT * FROM artists ORDER BY artists.id


553
554
555
556
557
558
559
560
561
562
# File 'lib/sequel/model/base.rb', line 553

def qualified_primary_key(qualifier=table_name)
  case key = @primary_key
  when Symbol
    SQL::QualifiedIdentifier.new(qualifier, key)
  when Array
    key.map{|k| SQL::QualifiedIdentifier.new(qualifier, k)}
  else
    raise(Error, "#{self} does not have a primary key")
  end
end

#qualified_primary_key_hash(value, qualifier = table_name) ⇒ Object

Return a hash where the keys are qualified column references. Uses the given qualifier if provided, or the table_name otherwise. This is useful if you plan to join other tables to this table and you want the column references to be qualified. If the model does not have a primary key, raises an Error.

Artist.where(Artist.qualified_primary_key_hash(1))
# SELECT * FROM artists WHERE (artists.id = 1)


571
572
573
574
575
576
577
578
579
580
581
582
# File 'lib/sequel/model/base.rb', line 571

def qualified_primary_key_hash(value, qualifier=table_name)
  case key = @primary_key
  when Symbol
    {SQL::QualifiedIdentifier.new(qualifier, key) => value}
  when Array
    hash = {}
    key.zip(Array(value)){|k,v| hash[SQL::QualifiedIdentifier.new(qualifier, k)] = v}
    hash
  else
    raise(Error, "#{self} does not have a primary key")
  end
end

#restrict_primary_keyObject

Restrict the setting of the primary key(s) when using mass assignment (e.g. set). Because this is the default, this only make sense to use in a subclass where the parent class has used unrestrict_primary_key.



587
588
589
590
# File 'lib/sequel/model/base.rb', line 587

def restrict_primary_key
  clear_setter_methods_cache
  @restrict_primary_key = true
end

#restrict_primary_key?Boolean

Whether or not setting the primary key(s) when using mass assignment (e.g. set) is restricted, true by default.

Returns:

  • (Boolean)


594
595
596
# File 'lib/sequel/model/base.rb', line 594

def restrict_primary_key?
  @restrict_primary_key
end

#set_dataset(ds, opts = OPTS) ⇒ Object

Sets the dataset associated with the Model class. ds can be a Symbol, LiteralString, SQL::Identifier, SQL::QualifiedIdentifier, SQL::AliasedExpression (all specifying a table name in the current database), or a Dataset. If a dataset is used, the model’s database is changed to the database of the given dataset. If a dataset is not used, a dataset is created from the current database with the table name given. Other arguments raise an Error. Returns self.

It also attempts to determine the database schema for the model, based on the given dataset.

Note that you should not use this to change the model’s dataset at runtime. If you have that need, you should look into Sequel’s sharding support, or creating a separate Model class per dataset

You should avoid calling this method directly if possible. Instead you should set the table name or dataset when creating the model class:

# table name
class Artist < Sequel::Model(:tbl_artists)
end

# dataset
class Artist < Sequel::Model(DB[:tbl_artists])
end


624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
# File 'lib/sequel/model/base.rb', line 624

def set_dataset(ds, opts=OPTS)
  inherited = opts[:inherited]
  @dataset = convert_input_dataset(ds)
  @require_modification = @dataset.provides_accurate_rows_matched? if require_modification.nil?
  if inherited
    self.simple_table = superclass.simple_table
    @columns = superclass.instance_variable_get(:@columns)
    @db_schema = superclass.instance_variable_get(:@db_schema)
  else
    @dataset = @dataset.with_extend(*@dataset_method_modules.reverse)
    @db_schema = get_db_schema
  end

  @fast_pk_lookup_sql = @fast_instance_delete_sql = nil unless @dataset.supports_placeholder_literalizer?
  reset_instance_dataset
  self
end

#set_model_instance_variables(*ivs) ⇒ Object

Set instance variables used by instances of this model class. Only has an effect if shape_friendly is true.



644
645
646
647
# File 'lib/sequel/model/base.rb', line 644

def set_model_instance_variables(*ivs)
  Plugins.model_instance_variables(singleton_class, *ivs)
  def_initialize_nil_instance_variables
end

#set_primary_key(key) ⇒ Object

Sets the primary key for this model. You can use either a regular or a composite primary key. To not use a primary key, set to nil or use no_primary_key. On most adapters, Sequel can automatically determine the primary key to use, so this method is not needed often.

class Person < Sequel::Model
  # regular key
  set_primary_key :person_id
end

class Tagging < Sequel::Model
  # composite key
  set_primary_key [:taggable_id, :tag_id]
end


663
664
665
666
667
668
669
670
671
672
673
674
675
676
# File 'lib/sequel/model/base.rb', line 663

def set_primary_key(key)
  clear_setter_methods_cache
  if key.is_a?(Array)
    if key.length < 2
      key = key.first
    else
      key = key.dup.freeze
    end
  end
  self.simple_pk = if key && !key.is_a?(Array)
    (@dataset || db).literal(key).freeze
  end
  @primary_key = key
end

#setter_methodsObject

Cache of setter methods to allow by default, in order to speed up mass assignment.



679
680
681
# File 'lib/sequel/model/base.rb', line 679

def setter_methods
  @setter_methods || (@setter_methods = get_setter_methods)
end

#table_nameObject

Returns name of primary table for the dataset. If the table for the dataset is aliased, returns the aliased name.

Artist.table_name # => :artists
Sequel::Model(:foo).table_name # => :foo
Sequel::Model(Sequel[:foo].as(:bar)).table_name # => :bar


695
696
697
# File 'lib/sequel/model/base.rb', line 695

def table_name
  dataset.first_source_alias
end

#unrestrict_primary_keyObject

Allow the setting of the primary key(s) when using the mass assignment methods. Using this method can open up security issues, be very careful before using it.

Artist.set(id: 1) # Error
Artist.unrestrict_primary_key
Artist.set(id: 1) # No Error


705
706
707
708
# File 'lib/sequel/model/base.rb', line 705

def unrestrict_primary_key
  clear_setter_methods_cache
  @restrict_primary_key = false
end

#with_pk(pk) ⇒ Object

Return the model instance with the primary key, or nil if there is no matching record.



711
712
713
# File 'lib/sequel/model/base.rb', line 711

def with_pk(pk)
  primary_key_lookup(pk)
end

#with_pk!(pk) ⇒ Object

Return the model instance with the primary key, or raise NoMatchingRow if there is no matching record.



716
717
718
# File 'lib/sequel/model/base.rb', line 716

def with_pk!(pk)
  with_pk(pk) || raise(NoMatchingRow.new(dataset))
end