Method: Sequel::Model::ClassMethods#allowed_columns

Defined in:
lib/sequel/model/base.rb

#allowed_columnsObject (readonly)

Which columns should be the only columns allowed in a call to a mass assignment method (e.g. set) (default: not set, so all columns not otherwise restricted are allowed).



13
14
15
# File 'lib/sequel/model/base.rb', line 13

def allowed_columns
  @allowed_columns
end