Module: Sequel::Model::DatasetMethods

Defined in:
lib/sequel/model/base.rb

Overview

DatasetMethods contains methods that all model datasets have.

Instance Method Summary collapse

Instance Method Details

#[](*args) ⇒ Object

Assume if a single integer is given that it is a lookup by primary key, and call with_pk with the argument.

Artist.dataset[1] # SELECT * FROM artists WHERE (id = 1) LIMIT 1


2287
2288
2289
2290
2291
2292
2293
# File 'lib/sequel/model/base.rb', line 2287

def [](*args)
  if args.length == 1 && (i = args[0]) && i.is_a?(Integer)
    with_pk(i)
  else
    super
  end
end

#as_hash(key_column = nil, value_column = nil, opts = OPTS) ⇒ Object

This allows you to call as_hash without any arguments, which will result in a hash with the primary key value being the key and the model object being the value.

Artist.dataset.as_hash # SELECT * FROM artists
# => {1=>#<Artist {:id=>1, ...}>,
#     2=>#<Artist {:id=>2, ...}>,
#     ...}


2347
2348
2349
2350
2351
2352
2353
2354
# File 'lib/sequel/model/base.rb', line 2347

def as_hash(key_column=nil, value_column=nil, opts=OPTS)
  if key_column
    super
  else
    raise(Sequel::Error, "No primary key for model") unless model && (pk = model.primary_key)
    super(pk, value_column, opts) 
  end
end

#destroyObject

Destroy each row in the dataset by instantiating it and then calling destroy on the resulting model object. This isn't as fast as deleting the dataset, which does a single SQL call, but this runs any destroy hooks on each object in the dataset.

Artist.dataset.destroy
# DELETE FROM artists WHERE (id = 1)
# DELETE FROM artists WHERE (id = 2)
# ...


2304
2305
2306
2307
2308
# File 'lib/sequel/model/base.rb', line 2304

def destroy
  @db.transaction(:server=>opts[:server], :skip_transaction=>model.use_transactions == false) do
    all(&:destroy).length
  end
end

#last(*a, &block) ⇒ Object

If there is no order already defined on this dataset, order it by the primary key and call last.

Album.last
# SELECT * FROM albums ORDER BY id DESC LIMIT 1


2315
2316
2317
2318
2319
2320
2321
# File 'lib/sequel/model/base.rb', line 2315

def last(*a, &block)
  if ds = _primary_key_order
    ds.last(*a, &block)
  else
    super
  end
end

#modelObject

The model class associated with this dataset

Artist.dataset.model # => Artist


2279
2280
2281
# File 'lib/sequel/model/base.rb', line 2279

def model
  @opts[:model]
end

#paged_each(*a, &block) ⇒ Object

If there is no order already defined on this dataset, order it by the primary key and call paged_each.

Album.paged_each{|row| }
# SELECT * FROM albums ORDER BY id LIMIT 1000 OFFSET 0
# SELECT * FROM albums ORDER BY id LIMIT 1000 OFFSET 1000
# SELECT * FROM albums ORDER BY id LIMIT 1000 OFFSET 2000
# ...


2331
2332
2333
2334
2335
2336
2337
# File 'lib/sequel/model/base.rb', line 2331

def paged_each(*a, &block)
  if ds = _primary_key_order
    ds.paged_each(*a, &block)
  else
    super
  end
end

#to_hash(*a) ⇒ Object

Alias of as_hash for backwards compatibility.



2357
2358
2359
# File 'lib/sequel/model/base.rb', line 2357

def to_hash(*a)
  as_hash(*a)
end

#with_pk(pk) ⇒ Object

Given a primary key value, return the first record in the dataset with that primary key value. If no records matches, returns nil.

# Single primary key
Artist.dataset.with_pk(1)
# SELECT * FROM artists WHERE (artists.id = 1) LIMIT 1

# Composite primary key
Artist.dataset.with_pk([1, 2])
# SELECT * FROM artists WHERE ((artists.id1 = 1) AND (artists.id2 = 2)) LIMIT 1


2371
2372
2373
2374
2375
2376
2377
# File 'lib/sequel/model/base.rb', line 2371

def with_pk(pk)
  if pk && (loader = _with_pk_loader)
    loader.first(*pk)
  else
    first(model.qualified_primary_key_hash(pk))
  end
end

#with_pk!(pk) ⇒ Object

Same as with_pk, but raises NoMatchingRow instead of returning nil if no row matches.



2381
2382
2383
# File 'lib/sequel/model/base.rb', line 2381

def with_pk!(pk)
  with_pk(pk) || raise(NoMatchingRow.new(self))
end