Class: Sequel::MySQL::Database

Inherits:
Database show all
Includes:
DatabaseMethods
Defined in:
lib/sequel/adapters/mysql.rb

Overview

Database class for MySQL databases used with Sequel.

Constant Summary collapse

MYSQL_DATABASE_DISCONNECT_ERRORS =

Mysql::Error messages that indicate the current connection should be disconnected

/\ACommands out of sync; you can't run this command now\z/

Constants included from DatabaseMethods

Sequel::MySQL::DatabaseMethods::AUTO_INCREMENT, Sequel::MySQL::DatabaseMethods::CAST_TYPES, Sequel::MySQL::DatabaseMethods::PRIMARY

Constants inherited from Database

Database::ADAPTERS, Database::AUTOINCREMENT, Database::CASCADE, Database::COMMA_SEPARATOR, Database::MYSQL_TIMESTAMP_RE, Database::NOT_NULL, Database::NO_ACTION, Database::NULL, Database::POSTGRES_DEFAULT_RE, Database::PRIMARY_KEY, Database::RESTRICT, Database::SET_DEFAULT, Database::SET_NULL, Database::SQL_BEGIN, Database::SQL_COMMIT, Database::SQL_RELEASE_SAVEPOINT, Database::SQL_ROLLBACK, Database::SQL_ROLLBACK_TO_SAVEPOINT, Database::SQL_SAVEPOINT, Database::STRING_DEFAULT_RE, Database::TEMPORARY, Database::TRANSACTION_BEGIN, Database::TRANSACTION_COMMIT, Database::TRANSACTION_ROLLBACK, Database::UNDERSCORE, Database::UNIQUE, Database::UNSIGNED

Instance Attribute Summary

Attributes inherited from Database

#default_schema, #loggers, #opts, #pool, #prepared_statements

Instance Method Summary collapse

Methods included from DatabaseMethods

#cast_type_literal, #database_type, #indexes, #supports_savepoints?, #tables, #use

Methods inherited from Database

#<<, #[], adapter_class, adapter_scheme, #add_column, #add_index, #alter_table, #call, #cast_type_literal, connect, #create_or_replace_view, #create_table, #create_table!, #create_table?, #create_view, #database_type, #disconnect, #drop_column, #drop_index, #drop_table, #drop_view, #dump_indexes_migration, #dump_schema_migration, #dump_table_schema, #execute_ddl, #execute_dui, #execute_insert, #fetch, #from, #get, #identifier_input_method, identifier_input_method, #identifier_input_method=, identifier_input_method=, identifier_output_method, #identifier_output_method, identifier_output_method=, #identifier_output_method=, #initialize, #inspect, #literal, #log_info, #logger=, #query, #quote_identifiers=, quote_identifiers=, #quote_identifiers?, #rename_column, #rename_table, #schema, #select, #serial_primary_key_options, #set_column_default, #set_column_type, single_threaded=, #single_threaded?, #supports_savepoints?, #synchronize, #table_exists?, #test_connection, #transaction, #typecast_value, #uri, #url

Methods included from Sequel::Metaprogramming

#meta_def

Constructor Details

This class inherits a constructor from Sequel::Database

Instance Method Details

#call_sproc(name, opts = {}, &block) ⇒ Object

Support stored procedures on MySQL



65
66
67
68
# File 'lib/sequel/adapters/mysql.rb', line 65

def call_sproc(name, opts={}, &block)
  args = opts[:args] || [] 
  execute("CALL #{name}#{args.empty? ? '()' : literal(args)}", opts.merge(:sproc=>false), &block)
end

#connect(server) ⇒ Object

Connect to the database. In addition to the usual database options, the following options have effect:

  • :auto_is_null - Set to true to use MySQL default behavior of having a filter for an autoincrement column equals NULL to return the last inserted row.
  • :charset - Same as :encoding (:encoding takes precendence)
  • :compress - Set to false to not compress results from the server
  • :encoding - Set all the related character sets for this connection (connection, client, database, server, and results).
  • :socket - Use a unix socket file instead of connecting via TCP/IP.
  • :timeout - Set the timeout in seconds before the server will disconnect this connection.


83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
# File 'lib/sequel/adapters/mysql.rb', line 83

def connect(server)
  opts = server_opts(server)
  conn = Mysql.init
  conn.options(Mysql::OPT_LOCAL_INFILE, "client")
  if encoding = opts[:encoding] || opts[:charset]
    # set charset _before_ the connect. using an option instead of "SET (NAMES|CHARACTER_SET_*)" works across reconnects
    conn.options(Mysql::SET_CHARSET_NAME, encoding)
  end
  conn.real_connect(
    opts[:host] || 'localhost',
    opts[:user],
    opts[:password],
    opts[:database],
    opts[:port],
    opts[:socket],
    Mysql::CLIENT_MULTI_RESULTS +
    Mysql::CLIENT_MULTI_STATEMENTS +
    (opts[:compress] == false ? 0 : Mysql::CLIENT_COMPRESS)
  )

  # increase timeout so mysql server doesn't disconnect us
  conn.query("set @@wait_timeout = #{opts[:timeout] || 2592000}")

  # By default, MySQL 'where id is null' selects the last inserted id
  conn.query("set SQL_AUTO_IS_NULL=0") unless opts[:auto_is_null]
  
  class << conn
    attr_accessor :prepared_statements
  end
  conn.prepared_statements = {}
  conn.reconnect = true
  conn
end

#dataset(opts = nil) ⇒ Object

Returns instance of Sequel::MySQL::Dataset with the given options.



118
119
120
# File 'lib/sequel/adapters/mysql.rb', line 118

def dataset(opts = nil)
  MySQL::Dataset.new(self, opts)
end

#execute(sql, opts = {}, &block) ⇒ Object

Executes the given SQL using an available connection, yielding the connection if the block is given.



124
125
126
127
128
129
130
131
132
# File 'lib/sequel/adapters/mysql.rb', line 124

def execute(sql, opts={}, &block)
  return call_sproc(sql, opts, &block) if opts[:sproc]
  return execute_prepared_statement(sql, opts, &block) if Symbol === sql
  begin
    synchronize(opts[:server]){|conn| _execute(conn, sql, opts, &block)}
  rescue Mysql::Error => e
    raise_error(e)
  end
end

#server_version(server = nil) ⇒ Object

Return the version of the MySQL server two which we are connecting.



135
136
137
# File 'lib/sequel/adapters/mysql.rb', line 135

def server_version(server=nil)
  @server_version ||= (synchronize(server){|conn| conn.server_version if conn.respond_to?(:server_version)} || super)
end