Class: Sequel::MySQL::Dataset

Inherits:
Dataset
  • Object
show all
Includes:
DatasetMethods, MysqlMysql2::DatasetMethods, PreparedStatements::DatasetMethods
Defined in:
lib/sequel/adapters/mysql.rb

Constant Summary

Constants included from PreparedStatements::DatasetMethods

PreparedStatements::DatasetMethods::PreparedStatementMethods

Constants included from MysqlMysql2::DatasetMethods

MysqlMysql2::DatasetMethods::StoredProcedureMethods

Constants included from DatasetMethods

Sequel::MySQL::DatasetMethods::MATCH_AGAINST, Sequel::MySQL::DatasetMethods::MATCH_AGAINST_BOOLEAN

Instance Method Summary collapse

Methods included from Dataset::StoredProcedures

#call_sproc, #prepare_sproc

Methods included from DatasetMethods

#calc_found_rows, #complex_expression_sql_append, #constant_sql_append, #delete_from, #distinct, #explain, #for_share, #full_text_search, #full_text_sql, #insert_ignore, #insert_select, #insert_select_sql, #on_duplicate_key_update, #quoted_identifier_append, #supports_cte?, #supports_derived_column_lists?, #supports_distinct_on?, #supports_group_rollup?, #supports_intersect_except?, #supports_limits_in_correlated_subqueries?, #supports_modifying_joins?, #supports_nowait?, #supports_ordered_distinct_on?, #supports_regexp?, #supports_returning?, #supports_skip_locked?, #supports_timestamp_usecs?, #supports_window_clause?, #supports_window_functions?, #update_ignore

Methods included from Dataset::Replace

#multi_replace, #replace, #replace_sql, #supports_replace?

Instance Method Details

#fetch_rows(sql) ⇒ Object

Yield all rows matching this dataset. If the dataset is set to split multiple statements, yield arrays of hashes one per statement instead of yielding results for all statements as hashes.



305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
# File 'lib/sequel/adapters/mysql.rb', line 305

def fetch_rows(sql)
  execute(sql) do |r|
    i = -1
    cps = db.conversion_procs
    cols = r.fetch_fields.map do |f| 
      # Pretend tinyint is another integer type if its length is not 1, to
      # avoid casting to boolean if convert_tinyint_to_bool is set.
      type_proc = f.type == 1 && cast_tinyint_integer?(f) ? cps[2] : cps[f.type]
      [output_identifier(f.name), type_proc, i+=1]
    end
    self.columns = cols.map(&:first)
    if opts[:split_multiple_result_sets]
      s = []
      yield_rows(r, cols){|h| s << h}
      yield s
    else
      yield_rows(r, cols){|h| yield h}
    end
  end
  self
end

#graphObject

Don't allow graphing a dataset that splits multiple statements

Raises:



328
329
330
331
# File 'lib/sequel/adapters/mysql.rb', line 328

def graph(*)
  raise(Error, "Can't graph a dataset that splits multiple result sets") if opts[:split_multiple_result_sets]
  super
end

#split_multiple_result_setsObject

Makes each yield arrays of rows, with each array containing the rows for a given result set. Does not work with graphing. So you can submit SQL with multiple statements and easily determine which statement returned which results.

Modifies the row_proc of the returned dataset so that it still works as expected (running on the hashes instead of on the arrays of hashes). If you modify the row_proc afterward, note that it will receive an array of hashes instead of a hash.

Raises:



342
343
344
345
346
347
# File 'lib/sequel/adapters/mysql.rb', line 342

def split_multiple_result_sets
  raise(Error, "Can't split multiple statements on a graphed dataset") if opts[:graph]
  ds = clone(:split_multiple_result_sets=>true)
  ds = ds.with_row_proc(proc{|x| x.map{|h| row_proc.call(h)}}) if row_proc
  ds
end