Method: Sequel::MySQL::Dataset#split_multiple_result_sets

Defined in:
lib/sequel/adapters/mysql.rb

#split_multiple_result_setsObject

Makes each yield arrays of rows, with each array containing the rows for a given result set. Does not work with graphing. So you can submit SQL with multiple statements and easily determine which statement returned which results.

Modifies the row_proc of the returned dataset so that it still works as expected (running on the hashes instead of on the arrays of hashes). If you modify the row_proc afterward, note that it will receive an array of hashes instead of a hash.

Raises:



317
318
319
320
321
322
# File 'lib/sequel/adapters/mysql.rb', line 317

def split_multiple_result_sets
  raise(Error, "Can't split multiple statements on a graphed dataset") if opts[:graph]
  ds = clone(:split_multiple_result_sets=>true)
  ds.row_proc = proc{|x| x.map{|h| row_proc.call(h)}} if row_proc
  ds
end