Module: Sequel::Plugins::DatasetAssociations::DatasetMethods

Defined in:
lib/sequel/plugins/dataset_associations.rb

Instance Method Summary collapse

Instance Method Details

#associated(name) ⇒ Object

For the association given by name, return a dataset of associated objects such that it would return the union of calling the association method on all objects returned by the current dataset.

This supports most options that are supported when eager loading. However, it will only work for limited associations or *_one associations with orders if the database supports window functions.

Raises:



83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
# File 'lib/sequel/plugins/dataset_associations.rb', line 83

def associated(name)
  raise Error, "unrecognized association name: #{name.inspect}" unless r = model.association_reflection(name)
  klass = r.associated_class
  sds = opts[:limit] ? self : unordered

  if r.send(:filter_by_associations_limit_strategy) == :lateral_subquery
    ds = r.send(:associated_eager_dataset)

    case r[:type]
    when :one_to_one, :one_to_many
      sds = sds.select(*Array(r.qualified_primary_key))
    else
      sds = sds.select(*r[:left_primary_keys])
      ds = ds.select_all(klass.table_name)
      update_select = true
    end

    ds = r.send(:apply_lateral_subquery_eager_limit_strategy, ds, sds, r.limit_and_offset)
    ds = ds.clone(:select=>ds.opts[:select][0,1]) if update_select
    return ds.clone(:eager=>nil, :eager_graph=>nil)
  end

  ds = klass.dataset
  ds = case r[:type]
  when :many_to_one
    ds.where(r.qualified_primary_key=>sds.select(*Array(r[:qualified_key])))
  when :one_to_one, :one_to_many
    r.send(:apply_filter_by_associations_limit_strategy, ds.where(r.qualified_key=>sds.select(*Array(r.qualified_primary_key))))
  when :many_to_many, :one_through_one
    mds = r.associated_class.dataset.
      join(r[:join_table], r[:right_keys].zip(r.right_primary_keys)).
      select(*Array(r.qualified_right_key)).
      where(r.qualify(r.join_table_alias, r[:left_keys])=>sds.select(*r.qualify(model.table_name, r[:left_primary_key_columns])))
    ds.where(r.qualified_right_primary_key=>r.send(:apply_filter_by_associations_limit_strategy, mds))
  when :many_through_many, :one_through_many
    if r.reverse_edges.empty?
      mds = r.associated_dataset
      fe = r.edges.first
      selection = Array(r.qualify(fe[:table], r.final_edge[:left]))
      predicate_key = r.qualify(fe[:table], fe[:right])
    else
      mds = model.dataset
      iq = model.table_name
      edges = r.edges.map(&:dup)
      edges << r.final_edge.dup
      edges.each do |e|
        alias_expr = e[:table]
        aliaz = mds.unused_table_alias(e[:table])
        unless aliaz == alias_expr
          alias_expr = Sequel.as(e[:table], aliaz)
        end
        e[:alias] = aliaz
        mds = mds.join(alias_expr, Array(e[:right]).zip(Array(e[:left])), :implicit_qualifier=>iq)
        iq = nil
      end
      fe, f1e, f2e = edges.values_at(0, -1, -2)
      selection = Array(r.qualify(f2e[:alias], f1e[:left]))
      predicate_key = r.qualify(fe[:alias], fe[:right])
    end

    mds = mds.
      select(*selection).
      where(predicate_key=>sds.select(*r.qualify(model.table_name, r[:left_primary_key_columns])))
    ds.where(r.qualified_right_primary_key=>r.send(:apply_filter_by_associations_limit_strategy, mds))
  when :pg_array_to_many
    ds.where(Sequel[r.primary_key=>sds.select{Sequel.pg_array_op(r.qualify(r[:model].table_name, r[:key])).unnest}])
  when :many_to_pg_array
    ds.where(Sequel.function(:coalesce, Sequel.pg_array_op(r[:key]).overlaps(sds.select{array_agg(r.qualify(r[:model].table_name, r.primary_key))}), false))
  else
    raise Error, "unrecognized association type for association #{name.inspect}: #{r[:type].inspect}"
  end

  ds = r.apply_eager_dataset_changes(ds).unlimited

  if r[:dataset_associations_join]
    case r[:type]
    when :many_to_many, :one_through_one
      ds = ds.join(r[:join_table], r[:right_keys].zip(r.right_primary_keys))
    when :many_through_many, :one_through_many
      (r.reverse_edges + [r.final_reverse_edge]).each{|e| ds = ds.join(e[:table], e.fetch(:only_conditions, (Array(e[:left]).zip(Array(e[:right])) + Array(e[:conditions]))), :table_alias=>ds.unused_table_alias(e[:table]), :qualify=>:deep, &e[:block])}
    end
  end

  ds
end