Module: Sequel::Plugins::ForbidLazyLoad::InstanceMethods
- Defined in:
- lib/sequel/plugins/forbid_lazy_load.rb
Instance Method Summary collapse
-
#allow_lazy_load ⇒ Object
Set this model instance to allow lazy loading of associations.
-
#forbid_lazy_load ⇒ Object
Set this model instance to not allow lazy loading of associations.
Instance Method Details
#allow_lazy_load ⇒ Object
Set this model instance to allow lazy loading of associations.
148 149 150 151 |
# File 'lib/sequel/plugins/forbid_lazy_load.rb', line 148 def allow_lazy_load @forbid_lazy_load = false self end |
#forbid_lazy_load ⇒ Object
Set this model instance to not allow lazy loading of associations.
154 155 156 157 |
# File 'lib/sequel/plugins/forbid_lazy_load.rb', line 154 def forbid_lazy_load @forbid_lazy_load = true self end |