Class: Sequel::Postgres::Database
- Includes:
- DatabaseMethods
- Defined in:
- lib/sequel/adapters/postgres.rb
Constant Summary
Constants included from DatabaseMethods
Sequel::Postgres::DatabaseMethods::FOREIGN_KEY_LIST_ON_DELETE_MAP, Sequel::Postgres::DatabaseMethods::ON_COMMIT, Sequel::Postgres::DatabaseMethods::PREPARED_ARG_PLACEHOLDER, Sequel::Postgres::DatabaseMethods::SELECT_CUSTOM_SEQUENCE_SQL, Sequel::Postgres::DatabaseMethods::SELECT_PK_SQL, Sequel::Postgres::DatabaseMethods::SELECT_SERIAL_SEQUENCE_SQL
Constants inherited from Database
Database::ADAPTERS, Database::COLUMN_DEFINITION_ORDER, Database::COLUMN_SCHEMA_DATETIME_TYPES, Database::COLUMN_SCHEMA_STRING_TYPES, Database::COMBINABLE_ALTER_TABLE_OPS, Database::DEFAULT_DATABASE_ERROR_REGEXPS, Database::DEFAULT_STRING_COLUMN_SIZE, Database::EXTENSIONS, Database::OPTS, Database::SCHEMA_TYPE_CLASSES, Database::TRANSACTION_ISOLATION_LEVELS
Instance Attribute Summary
Attributes included from DatabaseMethods
Attributes inherited from Database
#cache_schema, #dataset_class, #default_string_column_size, #log_connection_info, #log_warn_duration, #loggers, #opts, #pool, #prepared_statements, #sql_log_level, #timezone, #transaction_isolation_level
Instance Method Summary collapse
-
#bound_variable_arg(arg, conn) ⇒ Object
Convert given argument so that it can be used directly by pg.
-
#connect(server) ⇒ Object
Connects to the database.
-
#convert_infinite_timestamps ⇒ Object
Always false, support was moved to pg_extended_date_support extension.
-
#convert_infinite_timestamps=(v) ⇒ Object
Enable pg_extended_date_support extension if symbol or string is given.
-
#copy_into(table, opts = OPTS) ⇒ Object
copy_intouses PostgreSQL’s COPY FROM STDIN SQL statement to do very fast inserts into a table using input preformatting in either CSV or PostgreSQL text format. -
#copy_table(table, opts = OPTS) ⇒ Object
copy_tableuses PostgreSQL’s COPY TO STDOUT SQL statement to return formatted results directly to the caller. - #disconnect_connection(conn) ⇒ Object
-
#error_info(e) ⇒ Object
Return a hash of information about the related PGError (or Sequel::DatabaseError that wraps a PGError), with the following entries:.
- #execute(sql, opts = OPTS, &block) ⇒ Object
-
#listen(channels, opts = OPTS, &block) ⇒ Object
Listens on the given channel (or multiple channels if channel is an array), waiting for notifications.
Methods included from DatabaseMethods
#add_conversion_proc, #add_named_conversion_proc, #commit_prepared_transaction, #create_function, #create_language, #create_schema, #create_trigger, #database_type, #do, #drop_function, #drop_language, #drop_schema, #drop_trigger, #foreign_key_list, #freeze, #indexes, #locks, #notify, #primary_key, #primary_key_sequence, #refresh_view, #reset_primary_key_sequence, #rollback_prepared_transaction, #serial_primary_key_options, #server_version, #supports_create_table_if_not_exists?, #supports_deferrable_constraints?, #supports_deferrable_foreign_key_constraints?, #supports_drop_table_if_exists?, #supports_partial_indexes?, #supports_prepared_transactions?, #supports_savepoints?, #supports_transaction_isolation_levels?, #supports_transactional_ddl?, #supports_trigger_conditions?, #tables, #type_supported?, #values, #views
Methods inherited from Database
#<<, #[], adapter_class, #adapter_scheme, adapter_scheme, #add_column, #add_index, #add_servers, #after_commit, after_initialize, #after_rollback, #alter_table, #alter_table_generator, #call, #cast_type_literal, connect, #create_join_table, #create_join_table!, #create_join_table?, #create_or_replace_view, #create_table, #create_table!, #create_table?, #create_table_generator, #create_view, #database_type, #dataset, #disconnect, #drop_column, #drop_index, #drop_join_table, #drop_table, #drop_table?, #drop_view, #execute_ddl, #execute_dui, #execute_insert, #extend_datasets, #extension, extension, #fetch, #freeze, #from, #from_application_timestamp, #get, #global_index_namespace?, #in_transaction?, #initialize, #inspect, #literal, #literal_symbol, #literal_symbol_set, load_adapter, #log_connection_yield, #log_exception, #log_info, #logger=, #prepared_statement, #quote_identifier, register_extension, #remove_servers, #rename_column, #rename_table, #rollback_checker, #run, run_after_initialize, #schema, #schema_type_class, #select, #serial_primary_key_options, #servers, #set_column_default, #set_column_type, #set_prepared_statement, set_shared_adapter_scheme, #sharded?, #single_threaded?, #supports_create_table_if_not_exists?, #supports_deferrable_constraints?, #supports_deferrable_foreign_key_constraints?, #supports_drop_table_if_exists?, #supports_foreign_key_parsing?, #supports_index_parsing?, #supports_partial_indexes?, #supports_prepared_transactions?, #supports_savepoints?, #supports_savepoints_in_prepared_transactions?, #supports_schema_parsing?, #supports_table_listing?, #supports_transaction_isolation_levels?, #supports_transactional_ddl?, #supports_view_listing?, #supports_views_with_check_option?, #supports_views_with_local_check_option?, #synchronize, #table_exists?, #test_connection, #to_application_timestamp, #transaction, #typecast_value, #uri, #url, #valid_connection?
Constructor Details
This class inherits a constructor from Sequel::Database
Instance Method Details
#bound_variable_arg(arg, conn) ⇒ Object
Convert given argument so that it can be used directly by pg. Currently, pg doesn’t handle fractional seconds in Time/DateTime or blobs with “\0”. Only public for use by the adapter, shouldn’t be used by external code.
158 159 160 161 162 163 164 165 166 167 |
# File 'lib/sequel/adapters/postgres.rb', line 158 def bound_variable_arg(arg, conn) case arg when Sequel::SQL::Blob {:value=>arg, :type=>17, :format=>1} when DateTime, Time literal(arg) else arg end end |
#connect(server) ⇒ Object
Connects to the database. In addition to the standard database options, using the :encoding or :charset option changes the client encoding for the connection, :connect_timeout is a connection timeout in seconds, :sslmode sets whether postgres’s sslmode, and :notice_receiver handles server notices in a proc. :connect_timeout, :driver_options, :sslmode, and :notice_receiver are only supported if the pg driver is used.
176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 |
# File 'lib/sequel/adapters/postgres.rb', line 176 def connect(server) opts = server_opts(server) if USES_PG connection_params = { :host => opts[:host], :port => opts[:port] || 5432, :dbname => opts[:database], :user => opts[:user], :password => opts[:password], :connect_timeout => opts[:connect_timeout] || 20, :sslmode => opts[:sslmode], :sslrootcert => opts[:sslrootcert] }.delete_if { |key, value| blank_object?(value) } connection_params.merge!(opts[:driver_options]) if opts[:driver_options] conn = Adapter.connect(connection_params) conn.instance_variable_set(:@prepared_statements, {}) if receiver = opts[:notice_receiver] conn.set_notice_receiver(&receiver) end else unless typecast_value_boolean(@opts.fetch(:force_standard_strings, true)) raise Error, "Cannot create connection using postgres-pr unless force_standard_strings is set" end conn = Adapter.connect( (opts[:host] unless blank_object?(opts[:host])), opts[:port] || 5432, nil, '', opts[:database], opts[:user], opts[:password] ) end conn.instance_variable_set(:@db, self) if encoding = opts[:encoding] || opts[:charset] if conn.respond_to?(:set_client_encoding) conn.set_client_encoding(encoding) else conn.async_exec("set client_encoding to '#{encoding}'") end end connection_configuration_sqls.each{|sql| conn.execute(sql)} conn end |
#convert_infinite_timestamps ⇒ Object
Always false, support was moved to pg_extended_date_support extension. Needs to stay defined here so that sequel_pg works.
228 229 230 |
# File 'lib/sequel/adapters/postgres.rb', line 228 def false end |
#convert_infinite_timestamps=(v) ⇒ Object
Enable pg_extended_date_support extension if symbol or string is given.
233 234 235 236 237 238 239 |
# File 'lib/sequel/adapters/postgres.rb', line 233 def (v) case v when Symbol, String, true extension(:pg_extended_date_support) self. = v end end |
#copy_into(table, opts = OPTS) ⇒ Object
copy_into uses PostgreSQL’s COPY FROM STDIN SQL statement to do very fast inserts into a table using input preformatting in either CSV or PostgreSQL text format. This method is only supported if pg 0.14.0+ is the underlying ruby driver. This method should only be called if you want results returned to the client. If you are using COPY FROM with a filename, you should just use run instead of this method.
The following options are respected:
- :columns
-
The columns to insert into, with the same order as the columns in the input data. If this isn’t given, uses all columns in the table.
- :data
-
The data to copy to PostgreSQL, which should already be in CSV or PostgreSQL text format. This can be either a string, or any object that responds to each and yields string.
- :format
-
The format to use. text is the default, so this should be :csv or :binary.
- :options
-
An options SQL string to use, which should contain comma separated options.
- :server
-
The server on which to run the query.
If a block is provided and :data option is not, this will yield to the block repeatedly. The block should return a string, or nil to signal that it is finished.
354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 |
# File 'lib/sequel/adapters/postgres.rb', line 354 def copy_into(table, opts=OPTS) data = opts[:data] data = Array(data) if data.is_a?(String) if block_given? && data raise Error, "Cannot provide both a :data option and a block to copy_into" elsif !block_given? && !data raise Error, "Must provide either a :data option or a block to copy_into" end synchronize(opts[:server]) do |conn| conn.execute(copy_into_sql(table, opts)) begin if block_given? while buf = yield conn.put_copy_data(buf) end else data.each{|buff| conn.put_copy_data(buff)} end rescue Exception => e conn.put_copy_end("ruby exception occurred while copying data into PostgreSQL") ensure conn.put_copy_end unless e while res = conn.get_result raise e if e check_database_errors{res.check} end end end end |
#copy_table(table, opts = OPTS) ⇒ Object
copy_table uses PostgreSQL’s COPY TO STDOUT SQL statement to return formatted results directly to the caller. This method is only supported if pg is the underlying ruby driver. This method should only be called if you want results returned to the client. If you are using COPY TO with a filename, you should just use run instead of this method.
The table argument supports the following types:
- String
-
Uses the first argument directly as literal SQL. If you are using a version of PostgreSQL before 9.0, you will probably want to use a string if you are using any options at all, as the syntax Sequel uses for options is only compatible with PostgreSQL 9.0+. This should be the full COPY statement passed to PostgreSQL, not just the SELECT query. If a string is given, the :format and :options options are ignored.
- Dataset
-
Uses a query instead of a table name when copying.
- other
-
Uses a table name (usually a symbol) when copying.
The following options are respected:
- :format
-
The format to use. text is the default, so this should be :csv or :binary.
- :options
-
An options SQL string to use, which should contain comma separated options.
- :server
-
The server on which to run the query.
If a block is provided, the method continually yields to the block, one yield per row. If a block is not provided, a single string is returned with all of the data.
304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 |
# File 'lib/sequel/adapters/postgres.rb', line 304 def copy_table(table, opts=OPTS) synchronize(opts[:server]) do |conn| conn.execute(copy_table_sql(table, opts)) begin if block_given? while buf = conn.get_copy_data yield buf end b = nil else b = String.new b << buf while buf = conn.get_copy_data end res = conn.get_last_result if !res || res.result_status != 1 raise PG::NotAllCopyDataRetrieved, "Not all COPY data retrieved" end b rescue => e raise_error(e, :disconnect=>true) ensure if buf && !e raise DatabaseDisconnectError, "disconnecting as a partial COPY may leave the connection in an unusable state" end end end end |
#disconnect_connection(conn) ⇒ Object
241 242 243 244 245 |
# File 'lib/sequel/adapters/postgres.rb', line 241 def disconnect_connection(conn) conn.finish rescue PGError, IOError nil end |
#error_info(e) ⇒ Object
Return a hash of information about the related PGError (or Sequel::DatabaseError that wraps a PGError), with the following entries:
- :schema
-
The schema name related to the error
- :table
-
The table name related to the error
- :column
-
the column name related to the error
- :constraint
-
The constraint name related to the error
- :type
-
The datatype name related to the error
This requires a PostgreSQL 9.3+ server and 9.3+ client library, and ruby-pg 0.16.0+ to be supported.
259 260 261 262 263 264 265 266 267 268 269 |
# File 'lib/sequel/adapters/postgres.rb', line 259 def error_info(e) e = e.wrapped_exception if e.is_a?(DatabaseError) r = e.result h = {} h[:schema] = r.error_field(::PG::PG_DIAG_SCHEMA_NAME) h[:table] = r.error_field(::PG::PG_DIAG_TABLE_NAME) h[:column] = r.error_field(::PG::PG_DIAG_COLUMN_NAME) h[:constraint] = r.error_field(::PG::PG_DIAG_CONSTRAINT_NAME) h[:type] = r.error_field(::PG::PG_DIAG_DATATYPE_NAME) h end |
#execute(sql, opts = OPTS, &block) ⇒ Object
272 273 274 |
# File 'lib/sequel/adapters/postgres.rb', line 272 def execute(sql, opts=OPTS, &block) synchronize(opts[:server]){|conn| check_database_errors{_execute(conn, sql, opts, &block)}} end |
#listen(channels, opts = OPTS, &block) ⇒ Object
Listens on the given channel (or multiple channels if channel is an array), waiting for notifications. After a notification is received, or the timeout has passed, stops listening to the channel. Options:
- :after_listen
-
An object that responds to
callthat is called with the underlying connection after the LISTEN statement is sent, but before the connection starts waiting for notifications. - :loop
-
Whether to continually wait for notifications, instead of just waiting for a single notification. If this option is given, a block must be provided. If this object responds to
call, it is called with the underlying connection after each notification is received (after the block is called). If a :timeout option is used, and a callable object is given, the object will also be called if the timeout expires. If :loop is used and you want to stop listening, you can either break from inside the block given to #listen, or you can throw :stop from inside the :loop object’s call method or the block. - :server
-
The server on which to listen, if the sharding support is being used.
- :timeout
-
How long to wait for a notification, in seconds (can provide a float value for fractional seconds). If this object responds to
call, it will be called and should return the number of seconds to wait. If the loop option is also specified, the object will be called on each iteration to obtain a new timeout value. If not given or nil, waits indefinitely.
This method is only supported if pg is used as the underlying ruby driver. It returns the channel the notification was sent to (as a string), unless :loop was used, in which case it returns nil. If a block is given, it is yielded 3 arguments:
-
the channel the notification was sent to (as a string)
-
the backend pid of the notifier (as an integer),
-
and the payload of the notification (as a string or nil).
409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 |
# File 'lib/sequel/adapters/postgres.rb', line 409 def listen(channels, opts=OPTS, &block) check_database_errors do synchronize(opts[:server]) do |conn| begin channels = Array(channels) channels.each do |channel| sql = "LISTEN ".dup dataset.send(:identifier_append, sql, channel) conn.execute(sql) end opts[:after_listen].call(conn) if opts[:after_listen] timeout = opts[:timeout] if timeout timeout_block = timeout.respond_to?(:call) ? timeout : proc{timeout} end if l = opts[:loop] raise Error, 'calling #listen with :loop requires a block' unless block loop_call = l.respond_to?(:call) catch(:stop) do loop do t = timeout_block ? [timeout_block.call] : [] conn.wait_for_notify(*t, &block) l.call(conn) if loop_call end end nil else t = timeout_block ? [timeout_block.call] : [] conn.wait_for_notify(*t, &block) end ensure conn.execute("UNLISTEN *") end end end end |