Module: Sequel::Postgres::ExtendedIntegerSupport

Defined in:
lib/sequel/extensions/pg_extended_integer_support.rb

Instance Method Summary collapse

Instance Method Details

#integer_outside_bigint_range_strategy(strategy) ⇒ Object

Set the strategy for handling integers outside PostgreSQL bigint range. Supported values:

:quote :: Quote the integer value. PostgreSQL will treat the integer as a unknown type, implicitly casting to any other type as needed. This is the default value when using the pg_extended_integer_support extension. :raise :: Raise error when attempting to literalize the integer (the default behavior of Sequel on PostgreSQL when not using the pg_extended_integer_support extension). :raw :: Use raw integer value without quoting. PostgreSQL will treat the integer as a numeric. This was Sequel's historical behavior, but it is unlikely to be desired.



94
95
96
# File 'lib/sequel/extensions/pg_extended_integer_support.rb', line 94

def integer_outside_bigint_range_strategy(strategy)
  clone(:integer_outside_bigint_range_strategy=>strategy)
end