Method: Sequel::Postgres::PGArray::DatabaseMethods.extended

Defined in:
lib/sequel/extensions/pg_array.rb

.extended(db) ⇒ Object

Create the local hash of database type strings to schema type symbols, used for array types local to this database.



196
197
198
199
200
201
202
203
204
205
206
207
# File 'lib/sequel/extensions/pg_array.rb', line 196

def self.extended(db)
  db.instance_eval do
    @pg_array_schema_types ||= {}
    procs = conversion_procs
    procs[1115] = Creator.new("timestamp without time zone", procs[1114])
    procs[1185] = Creator.new("timestamp with time zone", procs[1184])
    copy_conversion_procs([1009, 1007, 1016, 1231, 1022, 1000, 1001, 1182, 1183, 1270, 1005, 1028, 1021, 1014, 1015])
    [:string_array, :integer_array, :decimal_array, :float_array, :boolean_array, :blob_array, :date_array, :time_array, :datetime_array].each do |v|
      @schema_type_classes[v] = PGArray
    end
  end
end