Class: Sequel::Postgres::PGArray::Parser

Inherits:
Object
  • Object
show all
Defined in:
lib/sequel/extensions/pg_array.rb

Overview

PostgreSQL array parser that handles all types of input.

This parser is very simple and unoptimized, but should still be O(n) where n is the length of the input string.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(source, converter = nil) ⇒ Parser

Returns a new instance of Parser.



294
295
296
297
298
299
300
301
302
# File 'lib/sequel/extensions/pg_array.rb', line 294

def initialize(source, converter=nil)
  @source = source
  @source_length = source.length
  @converter = converter 
  @pos = -1
  @entries = []
  @recorded = ""
  @dimension = 0
end

Instance Attribute Details

#posObject (readonly)

Returns the value of attribute pos.



288
289
290
# File 'lib/sequel/extensions/pg_array.rb', line 288

def pos
  @pos
end

Instance Method Details

#new_entry(include_empty = false) ⇒ Object



323
324
325
326
327
328
329
330
331
332
333
334
# File 'lib/sequel/extensions/pg_array.rb', line 323

def new_entry(include_empty=false)
  if !@recorded.empty? || include_empty
    entry = @recorded
    if entry == NULL && !include_empty
      entry = nil
    elsif @converter
      entry = @converter.call(entry)
    end
    @entries.push(entry)
    @recorded = ""
  end
end

#next_charObject



306
307
308
309
310
311
312
313
314
# File 'lib/sequel/extensions/pg_array.rb', line 306

def next_char
  @pos += 1
  if (c = @source[@pos..@pos]) == BACKSLASH
    @pos += 1
    [true, @source[@pos..@pos]]
  else
    [false, c]
  end
end

#parse(nested = false) ⇒ Object

Raises:



338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
# File 'lib/sequel/extensions/pg_array.rb', line 338

def parse(nested=false)
  # quote sets whether we are inside of a quoted string.
  quote = false
  until @pos >= @source_length
    escaped, char = next_char
    if char == OPEN_BRACE && !quote
      @dimension += 1
      if (@dimension > 1)
        # Multi-dimensional array encounter, use a subparser
        # to parse the next level down.
        subparser = self.class.new(@source[@pos..-1], @converter)
        @entries.push(subparser.parse(true))
        @pos += subparser.pos - 1
      end
    elsif char == CLOSE_BRACE && !quote
      @dimension -= 1
      if (@dimension == 0)
        new_entry
        # Exit early if inside a subparser, since the
        # text after parsing the current level should be
        # ignored as it is handled by the parent parser.
        return @entries if nested
      end
    elsif char == QUOTE && !escaped
      # If already inside the quoted string, this is the
      # ending quote, so add the entry.  Otherwise, this
      # is the opening quote, so set the quote flag.
      new_entry(true) if quote
      quote = !quote
    elsif char == COMMA && !quote
      # If not inside a string and a comma occurs, it indicates
      # the end of the entry, so add the entry.
      new_entry
    else
      # Add the character to the recorded character buffer.
      record(char)
    end
  end
  raise Sequel::Error, "array dimensions not balanced" unless @dimension == 0
  @entries
end

#record(c) ⇒ Object



317
318
319
# File 'lib/sequel/extensions/pg_array.rb', line 317

def record(c)
  @recorded << c
end