Method: Sequel::Postgres::StatementCache::AdapterMethods#deallocate
- Defined in:
- lib/sequel/extensions/pg_statement_cache.rb
#deallocate(name) ⇒ Object
Deallocate on the server the prepared statement with the given name.
242 243 244 245 246 247 248 |
# File 'lib/sequel/extensions/pg_statement_cache.rb', line 242 def deallocate(name) begin execute("DEALLOCATE #{name}") rescue PGError # table probably got removed, just ignore it end end |