Class: Sequel::SQL::JoinUsingClause
- Inherits:
-
JoinClause
- Object
- Expression
- JoinClause
- Sequel::SQL::JoinUsingClause
- Defined in:
- lib/sequel/sql.rb,
lib/sequel/extensions/eval_inspect.rb
Overview
Represents an SQL JOIN clause with USING conditions.
Instance Attribute Summary collapse
-
#using ⇒ Object
readonly
The columns that appear in both tables that should be equal for the conditions to match.
Attributes inherited from JoinClause
Instance Method Summary collapse
-
#initialize(cols, *args) ⇒ JoinUsingClause
constructor
Create an object with the given USING conditions and call super with the remaining args.
Methods inherited from JoinClause
#column_aliases, #table, #table_alias
Methods inherited from Expression
#==, attr_reader, #clone, #eql?, #hash, inherited, #inspect
Constructor Details
#initialize(cols, *args) ⇒ JoinUsingClause
Create an object with the given USING conditions and call super with the remaining args.
1581 1582 1583 1584 |
# File 'lib/sequel/sql.rb', line 1581 def initialize(cols, *args) @using = cols super(*args) end |
Instance Attribute Details
#using ⇒ Object (readonly)
The columns that appear in both tables that should be equal for the conditions to match.
1577 1578 1579 |
# File 'lib/sequel/sql.rb', line 1577 def using @using end |