Module: Sequel::SQL::QualifyingMethods
- Included in:
- Identifier, QualifiedIdentifier, Symbol
- Defined in:
- lib/sequel/sql.rb
Overview
Includes a qualify method that created QualifiedIdentifiers, used for qualifying column names with a table or table names with a schema, and the * method for returning all columns in the identifier if no arguments are given.
Instance Method Summary collapse
-
#*(ce = (arg=false;nil)) ⇒ Object
If no arguments are given, return an SQL::ColumnAll:.
-
#[](identifier) ⇒ Object
Qualify the receiver with the given
qualifier(table for column/schema for table). -
#qualify(qualifier) ⇒ Object
Qualify the receiver with the given
qualifier(table for column/schema for table).
Instance Method Details
#*(ce = (arg=false;nil)) ⇒ Object
If no arguments are given, return an SQL::ColumnAll:
Sequel[:a__b].* # a.b.*
949 950 951 952 953 954 955 |
# File 'lib/sequel/sql.rb', line 949 def *(ce=(arg=false;nil)) if arg == false Sequel::SQL::ColumnAll.new(self) else super(ce) end end |
#[](identifier) ⇒ Object
971 972 973 |
# File 'lib/sequel/sql.rb', line 971 def [](identifier) QualifiedIdentifier.new(self, identifier) end |
#qualify(qualifier) ⇒ Object
962 963 964 |
# File 'lib/sequel/sql.rb', line 962 def qualify(qualifier) QualifiedIdentifier.new(qualifier, self) end |