Method: Sequel::SQL::StringExpression.like
- Defined in:
- lib/sequel/sql.rb
.like(l, *ces) ⇒ Object
Creates a SQL pattern match exprssion. left (l) is the SQL string we are matching against, and ces are the patterns we are matching. The match succeeds if any of the patterns match (SQL OR).
If a regular expression is used as a pattern, an SQL regular expression will be used, which is currently only supported on MySQL and PostgreSQL. Be aware that MySQL and PostgreSQL regular expression syntax is similar to ruby regular expression syntax, but it not exactly the same, especially for advanced regular expression features. Sequel just uses the source of the ruby regular expression verbatim as the SQL regular expression string.
If any other object is used as a regular expression, the SQL LIKE operator will be used, and should be supported by most databases.
The pattern match will be case insensitive if the last argument is a hash with a key of :case_insensitive that is not false or nil. Also, if a case insensitive regular expression is used (//i), that particular pattern which will always be case insensitive.
StringExpression.like(:a, 'a%') # "a" LIKE 'a%' ESCAPE '\'
StringExpression.like(:a, 'a%', :case_insensitive=>true) # "a" ILIKE 'a%' ESCAPE '\'
StringExpression.like(:a, 'a%', /^a/i) # "a" LIKE 'a%' ESCAPE '\' OR "a" ~* '^a'
1637 1638 1639 1640 1641 1642 1643 1644 1645 |
# File 'lib/sequel/sql.rb', line 1637 def self.like(l, *ces) l, lre, lci = like_element(l) lci = (ces.last.is_a?(Hash) ? ces.pop : {})[:case_insensitive] ? true : lci ces.collect! do |ce| r, rre, rci = like_element(ce) BooleanExpression.new(LIKE_MAP[[lre||rre, lci||rci]], l, r) end ces.length == 1 ? ces.at(0) : BooleanExpression.new(:OR, *ces) end |