Class: Sequel::SQL::Subscript
- Inherits:
-
GenericExpression
- Object
- Expression
- GenericExpression
- Sequel::SQL::Subscript
- Defined in:
- lib/sequel/sql.rb
Overview
Represents an SQL array access, with multiple possible arguments.
Instance Attribute Summary collapse
-
#f ⇒ Object
readonly
The SQL array column.
-
#sub ⇒ Object
readonly
The array of subscripts to use (should be an array of numbers).
Instance Method Summary collapse
-
#initialize(f, sub) ⇒ Subscript
constructor
Set the array column and subscripts to the given arguments.
-
#|(sub) ⇒ Object
Create a new
Subscriptappending the given subscript(s) the the current array of subscripts.
Methods included from SubscriptMethods
Methods included from StringMethods
Methods included from OrderMethods
Methods included from NumericMethods
Methods included from ComplexExpressionMethods
#extract, #sql_boolean, #sql_number, #sql_string
Methods included from CastMethods
#cast, #cast_numeric, #cast_string
Methods included from BooleanMethods
Methods included from AliasMethods
Methods inherited from Expression
#==, attr_reader, comparison_attrs, #eql?, #hash, #inspect, #lit, #sql_literal
Constructor Details
#initialize(f, sub) ⇒ Subscript
Set the array column and subscripts to the given arguments
1020 1021 1022 |
# File 'lib/sequel/sql.rb', line 1020 def initialize(f, sub) @f, @sub = f, sub end |
Instance Attribute Details
#f ⇒ Object (readonly)
The SQL array column
1014 1015 1016 |
# File 'lib/sequel/sql.rb', line 1014 def f @f end |
#sub ⇒ Object (readonly)
The array of subscripts to use (should be an array of numbers)
1017 1018 1019 |
# File 'lib/sequel/sql.rb', line 1017 def sub @sub end |
Instance Method Details
#|(sub) ⇒ Object
Create a new Subscript appending the given subscript(s)
the the current array of subscripts.
1026 1027 1028 |
# File 'lib/sequel/sql.rb', line 1026 def |(sub) Subscript.new(@f, @sub + Array(sub)) end |