Method: Sequel::SQLite::JSONOp#extract

Defined in:
lib/sequel/extensions/sqlite_json_ops.rb

#extract(*a) ⇒ Object

Returns an expression for the JSON array element or object field at the specified path as a SQLite value, but only accept paths as arguments, and allow the use of multiple paths.

json_op.extract('$.a')        # json_extract(json, '$.a')
json_op.extract('$.a', '$.b') # json_extract(json, '$.a', '$.b')


104
105
106
# File 'lib/sequel/extensions/sqlite_json_ops.rb', line 104

def extract(*a)
  function(:extract, *a)
end