Class: Sequel::Schema::CreateTableGenerator

Inherits:
Object
  • Object
show all
Defined in:
lib/sequel/database/schema_generator.rb,
lib/sequel/extensions/schema_dumper.rb

Overview

Schema::CreateTableGenerator is an internal class that the user is not expected to instantiate directly. Instances are created by Database#create_table. It is used to specify table creation parameters. It takes a Database object and a block of column/index/constraint specifications, and gives the Database a table description, which the database uses to create a table.

Schema::CreateTableGenerator has some methods but also includes method_missing, allowing users to specify column type as a method instead of using the column method, which makes for a nicer DSL.

For more information on Sequel’s support for schema modification, see the “Schema Modification” guide.

Direct Known Subclasses

Postgres::CreateTableGenerator

Constant Summary collapse

GENERIC_TYPES =

Classes specifying generic types that Sequel will convert to database-specific types.

%w'String Integer Float Numeric BigDecimal Date DateTime Time File TrueClass FalseClass'.freeze

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(db, &block) ⇒ CreateTableGenerator

Set the database in which to create the table, and evaluate the block in the context of this object.



34
35
36
37
38
39
40
41
42
# File 'lib/sequel/database/schema_generator.rb', line 34

def initialize(db, &block)
  @db = db
  @columns = []
  @indexes = []
  @constraints = []
  @primary_key = nil
  instance_exec(&block) if block
  @columns.unshift(@primary_key) if @primary_key && !has_column?(primary_key_name)
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(type, name = nil, opts = OPTS) ⇒ Object

Add a column with the given type, name, and opts. See #column for available options.



247
248
249
# File 'lib/sequel/database/schema_generator.rb', line 247

def method_missing(type, name = nil, opts = OPTS)
  name ? column(name, type, opts) : super
end

Instance Attribute Details

#columnsObject (readonly)

Column hashes created by this generator



24
25
26
# File 'lib/sequel/database/schema_generator.rb', line 24

def columns
  @columns
end

#constraintsObject (readonly)

Constraint hashes created by this generator



27
28
29
# File 'lib/sequel/database/schema_generator.rb', line 27

def constraints
  @constraints
end

#indexesObject (readonly)

Index hashes created by this generator



30
31
32
# File 'lib/sequel/database/schema_generator.rb', line 30

def indexes
  @indexes
end

Class Method Details

.add_type_method(*types) ⇒ Object

Add a method for each of the given types that creates a column with that type as a constant. Types given should either already be constants/classes or a capitalized string/symbol with the same name as a constant/class.



60
61
62
63
64
65
66
67
68
69
70
71
72
73
# File 'lib/sequel/database/schema_generator.rb', line 60

def self.add_type_method(*types)
  types.each do |type|
    case type
    when Symbol, String
      method = type
      type = Object.const_get(type)
    else
      method = type.to_s
    end

    define_method(method){|name, opts=OPTS| column(name, type, opts)}
  end
  nil
end

Instance Method Details

#Bignum(name, opts = OPTS) ⇒ Object

Use custom Bignum method to use :Bignum instead of Bignum class, to work correctly in cases where Bignum is the same as Integer.



46
47
48
# File 'lib/sequel/database/schema_generator.rb', line 46

def Bignum(name, opts=OPTS)
  column(name, :Bignum, opts)
end

#check(*args, &block) ⇒ Object

Add an unnamed constraint, specified by the given block or args:

check(num: 1..5) # CHECK num >= 1 AND num <= 5
check{num > 5}   # CHECK num > 5


80
81
82
# File 'lib/sequel/database/schema_generator.rb', line 80

def check(*args, &block)
  constraint(nil, *args, &block)
end

#column(name, type, opts = OPTS) ⇒ Object

Add a column with the given name, type, and opts:

column :num, :integer
# num INTEGER

column :name, String, null: false, default: 'a'
# name varchar(255) NOT NULL DEFAULT 'a'

inet :ip
# ip inet

You can also create columns via method missing, so the following are equivalent:

column :number, :integer
integer :number

The following options are supported:

:collate

The collation to use for the column. For backwards compatibility, only symbols and string values are supported, and they are used verbatim. However, on PostgreSQL, symbols are literalized as regular identifiers, since unquoted collations are unlikely to be valid.

:default

The default value for the column.

:deferrable

For foreign key columns, this ensures referential integrity will work even if referencing table uses a foreign key value that does not yet exist on referenced table (but will exist before the transaction commits). Basically it adds DEFERRABLE INITIALLY DEFERRED on key creation. If you use :immediate as the value, uses DEFERRABLE INITIALLY IMMEDIATE.

:index

Create an index on this column. If given a hash, use the hash as the options for the index.

:key

For foreign key columns, the column in the associated table that this column references. Unnecessary if this column references the primary key of the associated table, except if you are using MySQL.

:null

Mark the column as allowing NULL values (if true), or not allowing NULL values (if false). The default is to allow NULL values.

:on_delete

Specify the behavior of this column when being deleted (:restrict, :cascade, :set_null, :set_default, :no_action).

:on_update

Specify the behavior of this column when being updated (:restrict, :cascade, :set_null, :set_default, :no_action).

:primary_key

Make the column as a single primary key column. This should not be used if you have a single, nonautoincrementing primary key column (use the primary_key method in that case).

:primary_key_constraint_name

The name to give the primary key constraint

:type

Overrides the type given as the argument. Generally not used by column itself, but can be passed as an option to other methods that call column.

:unique

Mark the column as unique, generally has the same effect as creating a unique index on the column.

:unique_constraint_name

The name to give the unique key constraint

MySQL specific options:

:generated_always_as

Specify a GENERATED ALWAYS AS column expression, if generated columns are supported.

:generated_type

Set the type of column when using :generated_always_as, should be :virtual or :stored to force a type.



140
141
142
143
144
145
146
# File 'lib/sequel/database/schema_generator.rb', line 140

def column(name, type, opts = OPTS)
  columns << {:name => name, :type => type}.merge!(opts)
  if index_opts = opts[:index]
    index(name, index_opts.is_a?(Hash) ? index_opts : OPTS)
  end
  nil
end

#constraint(name, *args, &block) ⇒ Object

Adds a named constraint (or unnamed if name is nil), with the given block or args. To provide options for the constraint, pass a hash as the first argument.

constraint(:blah, num: 1..5)
# CONSTRAINT blah CHECK num >= 1 AND num <= 5
constraint({name: :blah, deferrable: true}, num: 1..5)
# CONSTRAINT blah CHECK num >= 1 AND num <= 5 DEFERRABLE INITIALLY DEFERRED


156
157
158
159
160
# File 'lib/sequel/database/schema_generator.rb', line 156

def constraint(name, *args, &block)
  opts = name.is_a?(Hash) ? name : {:name=>name}
  constraints << opts.merge(:type=>:check, :check=>block || args)
  nil
end

#dump_columnsObject

Dump this generator’s columns to a string that could be evaled inside another instance to represent the same columns



390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
# File 'lib/sequel/extensions/schema_dumper.rb', line 390

def dump_columns
  strings = []
  cols = columns.dup
  cols.each do |x|
    x.delete(:on_delete) if x[:on_delete] == :no_action
    x.delete(:on_update) if x[:on_update] == :no_action
  end
  if (pkn = primary_key_name) && !@primary_key[:keep_order]
    cols.delete_if{|x| x[:name] == pkn}
    pk = @primary_key.dup
    pkname = pk.delete(:name)
    @db.serial_primary_key_options.each{|k,v| pk.delete(k) if v == pk[k]}
    strings << "primary_key #{pkname.inspect}#{opts_inspect(pk)}"
  end
  cols.each do |c|
    c = c.dup
    name = c.delete(:name)
    strings << if table = c.delete(:table)
      c.delete(:type) if c[:type] == Integer || c[:type] == 'integer'
      "foreign_key #{name.inspect}, #{table.inspect}#{opts_inspect(c)}"
    elsif pkn == name
      @db.serial_primary_key_options.each{|k,v| c.delete(k) if v == c[k]}
      "primary_key #{name.inspect}#{opts_inspect(c)}"
    else
      type = c.delete(:type)
      opts = opts_inspect(c)
      case type
      when Class
        "#{type.name} #{name.inspect}#{opts}"
      when :Bignum
        "Bignum #{name.inspect}#{opts}"
      else
        "column #{name.inspect}, #{type.inspect}#{opts}"
      end
    end
  end
  strings.join("\n")
end

#dump_constraintsObject

Dump this generator’s constraints to a string that could be evaled inside another instance to represent the same constraints



431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
# File 'lib/sequel/extensions/schema_dumper.rb', line 431

def dump_constraints
  cs = constraints.map do |c|
    c = c.dup
    type = c.delete(:type)
    case type
    when :check
      raise(Error, "can't dump check/constraint specified with Proc") if c[:check].is_a?(Proc)
      name = c.delete(:name)
      if !name and c[:check].length == 1 and c[:check].first.is_a?(Hash)
        "check #{c[:check].first.inspect[1...-1]}"
      else
        "#{name ? "constraint #{name.inspect}," : 'check'} #{c[:check].map(&:inspect).join(', ')}"
      end
    when :foreign_key
      c.delete(:on_delete) if c[:on_delete] == :no_action
      c.delete(:on_update) if c[:on_update] == :no_action
      c.delete(:deferrable) unless c[:deferrable]
      cols = c.delete(:columns)
      table = c.delete(:table)
      "#{type} #{cols.inspect}, #{table.inspect}#{opts_inspect(c)}"
    else
      cols = c.delete(:columns)
      "#{type} #{cols.inspect}#{opts_inspect(c)}"
    end
  end
  cs.join("\n")
end

#dump_indexes(options = OPTS) ⇒ Object

Dump this generator’s indexes to a string that could be evaled inside another instance to represent the same indexes. Options:

:add_index

Use add_index instead of index, so the methods can be called outside of a generator but inside a migration. The value of this option should be the table name to use.

:drop_index

Same as add_index, but create drop_index statements.

:ignore_errors

Add the ignore_errors option to the outputted indexes



466
467
468
469
470
471
472
473
474
475
476
477
478
# File 'lib/sequel/extensions/schema_dumper.rb', line 466

def dump_indexes(options=OPTS)
  is = indexes.map do |c|
    c = c.dup
    cols = c.delete(:columns)
    if table = options[:add_index] || options[:drop_index]
      "#{options[:drop_index] ? 'drop' : 'add'}_index #{table.inspect}, #{cols.inspect}#{', :ignore_errors=>true' if options[:ignore_errors]}#{opts_inspect(c)}"
    else
      "index #{cols.inspect}#{opts_inspect(c)}"
    end
  end
  is = is.reverse if options[:drop_index]
  is.join("\n")
end

#Fixnum(name, opts = OPTS) ⇒ Object

Use custom Fixnum method to use Integer instead of Fixnum class, to avoid warnings on ruby 2.4+.



52
53
54
# File 'lib/sequel/database/schema_generator.rb', line 52

def Fixnum(name, opts=OPTS)
  column(name, Integer, opts)
end

#foreign_key(name, table = nil, opts = OPTS) ⇒ Object

Add a foreign key in the table that references another table. See #column for available options.

foreign_key(:artist_id) # artist_id INTEGER
foreign_key(:artist_id, :artists) # artist_id INTEGER REFERENCES artists
foreign_key(:artist_id, :artists, key: :id) # artist_id INTEGER REFERENCES artists(id)
foreign_key(:artist_id, :artists, type: String) # artist_id varchar(255) REFERENCES artists(id)

Additional Options:

:foreign_key_constraint_name

The name to give the foreign key constraint

If you want a foreign key constraint without adding a column (usually because it is a composite foreign key), you can provide an array of columns as the first argument, and you can provide the :name option to name the constraint:

foreign_key([:artist_name, :artist_location], :artists, name: :artist_fk)
# ADD CONSTRAINT artist_fk FOREIGN KEY (artist_name, artist_location) REFERENCES artists


180
181
182
183
184
185
186
187
188
189
190
191
# File 'lib/sequel/database/schema_generator.rb', line 180

def foreign_key(name, table=nil, opts = OPTS)
  opts = case table
  when Hash
    table.merge(opts)
  when NilClass
    opts
  else
    opts.merge(:table=>table)
  end
  return composite_foreign_key(name, opts) if name.is_a?(Array)
  column(name, Integer, opts)
end

#full_text_index(columns, opts = OPTS) ⇒ Object

Add a full text index on the given columns.

PostgreSQL specific options:

:index_type

Can be set to :gist to use a GIST index instead of the default GIN index.

:language

Set a language to use for the index (default: simple).

Microsoft SQL Server specific options:

:key_index

The KEY INDEX to use for the full text index.



202
203
204
# File 'lib/sequel/database/schema_generator.rb', line 202

def full_text_index(columns, opts = OPTS)
  index(columns, opts.merge(:type => :full_text))
end

#has_column?(name) ⇒ Boolean

True if the generator includes the creation of a column with the given name.

Returns:

  • (Boolean)


207
208
209
# File 'lib/sequel/database/schema_generator.rb', line 207

def has_column?(name)
  columns.any?{|c| c[:name] == name}
end

#index(columns, opts = OPTS) ⇒ Object

Add an index on the given column(s) with the given options. General options:

:name

The name to use for the index. If not given, a default name based on the table and columns is used.

:type

The type of index to use (only supported by some databases)

:unique

Make the index unique, so duplicate values are not allowed.

:where

Create a partial index (only supported by some databases)

PostgreSQL specific options:

:concurrently

Create the index concurrently, so it doesn’t block operations on the table while the index is being built.

:opclass

Use a specific operator class in the index.

:include

Include additional column values in the index, without actually indexing on those values (PostgreSQL 11+).

:tablespace

Specify tablespace for index.

Microsoft SQL Server specific options:

:include

Include additional column values in the index, without actually indexing on those values.

index :name
# CREATE INDEX table_name_index ON table (name)

index [:artist_id, :name]
# CREATE INDEX table_artist_id_name_index ON table (artist_id, name)


240
241
242
243
# File 'lib/sequel/database/schema_generator.rb', line 240

def index(columns, opts = OPTS)
  indexes << {:columns => Array(columns)}.merge!(opts)
  nil
end

#primary_key(name, *args) ⇒ Object

Adds an autoincrementing primary key column or a primary key constraint. To just create a constraint, the first argument should be an array of column symbols specifying the primary key columns. To create an autoincrementing primary key column, a single symbol can be used. In both cases, an options hash can be used as the second argument.

If you want to create a primary key column that is not autoincrementing, you should not use this method. Instead, you should use the regular column method with a primary_key: true option.

If an array of column symbols is used, you can specify the :name option to name the constraint.

Options:

:keep_order

For non-composite primary keys, respects the existing order of columns, overriding the default behavior of making the primary key the first column.

Examples:

primary_key(:id)
primary_key(:id, type: :Bignum, keep_order: true)
primary_key([:street_number, :house_number], name: :some constraint_name)


278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
# File 'lib/sequel/database/schema_generator.rb', line 278

def primary_key(name, *args)
  return composite_primary_key(name, *args) if name.is_a?(Array)
  column = @db.serial_primary_key_options.merge({:name => name})
  
  if opts = args.pop
    opts = {:type => opts} unless opts.is_a?(Hash)
    if type = args.pop
      opts = opts.merge(:type => type)
    end
    column.merge!(opts)
  end

  @primary_key = column
  if column[:keep_order]
    columns << column
  else
    columns.unshift(column)
  end
  nil
end

#primary_key_nameObject

The name of the primary key for this generator, if it has a primary key.



300
301
302
# File 'lib/sequel/database/schema_generator.rb', line 300

def primary_key_name
  @primary_key[:name] if @primary_key
end

#respond_to_missing?(meth, include_private) ⇒ Boolean

This object responds to all methods.

Returns:

  • (Boolean)


252
253
254
# File 'lib/sequel/database/schema_generator.rb', line 252

def respond_to_missing?(meth, include_private)
  true
end

#spatial_index(columns, opts = OPTS) ⇒ Object

Add a spatial index on the given columns.



305
306
307
# File 'lib/sequel/database/schema_generator.rb', line 305

def spatial_index(columns, opts = OPTS)
  index(columns, opts.merge(:type => :spatial))
end

#unique(columns, opts = OPTS) ⇒ Object

Add a unique constraint on the given columns.

unique(:name) # UNIQUE (name)

Supports the same :deferrable option as #column. The :name option can be used to name the constraint.



315
316
317
318
# File 'lib/sequel/database/schema_generator.rb', line 315

def unique(columns, opts = OPTS)
  constraints << {:type => :unique, :columns => Array(columns)}.merge!(opts)
  nil
end