Class: Sequel::Trilogy::Database
- Includes:
- MySQL::DatabaseMethods
- Defined in:
- lib/sequel/adapters/trilogy.rb
Constant Summary collapse
- QUERY_FLAGS =
::Trilogy::QUERY_FLAGS_CAST | ::Trilogy::QUERY_FLAGS_CAST_BOOLEANS
- LOCAL_TIME_QUERY_FLAGS =
QUERY_FLAGS | ::Trilogy::QUERY_FLAGS_LOCAL_TIMEZONE
Constants included from MySQL::DatabaseMethods
MySQL::DatabaseMethods::CAST_TYPES, MySQL::DatabaseMethods::COLUMN_DEFINITION_ORDER
Instance Attribute Summary
Attributes included from MySQL::DatabaseMethods
#default_charset, #default_collate, #default_engine
Instance Method Summary collapse
-
#connect(server) ⇒ Object
Connect to the database.
- #disconnect_connection(c) ⇒ Object
-
#execute(sql, opts) ⇒ Object
Execute the given SQL on the given connection and yield the result.
- #execute_dui(sql, opts = OPTS) ⇒ Object
- #execute_insert(sql, opts = OPTS) ⇒ Object
- #freeze ⇒ Object
-
#server_version(_server = nil) ⇒ Object
Return the version of the MySQL server to which we are connecting.
Methods included from MySQL::DatabaseMethods
#cast_type_literal, #commit_prepared_transaction, #database_type, #foreign_key_list, #global_index_namespace?, #indexes, #mariadb?, #rename_tables, #rollback_prepared_transaction, #supports_create_table_if_not_exists?, #supports_generated_columns?, #supports_prepared_transactions?, #supports_savepoints?, #supports_savepoints_in_prepared_transactions?, #supports_timestamp_usecs?, #supports_transaction_isolation_levels?, #tables, #views, #with_advisory_lock
Instance Method Details
#connect(server) ⇒ Object
Connect to the database. See Trilogy documentation for options.
17 18 19 20 21 22 23 24 |
# File 'lib/sequel/adapters/trilogy.rb', line 17 def connect(server) opts = server_opts(server) opts[:username] ||= opts.delete(:user) opts[:found_rows] = true conn = ::Trilogy.new(opts) mysql_connection_setting_sqls.each{|sql| log_connection_yield(sql, conn){conn.query(sql)}} conn end |
#disconnect_connection(c) ⇒ Object
26 27 28 29 30 |
# File 'lib/sequel/adapters/trilogy.rb', line 26 def disconnect_connection(c) c.discard! rescue ::Trilogy::Error nil end |
#execute(sql, opts) ⇒ Object
Execute the given SQL on the given connection and yield the result.
33 34 35 36 37 38 39 40 41 42 |
# File 'lib/sequel/adapters/trilogy.rb', line 33 def execute(sql, opts) r = synchronize(opts[:server]) do |conn| log_connection_yield((log_sql = opts[:log_sql]) ? sql + log_sql : sql, conn) do conn.query_with_flags(sql, timezone.nil? || timezone == :local ? LOCAL_TIME_QUERY_FLAGS : QUERY_FLAGS) end end yield r rescue ::Trilogy::Error => e raise_error(e) end |
#execute_dui(sql, opts = OPTS) ⇒ Object
44 45 46 |
# File 'lib/sequel/adapters/trilogy.rb', line 44 def execute_dui(sql, opts=OPTS) execute(sql, opts, &:affected_rows) end |
#execute_insert(sql, opts = OPTS) ⇒ Object
48 49 50 |
# File 'lib/sequel/adapters/trilogy.rb', line 48 def execute_insert(sql, opts=OPTS) execute(sql, opts, &:last_insert_id) end |
#freeze ⇒ Object
52 53 54 55 |
# File 'lib/sequel/adapters/trilogy.rb', line 52 def freeze server_version super end |
#server_version(_server = nil) ⇒ Object
Return the version of the MySQL server to which we are connecting.
58 59 60 |
# File 'lib/sequel/adapters/trilogy.rb', line 58 def server_version(_server=nil) @server_version ||= super() end |