Method: Array#sql_expr
- Defined in:
- lib/sequel/core_sql.rb
#sql_expr ⇒ Object
Return a Sequel::SQL::BooleanExpression created from this array, matching all of the conditions. Rarely do you need to call this explicitly, as Sequel generally assumes that arrays of all two pairs specify this type of condition.
[[:a, true]].sql_expr # SQL: a IS TRUE
[[:a, 1], [:b, [2, 3]]].sql_expr # SQL: a = 1 AND b IN (2, 3)
52 53 54 |
# File 'lib/sequel/core_sql.rb', line 52 def sql_expr sql_expr_if_all_two_pairs end |