Method: Sequel::Dataset::PreparedStatementMethods#literal_symbol_append
- Defined in:
- lib/sequel/dataset/prepared_statements.rb
#literal_symbol_append(sql, v) ⇒ Object
Changes the values of symbols if they start with $ and prepared_args is present. If so, they are considered placeholders, and they are substituted using prepared_arg.
172 173 174 175 176 177 178 |
# File 'lib/sequel/dataset/prepared_statements.rb', line 172 def literal_symbol_append(sql, v) if @opts[:bind_vars] && /\A\$(.*)\z/ =~ v literal_append(sql, prepared_arg($1.to_sym)) else super end end |