Method: Sequel::Dataset#row_proc
- Defined in:
- lib/sequel/dataset/misc.rb
#row_proc ⇒ Object
The row_proc for this database, should be any object that responds to call with a single hash argument and returns the object you want #each to return.
179 180 181 |
# File 'lib/sequel/dataset/misc.rb', line 179 def row_proc @opts[:row_proc] end |