Method: Sequel::Dataset#with_sql_all
- Defined in:
- lib/sequel/dataset/actions.rb
#with_sql_all(sql, &block) ⇒ Object
Run the given SQL and return an array of all rows. If a block is given, each row is yielded to the block after all rows are loaded. See with_sql_each.
1078 1079 1080 |
# File 'lib/sequel/dataset/actions.rb', line 1078 def with_sql_all(sql, &block) _all(block){|a| with_sql_each(sql){|r| a << r}} end |