Method: Sequel::JDBC::Database#convert_types
- Defined in:
- lib/sequel/adapters/jdbc.rb
#convert_types ⇒ Object
Whether to convert some Java types to ruby types when retrieving rows. True by default, can be set to false to roughly double performance when fetching rows.
173 174 175 |
# File 'lib/sequel/adapters/jdbc.rb', line 173 def convert_types @convert_types end |