Method: Sequel::Model::Associations::AssociationReflection#cloneable?
- Defined in:
- lib/sequel/model/associations.rb
#cloneable?(ref) ⇒ Boolean
Whether you are able to clone from the given association type to the current association type, true by default only if the types match.
191 192 193 |
# File 'lib/sequel/model/associations.rb', line 191 def cloneable?(ref) ref[:type] == self[:type] end |