Method: Sequel::Model::Associations::AssociationReflection#inspect
- Defined in:
- lib/sequel/model/associations.rb
#inspect ⇒ Object
Show which type of reflection this is, and a guess at what code was used to create the association.
435 436 437 438 439 440 441 442 443 |
# File 'lib/sequel/model/associations.rb', line 435 def inspect o = self[:orig_opts].dup o.delete(:class) o.delete(:class_name) o.delete(:block) unless o[:block] o[:class] = self[:orig_class] if self[:orig_class] "#<#{self.class} #{self[:model]}.#{self[:type]} #{self[:name].inspect}#{", #{o.inspect[1...-1]}" unless o.empty?}>" end |