Method: Sequel::Model::Associations::AssociationReflection#reciprocal_array?
- Defined in:
- lib/sequel/model/associations.rb
#reciprocal_array? ⇒ Boolean
Whether the reciprocal of this association returns an array of objects instead of a single object, true by default.
533 534 535 |
# File 'lib/sequel/model/associations.rb', line 533 def reciprocal_array? true end |