Method: Sequel::Model::Associations::AssociationReflection#remove_before_destroy?
- Defined in:
- lib/sequel/model/associations.rb
#remove_before_destroy? ⇒ Boolean
Whether associated objects need to be removed from the association before being destroyed in order to preserve referential integrity.
544 545 546 |
# File 'lib/sequel/model/associations.rb', line 544 def remove_before_destroy? true end |