Method: Sequel::Model::Associations::ClassMethods#associate
- Defined in:
- lib/sequel/model/associations.rb
#associate(type, name, opts = OPTS, &block) ⇒ Object
Associates a related model with the current model. The following types are supported:
- :many_to_one
-
Foreign key in current model’s table points to associated model’s primary key. Each associated model object can be associated with more than one current model objects. Each current model object can be associated with only one associated model object.
- :one_to_many
-
Foreign key in associated model’s table points to this model’s primary key. Each current model object can be associated with more than one associated model objects. Each associated model object can be associated with only one current model object.
- :one_through_one
-
Similar to many_to_many in terms of foreign keys, but only one object is associated to the current object through the association. Provides only getter methods, no setter or modification methods.
- :one_to_one
-
Similar to one_to_many in terms of foreign keys, but only one object is associated to the current object through the association. The methods created are similar to many_to_one, except that the one_to_one setter method saves the passed object.
- :many_to_many
-
A join table is used that has a foreign key that points to this model’s primary key and a foreign key that points to the associated model’s primary key. Each current model object can be associated with many associated model objects, and each associated model object can be associated with many current model objects.
The following options can be supplied:
Multiple Types
- :adder
-
Proc used to define the private add* method for doing the database work to associate the given object to the current object (*_to_many assocations). Set to nil to not define a add_* method for the association.
- :after_add
-
Symbol, Proc, or array of both/either specifying a callback to call after a new item is added to the association.
- :after_load
-
Symbol, Proc, or array of both/either specifying a callback to call after the associated record(s) have been retrieved from the database.
- :after_remove
-
Symbol, Proc, or array of both/either specifying a callback to call after an item is removed from the association.
- :after_set
-
Symbol, Proc, or array of both/either specifying a callback to call after an item is set using the association setter method.
- :allow_eager
-
If set to false, you cannot load the association eagerly via eager or eager_graph
- :allow_eager_graph
-
If set to false, you cannot load the association eagerly via eager_graph.
- :allow_filtering_by
-
If set to false, you cannot use the association when filtering
- :before_add
-
Symbol, Proc, or array of both/either specifying a callback to call before a new item is added to the association.
- :before_remove
-
Symbol, Proc, or array of both/either specifying a callback to call before an item is removed from the association.
- :before_set
-
Symbol, Proc, or array of both/either specifying a callback to call before an item is set using the association setter method.
- :cartesian_product_number
-
the number of joins completed by this association that could cause more than one row for each row in the current table (default: 0 for many_to_one, one_to_one, and one_through_one associations, 1 for one_to_many and many_to_many associations).
- :class
-
The associated class or its name as a string or symbol. If not given, uses the association’s name, which is camelized (and singularized unless the type is :many_to_one, :one_to_one, or one_through_one). If this is specified as a string or symbol, you must specify the full class name (e.g. “::SomeModule::MyModel”).
- :class_namespace
-
If :class is given as a string or symbol, sets the default namespace in which to look for the class.
class: 'Foo', class_namespace: 'Bar'looks for::Bar::Foo.) - :clearer
-
Proc used to define the private remove_all* method for doing the database work to remove all objects associated to the current object (*_to_many assocations). Set to nil to not define a remove_all_* method for the association.
- :clone
-
Merge the current options and block into the options and block used in defining the given association. Can be used to DRY up a bunch of similar associations that all share the same options such as :class and :key, while changing the order and block used.
- :conditions
-
The conditions to use to filter the association, can be any argument passed to where. This option is not respected when using eager_graph or association_join, unless it is hash or array of two element arrays. Consider also specifying the :graph_block option if the value for this option is not a hash or array of two element arrays and you plan to use this association in eager_graph or association_join.
- :dataset
-
A proc that is used to define the method to get the base dataset to use (before the other options are applied). If the proc accepts an argument, it is passed the related association reflection. It is a best practice to always have the dataset accept an argument and use the argument to return the appropriate dataset.
- :distinct
-
Use the DISTINCT clause when selecting associating object, both when lazy loading and eager loading via .eager (but not when using .eager_graph).
- :eager
-
The associations to eagerly load via
eagerwhen loading the associated object(s). - :eager_block
-
If given, use the block instead of the default block when eagerly loading. To not use a block when eager loading (when one is used normally), set to nil.
- :eager_graph
-
The associations to eagerly load via
eager_graphwhen loading the associated object(s). many_to_many associations with this option cannot be eagerly loaded viaeager. - :eager_grapher
-
A proc to use to implement eager loading via
eager_graph, overriding the default. Takes an options hash with at least the entries :self (the receiver of the eager_graph call), :table_alias (the alias to use for table to graph into the association), and :implicit_qualifier (the alias that was used for the current table). Should return a copy of the dataset with the association graphed into it. - :eager_limit_strategy
-
Determines the strategy used for enforcing limits and offsets when eager loading associations via the
eagermethod. - :eager_loader
-
A proc to use to implement eager loading, overriding the default. Takes a single hash argument, with at least the keys: :rows, which is an array of current model instances, :associations, which is a hash of dependent associations, :self, which is the dataset doing the eager loading, :eager_block, which is a dynamic callback that should be called with the dataset, and :id_map, which is a mapping of key values to arrays of current model instances. In the proc, the associated records should be queried from the database and the associations cache for each record should be populated.
- :eager_loader_key
-
A symbol for the key column to use to populate the key_hash for the eager loader. Can be set to nil to not populate the key_hash.
- :eager_loading_predicate_transform
-
A callable object with which to transform the predicate key values used when eager loading. Called with two arguments, the array of predicate key values, and a the reflection for the association being eager loaded.
- :extend
-
A module or array of modules to extend the dataset with.
- :filter_limit_strategy
-
Determines the strategy used for enforcing limits and offsets when filtering by limited associations. Possible options are :window_function, :distinct_on, or :correlated_subquery depending on association type and database type.
- :graph_alias_base
-
The base name to use for the table alias when eager graphing. Defaults to the name of the association. If the alias name has already been used in the query, Sequel will create a unique alias by appending a numeric suffix (e.g. alias_0, alias_1, …) until the alias is unique.
- :graph_block
-
The block to pass to join_table when eagerly loading the association via
eager_graph. - :graph_conditions
-
The additional conditions to use on the SQL join when eagerly loading the association via
eager_graph. Should be a hash or an array of two element arrays. If not specified, the :conditions option is used if it is a hash or array of two element arrays. - :graph_join_type
-
The type of SQL join to use when eagerly loading the association via eager_graph. Defaults to :left_outer.
- :graph_only_conditions
-
The conditions to use on the SQL join when eagerly loading the association via
eager_graph, instead of the default conditions specified by the foreign/primary keys. This option causes the :graph_conditions option to be ignored. - :graph_order
-
the order to use when using eager_graph, instead of the default order. This should be used in the case where :order contains an identifier qualified by the table’s name, which may not match the alias used when eager graphing. By setting this to the unqualified identifier, it will be automatically qualified when using eager_graph.
- :graph_select
-
A column or array of columns to select from the associated table when eagerly loading the association via
eager_graph. Defaults to all columns in the associated table. - :graph_use_association_block
-
Makes eager_graph consider the association block. Without this, eager_graph ignores the bock and only use the :graph_* options.
- :instance_specific
-
Marks the association as instance specific. Should be used if the association block uses instance specific state, or transient state (accessing current date/time, etc.).
- :limit
-
Limit the number of records to the provided value. Use an array with two elements for the value to specify a limit (first element) and an offset (second element).
- :methods_module
-
The module that methods the association creates will be placed into. Defaults to the module containing the model’s columns.
- :no_association_method
-
Do not add a method for the association. This can save memory if the association method is never used.
- :no_dataset_method
-
Do not add a method for the association dataset. This can save memory if the dataset method is never used.
- :order
-
the column(s) by which to order the association dataset. Can be a singular column symbol or an array of column symbols.
- :order_eager_graph
-
Whether to add the association’s order to the graphed dataset’s order when graphing via
eager_graph. Defaults to true, so set to false to disable. - :read_only
-
Do not add a setter method (for many_to_one or one_to_one associations), or add_/remove_/remove_all_ methods (for one_to_many and many_to_many associations).
- :reciprocal
-
the symbol name of the reciprocal association, if it exists. By default, Sequel will try to determine it by looking at the associated model’s assocations for a association that matches the current association’s key(s). Set to nil to not use a reciprocal.
- :remover
-
Proc used to define the private remove* method for doing the database work to remove the association between the given object and the current object (*_to_many assocations). Set to nil to not define a remove_* method for the association.
- :select
-
the columns to select. Defaults to the associated class’s table_name.* in an association that uses joins, which means it doesn’t include the attributes from the join table. If you want to include the join table attributes, you can use this option, but beware that the join table attributes can clash with attributes from the model table, so you should alias any attributes that have the same name in both the join table and the associated table.
- :setter
-
Proc used to define the private _*= method for doing the work to setup the assocation between the given object and the current object (*_to_one associations). Set to nil to not define a setter method for the association.
- :subqueries_per_union
-
The number of subqueries to use in each UNION query, for eager loading limited associations using the default :union strategy.
- :use_placeholder_loader
-
Whether to use a placeholder loader when eager loading the association. Can be set to false to disable the use of a placeholder loader if one would be used by default.
- :validate
-
Set to false to not validate when implicitly saving any associated object.
:many_to_one
- :key
-
foreign key in current model’s table that references associated model’s primary key, as a symbol. Defaults to :“#name_id”. Can use an array of symbols for a composite key association.
- :key_column
-
Similar to, and usually identical to, :key, but :key refers to the model method to call, where :key_column refers to the underlying column. Should only be used if the model method differs from the foreign key column, in conjunction with defining a model alias method for the key column.
- :primary_key
-
column in the associated table that :key option references, as a symbol. Defaults to the primary key of the associated table. Can use an array of symbols for a composite key association.
- :primary_key_method
-
the method symbol or array of method symbols to call on the associated object to get the foreign key values. Defaults to :primary_key option.
- :qualify
-
Whether to use qualified primary keys when loading the association. The default is true, so you must set to false to not qualify. Qualification rarely causes problems, but it’s necessary to disable in some cases, such as when you are doing a JOIN USING operation on the column on Oracle.
:one_to_many and :one_to_one
- :key
-
foreign key in associated model’s table that references current model’s primary key, as a symbol. Defaults to :“#Sequel::Model::Associations::ClassMethods.selfself.nameself.name.underscore_id”. Can use an array of symbols for a composite key association.
- :key_method
-
the method symbol or array of method symbols to call on the associated object to get the foreign key values. Defaults to :key option.
- :primary_key
-
column in the current table that :key option references, as a symbol. Defaults to primary key of the current table. Can use an array of symbols for a composite key association.
- :primary_key_column
-
Similar to, and usually identical to, :primary_key, but :primary_key refers to the model method call, where :primary_key_column refers to the underlying column. Should only be used if the model method differs from the primary key column, in conjunction with defining a model alias method for the primary key column.
- :raise_on_save_failure
-
Do not raise exceptions for hook or validation failures when saving associated objects in the add/remove methods (return nil instead) [one_to_many only].
:many_to_many and :one_through_one
- :graph_join_table_block
-
The block to pass to
join_tablefor the join table when eagerly loading the association viaeager_graph. - :graph_join_table_conditions
-
The additional conditions to use on the SQL join for the join table when eagerly loading the association via
eager_graph. Should be a hash or an array of two element arrays. - :graph_join_table_join_type
-
The type of SQL join to use for the join table when eagerly loading the association via
eager_graph. Defaults to the :graph_join_type option or :left_outer. - :graph_join_table_only_conditions
-
The conditions to use on the SQL join for the join table when eagerly loading the association via
eager_graph, instead of the default conditions specified by the foreign/primary keys. This option causes the :graph_join_table_conditions option to be ignored. - :join_table
-
name of table that includes the foreign keys to both the current model and the associated model, as a symbol. Defaults to the name of current model and name of associated model, pluralized, underscored, sorted, and joined with ‘_’.
- :join_table_block
-
proc that can be used to modify the dataset used in the add/remove/remove_all methods. Should accept a dataset argument and return a modified dataset if present.
- :join_table_db
-
When retrieving records when using lazy loading or eager loading via
eager, instead of a join between to the join table and the associated table, use a separate query for the join table using the given Database object. - :left_key
-
foreign key in join table that points to current model’s primary key, as a symbol. Defaults to :“#Sequel::Model::Associations::ClassMethods.selfself.nameself.name.underscore_id”. Can use an array of symbols for a composite key association.
- :left_primary_key
-
column in current table that :left_key points to, as a symbol. Defaults to primary key of current table. Can use an array of symbols for a composite key association.
- :left_primary_key_column
-
Similar to, and usually identical to, :left_primary_key, but :left_primary_key refers to the model method to call, where :left_primary_key_column refers to the underlying column. Should only be used if the model method differs from the left primary key column, in conjunction with defining a model alias method for the left primary key column.
- :right_key
-
foreign key in join table that points to associated model’s primary key, as a symbol. Defaults to :“#Sequel::Model::Associations::ClassMethods.namename.to_sname.to_s.singularize_id”. Can use an array of symbols for a composite key association.
- :right_primary_key
-
column in associated table that :right_key points to, as a symbol. Defaults to primary key of the associated table. Can use an array of symbols for a composite key association.
- :right_primary_key_method
-
the method symbol or array of method symbols to call on the associated object to get the foreign key values for the join table. Defaults to :right_primary_key option.
- :uniq
-
Adds a after_load callback that makes the array of objects unique.
1872 1873 1874 1875 1876 1877 1878 1879 1880 1881 1882 1883 1884 1885 1886 1887 1888 1889 1890 1891 1892 1893 1894 1895 1896 1897 1898 1899 1900 1901 1902 1903 1904 1905 1906 1907 1908 1909 1910 1911 1912 1913 1914 1915 1916 1917 1918 1919 1920 1921 1922 1923 1924 1925 1926 1927 1928 1929 1930 1931 1932 1933 1934 1935 1936 1937 1938 1939 1940 1941 1942 1943 1944 1945 1946 1947 1948 1949 1950 |
# File 'lib/sequel/model/associations.rb', line 1872 def associate(type, name, opts = OPTS, &block) raise(Error, 'invalid association type') unless assoc_class = Sequel.synchronize{ASSOCIATION_TYPES[type]} raise(Error, 'Model.associate name argument must be a symbol') unless name.is_a?(Symbol) # dup early so we don't modify opts orig_opts = opts.dup if opts[:clone] cloned_assoc = association_reflection(opts[:clone]) remove_class_name = orig_opts[:class] && !orig_opts[:class_name] orig_opts = cloned_assoc[:orig_opts].merge(orig_opts) orig_opts.delete(:class_name) if remove_class_name end opts = Hash[] if = [type] opts.merge!() end opts.merge!(orig_opts) opts.merge!(:type => type, :name => name, :cache=>({} if cache_associations), :model => self) opts[:block] = block if block opts[:instance_specific] = true if orig_opts[:dataset] if !opts.has_key?(:instance_specific) && (block || orig_opts[:block]) # It's possible the association is instance specific, in that it depends on # values other than the foreign key value. This needs to be checked for # in certain places to disable optimizations. opts[:instance_specific] = _association_instance_specific_default(name) end if (orig_opts[:instance_specific] || orig_opts[:dataset]) && !opts.has_key?(:allow_eager) && !opts[:eager_loader] # For associations explicitly marked as instance specific, or that use the # :dataset option, where :allow_eager is not set, and no :eager_loader is # provided, disallow eager loading. In these cases, eager loading is # unlikely to work. This is not done for implicit setting of :instance_specific, # because implicit use is done by default for all associations with blocks, # and the vast majority of associations with blocks use the block for filtering # in a manner compatible with eager loading. opts[:allow_eager] = false end opts = assoc_class.new.merge!(opts) if opts[:clone] && !opts.cloneable?(cloned_assoc) raise(Error, "cannot clone an association to an association of different type (association #{name} with type #{type} cloning #{opts[:clone]} with type #{cloned_assoc[:type]})") end opts[:use_placeholder_loader] = !opts[:instance_specific] && !opts[:eager_graph] unless opts.include?(:use_placeholder_loader) opts[:eager_block] = opts[:block] unless opts.include?(:eager_block) opts[:graph_join_type] ||= :left_outer opts[:order_eager_graph] = true unless opts.include?(:order_eager_graph) conds = opts[:conditions] opts[:graph_alias_base] ||= name opts[:graph_conditions] = conds if !opts.include?(:graph_conditions) and Sequel.condition_specifier?(conds) opts[:graph_conditions] = opts.fetch(:graph_conditions, []).to_a opts[:graph_select] = Array(opts[:graph_select]) if opts[:graph_select] [:before_add, :before_remove, :after_add, :after_remove, :after_load, :before_set, :after_set].each do |cb_type| opts[cb_type] = Array(opts[cb_type]) if opts[cb_type] end if opts[:extend] opts[:extend] = Array(opts[:extend]) opts[:reverse_extend] = opts[:extend].reverse end late_binding_class_option(opts, opts.returns_array? ? singularize(name) : name) # Remove :class entry if it exists and is nil, to work with cached_fetch opts.delete(:class) unless opts[:class] opts[:_hash] = [self, name].hash def_association(opts) orig_opts.delete(:clone) opts[:orig_class] = orig_opts[:class] || orig_opts[:class_name] orig_opts.merge!(:class_name=>opts[:class_name], :class=>opts[:class], :block=>opts[:block]) opts[:orig_opts] = orig_opts # don't add to association_reflections until we are sure there are no errors association_reflections[name] = opts end |