Method: Sequel::Model::Associations::ClassMethods#autoreloading_associations
- Defined in:
- lib/sequel/model/associations.rb
#autoreloading_associations ⇒ Object (readonly)
Hash with column symbol keys and arrays of many_to_one association symbols that should be cleared when the column value changes.
1605 1606 1607 |
# File 'lib/sequel/model/associations.rb', line 1605 def autoreloading_associations @autoreloading_associations end |