Method: Sequel::Model::Associations::DatasetMethods#ungraphed
- Defined in:
- lib/sequel/model/associations.rb
#ungraphed ⇒ Object
Do not attempt to split the result set into associations, just return results as simple objects. This is useful if you want to use eager_graph as a shortcut to have all of the joins and aliasing set up, but want to do something else with the dataset.
3366 3367 3368 3369 3370 3371 3372 |
# File 'lib/sequel/model/associations.rb', line 3366 def ungraphed ds = super.clone(:eager_graph=>nil) if (eg = @opts[:eager_graph]) && (rp = eg[:row_proc]) ds = ds.with_row_proc(rp) end ds end |