Method: Sequel::Model::Associations::EagerGraphLoader#records_map
- Defined in:
- lib/sequel/model/associations.rb
#records_map ⇒ Object (readonly)
Hash with table alias symbol keys and subhash values mapping primary key symbols (or array of symbols) to model instances. Used so that only a single model instance is created for each object.
3779 3780 3781 |
# File 'lib/sequel/model/associations.rb', line 3779 def records_map @records_map end |