Method: Sequel::Plugins::BlacklistSecurity::InstanceMethods#set_except
- Defined in:
- lib/sequel/plugins/blacklist_security.rb
#set_except(hash, *except) ⇒ Object
Set all values using the entries in the hash, except for the keys given in except. You should probably use set_fields instead of this method, as blacklist approaches to security are a bad idea.
artist.set_except({name: 'Jim'}, :hometown)
artist.name # => 'Jim'
75 76 77 |
# File 'lib/sequel/plugins/blacklist_security.rb', line 75 def set_except(hash, *except) set_restricted(hash, ExceptionList.new(except.flatten)) end |