Method: Sequel::Plugins::List::InstanceMethods#next
- Defined in:
- lib/sequel/plugins/list.rb
#next(n = 1) ⇒ Object
The model instance the given number of places below this model instance in the list, or 1 place below if no argument is given.
172 173 174 |
# File 'lib/sequel/plugins/list.rb', line 172 def next(n = 1) n == 0 ? self : at_position(position_value + n) end |