Method: Sequel::Plugins::PgArrayAssociations::ManyToPgArrayAssociationReflection#primary_key
- Defined in:
- lib/sequel/plugins/pg_array_associations.rb
#primary_key ⇒ Object
The column in the current table that the keys in the array column in the associated table reference.
144 145 146 |
# File 'lib/sequel/plugins/pg_array_associations.rb', line 144 def primary_key self[:primary_key] end |