Method: Sequel::Plugins::SingleTableInheritance::ClassMethods#sti_key_chooser
- Defined in:
- lib/sequel/plugins/single_table_inheritance.rb
#sti_key_chooser ⇒ Object (readonly)
A proc which returns the value to use for new instances. This defaults to a lookup in the key map.
151 152 153 |
# File 'lib/sequel/plugins/single_table_inheritance.rb', line 151 def sti_key_chooser @sti_key_chooser end |