Method: Sequel::Plugins::SingleTableInheritance::ClassMethods#sti_subclass_added
- Defined in:
- lib/sequel/plugins/single_table_inheritance.rb
#sti_subclass_added(key) ⇒ Object
Make sure that all subclasses of the parent class correctly include keys for all of their descendant classes.
179 180 181 182 183 184 185 |
# File 'lib/sequel/plugins/single_table_inheritance.rb', line 179 def sti_subclass_added(key) if sti_key_array key_array = Array(key) Sequel.synchronize{sti_key_array.push(*key_array)} superclass.sti_subclass_added(key) end end |