Method: Sequel::Postgres::DatasetMethods#merge_delete_when_not_matched_by_source
- Defined in:
- lib/sequel/adapters/shared/postgres.rb
#merge_delete_when_not_matched_by_source(&block) ⇒ Object
Return a dataset with a WHEN NOT MATCHED BY SOURCE THEN DELETE clause added to the MERGE statement. If a block is passed, treat it as a virtual row and use it as additional conditions for the match.
merge_delete_not_matched_by_source
# WHEN NOT MATCHED BY SOURCE THEN DELETE
merge_delete_not_matched_by_source{a > 30}
# WHEN NOT MATCHED BY SOURCE AND (a > 30) THEN DELETE
2339 2340 2341 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 2339 def merge_delete_when_not_matched_by_source(&block) _merge_when(:type=>:delete_not_matched_by_source, &block) end |