Method: Sequel::Postgres::ExtendedDateSupport#convert_infinite_timestamps
- Defined in:
- lib/sequel/extensions/pg_extended_date_support.rb
#convert_infinite_timestamps ⇒ Object
Whether infinite timestamps/dates should be converted on retrieval. By default, no conversion is done, so an error is raised if you attempt to retrieve an infinite timestamp/date. You can set this to :nil to convert to nil, :string to leave as a string, or :float to convert to an infinite float.
67 68 69 |
# File 'lib/sequel/extensions/pg_extended_date_support.rb', line 67 def @convert_infinite_timestamps end |