Method: Sequel::Schema::AlterTableGenerator#initialize
- Defined in:
- lib/sequel/database/schema_generator.rb
#initialize(db, &block) ⇒ AlterTableGenerator
Set the Database object to which to apply the changes, and evaluate the block in the context of this object.
437 438 439 440 441 |
# File 'lib/sequel/database/schema_generator.rb', line 437 def initialize(db, &block) @db = db @operations = [] instance_exec(&block) if block end |