Method: Sequel::Schema::CreateTableGenerator#Bignum
- Defined in:
- lib/sequel/database/schema_generator.rb
#Bignum(name, opts = OPTS) ⇒ Object
Use custom Bignum method to use :Bignum instead of Bignum class, to work correctly in cases where Bignum is the same as Integer.
45 46 47 |
# File 'lib/sequel/database/schema_generator.rb', line 45 def Bignum(name, opts=OPTS) column(name, :Bignum, opts) end |