Method: Sequel::SequelMethods#convert_two_digit_years
- Defined in:
- lib/sequel/core.rb
#convert_two_digit_years ⇒ Object
Sequel converts two digit years in Dates and DateTimes by default, so 01/02/03 is interpreted at January 2nd, 2003, and 12/13/99 is interpreted as December 13, 1999. You can override this to treat those dates as January 2nd, 0003 and December 13, 0099, respectively, by:
Sequel.convert_two_digit_years = false
48 49 50 |
# File 'lib/sequel/core.rb', line 48 def convert_two_digit_years @convert_two_digit_years end |