Class: Sequel::BasicObject
Overview
If on Ruby 1.8, create a Sequel::BasicObject class that is similar to the the Ruby 1.9 BasicObject class. This is used in a few places where proxy objects are needed that respond to any method call.
Direct Known Subclasses
Plugins::AssociationProxies::AssociationProxy, SQL::VirtualRow