Class: Sequel::JDBC::Database

Inherits:
Database show all
Defined in:
lib/sequel/adapters/jdbc.rb

Overview

JDBC Databases offer a fairly uniform interface that does not change much based on the sub adapter.

Constant Summary collapse

DatasetClass =
self

Constants inherited from Database

Database::ADAPTERS, Database::AUTOINCREMENT, Database::COLUMN_DEFINITION_ORDER, Database::COLUMN_SCHEMA_DATETIME_TYPES, Database::COLUMN_SCHEMA_STRING_TYPES, Database::COMBINABLE_ALTER_TABLE_OPS, Database::COMMA_SEPARATOR, Database::CURRENT_TIMESTAMP_RE, Database::DEFAULT_DATABASE_ERROR_REGEXPS, Database::DEFAULT_JOIN_TABLE_COLUMN_OPTIONS, Database::DEFAULT_STRING_COLUMN_SIZE, Database::EXTENSIONS, Database::NOT_NULL, Database::NULL, Database::OPTS, Database::PRIMARY_KEY, Database::SCHEMA_TYPE_CLASSES, Database::SQL_BEGIN, Database::SQL_COMMIT, Database::SQL_RELEASE_SAVEPOINT, Database::SQL_ROLLBACK, Database::SQL_ROLLBACK_TO_SAVEPOINT, Database::SQL_SAVEPOINT, Database::STRING_DEFAULT_RE, Database::TEMPORARY, Database::TRANSACTION_BEGIN, Database::TRANSACTION_COMMIT, Database::TRANSACTION_ISOLATION_LEVELS, Database::TRANSACTION_ROLLBACK, Database::UNDERSCORE, Database::UNIQUE, Database::UNSIGNED

Instance Attribute Summary collapse

Attributes inherited from Database

#cache_schema, #dataset_class, #default_string_column_size, #log_connection_info, #log_warn_duration, #loggers, #opts, #pool, #prepared_statements, #sql_log_level, #timezone, #transaction_isolation_level

Instance Method Summary collapse

Methods inherited from Database

#<<, #[], adapter_class, adapter_scheme, #adapter_scheme, #add_column, #add_index, #add_servers, #after_commit, after_initialize, #after_rollback, #alter_table, #alter_table_generator, #call, #cast_type_literal, connect, #create_join_table, #create_join_table!, #create_join_table?, #create_or_replace_view, #create_table, #create_table!, #create_table?, #create_table_generator, #create_view, #dataset, #disconnect, #drop_column, #drop_index, #drop_join_table, #drop_table, #drop_table?, #drop_view, #each_server, #extend_datasets, #extension, extension, #fetch, #from, #from_application_timestamp, #get, #global_index_namespace?, #in_transaction?, #initialize, #initialize_copy, #inspect, #literal, #literal_symbol, #literal_symbol_set, load_adapter, #log_connection_yield, #log_exception, #log_info, #log_yield, #logger=, #prepared_statement, #quote_identifier, register_extension, #remove_servers, #rename_column, #rename_table, #rollback_checker, #run, run_after_initialize, #schema, #schema_type_class, #select, #serial_primary_key_options, #servers, #set_column_default, #set_column_type, #set_prepared_statement, set_shared_adapter_scheme, #sharded?, single_threaded, single_threaded=, #single_threaded?, #supports_create_table_if_not_exists?, #supports_deferrable_constraints?, #supports_deferrable_foreign_key_constraints?, #supports_drop_table_if_exists?, #supports_foreign_key_parsing?, #supports_index_parsing?, #supports_partial_indexes?, #supports_prepared_transactions?, #supports_savepoints?, #supports_savepoints_in_prepared_transactions?, #supports_schema_parsing?, #supports_table_listing?, #supports_transaction_isolation_levels?, #supports_transactional_ddl?, #supports_view_listing?, #supports_views_with_check_option?, #supports_views_with_local_check_option?, #synchronize, #table_exists?, #test_connection, #to_application_timestamp, #transaction, #typecast_value, #url, #valid_connection?

Methods included from Metaprogramming

#meta_def

Constructor Details

This class inherits a constructor from Sequel::Database

Instance Attribute Details

#basic_type_convertor_mapObject (readonly)

Map of JDBC type ids to callable objects that return appropriate ruby or java values.



171
172
173
# File 'lib/sequel/adapters/jdbc.rb', line 171

def basic_type_convertor_map
  @basic_type_convertor_map
end

#convert_typesObject

Whether to convert some Java types to ruby types when retrieving rows. True by default, can be set to false to roughly double performance when fetching rows.



161
162
163
# File 'lib/sequel/adapters/jdbc.rb', line 161

def convert_types
  @convert_types
end

#database_typeObject (readonly)

The type of database we are connecting to



153
154
155
# File 'lib/sequel/adapters/jdbc.rb', line 153

def database_type
  @database_type
end

#driverObject (readonly)

The Java database driver we are using (should be a Java class)



156
157
158
# File 'lib/sequel/adapters/jdbc.rb', line 156

def driver
  @driver
end

#fetch_sizeObject

The fetch size to use for JDBC Statement objects created by this database. By default, this is nil so a fetch size is not set explicitly.



165
166
167
# File 'lib/sequel/adapters/jdbc.rb', line 165

def fetch_size
  @fetch_size
end

#type_convertor_mapObject (readonly)

Map of JDBC type ids to callable objects that return appropriate ruby values.



168
169
170
# File 'lib/sequel/adapters/jdbc.rb', line 168

def type_convertor_map
  @type_convertor_map
end

Instance Method Details

#call_sproc(name, opts = OPTS) ⇒ Object

Execute the given stored procedure with the give name. If a block is given, the stored procedure should return rows.



175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
# File 'lib/sequel/adapters/jdbc.rb', line 175

def call_sproc(name, opts = OPTS)
  args = opts[:args] || []
  sql = "{call #{name}(#{args.map{'?'}.join(',')})}"
  synchronize(opts[:server]) do |conn|
    cps = conn.prepareCall(sql)

    i = 0
    args.each{|arg| set_ps_arg(cps, arg, i+=1)}

    begin
      if block_given?
        yield log_connection_yield(sql, conn){cps.executeQuery}
      else
        case opts[:type]
        when :insert
          log_connection_yield(sql, conn){cps.executeUpdate}
          last_insert_id(conn, opts)
        else
          log_connection_yield(sql, conn){cps.executeUpdate}
        end
      end
    rescue NativeException, JavaSQL::SQLException => e
      raise_error(e)
    ensure
      cps.close
    end
  end
end

#connect(server) ⇒ Object

Connect to the database using JavaSQL::DriverManager.getConnection.



205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
# File 'lib/sequel/adapters/jdbc.rb', line 205

def connect(server)
  opts = server_opts(server)
  conn = if jndi?
    get_connection_from_jndi
  else
    args = [uri(opts)]
    args.concat([opts[:user], opts[:password]]) if opts[:user] && opts[:password]
    begin
      JavaSQL::DriverManager.setLoginTimeout(opts[:login_timeout]) if opts[:login_timeout]
      raise StandardError, "skipping regular connection" if opts[:jdbc_properties]
      JavaSQL::DriverManager.getConnection(*args)
    rescue JavaSQL::SQLException, NativeException, StandardError => e
      raise e unless driver
      # If the DriverManager can't get the connection - use the connect
      # method of the driver. (This happens under Tomcat for instance)
      props = java.util.Properties.new
      if opts && opts[:user] && opts[:password]
        props.setProperty("user", opts[:user])
        props.setProperty("password", opts[:password])
      end
      opts[:jdbc_properties].each{|k,v| props.setProperty(k.to_s, v)} if opts[:jdbc_properties]
      begin
        c = driver.new.connect(args[0], props)
        raise(Sequel::DatabaseError, 'driver.new.connect returned nil: probably bad JDBC connection string') unless c
        c
      rescue JavaSQL::SQLException, NativeException, StandardError => e2
        if e2.respond_to?(:message=) && e2.message != e.message
          e2.message = "#{e2.message}\n#{e.class.name}: #{e.message}"
        end
        raise e2
      end
    end
  end
  setup_connection(conn)
end

#disconnect_connection(c) ⇒ Object

Close given adapter connections, and delete any related prepared statements.



242
243
244
245
# File 'lib/sequel/adapters/jdbc.rb', line 242

def disconnect_connection(c)
  @connection_prepared_statements_mutex.synchronize{@connection_prepared_statements.delete(c)}
  c.close
end

#execute(sql, opts = OPTS, &block) ⇒ Object Also known as: execute_dui

Execute the given SQL. If a block is given, if should be a SELECT statement or something else that returns rows.



249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
# File 'lib/sequel/adapters/jdbc.rb', line 249

def execute(sql, opts=OPTS, &block)
  return call_sproc(sql, opts, &block) if opts[:sproc]
  return execute_prepared_statement(sql, opts, &block) if [Symbol, Dataset].any?{|c| sql.is_a?(c)}
  synchronize(opts[:server]) do |conn|
    statement(conn) do |stmt|
      if block
        if size = fetch_size
          stmt.setFetchSize(size)
        end
        yield log_connection_yield(sql, conn){stmt.executeQuery(sql)}
      else
        case opts[:type]
        when :ddl
          log_connection_yield(sql, conn){stmt.execute(sql)}
        when :insert
          log_connection_yield(sql, conn){execute_statement_insert(stmt, sql)}
          last_insert_id(conn, Hash[opts].merge!(:stmt=>stmt))
        else
          log_connection_yield(sql, conn){stmt.executeUpdate(sql)}
        end
      end
    end
  end
end

#execute_ddl(sql, opts = OPTS) ⇒ Object

Execute the given DDL SQL, which should not return any values or rows.



277
278
279
280
281
# File 'lib/sequel/adapters/jdbc.rb', line 277

def execute_ddl(sql, opts=OPTS)
  opts = Hash[opts]
  opts[:type] = :ddl
  execute(sql, opts)
end

#execute_insert(sql, opts = OPTS) ⇒ Object

Execute the given INSERT SQL, returning the last inserted row id.



285
286
287
288
289
# File 'lib/sequel/adapters/jdbc.rb', line 285

def execute_insert(sql, opts=OPTS)
  opts = Hash[opts]
  opts[:type] = :insert
  execute(sql, opts)
end

#foreign_key_list(table, opts = OPTS) ⇒ Object

Use the JDBC metadata to get a list of foreign keys for the table.



298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
# File 'lib/sequel/adapters/jdbc.rb', line 298

def foreign_key_list(table, opts=OPTS)
  m = output_identifier_meth
  schema, table = (table, opts)
  foreign_keys = {}
  (:getImportedKeys, nil, schema, table) do |r|
    if fk = foreign_keys[r[:fk_name]]
      fk[:columns] << [r[:key_seq], m.call(r[:fkcolumn_name])]
      fk[:key] << [r[:key_seq], m.call(r[:pkcolumn_name])]
    elsif r[:fk_name]
      foreign_keys[r[:fk_name]] = {:name=>m.call(r[:fk_name]), :columns=>[[r[:key_seq], m.call(r[:fkcolumn_name])]], :table=>m.call(r[:pktable_name]), :key=>[[r[:key_seq], m.call(r[:pkcolumn_name])]]}
    end
  end
  foreign_keys.values.each do |fk|
    [:columns, :key].each do |k|
      fk[k] = fk[k].sort.map{|_, v| v}
    end
  end
end

#freezeObject



291
292
293
294
295
# File 'lib/sequel/adapters/jdbc.rb', line 291

def freeze
  @type_convertor_map.freeze
  @basic_type_convertor_map.freeze
  super
end

#indexes(table, opts = OPTS) ⇒ Object

Use the JDBC metadata to get the index information for the table.



318
319
320
321
322
323
324
325
326
327
328
329
# File 'lib/sequel/adapters/jdbc.rb', line 318

def indexes(table, opts=OPTS)
  m = output_identifier_meth
  schema, table = (table, opts)
  indexes = {}
  (:getIndexInfo, nil, schema, table, false, true) do |r|
    next unless name = r[:column_name]
    next if respond_to?(:primary_key_index_re, true) and r[:index_name] =~ primary_key_index_re 
    i = indexes[m.call(r[:index_name])] ||= {:columns=>[], :unique=>[false, 0].include?(r[:non_unique])}
    i[:columns] << m.call(name)
  end
  indexes
end

#jndi?Boolean

Whether or not JNDI is being used for this connection.

Returns:

  • (Boolean)


332
333
334
# File 'lib/sequel/adapters/jdbc.rb', line 332

def jndi?
  !!(uri =~ JNDI_URI_REGEXP)
end

#tables(opts = OPTS) ⇒ Object

All tables in this database



337
338
339
# File 'lib/sequel/adapters/jdbc.rb', line 337

def tables(opts=OPTS)
  get_tables('TABLE', opts)
end

#uri(opts = OPTS) ⇒ Object

The uri for this connection. You can specify the uri using the :uri, :url, or :database options. You don’t need to worry about this if you use Sequel.connect with the JDBC connectrion strings.



345
346
347
348
349
# File 'lib/sequel/adapters/jdbc.rb', line 345

def uri(opts=OPTS)
  opts = @opts.merge(opts)
  ur = opts[:uri] || opts[:url] || opts[:database]
  ur =~ /^\Ajdbc:/ ? ur : "jdbc:#{ur}"
end

#views(opts = OPTS) ⇒ Object

All views in this database



352
353
354
# File 'lib/sequel/adapters/jdbc.rb', line 352

def views(opts=OPTS)
  get_tables('VIEW', opts)
end