Module: Sequel::MSSQL::DatasetMethods
- Includes:
- EmulateOffsetWithRowNumber
- Included in:
- ADO::MSSQL::Dataset, JDBC::JTDS::Dataset, ODBC::MSSQL::Dataset, TinyTDS::Dataset
- Defined in:
- lib/sequel/adapters/shared/mssql.rb
Constant Summary collapse
- BOOL_TRUE =
'1'.freeze
- BOOL_FALSE =
'0'.freeze
- COMMA_SEPARATOR =
', '.freeze
- NOLOCK =
' WITH (NOLOCK)'.freeze
- UPDLOCK =
' WITH (UPDLOCK)'.freeze
- WILDCARD =
LiteralString.new('*').freeze
- CONSTANT_MAP =
{:CURRENT_DATE=>'CAST(CURRENT_TIMESTAMP AS DATE)'.freeze, :CURRENT_TIME=>'CAST(CURRENT_TIMESTAMP AS TIME)'.freeze}
- EXTRACT_MAP =
{:year=>"yy", :month=>"m", :day=>"d", :hour=>"hh", :minute=>"n", :second=>"s"}
- BRACKET_CLOSE =
Dataset::BRACKET_CLOSE
- BRACKET_OPEN =
Dataset::BRACKET_OPEN
- COMMA =
Dataset::COMMA
- PAREN_CLOSE =
Dataset::PAREN_CLOSE
- PAREN_SPACE_OPEN =
Dataset::PAREN_SPACE_OPEN
- SPACE =
Dataset::SPACE
- FROM =
Dataset::FROM
- APOS =
Dataset::APOS
- APOS_RE =
Dataset::APOS_RE
- DOUBLE_APOS =
Dataset::DOUBLE_APOS
- INTO =
Dataset::INTO
- DOUBLE_BRACKET_CLOSE =
']]'.freeze
- DATEPART_SECOND_OPEN =
"CAST((datepart(".freeze
- DATEPART_SECOND_MIDDLE =
') + datepart(ns, '.freeze
- DATEPART_SECOND_CLOSE =
")/1000000000.0) AS double precision)".freeze
- DATEPART_OPEN =
"datepart(".freeze
- TIMESTAMP_USEC_FORMAT =
".%03d".freeze
- OUTPUT_INSERTED =
" OUTPUT INSERTED.*".freeze
- HEX_START =
'0x'.freeze
- UNICODE_STRING_START =
"N'".freeze
- BACKSLASH_CRLF_RE =
/\\((?:\r\n)|\n)/.freeze
- BACKSLASH_CRLF_REPLACE =
'\\\\\\\\\\1\\1'.freeze
- TOP_PAREN =
" TOP (".freeze
- TOP =
" TOP ".freeze
- OUTPUT =
" OUTPUT ".freeze
- HSTAR =
"H*".freeze
- CASE_SENSITIVE_COLLATION =
'Latin1_General_CS_AS'.freeze
- CASE_INSENSITIVE_COLLATION =
'Latin1_General_CI_AS'.freeze
- DEFAULT_TIMESTAMP_FORMAT =
"'%Y-%m-%dT%H:%M:%S%N%z'".freeze
- FORMAT_DATE =
"'%Y%m%d'".freeze
- CROSS_APPLY =
'CROSS APPLY'.freeze
- OUTER_APPLY =
'OUTER APPLY'.freeze
- OFFSET =
" OFFSET ".freeze
- ROWS =
" ROWS".freeze
- ROWS_ONLY =
" ROWS ONLY".freeze
- FETCH_NEXT =
" FETCH NEXT ".freeze
Instance Attribute Summary collapse
-
#mssql_unicode_strings ⇒ Object
Use the database's mssql_unicode_strings setting if the dataset hasn't overridden it.
Instance Method Summary collapse
-
#complex_expression_sql_append(sql, op, args) ⇒ Object
MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
-
#constant_sql_append(sql, constant) ⇒ Object
MSSQL doesn't support the SQL standard CURRENT_DATE or CURRENT_TIME.
-
#cross_apply(table) ⇒ Object
Uses CROSS APPLY to join the given table into the current dataset.
-
#disable_insert_output ⇒ Object
Disable the use of INSERT OUTPUT.
-
#escape_like(string) ⇒ Object
MSSQL treats [] as a metacharacter in LIKE expresions.
-
#full_text_search(cols, terms, opts = OPTS) ⇒ Object
MSSQL uses the CONTAINS keyword for full text search.
-
#insert_select(*values) ⇒ Object
Use the OUTPUT clause to get the value of all columns for the newly inserted record.
-
#into(table) ⇒ Object
Specify a table for a SELECT ...
-
#nolock ⇒ Object
Allows you to do a dirty read of uncommitted data using WITH (NOLOCK).
-
#outer_apply(table) ⇒ Object
Uses OUTER APPLY to join the given table into the current dataset.
-
#output(into, values) ⇒ Object
Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
-
#quoted_identifier_append(sql, name) ⇒ Object
MSSQL uses [] to quote identifiers.
-
#select_sql ⇒ Object
On MSSQL 2012+ add a default order to the current dataset if an offset is used.
-
#server_version ⇒ Object
The version of the database server.
- #supports_cte?(type = :select) ⇒ Boolean
-
#supports_group_cube? ⇒ Boolean
MSSQL 2005+ supports GROUP BY CUBE.
-
#supports_group_rollup? ⇒ Boolean
MSSQL 2005+ supports GROUP BY ROLLUP.
-
#supports_insert_select? ⇒ Boolean
MSSQL supports insert_select via the OUTPUT clause.
-
#supports_intersect_except? ⇒ Boolean
MSSQL 2005+ supports INTERSECT and EXCEPT.
-
#supports_is_true? ⇒ Boolean
MSSQL does not support IS TRUE.
-
#supports_join_using? ⇒ Boolean
MSSQL doesn't support JOIN USING.
-
#supports_modifying_joins? ⇒ Boolean
MSSQL 2005+ supports modifying joined datasets.
-
#supports_multiple_column_in? ⇒ Boolean
MSSQL does not support multiple columns for the IN/NOT IN operators.
-
#supports_offsets_in_correlated_subqueries? ⇒ Boolean
MSSQL 2012+ supports offsets in correlated subqueries.
-
#supports_output_clause? ⇒ Boolean
MSSQL 2005+ supports the output clause.
-
#supports_where_true? ⇒ Boolean
MSSQL cannot use WHERE 1.
-
#supports_window_functions? ⇒ Boolean
MSSQL 2005+ supports window functions.
Instance Attribute Details
#mssql_unicode_strings ⇒ Object
Use the database's mssql_unicode_strings setting if the dataset hasn't overridden it.
559 560 561 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 559 def mssql_unicode_strings defined?(@mssql_unicode_strings) ? @mssql_unicode_strings : (@mssql_unicode_strings = db.mssql_unicode_strings) end |
Instance Method Details
#complex_expression_sql_append(sql, op, args) ⇒ Object
MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 564 def complex_expression_sql_append(sql, op, args) case op when :'||' super(sql, :+, args) when :LIKE, :"NOT LIKE" super(sql, op, args.map{|a| Sequel.lit(["(", " COLLATE #{CASE_SENSITIVE_COLLATION})"], a)}) when :ILIKE, :"NOT ILIKE" super(sql, (op == :ILIKE ? :LIKE : :"NOT LIKE"), args.map{|a| Sequel.lit(["(", " COLLATE #{CASE_INSENSITIVE_COLLATION})"], a)}) when :<<, :>> complex_expression_emulate_append(sql, op, args) when :extract part = args.at(0) raise(Sequel::Error, "unsupported extract argument: #{part.inspect}") unless format = EXTRACT_MAP[part] if part == :second expr = args.at(1) sql << DATEPART_SECOND_OPEN << format.to_s << COMMA literal_append(sql, expr) sql << DATEPART_SECOND_MIDDLE literal_append(sql, expr) sql << DATEPART_SECOND_CLOSE else sql << DATEPART_OPEN << format.to_s << COMMA literal_append(sql, args.at(1)) sql << PAREN_CLOSE end else super end end |
#constant_sql_append(sql, constant) ⇒ Object
MSSQL doesn't support the SQL standard CURRENT_DATE or CURRENT_TIME
595 596 597 598 599 600 601 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 595 def constant_sql_append(sql, constant) if c = CONSTANT_MAP[constant] sql << c else super end end |
#cross_apply(table) ⇒ Object
Uses CROSS APPLY to join the given table into the current dataset.
604 605 606 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 604 def cross_apply(table) join_table(:cross_apply, table) end |
#disable_insert_output ⇒ Object
Disable the use of INSERT OUTPUT
609 610 611 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 609 def disable_insert_output clone(:disable_insert_output=>true) end |
#escape_like(string) ⇒ Object
MSSQL treats [] as a metacharacter in LIKE expresions.
614 615 616 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 614 def escape_like(string) string.gsub(/[\\%_\[\]]/){|m| "\\#{m}"} end |
#full_text_search(cols, terms, opts = OPTS) ⇒ Object
MSSQL uses the CONTAINS keyword for full text search
619 620 621 622 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 619 def full_text_search(cols, terms, opts = OPTS) terms = "\"#{terms.join('" OR "')}\"" if terms.is_a?(Array) filter("CONTAINS (?, ?)", cols, terms) end |
#insert_select(*values) ⇒ Object
Use the OUTPUT clause to get the value of all columns for the newly inserted record.
625 626 627 628 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 625 def insert_select(*values) return unless supports_insert_select? naked.clone(default_server_opts(:sql=>output(nil, [SQL::ColumnAll.new(:inserted)]).insert_sql(*values))).single_record end |
#into(table) ⇒ Object
Specify a table for a SELECT ... INTO query.
631 632 633 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 631 def into(table) clone(:into => table) end |
#nolock ⇒ Object
Allows you to do a dirty read of uncommitted data using WITH (NOLOCK).
636 637 638 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 636 def nolock lock_style(:dirty) end |
#outer_apply(table) ⇒ Object
Uses OUTER APPLY to join the given table into the current dataset.
641 642 643 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 641 def outer_apply(table) join_table(:outer_apply, table) end |
#output(into, values) ⇒ Object
Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
The first argument is the table to output into, and the second argument is either an Array of column values to select, or a Hash which maps output column names to selected values, in the style of #insert or #update.
Output into a returned result set is not currently supported.
Examples:
dataset.output(:output_table, [:deleted__id, :deleted__name])
dataset.output(:output_table, :id => :inserted__id, :name => :inserted__name)
657 658 659 660 661 662 663 664 665 666 667 668 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 657 def output(into, values) raise(Error, "SQL Server versions 2000 and earlier do not support the OUTPUT clause") unless supports_output_clause? output = {} case values when Hash output[:column_list], output[:select_list] = values.keys, values.values when Array output[:select_list] = values end output[:into] = into clone({:output => output}) end |
#quoted_identifier_append(sql, name) ⇒ Object
MSSQL uses [] to quote identifiers.
671 672 673 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 671 def quoted_identifier_append(sql, name) sql << BRACKET_OPEN << name.to_s.gsub(/\]/, DOUBLE_BRACKET_CLOSE) << BRACKET_CLOSE end |
#select_sql ⇒ Object
On MSSQL 2012+ add a default order to the current dataset if an offset is used. The default offset emulation using a subquery would be used in the unordered case by default, and that also adds a default order, so it's better to just avoid the subquery.
679 680 681 682 683 684 685 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 679 def select_sql if @opts[:offset] && !@opts[:order] && is_2012_or_later? order(1).select_sql else super end end |
#server_version ⇒ Object
The version of the database server.
688 689 690 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 688 def server_version db.server_version(@opts[:server]) end |
#supports_cte?(type = :select) ⇒ Boolean
692 693 694 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 692 def supports_cte?(type=:select) is_2005_or_later? end |
#supports_group_cube? ⇒ Boolean
MSSQL 2005+ supports GROUP BY CUBE.
697 698 699 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 697 def supports_group_cube? is_2005_or_later? end |
#supports_group_rollup? ⇒ Boolean
MSSQL 2005+ supports GROUP BY ROLLUP
702 703 704 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 702 def supports_group_rollup? is_2005_or_later? end |
#supports_insert_select? ⇒ Boolean
MSSQL supports insert_select via the OUTPUT clause.
707 708 709 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 707 def supports_insert_select? supports_output_clause? && !opts[:disable_insert_output] end |
#supports_intersect_except? ⇒ Boolean
MSSQL 2005+ supports INTERSECT and EXCEPT
712 713 714 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 712 def supports_intersect_except? is_2005_or_later? end |
#supports_is_true? ⇒ Boolean
MSSQL does not support IS TRUE
717 718 719 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 717 def supports_is_true? false end |
#supports_join_using? ⇒ Boolean
MSSQL doesn't support JOIN USING
722 723 724 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 722 def supports_join_using? false end |
#supports_modifying_joins? ⇒ Boolean
MSSQL 2005+ supports modifying joined datasets
727 728 729 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 727 def is_2005_or_later? end |
#supports_multiple_column_in? ⇒ Boolean
MSSQL does not support multiple columns for the IN/NOT IN operators
732 733 734 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 732 def supports_multiple_column_in? false end |
#supports_offsets_in_correlated_subqueries? ⇒ Boolean
MSSQL 2012+ supports offsets in correlated subqueries.
737 738 739 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 737 def is_2012_or_later? end |
#supports_output_clause? ⇒ Boolean
MSSQL 2005+ supports the output clause.
742 743 744 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 742 def supports_output_clause? is_2005_or_later? end |
#supports_where_true? ⇒ Boolean
MSSQL cannot use WHERE 1.
752 753 754 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 752 def supports_where_true? false end |
#supports_window_functions? ⇒ Boolean
MSSQL 2005+ supports window functions
747 748 749 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 747 def supports_window_functions? true end |