Module: Sequel::MSSQL::DatasetMethods
- Includes:
- EmulateOffsetWithRowNumber
- Included in:
- ADO::MSSQL::Dataset, JDBC::JTDS::Dataset, ODBC::MSSQL::Dataset, TinyTDS::Dataset
- Defined in:
- lib/sequel/adapters/shared/mssql.rb
Constant Summary collapse
- CONSTANT_MAP =
.freeze # SEQUEL5
{:CURRENT_DATE=>'CAST(CURRENT_TIMESTAMP AS DATE)'.freeze, :CURRENT_TIME=>'CAST(CURRENT_TIMESTAMP AS TIME)'.freeze}
- EXTRACT_MAP =
.freeze # SEQUEL5
{:year=>"yy", :month=>"m", :day=>"d", :hour=>"hh", :minute=>"n", :second=>"s"}
- NON_SQL_OPTIONS =
EXTRACT_MAP.each_value(&:freeze) # SEQUEL5
(Dataset::NON_SQL_OPTIONS + [:disable_insert_output, :mssql_unicode_strings]).freeze
- LIMIT_ALL =
Object.new.freeze
- BOOL_TRUE =
'1'.freeze
- BOOL_FALSE =
'0'.freeze
- COMMA_SEPARATOR =
', '.freeze
- TABLE_HINT =
" WITH (".freeze
- READPAST =
"READPAST".freeze
- NOLOCK =
'NOLOCK'.freeze
- UPDLOCK =
'UPDLOCK'.freeze
- WILDCARD =
LiteralString.new('*').freeze
- BRACKET_CLOSE =
']'.freeze
- BRACKET_OPEN =
'['.freeze
- COMMA =
', '.freeze
- PAREN_CLOSE =
')'.freeze
- PAREN_SPACE_OPEN =
' ('.freeze
- SPACE =
' '.freeze
- FROM =
' FROM '.freeze
- APOS =
"'".freeze
- APOS_RE =
/'/.freeze
- DOUBLE_APOS =
"''".freeze
- INTO =
" INTO ".freeze
- DOUBLE_BRACKET_CLOSE =
']]'.freeze
- DATEPART_SECOND_OPEN =
"CAST((datepart(".freeze
- DATEPART_SECOND_MIDDLE =
') + datepart(ns, '.freeze
- DATEPART_SECOND_CLOSE =
")/1000000000.0) AS double precision)".freeze
- DATEPART_OPEN =
"datepart(".freeze
- OUTPUT_INSERTED =
" OUTPUT INSERTED.*".freeze
- HEX_START =
'0x'.freeze
- UNICODE_STRING_START =
"N'".freeze
- BACKSLASH_CRLF_RE =
/\\((?:\r\n)|\n)/.freeze
- BACKSLASH_CRLF_REPLACE =
'\\\\\\\\\\1\\1'.freeze
- TOP_PAREN =
" TOP (".freeze
- TOP =
" TOP ".freeze
- OUTPUT =
" OUTPUT ".freeze
- HSTAR =
"H*".freeze
- CASE_SENSITIVE_COLLATION =
'Latin1_General_CS_AS'.freeze
- CASE_INSENSITIVE_COLLATION =
'Latin1_General_CI_AS'.freeze
- DEFAULT_TIMESTAMP_FORMAT =
"'%Y-%m-%dT%H:%M:%S%N%z'".freeze
- FORMAT_DATE =
"'%Y%m%d'".freeze
- CROSS_APPLY =
'CROSS APPLY'.freeze
- OUTER_APPLY =
'OUTER APPLY'.freeze
- OFFSET =
" OFFSET ".freeze
- ROWS =
" ROWS".freeze
- ROWS_ONLY =
" ROWS ONLY".freeze
- FETCH_NEXT =
" FETCH NEXT ".freeze
Instance Method Summary collapse
-
#complex_expression_sql_append(sql, op, args) ⇒ Object
MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
-
#constant_sql_append(sql, constant) ⇒ Object
MSSQL doesn’t support the SQL standard CURRENT_DATE or CURRENT_TIME.
-
#cross_apply(table) ⇒ Object
Uses CROSS APPLY to join the given table into the current dataset.
-
#disable_insert_output ⇒ Object
Disable the use of INSERT OUTPUT.
-
#escape_like(string) ⇒ Object
MSSQL treats [] as a metacharacter in LIKE expresions.
-
#full_text_search(cols, terms, opts = OPTS) ⇒ Object
MSSQL uses the CONTAINS keyword for full text search.
-
#insert_select(*values) ⇒ Object
Use the OUTPUT clause to get the value of all columns for the newly inserted record.
-
#insert_select_sql(*values) ⇒ Object
Add OUTPUT clause unless there is already an existing output clause, then return the SQL to insert.
-
#into(table) ⇒ Object
Specify a table for a SELECT …
-
#mssql_unicode_strings ⇒ Object
Use the database’s mssql_unicode_strings setting if the dataset hasn’t overridden it.
-
#mssql_unicode_strings=(v) ⇒ Object
Allow overriding of the mssql_unicode_strings option at the dataset level.
-
#nolock ⇒ Object
Allows you to do a dirty read of uncommitted data using WITH (NOLOCK).
-
#outer_apply(table) ⇒ Object
Uses OUTER APPLY to join the given table into the current dataset.
-
#output(into, values) ⇒ Object
Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
-
#quoted_identifier_append(sql, name) ⇒ Object
MSSQL uses [] to quote identifiers.
-
#returning(*values) ⇒ Object
Emulate RETURNING using the output clause.
-
#select_sql ⇒ Object
On MSSQL 2012+ add a default order to the current dataset if an offset is used.
-
#server_version ⇒ Object
The version of the database server.
- #supports_cte?(type = :select) ⇒ Boolean
-
#supports_group_cube? ⇒ Boolean
MSSQL 2005+ supports GROUP BY CUBE.
-
#supports_group_rollup? ⇒ Boolean
MSSQL 2005+ supports GROUP BY ROLLUP.
-
#supports_grouping_sets? ⇒ Boolean
MSSQL 2008+ supports GROUPING SETS.
-
#supports_insert_select? ⇒ Boolean
MSSQL supports insert_select via the OUTPUT clause.
-
#supports_intersect_except? ⇒ Boolean
MSSQL 2005+ supports INTERSECT and EXCEPT.
-
#supports_is_true? ⇒ Boolean
MSSQL does not support IS TRUE.
-
#supports_join_using? ⇒ Boolean
MSSQL doesn’t support JOIN USING.
-
#supports_modifying_joins? ⇒ Boolean
MSSQL 2005+ supports modifying joined datasets.
-
#supports_multiple_column_in? ⇒ Boolean
MSSQL does not support multiple columns for the IN/NOT IN operators.
-
#supports_offsets_in_correlated_subqueries? ⇒ Boolean
MSSQL 2012+ supports offsets in correlated subqueries.
-
#supports_output_clause? ⇒ Boolean
MSSQL 2005+ supports the OUTPUT clause.
-
#supports_returning?(type) ⇒ Boolean
MSSQL 2005+ can emulate RETURNING via the OUTPUT clause.
-
#supports_skip_locked? ⇒ Boolean
MSSQL uses READPAST to skip locked rows.
-
#supports_where_true? ⇒ Boolean
MSSQL cannot use WHERE 1.
-
#supports_window_functions? ⇒ Boolean
MSSQL 2005+ supports window functions.
-
#with_mssql_unicode_strings(v) ⇒ Object
Return a cloned dataset with the mssql_unicode_strings option set.
Methods included from EmulateOffsetWithRowNumber
Instance Method Details
#complex_expression_sql_append(sql, op, args) ⇒ Object
MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 654 def complex_expression_sql_append(sql, op, args) case op when :'||' super(sql, :+, args) when :LIKE, :"NOT LIKE" super(sql, op, args.map{|a| Sequel.lit(["(", " COLLATE Latin1_General_CS_AS)"], a)}) when :ILIKE, :"NOT ILIKE" super(sql, (op == :ILIKE ? :LIKE : :"NOT LIKE"), args.map{|a| Sequel.lit(["(", " COLLATE Latin1_General_CI_AS)"], a)}) when :<<, :>> complex_expression_emulate_append(sql, op, args) when :extract part = args[0] raise(Sequel::Error, "unsupported extract argument: #{part.inspect}") unless format = EXTRACT_MAP[part] if part == :second expr = args[1] sql << "CAST((datepart(" << format.to_s << ', ' literal_append(sql, expr) sql << ') + datepart(ns, ' literal_append(sql, expr) sql << ")/1000000000.0) AS double precision)" else sql << "datepart(" << format.to_s << ', ' literal_append(sql, args[1]) sql << ')' end else super end end |
#constant_sql_append(sql, constant) ⇒ Object
MSSQL doesn’t support the SQL standard CURRENT_DATE or CURRENT_TIME
685 686 687 688 689 690 691 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 685 def constant_sql_append(sql, constant) if c = CONSTANT_MAP[constant] sql << c else super end end |
#cross_apply(table) ⇒ Object
Uses CROSS APPLY to join the given table into the current dataset.
694 695 696 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 694 def cross_apply(table) join_table(:cross_apply, table) end |
#disable_insert_output ⇒ Object
Disable the use of INSERT OUTPUT
699 700 701 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 699 def disable_insert_output clone(:disable_insert_output=>true) end |
#escape_like(string) ⇒ Object
MSSQL treats [] as a metacharacter in LIKE expresions.
704 705 706 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 704 def escape_like(string) string.gsub(/[\\%_\[\]]/){|m| "\\#{m}"} end |
#full_text_search(cols, terms, opts = OPTS) ⇒ Object
MSSQL uses the CONTAINS keyword for full text search
709 710 711 712 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 709 def full_text_search(cols, terms, opts = OPTS) terms = "\"#{terms.join('" OR "')}\"" if terms.is_a?(Array) where(Sequel.lit("CONTAINS (?, ?)", cols, terms)) end |
#insert_select(*values) ⇒ Object
Use the OUTPUT clause to get the value of all columns for the newly inserted record.
715 716 717 718 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 715 def insert_select(*values) return unless supports_insert_select? with_sql_first(insert_select_sql(*values)) end |
#insert_select_sql(*values) ⇒ Object
Add OUTPUT clause unless there is already an existing output clause, then return the SQL to insert.
722 723 724 725 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 722 def insert_select_sql(*values) ds = (opts[:output] || opts[:returning]) ? self : output(nil, [SQL::ColumnAll.new(:inserted)]) ds.insert_sql(*values) end |
#into(table) ⇒ Object
Specify a table for a SELECT … INTO query.
728 729 730 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 728 def into(table) clone(:into => table) end |
#mssql_unicode_strings ⇒ Object
Use the database’s mssql_unicode_strings setting if the dataset hasn’t overridden it.
644 645 646 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 644 def mssql_unicode_strings opts.has_key?(:mssql_unicode_strings) ? opts[:mssql_unicode_strings] : db.mssql_unicode_strings end |
#mssql_unicode_strings=(v) ⇒ Object
Allow overriding of the mssql_unicode_strings option at the dataset level.
639 640 641 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 639 def mssql_unicode_strings=(v) @opts[:mssql_unicode_strings] = v end |
#nolock ⇒ Object
Allows you to do a dirty read of uncommitted data using WITH (NOLOCK).
733 734 735 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 733 def nolock lock_style(:dirty) end |
#outer_apply(table) ⇒ Object
Uses OUTER APPLY to join the given table into the current dataset.
738 739 740 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 738 def outer_apply(table) join_table(:outer_apply, table) end |
#output(into, values) ⇒ Object
Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
The first argument is the table to output into, and the second argument is either an Array of column values to select, or a Hash which maps output column names to selected values, in the style of #insert or #update.
Output into a returned result set is not currently supported.
Examples:
dataset.output(:output_table, [:deleted__id, :deleted__name])
dataset.output(:output_table, :id => :inserted__id, :name => :inserted__name)
754 755 756 757 758 759 760 761 762 763 764 765 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 754 def output(into, values) raise(Error, "SQL Server versions 2000 and earlier do not support the OUTPUT clause") unless supports_output_clause? output = {} case values when Hash output[:column_list], output[:select_list] = values.keys, values.values when Array output[:select_list] = values end output[:into] = into clone(:output => output) end |
#quoted_identifier_append(sql, name) ⇒ Object
MSSQL uses [] to quote identifiers.
768 769 770 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 768 def quoted_identifier_append(sql, name) sql << '[' << name.to_s.gsub(/\]/, ']]') << ']' end |
#returning(*values) ⇒ Object
Emulate RETURNING using the output clause. This only handles values that are simple column references.
773 774 775 776 777 778 779 780 781 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 773 def returning(*values) values = values.map do |v| unless r = unqualified_column_for(v) raise(Error, "cannot emulate RETURNING via OUTPUT for value: #{v.inspect}") end r end clone(:returning=>values) end |
#select_sql ⇒ Object
On MSSQL 2012+ add a default order to the current dataset if an offset is used. The default offset emulation using a subquery would be used in the unordered case by default, and that also adds a default order, so it’s better to just avoid the subquery.
787 788 789 790 791 792 793 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 787 def select_sql if @opts[:offset] && !@opts[:order] && is_2012_or_later? order(1).select_sql else super end end |
#server_version ⇒ Object
The version of the database server.
796 797 798 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 796 def server_version db.server_version(@opts[:server]) end |
#supports_cte?(type = :select) ⇒ Boolean
800 801 802 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 800 def supports_cte?(type=:select) is_2005_or_later? end |
#supports_group_cube? ⇒ Boolean
MSSQL 2005+ supports GROUP BY CUBE.
805 806 807 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 805 def supports_group_cube? is_2005_or_later? end |
#supports_group_rollup? ⇒ Boolean
MSSQL 2005+ supports GROUP BY ROLLUP
810 811 812 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 810 def supports_group_rollup? is_2005_or_later? end |
#supports_grouping_sets? ⇒ Boolean
MSSQL 2008+ supports GROUPING SETS
815 816 817 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 815 def supports_grouping_sets? is_2008_or_later? end |
#supports_insert_select? ⇒ Boolean
MSSQL supports insert_select via the OUTPUT clause.
820 821 822 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 820 def supports_insert_select? supports_output_clause? && !opts[:disable_insert_output] end |
#supports_intersect_except? ⇒ Boolean
MSSQL 2005+ supports INTERSECT and EXCEPT
825 826 827 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 825 def supports_intersect_except? is_2005_or_later? end |
#supports_is_true? ⇒ Boolean
MSSQL does not support IS TRUE
830 831 832 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 830 def supports_is_true? false end |
#supports_join_using? ⇒ Boolean
MSSQL doesn’t support JOIN USING
835 836 837 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 835 def supports_join_using? false end |
#supports_modifying_joins? ⇒ Boolean
MSSQL 2005+ supports modifying joined datasets
840 841 842 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 840 def is_2005_or_later? end |
#supports_multiple_column_in? ⇒ Boolean
MSSQL does not support multiple columns for the IN/NOT IN operators
845 846 847 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 845 def supports_multiple_column_in? false end |
#supports_offsets_in_correlated_subqueries? ⇒ Boolean
MSSQL 2012+ supports offsets in correlated subqueries.
850 851 852 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 850 def is_2012_or_later? end |
#supports_output_clause? ⇒ Boolean
MSSQL 2005+ supports the OUTPUT clause.
855 856 857 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 855 def supports_output_clause? is_2005_or_later? end |
#supports_returning?(type) ⇒ Boolean
MSSQL 2005+ can emulate RETURNING via the OUTPUT clause.
860 861 862 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 860 def supports_returning?(type) supports_insert_select? end |
#supports_skip_locked? ⇒ Boolean
MSSQL uses READPAST to skip locked rows.
865 866 867 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 865 def supports_skip_locked? true end |
#supports_where_true? ⇒ Boolean
MSSQL cannot use WHERE 1.
875 876 877 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 875 def supports_where_true? false end |
#supports_window_functions? ⇒ Boolean
MSSQL 2005+ supports window functions
870 871 872 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 870 def supports_window_functions? true end |
#with_mssql_unicode_strings(v) ⇒ Object
Return a cloned dataset with the mssql_unicode_strings option set.
649 650 651 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 649 def with_mssql_unicode_strings(v) clone(:mssql_unicode_strings=>v) end |