Module: Sequel::MSSQL::DatasetMethods
- Includes:
- EmulateOffsetWithRowNumber
- Included in:
- ADO::MSSQL::Dataset, JDBC::JTDS::Dataset, ODBC::MSSQL::Dataset, TinyTDS::Dataset
- Defined in:
- lib/sequel/adapters/shared/mssql.rb
Constant Summary collapse
- BOOL_TRUE =
'1'.freeze
- BOOL_FALSE =
'0'.freeze
- COMMA_SEPARATOR =
', '.freeze
- DELETE_CLAUSE_METHODS =
Dataset.clause_methods(:delete, %w'with delete from output from2 where')
- INSERT_CLAUSE_METHODS =
Dataset.clause_methods(:insert, %w'with insert into columns output values')
- SELECT_CLAUSE_METHODS =
Dataset.clause_methods(:select, %w'with select distinct limit columns into from lock join where group having order compounds')
- UPDATE_CLAUSE_METHODS =
Dataset.clause_methods(:update, %w'with update limit table set output from where')
- UPDATE_CLAUSE_METHODS_2000 =
Dataset.clause_methods(:update, %w'update table set output from where')
- NOLOCK =
' WITH (NOLOCK)'.freeze
- UPDLOCK =
' WITH (UPDLOCK)'.freeze
- WILDCARD =
LiteralString.new('*').freeze
- CONSTANT_MAP =
{:CURRENT_DATE=>'CAST(CURRENT_TIMESTAMP AS DATE)'.freeze, :CURRENT_TIME=>'CAST(CURRENT_TIMESTAMP AS TIME)'.freeze}
- EXTRACT_MAP =
{:year=>"yy", :month=>"m", :day=>"d", :hour=>"hh", :minute=>"n", :second=>"s"}
- BRACKET_CLOSE =
Dataset::BRACKET_CLOSE
- BRACKET_OPEN =
Dataset::BRACKET_OPEN
- COMMA =
Dataset::COMMA
- PAREN_CLOSE =
Dataset::PAREN_CLOSE
- PAREN_SPACE_OPEN =
Dataset::PAREN_SPACE_OPEN
- SPACE =
Dataset::SPACE
- FROM =
Dataset::FROM
- APOS =
Dataset::APOS
- APOS_RE =
Dataset::APOS_RE
- DOUBLE_APOS =
Dataset::DOUBLE_APOS
- INTO =
Dataset::INTO
- DOUBLE_BRACKET_CLOSE =
']]'.freeze
- DATEPART_SECOND_OPEN =
"CAST((datepart(".freeze
- DATEPART_SECOND_MIDDLE =
') + datepart(ns, '.freeze
- DATEPART_SECOND_CLOSE =
")/1000000000.0) AS double precision)".freeze
- DATEPART_OPEN =
"datepart(".freeze
- UNION_ALL =
' UNION ALL '.freeze
- SELECT_SPACE =
'SELECT '.freeze
- TIMESTAMP_USEC_FORMAT =
".%03d".freeze
- OUTPUT_INSERTED =
" OUTPUT INSERTED.*".freeze
- HEX_START =
'0x'.freeze
- UNICODE_STRING_START =
"N'".freeze
- BACKSLASH_CRLF_RE =
/\\((?:\r\n)|\n)/.freeze
- BACKSLASH_CRLF_REPLACE =
'\\\\\\\\\\1\\1'.freeze
- TOP_PAREN =
" TOP (".freeze
- TOP =
" TOP ".freeze
- OUTPUT =
" OUTPUT ".freeze
- HSTAR =
"H*".freeze
- CASE_SENSITIVE_COLLATION =
'Latin1_General_CS_AS'.freeze
- CASE_INSENSITIVE_COLLATION =
'Latin1_General_CI_AS'.freeze
- DEFAULT_TIMESTAMP_FORMAT =
"'%Y-%m-%dT%H:%M:%S%N%z'".freeze
- FORMAT_DATE =
"'%Y%m%d'".freeze
- CROSS_APPLY =
'CROSS APPLY'.freeze
- OUTER_APPLY =
'OUTER APPLY'.freeze
Instance Attribute Summary collapse
-
#mssql_unicode_strings ⇒ Object
Allow overriding of the mssql_unicode_strings option at the dataset level.
Instance Method Summary collapse
-
#complex_expression_sql_append(sql, op, args) ⇒ Object
MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
-
#constant_sql_append(sql, constant) ⇒ Object
MSSQL doesn’t support the SQL standard CURRENT_DATE or CURRENT_TIME.
-
#cross_apply(table) ⇒ Object
Uses CROSS APPLY to join the given table into the current dataset.
-
#disable_insert_output ⇒ Object
Disable the use of INSERT OUTPUT.
-
#emulated_function_sql_append(sql, f) ⇒ Object
There is no function on Microsoft SQL Server that does character length and respects trailing spaces (datalength respects trailing spaces, but counts bytes instead of characters).
-
#escape_like(string) ⇒ Object
MSSQL treats [] as a metacharacter in LIKE expresions.
-
#full_text_search(cols, terms, opts = {}) ⇒ Object
MSSQL uses the CONTAINS keyword for full text search.
-
#initialize(db, opts = {}) ⇒ Object
Copy the mssql_unicode_strings option from the
dbobject. -
#insert_select(*values) ⇒ Object
Use the OUTPUT clause to get the value of all columns for the newly inserted record.
-
#into(table) ⇒ Object
Specify a table for a SELECT …
-
#multi_insert_sql(columns, values) ⇒ Object
MSSQL uses a UNION ALL statement to insert multiple values at once.
-
#nolock ⇒ Object
Allows you to do a dirty read of uncommitted data using WITH (NOLOCK).
-
#outer_apply(table) ⇒ Object
Uses OUTER APPLY to join the given table into the current dataset.
-
#output(into, values) ⇒ Object
Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
-
#quoted_identifier_append(sql, name) ⇒ Object
MSSQL uses [] to quote identifiers.
-
#server_version ⇒ Object
The version of the database server.
-
#supports_group_cube? ⇒ Boolean
MSSQL 2005+ supports GROUP BY CUBE.
-
#supports_group_rollup? ⇒ Boolean
MSSQL 2005+ supports GROUP BY ROLLUP.
-
#supports_insert_select? ⇒ Boolean
MSSQL supports insert_select via the OUTPUT clause.
-
#supports_intersect_except? ⇒ Boolean
MSSQL 2005+ supports INTERSECT and EXCEPT.
-
#supports_is_true? ⇒ Boolean
MSSQL does not support IS TRUE.
-
#supports_join_using? ⇒ Boolean
MSSQL doesn’t support JOIN USING.
-
#supports_modifying_joins? ⇒ Boolean
MSSQL 2005+ supports modifying joined datasets.
-
#supports_multiple_column_in? ⇒ Boolean
MSSQL does not support multiple columns for the IN/NOT IN operators.
-
#supports_output_clause? ⇒ Boolean
MSSQL 2005+ supports the output clause.
-
#supports_where_true? ⇒ Boolean
MSSQL cannot use WHERE 1.
-
#supports_window_functions? ⇒ Boolean
MSSQL 2005+ supports window functions.
Methods included from EmulateOffsetWithRowNumber
Instance Attribute Details
#mssql_unicode_strings ⇒ Object
Allow overriding of the mssql_unicode_strings option at the dataset level.
444 445 446 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 444 def mssql_unicode_strings @mssql_unicode_strings end |
Instance Method Details
#complex_expression_sql_append(sql, op, args) ⇒ Object
MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 453 def complex_expression_sql_append(sql, op, args) case op when :'||' super(sql, :+, args) when :LIKE, :"NOT LIKE" super(sql, op, args.map{|a| LiteralString.new("(#{literal(a)} COLLATE #{CASE_SENSITIVE_COLLATION})")}) when :ILIKE, :"NOT ILIKE" super(sql, (op == :ILIKE ? :LIKE : :"NOT LIKE"), args.map{|a| LiteralString.new("(#{literal(a)} COLLATE #{CASE_INSENSITIVE_COLLATION})")}) when :<< sql << complex_expression_arg_pairs(args){|a, b| "(#{literal(a)} * POWER(2, #{literal(b)}))"} when :>> sql << complex_expression_arg_pairs(args){|a, b| "(#{literal(a)} / POWER(2, #{literal(b)}))"} when :extract part = args.at(0) raise(Sequel::Error, "unsupported extract argument: #{part.inspect}") unless format = EXTRACT_MAP[part] if part == :second expr = literal(args.at(1)) sql << DATEPART_SECOND_OPEN << format.to_s << COMMA << expr << DATEPART_SECOND_MIDDLE << expr << DATEPART_SECOND_CLOSE else sql << DATEPART_OPEN << format.to_s << COMMA literal_append(sql, args.at(1)) sql << PAREN_CLOSE end else super end end |
#constant_sql_append(sql, constant) ⇒ Object
MSSQL doesn’t support the SQL standard CURRENT_DATE or CURRENT_TIME
482 483 484 485 486 487 488 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 482 def constant_sql_append(sql, constant) if c = CONSTANT_MAP[constant] sql << c else super end end |
#cross_apply(table) ⇒ Object
Uses CROSS APPLY to join the given table into the current dataset.
491 492 493 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 491 def cross_apply(table) join_table(:cross_apply, table) end |
#disable_insert_output ⇒ Object
Disable the use of INSERT OUTPUT
496 497 498 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 496 def disable_insert_output clone(:disable_insert_output=>true) end |
#emulated_function_sql_append(sql, f) ⇒ Object
There is no function on Microsoft SQL Server that does character length and respects trailing spaces (datalength respects trailing spaces, but counts bytes instead of characters). Use a hack to work around the trailing spaces issue.
509 510 511 512 513 514 515 516 517 518 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 509 def emulated_function_sql_append(sql, f) case f.f when :char_length literal_append(sql, SQL::Function.new(:len, Sequel.join([f.args.first, 'x'])) - 1) when :trim literal_append(sql, SQL::Function.new(:ltrim, SQL::Function.new(:rtrim, f.args.first))) else super end end |
#escape_like(string) ⇒ Object
MSSQL treats [] as a metacharacter in LIKE expresions.
501 502 503 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 501 def escape_like(string) string.gsub(/[\\%_\[\]]/){|m| "\\#{m}"} end |
#full_text_search(cols, terms, opts = {}) ⇒ Object
MSSQL uses the CONTAINS keyword for full text search
521 522 523 524 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 521 def full_text_search(cols, terms, opts = {}) terms = "\"#{terms.join('" OR "')}\"" if terms.is_a?(Array) filter("CONTAINS (?, ?)", cols, terms) end |
#initialize(db, opts = {}) ⇒ Object
Copy the mssql_unicode_strings option from the db object.
447 448 449 450 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 447 def initialize(db, opts={}) super @mssql_unicode_strings = db.mssql_unicode_strings end |
#insert_select(*values) ⇒ Object
Use the OUTPUT clause to get the value of all columns for the newly inserted record.
527 528 529 530 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 527 def insert_select(*values) return unless supports_insert_select? naked.clone(default_server_opts(:sql=>output(nil, [SQL::ColumnAll.new(:inserted)]).insert_sql(*values))).single_record end |
#into(table) ⇒ Object
Specify a table for a SELECT … INTO query.
533 534 535 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 533 def into(table) clone(:into => table) end |
#multi_insert_sql(columns, values) ⇒ Object
MSSQL uses a UNION ALL statement to insert multiple values at once.
538 539 540 541 542 543 544 545 546 547 548 549 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 538 def multi_insert_sql(columns, values) c = false sql = LiteralString.new('') u = UNION_ALL values.each do |v| sql << u if c sql << SELECT_SPACE expression_list_append(sql, v) c ||= true end [insert_sql(columns, sql)] end |
#nolock ⇒ Object
Allows you to do a dirty read of uncommitted data using WITH (NOLOCK).
552 553 554 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 552 def nolock lock_style(:dirty) end |
#outer_apply(table) ⇒ Object
Uses OUTER APPLY to join the given table into the current dataset.
557 558 559 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 557 def outer_apply(table) join_table(:outer_apply, table) end |
#output(into, values) ⇒ Object
Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
The first argument is the table to output into, and the second argument is either an Array of column values to select, or a Hash which maps output column names to selected values, in the style of #insert or #update.
Output into a returned result set is not currently supported.
Examples:
dataset.output(:output_table, [:deleted__id, :deleted__name])
dataset.output(:output_table, :id => :inserted__id, :name => :inserted__name)
573 574 575 576 577 578 579 580 581 582 583 584 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 573 def output(into, values) raise(Error, "SQL Server versions 2000 and earlier do not support the OUTPUT clause") unless supports_output_clause? output = {} case values when Hash output[:column_list], output[:select_list] = values.keys, values.values when Array output[:select_list] = values end output[:into] = into clone({:output => output}) end |
#quoted_identifier_append(sql, name) ⇒ Object
MSSQL uses [] to quote identifiers.
587 588 589 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 587 def quoted_identifier_append(sql, name) sql << BRACKET_OPEN << name.to_s.gsub(/\]/, DOUBLE_BRACKET_CLOSE) << BRACKET_CLOSE end |
#server_version ⇒ Object
The version of the database server.
592 593 594 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 592 def server_version db.server_version(@opts[:server]) end |
#supports_group_cube? ⇒ Boolean
MSSQL 2005+ supports GROUP BY CUBE.
597 598 599 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 597 def supports_group_cube? is_2005_or_later? end |
#supports_group_rollup? ⇒ Boolean
MSSQL 2005+ supports GROUP BY ROLLUP
602 603 604 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 602 def supports_group_rollup? is_2005_or_later? end |
#supports_insert_select? ⇒ Boolean
MSSQL supports insert_select via the OUTPUT clause.
607 608 609 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 607 def supports_insert_select? supports_output_clause? && !opts[:disable_insert_output] end |
#supports_intersect_except? ⇒ Boolean
MSSQL 2005+ supports INTERSECT and EXCEPT
612 613 614 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 612 def supports_intersect_except? is_2005_or_later? end |
#supports_is_true? ⇒ Boolean
MSSQL does not support IS TRUE
617 618 619 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 617 def supports_is_true? false end |
#supports_join_using? ⇒ Boolean
MSSQL doesn’t support JOIN USING
622 623 624 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 622 def supports_join_using? false end |
#supports_modifying_joins? ⇒ Boolean
MSSQL 2005+ supports modifying joined datasets
627 628 629 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 627 def is_2005_or_later? end |
#supports_multiple_column_in? ⇒ Boolean
MSSQL does not support multiple columns for the IN/NOT IN operators
632 633 634 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 632 def supports_multiple_column_in? false end |
#supports_output_clause? ⇒ Boolean
MSSQL 2005+ supports the output clause.
637 638 639 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 637 def supports_output_clause? is_2005_or_later? end |
#supports_where_true? ⇒ Boolean
MSSQL cannot use WHERE 1.
647 648 649 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 647 def supports_where_true? false end |
#supports_window_functions? ⇒ Boolean
MSSQL 2005+ supports window functions
642 643 644 |
# File 'lib/sequel/adapters/shared/mssql.rb', line 642 def supports_window_functions? true end |