Class: Sequel::Model::Associations::AssociationReflection

Inherits:
Hash
  • Object
show all
Includes:
Inflections
Defined in:
lib/sequel/model/associations.rb

Overview

AssociationReflection is a Hash subclass that keeps information on Sequel::Model associations. It provides methods to reduce internal code duplication. It should not be instantiated by the user.

Constant Summary collapse

ASSOCIATION_DATASET_PROC =
proc{|r| r.association_dataset_for(self)}
FINALIZE_SETTINGS =

Map of methods to cache keys used for finalizing associations.

{
  :associated_class=>:class,
  :associated_dataset=>:_dataset,
  :associated_eager_dataset=>:associated_eager_dataset,
  :eager_limit_strategy=>:_eager_limit_strategy,
  :filter_by_associations_conditions_dataset=>:filter_by_associations_conditions_dataset,
  :placeholder_loader=>:placeholder_loader,
  :predicate_key=>:predicate_key,
  :predicate_keys=>:predicate_keys,
  :reciprocal=>:reciprocal,
}.freeze

Constants included from Inflections

Inflections::CAMELIZE_CONVERT_REGEXP, Inflections::CAMELIZE_MODULE_REGEXP, Inflections::DASH, Inflections::DEMODULIZE_CONVERT_REGEXP, Inflections::EMPTY_STRING, Inflections::SLASH, Inflections::UNDERSCORE, Inflections::UNDERSCORE_CONVERT_REGEXP1, Inflections::UNDERSCORE_CONVERT_REGEXP2, Inflections::UNDERSCORE_CONVERT_REPLACE, Inflections::UNDERSCORE_MODULE_REGEXP, Inflections::VALID_CONSTANT_NAME_REGEXP

Instance Method Summary collapse

Methods included from Inflections

clear, irregular, plural, singular, uncountable

Methods inherited from Hash

#&, #case, #hstore, #pg_json, #pg_jsonb, #sql_expr, #sql_negate, #sql_or, #|, #~

Instance Method Details

#_add_methodObject

Name symbol for the _add internal association method



34
35
36
# File 'lib/sequel/model/associations.rb', line 34

def _add_method
  :"_add_#{singularize(self[:name])}"
end

#_remove_all_methodObject

Name symbol for the _remove_all internal association method



39
40
41
# File 'lib/sequel/model/associations.rb', line 39

def _remove_all_method
  :"_remove_all_#{self[:name]}"
end

#_remove_methodObject

Name symbol for the _remove internal association method



44
45
46
# File 'lib/sequel/model/associations.rb', line 44

def _remove_method
  :"_remove_#{singularize(self[:name])}"
end

#_setter_methodObject

Name symbol for the _setter association method



49
50
51
# File 'lib/sequel/model/associations.rb', line 49

def _setter_method
  :"_#{self[:name]}="
end

#add_methodObject

Name symbol for the add association method



54
55
56
# File 'lib/sequel/model/associations.rb', line 54

def add_method
  :"add_#{singularize(self[:name])}"
end

#apply_dataset_changes(ds) ⇒ Object

Apply all non-instance specific changes to the given dataset and return it.



82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
# File 'lib/sequel/model/associations.rb', line 82

def apply_dataset_changes(ds)
  ds = ds.with_extend(AssociationDatasetMethods).
    clone(:association_reflection => self)
  self[:extend].each{|m| ds = ds.with_extend(m)}
  ds = ds.select(*select) if select
  if c = self[:conditions]
    ds = (c.is_a?(Array) && !Sequel.condition_specifier?(c)) ? ds.where(*c) : ds.where(c)
  end
  ds = ds.order(*self[:order]) if self[:order]
  ds = ds.limit(*self[:limit]) if self[:limit]
  ds = ds.limit(1).skip_limit_check if limit_to_single_row?
  ds = ds.eager(self[:eager]) if self[:eager]
  ds = ds.distinct if self[:distinct]
  ds
end

#apply_distinct_on_eager_limit_strategy(ds) ⇒ Object

Use DISTINCT ON and ORDER BY clauses to limit the results to the first record with matching keys.



135
136
137
138
# File 'lib/sequel/model/associations.rb', line 135

def apply_distinct_on_eager_limit_strategy(ds)
  keys = predicate_key
  ds.distinct(*keys).order_prepend(*keys)
end

#apply_eager_dataset_changes(ds) ⇒ Object

Apply all non-instance specific changes and the eager_block option to the given dataset and return it.



100
101
102
103
104
105
106
# File 'lib/sequel/model/associations.rb', line 100

def apply_eager_dataset_changes(ds)
  ds = apply_dataset_changes(ds)
  if block = self[:eager_block]
    ds = block.call(ds)
  end
  ds
end

#apply_eager_graph_limit_strategy(strategy, ds) ⇒ Object

Apply the eager graph limit strategy to the dataset to graph into the current dataset, or return the dataset unmodified if no SQL limit strategy is needed.



110
111
112
113
114
115
116
117
118
119
# File 'lib/sequel/model/associations.rb', line 110

def apply_eager_graph_limit_strategy(strategy, ds)
  case strategy
  when :distinct_on
    apply_distinct_on_eager_limit_strategy(ds.order_prepend(*self[:order]))
  when :window_function
    apply_window_function_eager_limit_strategy(ds.order_prepend(*self[:order])).select(*ds.columns)
  else
    ds
  end
end

#apply_eager_limit_strategy(ds, strategy = eager_limit_strategy, limit_and_offset = limit_and_offset()) ⇒ Object

Apply an eager limit strategy to the dataset, or return the dataset unmodified if it doesn’t need an eager limit strategy.



123
124
125
126
127
128
129
130
131
132
# File 'lib/sequel/model/associations.rb', line 123

def apply_eager_limit_strategy(ds, strategy=eager_limit_strategy, limit_and_offset=limit_and_offset())
  case strategy
  when :distinct_on
    apply_distinct_on_eager_limit_strategy(ds)
  when :window_function
    apply_window_function_eager_limit_strategy(ds, limit_and_offset)
  else
    ds
  end
end

#apply_ruby_eager_limit_strategy(rows, limit_and_offset = limit_and_offset()) ⇒ Object

If the ruby eager limit strategy is being used, slice the array using the slice range to return the object(s) at the correct offset/limit.



161
162
163
164
165
166
167
168
169
170
# File 'lib/sequel/model/associations.rb', line 161

def apply_ruby_eager_limit_strategy(rows, limit_and_offset = limit_and_offset())
  name = self[:name]
  if returns_array?
    range = slice_range(limit_and_offset)
    rows.each{|o| o.associations[name] = o.associations[name][range] || []}
  elsif sr = slice_range(limit_and_offset)
    offset = sr.begin
    rows.each{|o| o.associations[name] = o.associations[name][offset]}
  end
end

#apply_window_function_eager_limit_strategy(ds, limit_and_offset = limit_and_offset()) ⇒ Object

Use a window function to limit the results of the eager loading dataset.



141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
# File 'lib/sequel/model/associations.rb', line 141

def apply_window_function_eager_limit_strategy(ds, limit_and_offset=limit_and_offset())
  rn = ds.row_number_column 
  limit, offset = limit_and_offset
  ds = ds.unordered.select_append{|o| o.row_number.function.over(:partition=>predicate_key, :order=>ds.opts[:order]).as(rn)}.from_self
  ds = if !returns_array?
    ds.where(rn => offset ? offset+1 : 1)
  elsif offset
    offset += 1
    if limit
      ds.where(rn => (offset...(offset+limit))) 
    else
      ds.where{SQL::Identifier.new(rn) >= offset} 
    end
  else
    ds.where{SQL::Identifier.new(rn) <= limit} 
  end
end

#assign_singular?Boolean

Whether the associations cache should use an array when storing the associated records during eager loading.

Returns:

  • (Boolean)


174
175
176
# File 'lib/sequel/model/associations.rb', line 174

def assign_singular?
  !returns_array?
end

#associated_classObject

The class associated to the current model class via this association



64
65
66
67
68
69
70
71
72
# File 'lib/sequel/model/associations.rb', line 64

def associated_class
  cached_fetch(:class) do
    begin
      constantize(self[:class_name])
    rescue NameError => e
      raise NameError, "#{e.message} (this happened when attempting to find the associated class for #{inspect})", e.backtrace
    end
  end
end

#associated_datasetObject

The dataset associated via this association, with the non-instance specific changes already applied. This will be a joined dataset if the association requires joining tables.



77
78
79
# File 'lib/sequel/model/associations.rb', line 77

def associated_dataset
  cached_fetch(:_dataset){apply_dataset_changes(_associated_dataset)}
end

#association_dataset_for(object) ⇒ Object

Return an dataset that will load the appropriate associated objects for the given object using this association.



211
212
213
214
215
216
217
218
219
# File 'lib/sequel/model/associations.rb', line 211

def association_dataset_for(object)
  condition = if can_have_associated_objects?(object)
    predicate_keys.zip(predicate_key_values(object))
  else
    false
  end

  associated_dataset.where(condition)
end

#association_dataset_procObject

Proc used to create the association dataset method.



223
224
225
# File 'lib/sequel/model/associations.rb', line 223

def association_dataset_proc
  ASSOCIATION_DATASET_PROC
end

#association_methodObject

Name symbol for association method, the same as the name of the association.



59
60
61
# File 'lib/sequel/model/associations.rb', line 59

def association_method
  self[:name]
end

#can_have_associated_objects?(obj) ⇒ Boolean

Whether this association can have associated objects, given the current object. Should be false if obj cannot have associated objects because the necessary key columns are NULL.

Returns:

  • (Boolean)


181
182
183
# File 'lib/sequel/model/associations.rb', line 181

def can_have_associated_objects?(obj)
  true
end

#cloneable?(ref) ⇒ Boolean

Whether you are able to clone from the given association type to the current association type, true by default only if the types match.

Returns:

  • (Boolean)


187
188
189
# File 'lib/sequel/model/associations.rb', line 187

def cloneable?(ref)
  ref[:type] == self[:type]
end

#dataset_methodObject

Name symbol for the dataset association method



192
193
194
# File 'lib/sequel/model/associations.rb', line 192

def dataset_method
  :"#{self[:name]}_dataset"
end

#dataset_need_primary_key?Boolean

Whether the dataset needs a primary key to function, true by default.

Returns:

  • (Boolean)


197
198
199
# File 'lib/sequel/model/associations.rb', line 197

def dataset_need_primary_key?
  true
end

#delete_row_number_column(ds = associated_dataset) ⇒ Object

Return the symbol used for the row number column if the window function eager limit strategy is being used, or nil otherwise.



203
204
205
206
207
# File 'lib/sequel/model/associations.rb', line 203

def delete_row_number_column(ds=associated_dataset)
  if eager_limit_strategy == :window_function
    ds.row_number_column 
  end
end

#eager_graph_lazy_dataset?Boolean

Whether to eagerly graph a lazy dataset, true by default. If this is false, the association won’t respect the :eager_graph option when loading the association for a single record.

Returns:

  • (Boolean)


335
336
337
# File 'lib/sequel/model/associations.rb', line 335

def eager_graph_lazy_dataset?
  true
end

#eager_graph_limit_strategy(strategy) ⇒ Object

The eager_graph limit strategy to use for this dataset



228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
# File 'lib/sequel/model/associations.rb', line 228

def eager_graph_limit_strategy(strategy)
  if self[:limit] || !returns_array?
    strategy = strategy[self[:name]] if strategy.is_a?(Hash)
    case strategy
    when true
      true_eager_graph_limit_strategy
    when Symbol
      strategy
    else
      if returns_array? || offset
        :ruby
      end
    end
  end
end

#eager_limit_strategyObject

The eager limit strategy to use for this dataset.



245
246
247
248
249
250
251
252
253
254
255
256
# File 'lib/sequel/model/associations.rb', line 245

def eager_limit_strategy
  cached_fetch(:_eager_limit_strategy) do
    if self[:limit] || !returns_array?
      case s = cached_fetch(:eager_limit_strategy){default_eager_limit_strategy}
      when true
        true_eager_limit_strategy
      else
        s
      end
    end
  end
end

#eager_load_results(eo, &block) ⇒ Object

Eager load the associated objects using the hash of eager options, yielding each row to the block.



260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
# File 'lib/sequel/model/associations.rb', line 260

def eager_load_results(eo, &block)
  rows = eo[:rows]
  initialize_association_cache(rows) unless eo[:initialize_rows] == false
  if eo[:id_map]
    ids = eo[:id_map].keys
    return ids if ids.empty?
  end
  strategy = eager_limit_strategy
  cascade = eo[:associations]
  eager_limit = nil

  if eo[:eager_block] || eo[:loader] == false
    ds = eager_loading_dataset(eo)

    strategy = ds.opts[:eager_limit_strategy] || strategy

    eager_limit =
      if el = ds.opts[:eager_limit]
        raise Error, "The :eager_limit dataset option is not supported for associations returning a single record" unless returns_array?
        strategy ||= true_eager_graph_limit_strategy
        if el.is_a?(Array)
          el
        else
          [el, nil]
        end
      else
        limit_and_offset
      end

    strategy = true_eager_graph_limit_strategy if strategy == :union
    # Correlated subqueries are not supported for regular eager loading
    strategy = :ruby if strategy == :correlated_subquery
    strategy = nil if strategy == :ruby && assign_singular?
    objects = apply_eager_limit_strategy(ds, strategy, eager_limit).all
  elsif strategy == :union
    objects = []
    ds = associated_dataset
    loader = union_eager_loader
    joiner = " UNION ALL "
    ids.each_slice(subqueries_per_union).each do |slice|
      objects.concat(ds.with_sql(slice.map{|k| loader.sql(*k)}.join(joiner)).to_a)
    end
    ds = ds.eager(cascade) if cascade
    ds.send(:post_load, objects)
  else
    loader = placeholder_eager_loader
    loader = loader.with_dataset{|dataset| dataset.eager(cascade)} if cascade
    objects = loader.all(ids)
  end

  objects.each(&block)
  if strategy == :ruby
    apply_ruby_eager_limit_strategy(rows, eager_limit || limit_and_offset)
  end
end

#eager_loader_keyObject

The key to use for the key hash when eager loading



317
318
319
# File 'lib/sequel/model/associations.rb', line 317

def eager_loader_key
  self[:eager_loader_key]
end

#eager_loading_predicate_keyObject

Alias of predicate_key, only for backwards compatibility.



328
329
330
# File 'lib/sequel/model/associations.rb', line 328

def eager_loading_predicate_key
  predicate_key
end

#eager_loading_use_associated_key?Boolean

By default associations do not need to select a key in an associated table to eagerly load.

Returns:

  • (Boolean)


323
324
325
# File 'lib/sequel/model/associations.rb', line 323

def eager_loading_use_associated_key?
  false
end

#filter_by_associations_add_conditions?Boolean

Whether additional conditions should be added when using the filter by associations support.

Returns:

  • (Boolean)


341
342
343
# File 'lib/sequel/model/associations.rb', line 341

def filter_by_associations_add_conditions?
  self[:conditions] || self[:eager_block] || self[:limit]
end

#filter_by_associations_conditions_expression(obj) ⇒ Object

The expression to use for the additional conditions to be added for the filter by association support, when the association itself is filtered. Works by using a subquery to test that the objects passed also meet the association filter criteria.



349
350
351
352
# File 'lib/sequel/model/associations.rb', line 349

def filter_by_associations_conditions_expression(obj)
  ds = filter_by_associations_conditions_dataset.where(filter_by_associations_conditions_subquery_conditions(obj))
  {filter_by_associations_conditions_key=>ds}
end

#finalizeObject

Finalize the association by first attempting to populate the thread-safe cache, and then transfering the thread-safe cache value to the association itself, so that a mutex is not needed to get the value.



357
358
359
360
361
362
363
364
365
366
367
368
# File 'lib/sequel/model/associations.rb', line 357

def finalize
  return unless cache = self[:cache]

  finalize_settings.each do |meth, key|
    next if has_key?(key)

    send(meth)
    self[key] = cache.delete(key) if cache.has_key?(key)
  end

  nil
end

#finalize_settingsObject



382
383
384
# File 'lib/sequel/model/associations.rb', line 382

def finalize_settings
  FINALIZE_SETTINGS
end

#handle_silent_modification_failure?Boolean

Whether to handle silent modification failure when adding/removing associated records, false by default.

Returns:

  • (Boolean)


388
389
390
# File 'lib/sequel/model/associations.rb', line 388

def handle_silent_modification_failure?
  false
end

#initialize_association_cache(objects) ⇒ Object

Initialize the associations cache for the current association for the given objects.



393
394
395
396
397
398
399
400
# File 'lib/sequel/model/associations.rb', line 393

def initialize_association_cache(objects)
  name = self[:name]
  if assign_singular?
    objects.each{|object| object.associations[name] = nil}
  else
    objects.each{|object| object.associations[name] = []}
  end
end

#inspectObject

Show which type of reflection this is, and a guess at what line was used to create the association.



404
405
406
407
408
409
410
411
412
# File 'lib/sequel/model/associations.rb', line 404

def inspect
  o = self[:orig_opts].dup
  o.delete(:class)
  o.delete(:class_name)
  o.delete(:block) unless o[:block]
  o[:class] = self[:orig_class] if self[:orig_class]

  "#<#{self.class} #{self[:model]}.#{self[:type]} #{self[:name].inspect}#{", #{o.inspect[1...-1]}" unless o.empty?}>"
end

#limit_and_offsetObject

The limit and offset for this association (returned as a two element array).



415
416
417
418
419
420
421
# File 'lib/sequel/model/associations.rb', line 415

def limit_and_offset
  if (v = self[:limit]).is_a?(Array)
    v
  else
    [v, nil]
  end
end

#need_associated_primary_key?Boolean

Whether the associated object needs a primary key to be added/removed, false by default.

Returns:

  • (Boolean)


425
426
427
# File 'lib/sequel/model/associations.rb', line 425

def need_associated_primary_key?
  false
end

#placeholder_loaderObject

A placeholder literalizer that can be used to lazily load the association. If one can’t be used, returns nil.



431
432
433
434
435
436
437
438
439
# File 'lib/sequel/model/associations.rb', line 431

def placeholder_loader
  if use_placeholder_loader?
    cached_fetch(:placeholder_loader) do
      Sequel::Dataset::PlaceholderLiteralizer.loader(associated_dataset) do |pl, ds|
        ds.where(Sequel.&(*predicate_keys.map{|k| SQL::BooleanExpression.new(:'=', k, pl.arg)}))
      end
    end
  end
end

#predicate_key_values(object) ⇒ Object

The values that predicate_keys should match for objects to be associated.



447
448
449
# File 'lib/sequel/model/associations.rb', line 447

def predicate_key_values(object)
  predicate_key_methods.map{|k| object.get_column_value(k)}
end

#predicate_keysObject

The keys to use for loading of the regular dataset, as an array.



442
443
444
# File 'lib/sequel/model/associations.rb', line 442

def predicate_keys
  cached_fetch(:predicate_keys){Array(predicate_key)}
end

#qualify(table, col) ⇒ Object

Qualify col with the given table name. If col is an array of columns, return an array of qualified columns. Only qualifies Symbols and SQL::Identifier values, other values are not modified.



454
455
456
457
458
459
460
461
462
463
# File 'lib/sequel/model/associations.rb', line 454

def qualify(table, col)
  transform(col) do |k|
    case k
    when Symbol, SQL::Identifier
      SQL::QualifiedIdentifier.new(table, k)
    else
      Sequel::Qualifier.new(table).transform(k)
    end
  end
end

#qualify_assoc(col) ⇒ Object

Qualify col with the associated model’s table name.



466
467
468
# File 'lib/sequel/model/associations.rb', line 466

def qualify_assoc(col)
  qualify(associated_class.table_name, col)
end

#qualify_cur(col) ⇒ Object

Qualify col with the current model’s table name.



471
472
473
# File 'lib/sequel/model/associations.rb', line 471

def qualify_cur(col)
  qualify(self[:model].table_name, col)
end

#reciprocalObject

Returns the reciprocal association variable, if one exists. The reciprocal association is the association in the associated class that is the opposite of the current association. For example, Album.many_to_one :artist and Artist.one_to_many :albums are reciprocal associations. This information is to populate reciprocal associations. For example, when you do this_artist.add_album(album) it sets album.artist to this_artist.



481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
# File 'lib/sequel/model/associations.rb', line 481

def reciprocal
  cached_fetch(:reciprocal) do
    possible_recips = []

    associated_class.all_association_reflections.each do |assoc_reflect|
      if reciprocal_association?(assoc_reflect)
        possible_recips << assoc_reflect
      end
    end

    if possible_recips.length == 1
      cached_set(:reciprocal_type, possible_recips.first[:type]) if ambiguous_reciprocal_type?
      possible_recips.first[:name]
    end
  end
end

#reciprocal_array?Boolean

Whether the reciprocal of this association returns an array of objects instead of a single object, true by default.

Returns:

  • (Boolean)


500
501
502
# File 'lib/sequel/model/associations.rb', line 500

def reciprocal_array?
  true
end

#remove_all_methodObject

Name symbol for the remove_all_ association method



505
506
507
# File 'lib/sequel/model/associations.rb', line 505

def remove_all_method
  :"remove_all_#{self[:name]}"
end

#remove_before_destroy?Boolean

Whether associated objects need to be removed from the association before being destroyed in order to preserve referential integrity.

Returns:

  • (Boolean)


511
512
513
# File 'lib/sequel/model/associations.rb', line 511

def remove_before_destroy?
  true
end

#remove_methodObject

Name symbol for the remove_ association method



516
517
518
# File 'lib/sequel/model/associations.rb', line 516

def remove_method
  :"remove_#{singularize(self[:name])}"
end

#remove_should_check_existing?Boolean

Whether to check that an object to be disassociated is already associated to this object, false by default.

Returns:

  • (Boolean)


521
522
523
# File 'lib/sequel/model/associations.rb', line 521

def remove_should_check_existing?
  false
end

#returns_array?Boolean

Whether this association returns an array of objects instead of a single object, true by default.

Returns:

  • (Boolean)


527
528
529
# File 'lib/sequel/model/associations.rb', line 527

def returns_array?
  true
end

#selectObject

The columns to select when loading the association.



532
533
534
# File 'lib/sequel/model/associations.rb', line 532

def select
  self[:select]
end

#set_reciprocal_to_self?Boolean

Whether to set the reciprocal association to self when loading associated records, false by default.

Returns:

  • (Boolean)


538
539
540
# File 'lib/sequel/model/associations.rb', line 538

def set_reciprocal_to_self?
  false
end

#setter_methodObject

Name symbol for the setter association method



543
544
545
# File 'lib/sequel/model/associations.rb', line 543

def setter_method
  :"#{self[:name]}="
end

#slice_range(limit_and_offset = limit_and_offset()) ⇒ Object

The range used for slicing when using the :ruby eager limit strategy.



548
549
550
551
552
553
# File 'lib/sequel/model/associations.rb', line 548

def slice_range(limit_and_offset = limit_and_offset())
  limit, offset = limit_and_offset
  if limit || offset
    (offset||0)..(limit ? (offset||0)+limit-1 : -1)
  end
end