Module: Sequel::Model::Associations::InstanceMethods
- Defined in:
- lib/sequel/model/associations.rb
Overview
Instance methods used to implement the associations support.
Instance Method Summary collapse
-
#associations ⇒ Object
The currently cached associations.
-
#freeze ⇒ Object
Freeze the associations cache when freezing the object.
Instance Method Details
#associations ⇒ Object
The currently cached associations. A hash with the keys being the association name symbols and the values being the associated object or nil (many_to_one), or the array of associated objects (*_to_many).
2697 2698 2699 |
# File 'lib/sequel/model/associations.rb', line 2697 def associations @associations ||= {} end |
#freeze ⇒ Object
Freeze the associations cache when freezing the object. Note that retrieving associations after freezing will still work in most cases, but the associations will not be cached in the association cache.
2704 2705 2706 2707 2708 2709 |
# File 'lib/sequel/model/associations.rb', line 2704 def freeze associations super associations.freeze self end |