Module: Sequel::Model::Associations::InstanceMethods

Defined in:
lib/sequel/model/associations.rb

Overview

Instance methods used to implement the associations support.

Instance Method Summary collapse

Instance Method Details

#associations ⇒ Object

The currently cached associations. A hash with the keys being the association name symbols and the values being the associated object or nil (many_to_one), or the array of associated objects (*_to_many).



1971
1972
1973
# File 'lib/sequel/model/associations.rb', line 1971

def associations
  @associations ||= {}
end

#freeze ⇒ Object

Freeze the associations cache when freezing the object. Note that retrieving associations after freezing will still work in most cases, but the associations will not be cached in the association cache.



1978
1979
1980
1981
# File 'lib/sequel/model/associations.rb', line 1978

def freeze
  associations.freeze
  super
end