Module: Sequel::Model::Associations::InstanceMethods
- Defined in:
- lib/sequel/model/associations.rb
Overview
Instance methods used to implement the associations support.
Instance Method Summary collapse
-
#associations ⇒ Object
The currently cached associations.
-
#freeze ⇒ Object
Freeze the associations cache when freezing the object.
Instance Method Details
#associations ⇒ Object
The currently cached associations. A hash with the keys being the association name symbols and the values being the associated object or nil (many_to_one), or the array of associated objects (*_to_many).
2224 2225 2226 |
# File 'lib/sequel/model/associations.rb', line 2224 def associations @associations ||= {} end |
#freeze ⇒ Object
Freeze the associations cache when freezing the object. Note that retrieving associations after freezing will still work in most cases, but the associations will not be cached in the association cache.
2231 2232 2233 2234 2235 2236 |
# File 'lib/sequel/model/associations.rb', line 2231 def freeze associations super associations.freeze self end |