Class: Sequel::Model::Associations::ManyToManyAssociationReflection
- Inherits:
-
AssociationReflection
- Object
- Hash
- AssociationReflection
- Sequel::Model::Associations::ManyToManyAssociationReflection
- Defined in:
- lib/sequel/model/associations.rb
Direct Known Subclasses
Plugins::ManyThroughMany::ManyThroughManyAssociationReflection
Constant Summary
Constants included from Inflections
Inflections::CAMELIZE_CONVERT_REGEXP, Inflections::CAMELIZE_MODULE_REGEXP, Inflections::DASH, Inflections::DEMODULIZE_CONVERT_REGEXP, Inflections::EMPTY_STRING, Inflections::SLASH, Inflections::UNDERSCORE, Inflections::UNDERSCORE_CONVERT_REGEXP1, Inflections::UNDERSCORE_CONVERT_REGEXP2, Inflections::UNDERSCORE_CONVERT_REPLACE, Inflections::UNDERSCORE_MODULE_REGEXP, Inflections::VALID_CONSTANT_NAME_REGEXP
Instance Method Summary collapse
-
#associated_key_alias ⇒ Object
The alias to use for the associated key when eagerly loading.
-
#associated_key_column ⇒ Object
The column to use for the associated key when eagerly loading.
-
#associated_key_table ⇒ Object
The table containing the column to use for the associated key when eagerly loading.
-
#default_associated_key_alias ⇒ Object
The default associated key alias.
-
#default_join_table ⇒ Object
Default name symbol for the join table.
-
#default_left_key ⇒ Object
Default foreign key name symbol for key in join table that points to current table's primary key (or :left_primary_key column).
-
#default_right_key ⇒ Object
Default foreign key name symbol for foreign key in join table that points to the association's table's primary key (or :right_primary_key column).
-
#eager_loader_key ⇒ Object
The key to use for the key hash when eager loading.
-
#eager_loading_use_associated_key? ⇒ Boolean
many_to_many associations need to select a key in an associated table to eagerly load.
-
#need_associated_primary_key? ⇒ Boolean
Whether the associated object needs a primary key to be added/removed, true for many_to_many associations.
-
#reciprocal ⇒ Object
Returns the reciprocal association symbol, if one exists.
-
#right_primary_key ⇒ Object
The primary key column to use in the associated table.
-
#select ⇒ Object
The columns to select when loading the association, associated_class.table_name.* by default.
Methods inherited from AssociationReflection
#_add_method, #_dataset_method, #_remove_all_method, #_remove_method, #_setter_method, #add_method, #associated_class, #association_method, #dataset_helper_method, #dataset_method, #dataset_need_primary_key?, #eager_graph_lazy_dataset?, #reciprocal_array?, #remove_all_method, #remove_method, #returns_array?, #set_reciprocal_to_self?, #setter_method
Methods included from Inflections
clear, irregular, plural, singular, uncountable
Methods inherited from Hash
#&, #case, #sql_expr, #sql_negate, #sql_or, #|, #~
Instance Method Details
#associated_key_alias ⇒ Object
The alias to use for the associated key when eagerly loading
234 235 236 |
# File 'lib/sequel/model/associations.rb', line 234 def associated_key_alias self[:left_key_alias] end |
#associated_key_column ⇒ Object
The column to use for the associated key when eagerly loading
239 240 241 |
# File 'lib/sequel/model/associations.rb', line 239 def associated_key_column self[:left_key] end |
#associated_key_table ⇒ Object
The table containing the column to use for the associated key when eagerly loading
244 245 246 |
# File 'lib/sequel/model/associations.rb', line 244 def associated_key_table self[:join_table] end |
#default_associated_key_alias ⇒ Object
The default associated key alias
249 250 251 |
# File 'lib/sequel/model/associations.rb', line 249 def default_associated_key_alias :x_foreign_key_x end |
#default_join_table ⇒ Object
Default name symbol for the join table.
254 255 256 |
# File 'lib/sequel/model/associations.rb', line 254 def default_join_table [self[:class_name], self[:model].name].map{|i| underscore(pluralize(demodulize(i)))}.sort.join('_').to_sym end |
#default_left_key ⇒ Object
Default foreign key name symbol for key in join table that points to current table's primary key (or :left_primary_key column).
260 261 262 |
# File 'lib/sequel/model/associations.rb', line 260 def default_left_key :"#{underscore(demodulize(self[:model].name))}_id" end |
#default_right_key ⇒ Object
Default foreign key name symbol for foreign key in join table that points to the association's table's primary key (or :right_primary_key column).
266 267 268 |
# File 'lib/sequel/model/associations.rb', line 266 def default_right_key :"#{singularize(self[:name])}_id" end |
#eager_loader_key ⇒ Object
The key to use for the key hash when eager loading
271 272 273 |
# File 'lib/sequel/model/associations.rb', line 271 def eager_loader_key self[:left_primary_key] end |
#eager_loading_use_associated_key? ⇒ Boolean
many_to_many associations need to select a key in an associated table to eagerly load
276 277 278 |
# File 'lib/sequel/model/associations.rb', line 276 def eager_loading_use_associated_key? true end |
#need_associated_primary_key? ⇒ Boolean
Whether the associated object needs a primary key to be added/removed, true for many_to_many associations.
282 283 284 |
# File 'lib/sequel/model/associations.rb', line 282 def need_associated_primary_key? true end |
#reciprocal ⇒ Object
Returns the reciprocal association symbol, if one exists.
287 288 289 290 291 292 293 294 295 296 297 298 299 300 |
# File 'lib/sequel/model/associations.rb', line 287 def reciprocal return self[:reciprocal] if include?(:reciprocal) left_key = self[:left_key] right_key = self[:right_key] join_table = self[:join_table] associated_class.all_association_reflections.each do |assoc_reflect| if assoc_reflect[:type] == :many_to_many && assoc_reflect[:left_key] == right_key && assoc_reflect[:right_key] == left_key && assoc_reflect[:join_table] == join_table && assoc_reflect.associated_class == self[:model] return self[:reciprocal] = assoc_reflect[:name] end end self[:reciprocal] = nil end |
#right_primary_key ⇒ Object
The primary key column to use in the associated table.
303 304 305 |
# File 'lib/sequel/model/associations.rb', line 303 def right_primary_key self[:right_primary_key] ||= associated_class.primary_key end |
#select ⇒ Object
The columns to select when loading the association, associated_class.table_name.* by default.
308 309 310 311 |
# File 'lib/sequel/model/associations.rb', line 308 def select return self[:select] if include?(:select) self[:select] ||= Sequel::SQL::ColumnAll.new(associated_class.table_name) end |