Module: Sequel::Model::ClassMethods
- Included in:
- Sequel::Model
- Defined in:
- lib/sequel/model/base.rb
Overview
Class methods for Sequel::Model that implement basic model functionality.
-
All of the method names in Model::DATASET_METHODS have class methods created that call the Model’s dataset with the method of the same name with the given arguments.
Instance Attribute Summary collapse
-
#allowed_columns ⇒ Object
readonly
Which columns should be the only columns allowed in a call to a mass assignment method (e.g. set) (default: not set, so all columns not otherwise restricted are allowed).
-
#cache_anonymous_models ⇒ Object
Whether to cache the anonymous models created by Sequel::Model().
-
#dataset_method_modules ⇒ Object
readonly
Array of modules that extend this model’s dataset.
-
#default_set_fields_options ⇒ Object
The default options to use for Model#set_fields.
-
#fast_instance_delete_sql ⇒ Object
readonly
SQL string fragment used for faster DELETE statement creation when deleting/destroying model instances, or nil if the optimization should not be used.
-
#instance_dataset ⇒ Object
readonly
The dataset that instance datasets (#this) are based on.
-
#plugins ⇒ Object
readonly
Array of plugin modules loaded by this class.
-
#primary_key ⇒ Object
readonly
The primary key for the class.
-
#raise_on_save_failure ⇒ Object
Whether to raise an error instead of returning nil on a failure to save/create/save_changes/etc due to a validation failure or a before_* hook returning false.
-
#raise_on_typecast_failure ⇒ Object
Whether to raise an error when unable to typecast data for a column (default: true).
-
#require_modification ⇒ Object
Whether to raise an error if an UPDATE or DELETE query related to a model instance does not modify exactly 1 row.
-
#restricted_columns ⇒ Object
readonly
Which columns are specifically restricted in a call to set/update/new/etc.
-
#simple_pk ⇒ Object
readonly
Should be the literal primary key column name if this Model’s table has a simple primary key, or nil if the model has a compound primary key or no primary key.
-
#simple_table ⇒ Object
readonly
Should be the literal table name if this Model’s dataset is a simple table (no select, order, join, etc.), or nil otherwise.
-
#strict_param_setting ⇒ Object
Whether new/set/update and their variants should raise an error if an invalid key is used.
-
#typecast_empty_string_to_nil ⇒ Object
Whether to typecast the empty string (”) to nil for columns that are not string or blob.
-
#typecast_on_assignment ⇒ Object
Whether to typecast attribute values on assignment (default: true).
-
#use_after_commit_rollback ⇒ Object
Whether to enable the after_commit and after_rollback hooks when saving/destroying instances.
-
#use_transactions ⇒ Object
Whether to use a transaction by default when saving/deleting records (default: true).
Instance Method Summary collapse
-
#[](*args) ⇒ Object
Returns the first record from the database matching the conditions.
-
#call(values) ⇒ Object
Initializes a model instance as an existing record.
-
#clear_setter_methods_cache ⇒ Object
Clear the setter_methods cache.
-
#columns ⇒ Object
Returns the columns in the result set in their original order.
-
#create(values = {}, &block) ⇒ Object
Creates instance using new with the given values and block, and saves it.
-
#dataset ⇒ Object
Returns the dataset associated with the Model class.
-
#dataset=(ds) ⇒ Object
Alias of set_dataset.
-
#dataset_module(mod = nil) ⇒ Object
Extend the dataset with a module, similar to adding a plugin with the methods defined in DatasetMethods.
-
#db ⇒ Object
Returns the database associated with the Model class.
-
#db=(db) ⇒ Object
Sets the database associated with the Model class.
-
#db_schema ⇒ Object
Returns the cached schema information if available or gets it from the database.
-
#def_column_alias(meth, column) ⇒ Object
Create a column alias, where the column methods have one name, but the underlying storage uses a different name.
-
#def_dataset_method(*args, &block) ⇒ Object
If a block is given, define a method on the dataset (if the model currently has an dataset) with the given argument name using the given block.
-
#find(*args, &block) ⇒ Object
Finds a single record according to the supplied filter.
-
#find_or_create(cond, &block) ⇒ Object
Like
findbut invokes create with given conditions when record does not exist. -
#implicit_table_name ⇒ Object
Returns the implicit table name for the model class, which is the demodulized, underscored, pluralized name of the class.
-
#include(mod) ⇒ Object
Clear the setter_methods cache when a module is included, as it may contain setter methods.
-
#inherited(subclass) ⇒ Object
If possible, set the dataset for the model subclass as soon as it is created.
-
#load(values) ⇒ Object
Calls #call with the values hash.
-
#method_added(meth) ⇒ Object
Clear the setter_methods cache when a setter method is added.
-
#no_primary_key ⇒ Object
Mark the model as not having a primary key.
-
#plugin(plugin, *args, &block) ⇒ Object
Loads a plugin for use with the model class, passing optional arguments to the plugin.
-
#primary_key_hash(value) ⇒ Object
Returns primary key attribute hash.
-
#qualified_primary_key_hash(value, qualifier = table_name) ⇒ Object
Return a hash where the keys are qualified column references.
-
#restrict_primary_key ⇒ Object
Restrict the setting of the primary key(s) when using mass assignment (e.g.
set). -
#restrict_primary_key? ⇒ Boolean
Whether or not setting the primary key(s) when using mass assignment (e.g.
set) is restricted, true by default. -
#set_allowed_columns(*cols) ⇒ Object
Set the columns to allow when using mass assignment (e.g.
set). -
#set_dataset(ds, opts = {}) ⇒ Object
Sets the dataset associated with the Model class.
-
#set_primary_key(*key) ⇒ Object
Sets the primary key for this model.
-
#set_restricted_columns(*cols) ⇒ Object
Set the columns to restrict when using mass assignment (e.g.
set). -
#setter_methods ⇒ Object
Cache of setter methods to allow by default, in order to speed up new/set/update instance methods.
-
#subset(name, *args, &block) ⇒ Object
Sets up a dataset method that returns a filtered dataset.
-
#table_name ⇒ Object
Returns name of primary table for the dataset.
-
#unrestrict_primary_key ⇒ Object
Allow the setting of the primary key(s) when using the mass assignment methods.
Instance Attribute Details
#allowed_columns ⇒ Object (readonly)
Which columns should be the only columns allowed in a call to a mass assignment method (e.g. set) (default: not set, so all columns not otherwise restricted are allowed).
13 14 15 |
# File 'lib/sequel/model/base.rb', line 13 def allowed_columns @allowed_columns end |
#cache_anonymous_models ⇒ Object
Whether to cache the anonymous models created by Sequel::Model(). This is required for reloading them correctly (avoiding the superclass mismatch). True by default for backwards compatibility.
18 19 20 |
# File 'lib/sequel/model/base.rb', line 18 def cache_anonymous_models @cache_anonymous_models end |
#dataset_method_modules ⇒ Object (readonly)
Array of modules that extend this model’s dataset. Stored so that if the model’s dataset is changed, it will be extended with all of these modules.
23 24 25 |
# File 'lib/sequel/model/base.rb', line 23 def dataset_method_modules @dataset_method_modules end |
#default_set_fields_options ⇒ Object
The default options to use for Model#set_fields. These are merged with the options given to set_fields.
27 28 29 |
# File 'lib/sequel/model/base.rb', line 27 def end |
#fast_instance_delete_sql ⇒ Object (readonly)
SQL string fragment used for faster DELETE statement creation when deleting/destroying model instances, or nil if the optimization should not be used. For internal use only.
31 32 33 |
# File 'lib/sequel/model/base.rb', line 31 def fast_instance_delete_sql @fast_instance_delete_sql end |
#instance_dataset ⇒ Object (readonly)
The dataset that instance datasets (#this) are based on. Generally a naked version of the model’s dataset limited to one row. For internal use only.
35 36 37 |
# File 'lib/sequel/model/base.rb', line 35 def instance_dataset @instance_dataset end |
#plugins ⇒ Object (readonly)
Array of plugin modules loaded by this class
Sequel::Model.plugins
# => [Sequel::Model, Sequel::Model::Associations]
41 42 43 |
# File 'lib/sequel/model/base.rb', line 41 def plugins @plugins end |
#primary_key ⇒ Object (readonly)
The primary key for the class. Sequel can determine this automatically for many databases, but not all, so you may need to set it manually. If not determined automatically, the default is :id.
46 47 48 |
# File 'lib/sequel/model/base.rb', line 46 def primary_key @primary_key end |
#raise_on_save_failure ⇒ Object
Whether to raise an error instead of returning nil on a failure to save/create/save_changes/etc due to a validation failure or a before_* hook returning false.
51 52 53 |
# File 'lib/sequel/model/base.rb', line 51 def raise_on_save_failure @raise_on_save_failure end |
#raise_on_typecast_failure ⇒ Object
Whether to raise an error when unable to typecast data for a column (default: true). This should be set to false if you want to use validations to display nice error messages to the user (e.g. most web applications). You can use the validates_schema_types validation (from the validation_helpers plugin) in connection with this setting to check for typecast failures during validation.
59 60 61 |
# File 'lib/sequel/model/base.rb', line 59 def raise_on_typecast_failure @raise_on_typecast_failure end |
#require_modification ⇒ Object
Whether to raise an error if an UPDATE or DELETE query related to a model instance does not modify exactly 1 row. If set to false, Sequel will not check the number of rows modified (default: true).
64 65 66 |
# File 'lib/sequel/model/base.rb', line 64 def require_modification @require_modification end |
#restricted_columns ⇒ Object (readonly)
Which columns are specifically restricted in a call to set/update/new/etc. (default: not set). Some columns are restricted regardless of this setting, such as the primary key column and columns in Model::RESTRICTED_SETTER_METHODS.
69 70 71 |
# File 'lib/sequel/model/base.rb', line 69 def restricted_columns @restricted_columns end |
#simple_pk ⇒ Object
Should be the literal primary key column name if this Model’s table has a simple primary key, or nil if the model has a compound primary key or no primary key.
73 74 75 |
# File 'lib/sequel/model/base.rb', line 73 def simple_pk @simple_pk end |
#simple_table ⇒ Object
Should be the literal table name if this Model’s dataset is a simple table (no select, order, join, etc.), or nil otherwise. This and simple_pk are used for an optimization in Model.[].
77 78 79 |
# File 'lib/sequel/model/base.rb', line 77 def simple_table @simple_table end |
#strict_param_setting ⇒ Object
Whether new/set/update and their variants should raise an error if an invalid key is used. A key is invalid if no setter method exists for that key or the access to the setter method is restricted (e.g. due to it being a primary key field). If set to false, silently skip any key where the setter method doesn’t exist or access to it is restricted.
84 85 86 |
# File 'lib/sequel/model/base.rb', line 84 def strict_param_setting @strict_param_setting end |
#typecast_empty_string_to_nil ⇒ Object
Whether to typecast the empty string (”) to nil for columns that are not string or blob. In most cases the empty string would be the way to specify a NULL SQL value in string form (nil.to_s == ”), and an empty string would not usually be typecast correctly for other types, so the default is true.
91 92 93 |
# File 'lib/sequel/model/base.rb', line 91 def typecast_empty_string_to_nil @typecast_empty_string_to_nil end |
#typecast_on_assignment ⇒ Object
Whether to typecast attribute values on assignment (default: true). If set to false, no typecasting is done, so it will be left up to the database to typecast the value correctly.
96 97 98 |
# File 'lib/sequel/model/base.rb', line 96 def typecast_on_assignment @typecast_on_assignment end |
#use_after_commit_rollback ⇒ Object
Whether to enable the after_commit and after_rollback hooks when saving/destroying instances. On by default, can be turned off for performance reasons or when using prepared transactions (which aren’t compatible with after commit/rollback).
101 102 103 |
# File 'lib/sequel/model/base.rb', line 101 def use_after_commit_rollback @use_after_commit_rollback end |
#use_transactions ⇒ Object
Whether to use a transaction by default when saving/deleting records (default: true). If you are sending database queries in before_* or after_* hooks, you shouldn’t change the default setting without a good reason.
106 107 108 |
# File 'lib/sequel/model/base.rb', line 106 def use_transactions @use_transactions end |
Instance Method Details
#[](*args) ⇒ Object
Returns the first record from the database matching the conditions. If a hash is given, it is used as the conditions. If another object is given, it finds the first record whose primary key(s) match the given argument(s). If no object is returned by the dataset, returns nil.
Artist[1] # SELECT * FROM artists WHERE id = 1
# => #<Artist {:id=>1, ...}>
Artist[:name=>'Bob'] # SELECT * FROM artists WHERE (name = 'Bob') LIMIT 1
# => #<Artist {:name=>'Bob', ...}>
118 119 120 121 |
# File 'lib/sequel/model/base.rb', line 118 def [](*args) args = args.first if args.size <= 1 args.is_a?(Hash) ? dataset[args] : (primary_key_lookup(args) unless args.nil?) end |
#call(values) ⇒ Object
Initializes a model instance as an existing record. This constructor is used by Sequel to initialize model instances when fetching records. Requires that values be a hash where all keys are symbols. It probably should not be used by external code.
127 128 129 130 131 132 |
# File 'lib/sequel/model/base.rb', line 127 def call(values) o = allocate o.set_values(values) o.after_initialize o end |
#clear_setter_methods_cache ⇒ Object
Clear the setter_methods cache
135 136 137 |
# File 'lib/sequel/model/base.rb', line 135 def clear_setter_methods_cache @setter_methods = nil end |
#columns ⇒ Object
Returns the columns in the result set in their original order. Generally, this will use the columns determined via the database schema, but in certain cases (e.g. models that are based on a joined dataset) it will use Dataset#columns to find the columns.
Artist.columns
# => [:id, :name]
146 147 148 |
# File 'lib/sequel/model/base.rb', line 146 def columns @columns || set_columns(dataset.naked.columns) end |
#create(values = {}, &block) ⇒ Object
Creates instance using new with the given values and block, and saves it.
Artist.create(:name=>'Bob')
# INSERT INTO artists (name) VALUES ('Bob')
Artist.create do |a|
a.name = 'Jim'
end # INSERT INTO artists (name) VALUES ('Jim')
158 159 160 |
# File 'lib/sequel/model/base.rb', line 158 def create(values = {}, &block) new(values, &block).save end |
#dataset ⇒ Object
Returns the dataset associated with the Model class. Raises an Error if there is no associated dataset for this class. In most cases, you don’t need to call this directly, as Model proxies many dataset methods to the underlying dataset.
Artist.dataset.all # SELECT * FROM artists
168 169 170 |
# File 'lib/sequel/model/base.rb', line 168 def dataset @dataset || raise(Error, "No dataset associated with #{self}") end |
#dataset=(ds) ⇒ Object
Alias of set_dataset
173 174 175 |
# File 'lib/sequel/model/base.rb', line 173 def dataset=(ds) set_dataset(ds) end |
#dataset_module(mod = nil) ⇒ Object
Extend the dataset with a module, similar to adding a plugin with the methods defined in DatasetMethods. This is the recommended way to add methods to model datasets.
If an argument, it should be a module, and is used to extend the underlying dataset. Otherwise an anonymous module is created, and if a block is given, it is module_evaled, allowing you do define dataset methods directly using the standard ruby def syntax. Returns the module given or the anonymous module created.
# Usage with existing module
Artist.dataset_module Sequel::ColumnsIntrospection
# Usage with anonymous module
Artist.dataset_module do
def foo
:bar
end
end
Artist.dataset.foo
# => :bar
Artist.foo
# => :bar
Any anonymous modules created are actually instances of Sequel::Model::DatasetModule (a Module subclass), which allows you to call the subset method on them:
Artist.dataset_module do
subset :released, Sequel.identifier(release_date) > Sequel::CURRENT_DATE
end
Any public methods in the dataset module will have class methods created that call the method on the dataset, assuming that the class method is not already defined.
211 212 213 214 215 216 217 218 219 220 221 222 |
# File 'lib/sequel/model/base.rb', line 211 def dataset_module(mod = nil) if mod raise Error, "can't provide both argument and block to Model.dataset_module" if block_given? dataset_extend(mod) mod else @dataset_module ||= DatasetModule.new(self) @dataset_module.module_eval(&Proc.new) if block_given? dataset_extend(@dataset_module) @dataset_module end end |
#db ⇒ Object
Returns the database associated with the Model class. If this model doesn’t have a database associated with it, assumes the superclass’s database, or the first object in Sequel::DATABASES. If no Sequel::Database object has been created, raises an error.
Artist.db.transaction do # BEGIN
Artist.create(:name=>'Bob')
# INSERT INTO artists (name) VALUES ('Bob')
end # COMMIT
234 235 236 237 238 239 |
# File 'lib/sequel/model/base.rb', line 234 def db return @db if @db @db = self == Model ? Sequel.synchronize{DATABASES.first} : superclass.db raise(Error, "No database associated with #{self}: have you called Sequel.connect or #{self}.db= ?") unless @db @db end |
#db=(db) ⇒ Object
Sets the database associated with the Model class. If the model has an associated dataset, sets the model’s dataset to a dataset on the new database with the same options used by the current dataset. This can be used directly on Sequel::Model to set the default database to be used by subclasses, or to override the database used for specific models:
Sequel::Model.db = DB1
Artist.db = DB2
Note that you should not use this to change the model’s database at runtime. If you have that need, you should look into Sequel’s sharding support.
255 256 257 258 |
# File 'lib/sequel/model/base.rb', line 255 def db=(db) @db = db set_dataset(db.dataset(@dataset.opts)) if @dataset end |
#db_schema ⇒ Object
Returns the cached schema information if available or gets it from the database. This is a hash where keys are column symbols and values are hashes of information related to the column. See Database#schema.
Artist.db_schema
# {:id=>{:type=>:integer, :primary_key=>true, ...},
# :name=>{:type=>:string, :primary_key=>false, ...}}
268 269 270 |
# File 'lib/sequel/model/base.rb', line 268 def db_schema @db_schema ||= get_db_schema end |
#def_column_alias(meth, column) ⇒ Object
Create a column alias, where the column methods have one name, but the underlying storage uses a different name.
274 275 276 277 278 279 280 |
# File 'lib/sequel/model/base.rb', line 274 def def_column_alias(meth, column) clear_setter_methods_cache overridable_methods_module.module_eval do define_method(meth){self[column]} define_method("#{meth}="){|v| self[column] = v} end end |
#def_dataset_method(*args, &block) ⇒ Object
If a block is given, define a method on the dataset (if the model currently has an dataset) with the given argument name using the given block. Also define a class method on the model that calls the dataset method. Stores the method name and block so that it can be reapplied if the model’s dataset changes.
If a block is not given, just define a class method on the model for each argument that calls the dataset method of the same argument name.
It is recommended that you define methods inside a block passed to #dataset_module instead of using this method, as #dataset_module allows you to use normal ruby def syntax.
# Add new dataset method and class method that calls it
Artist.def_dataset_method(:by_name){order(:name)}
Artist.filter(:name.like('A%')).by_name
Artist.by_name.filter(:name.like('A%'))
# Just add a class method that calls an existing dataset method
Artist.def_dataset_method(:server!)
Artist.server!(:server1)
302 303 304 305 306 307 308 309 310 311 |
# File 'lib/sequel/model/base.rb', line 302 def def_dataset_method(*args, &block) raise(Error, "No arguments given") if args.empty? if block raise(Error, "Defining a dataset method using a block requires only one argument") if args.length > 1 dataset_module{define_method(args.first, &block)} else args.each{|arg| def_model_dataset_method(arg)} end end |
#find(*args, &block) ⇒ Object
Finds a single record according to the supplied filter. You are encouraged to use Model.[] or Model.first instead of this method.
Artist.find(:name=>'Bob')
# SELECT * FROM artists WHERE (name = 'Bob') LIMIT 1
Artist.find{name > 'M'}
# SELECT * FROM artists WHERE (name > 'M') LIMIT 1
321 322 323 |
# File 'lib/sequel/model/base.rb', line 321 def find(*args, &block) filter(*args, &block).first end |
#find_or_create(cond, &block) ⇒ Object
Like find but invokes create with given conditions when record does not exist. Unlike find in that the block used in this method is not passed to find, but instead is passed to create only if find does not return an object.
Artist.find_or_create(:name=>'Bob')
# SELECT * FROM artists WHERE (name = 'Bob') LIMIT 1
# INSERT INTO artists (name) VALUES ('Bob')
Artist.find_or_create(:name=>'Jim'){|a| a.hometown = 'Sactown'}
# SELECT * FROM artists WHERE (name = 'Jim') LIMIT 1
# INSERT INTO artists (name, hometown) VALUES ('Jim', 'Sactown')
337 338 339 |
# File 'lib/sequel/model/base.rb', line 337 def find_or_create(cond, &block) find(cond) || create(cond, &block) end |
#implicit_table_name ⇒ Object
Returns the implicit table name for the model class, which is the demodulized, underscored, pluralized name of the class.
Artist.implicit_table_name # => :artists
Foo::ArtistAlias.implicit_table_name # => :artist_aliases
396 397 398 |
# File 'lib/sequel/model/base.rb', line 396 def implicit_table_name pluralize(underscore(demodulize(name))).to_sym end |
#include(mod) ⇒ Object
Clear the setter_methods cache when a module is included, as it may contain setter methods.
343 344 345 346 |
# File 'lib/sequel/model/base.rb', line 343 def include(mod) clear_setter_methods_cache super end |
#inherited(subclass) ⇒ Object
If possible, set the dataset for the model subclass as soon as it is created. Also, make sure the inherited class instance variables are copied into the subclass.
Sequel queries the database to get schema information as soon as a model class is created:
class Artist < Sequel::Model # Causes schema query
end
357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 |
# File 'lib/sequel/model/base.rb', line 357 def inherited(subclass) super ivs = subclass.instance_variables.collect{|x| x.to_s} inherited_instance_variables.each do |iv, dup| next if ivs.include?(iv.to_s) if (sup_class_value = instance_variable_get(iv)) && dup sup_class_value = case dup when :dup sup_class_value.dup when :hash_dup h = {} sup_class_value.each{|k,v| h[k] = v.dup} h when Proc dup.call(sup_class_value) else raise Error, "bad inherited instance variable type: #{dup.inspect}" end end subclass.instance_variable_set(iv, sup_class_value) end unless ivs.include?("@dataset") if self == Model || !@dataset n = subclass.name unless n.nil? || n.empty? db subclass.set_dataset(subclass.implicit_table_name) rescue nil end elsif @dataset subclass.set_dataset(@dataset.clone, :inherited=>true) rescue nil end end end |
#load(values) ⇒ Object
Calls #call with the values hash. Only for backwards compatibility.
401 402 403 |
# File 'lib/sequel/model/base.rb', line 401 def load(values) call(values) end |
#method_added(meth) ⇒ Object
Clear the setter_methods cache when a setter method is added
406 407 408 409 |
# File 'lib/sequel/model/base.rb', line 406 def method_added(meth) clear_setter_methods_cache if meth.to_s =~ SETTER_METHOD_REGEXP super end |
#no_primary_key ⇒ Object
Mark the model as not having a primary key. Not having a primary key can cause issues, among which is that you won’t be able to update records.
Artist.primary_key # => :id
Artist.no_primary_key
Artist.primary_key # => nil
417 418 419 420 |
# File 'lib/sequel/model/base.rb', line 417 def no_primary_key clear_setter_methods_cache self.simple_pk = @primary_key = nil end |
#plugin(plugin, *args, &block) ⇒ Object
Loads a plugin for use with the model class, passing optional arguments to the plugin. If the plugin is a module, load it directly. Otherwise, require the plugin from either sequel/plugins/##plugin or sequel_##plugin, and then attempt to load the module using a the camelized plugin name under Sequel::Plugins.
427 428 429 430 431 432 433 434 435 436 437 |
# File 'lib/sequel/model/base.rb', line 427 def plugin(plugin, *args, &block) m = plugin.is_a?(Module) ? plugin : plugin_module(plugin) unless @plugins.include?(m) @plugins << m m.apply(self, *args, &block) if m.respond_to?(:apply) include(m::InstanceMethods) if plugin_module_defined?(m, :InstanceMethods) extend(m::ClassMethods)if plugin_module_defined?(m, :ClassMethods) dataset_extend(m::DatasetMethods) if plugin_module_defined?(m, :DatasetMethods) end m.configure(self, *args, &block) if m.respond_to?(:configure) end |
#primary_key_hash(value) ⇒ Object
Returns primary key attribute hash. If using a composite primary key value such be an array with values for each primary key in the correct order. For a standard primary key, value should be an object with a compatible type for the key. If the model does not have a primary key, raises an Error.
Artist.primary_key_hash(1) # => {:id=>1}
Artist.primary_key_hash([1, 2]) # => {:id1=>1, :id2=>2}
447 448 449 450 451 452 453 454 455 456 457 |
# File 'lib/sequel/model/base.rb', line 447 def primary_key_hash(value) raise(Error, "#{self} does not have a primary key") unless key = @primary_key case key when Array hash = {} key.each_with_index{|k,i| hash[k] = value[i]} hash else {key => value} end end |
#qualified_primary_key_hash(value, qualifier = table_name) ⇒ Object
Return a hash where the keys are qualified column references. Uses the given qualifier if provided, or the table_name otherwise. This is useful if you plan to join other tables to this table and you want the column references to be qualified.
Artist.filter(Artist.qualified_primary_key_hash(1))
# SELECT * FROM artists WHERE (artists.id = 1)
466 467 468 469 470 |
# File 'lib/sequel/model/base.rb', line 466 def qualified_primary_key_hash(value, qualifier=table_name) h = primary_key_hash(value) h.to_a.each{|k,v| h[SQL::QualifiedIdentifier.new(qualifier, k)] = h.delete(k)} h end |
#restrict_primary_key ⇒ Object
Restrict the setting of the primary key(s) when using mass assignment (e.g. set). Because this is the default, this only make sense to use in a subclass where the parent class has used unrestrict_primary_key.
475 476 477 478 |
# File 'lib/sequel/model/base.rb', line 475 def restrict_primary_key clear_setter_methods_cache @restrict_primary_key = true end |
#restrict_primary_key? ⇒ Boolean
Whether or not setting the primary key(s) when using mass assignment (e.g. set) is restricted, true by default.
482 483 484 |
# File 'lib/sequel/model/base.rb', line 482 def restrict_primary_key? @restrict_primary_key end |
#set_allowed_columns(*cols) ⇒ Object
Set the columns to allow when using mass assignment (e.g. set). Using this means that any columns not listed here will not be modified. If you have any virtual setter methods (methods that end in =) that you want to be used during mass assignment, they need to be listed here as well (without the =).
It may be better to use a method such as set_only or set_fields that lets you specify the allowed fields per call.
Artist.set_allowed_columns(:name, :hometown)
Artist.set(:name=>'Bob', :hometown=>'Sactown') # No Error
Artist.set(:name=>'Bob', :records_sold=>30000) # Error
497 498 499 500 |
# File 'lib/sequel/model/base.rb', line 497 def set_allowed_columns(*cols) clear_setter_methods_cache @allowed_columns = cols end |
#set_dataset(ds, opts = {}) ⇒ Object
Sets the dataset associated with the Model class. ds can be a Symbol, LiteralString, SQL::Identifier, SQL::QualifiedIdentifier, SQL::AliasedExpression (all specifying a table name in the current database), or a Dataset. If a dataset is used, the model’s database is changed to the database of the given dataset. If a dataset is not used, a dataset is created from the current database with the table name given. Other arguments raise an Error. Returns self.
This changes the row_proc of the dataset to return model objects and extends the dataset with the dataset_method_modules. It also attempts to determine the database schema for the model, based on the given dataset.
Artist.set_dataset(:tbl_artists)
Artist.set_dataset(DB[:artists])
Note that you should not use this to change the model’s dataset at runtime. If you have that need, you should look into Sequel’s sharding support.
522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 |
# File 'lib/sequel/model/base.rb', line 522 def set_dataset(ds, opts={}) inherited = opts[:inherited] @dataset = case ds when Symbol, SQL::Identifier, SQL::QualifiedIdentifier, SQL::AliasedExpression, LiteralString self.simple_table = db.literal(ds) db.from(ds) when Dataset self.simple_table = if ds.send(:simple_select_all?) ds.literal(ds.first_source_table) else nil end @db = ds.db ds else raise(Error, "Model.set_dataset takes one of the following classes as an argument: Symbol, LiteralString, SQL::Identifier, SQL::QualifiedIdentifier, SQL::AliasedExpression, Dataset") end @dataset.row_proc = self @require_modification = Sequel::Model.require_modification.nil? ? @dataset.provides_accurate_rows_matched? : Sequel::Model.require_modification if inherited self.simple_table = superclass.simple_table @columns = @dataset.columns rescue nil else @dataset_method_modules.each{|m| @dataset.extend(m)} if @dataset_method_modules end @dataset.model = self if @dataset.respond_to?(:model=) check_non_connection_error{@db_schema = (inherited ? superclass.db_schema : get_db_schema)} reset_instance_dataset self end |
#set_primary_key(*key) ⇒ Object
Sets the primary key for this model. You can use either a regular or a composite primary key. To not use a primary key, set to nil or use no_primary_key. On most adapters, Sequel can automatically determine the primary key to use, so this method is not needed often.
class Person < Sequel::Model
# regular key
set_primary_key :person_id
end
class Tagging < Sequel::Model
# composite key
set_primary_key [:taggable_id, :tag_id]
end
567 568 569 570 571 572 573 574 575 576 |
# File 'lib/sequel/model/base.rb', line 567 def set_primary_key(*key) clear_setter_methods_cache key = key.flatten self.simple_pk = if key.length == 1 (@dataset || db).literal(key.first) else nil end @primary_key = (key.length == 1) ? key[0] : key end |
#set_restricted_columns(*cols) ⇒ Object
Set the columns to restrict when using mass assignment (e.g. set). Using this means that attempts to call setter methods for the columns listed here will cause an exception or be silently skipped (based on the strict_param_setting setting). If you have any virtual setter methods (methods that end in =) that you want not to be used during mass assignment, they need to be listed here as well (without the =).
It’s generally a bad idea to rely on a blacklist approach for security. Using a whitelist approach such as set_allowed_columns or the instance level set_only or set_fields methods is usually a better choice. So use of this method is generally a bad idea.
Artist.set_restricted_columns(:records_sold)
Artist.set(:name=>'Bob', :hometown=>'Sactown') # No Error
Artist.set(:name=>'Bob', :records_sold=>30000) # Error
591 592 593 594 |
# File 'lib/sequel/model/base.rb', line 591 def set_restricted_columns(*cols) clear_setter_methods_cache @restricted_columns = cols end |
#setter_methods ⇒ Object
Cache of setter methods to allow by default, in order to speed up new/set/update instance methods.
597 598 599 600 601 602 603 604 605 606 |
# File 'lib/sequel/model/base.rb', line 597 def setter_methods @setter_methods ||= if allowed_columns allowed_columns.map{|x| "#{x}="} else meths = instance_methods.collect{|x| x.to_s}.grep(SETTER_METHOD_REGEXP) - RESTRICTED_SETTER_METHODS meths -= Array(primary_key).map{|x| "#{x}="} if primary_key && restrict_primary_key? meths -= restricted_columns.map{|x| "#{x}="} if restricted_columns meths end end |
#subset(name, *args, &block) ⇒ Object
Sets up a dataset method that returns a filtered dataset. Sometimes thought of as a scope, and like most dataset methods, they can be chained. For example:
Topic.subset(:joes, :username.like('%joe%'))
Topic.subset(:popular){num_posts > 100}
Topic.subset(:recent){created_on > Date.today - 7}
Allows you to do:
Topic.joes.recent.popular
to get topics with a username that includes joe that have more than 100 posts and were created less than 7 days ago.
Both the args given and the block are passed to Dataset#filter.
This method creates dataset methods that do not accept arguments. To create dataset methods that accept arguments, you should use define a method directly inside a #dataset_module block.
630 631 632 |
# File 'lib/sequel/model/base.rb', line 630 def subset(name, *args, &block) dataset_module.subset(name, *args, &block) end |
#table_name ⇒ Object
Returns name of primary table for the dataset. If the table for the dataset is aliased, returns the aliased name.
Artist.table_name # => :artists
Sequel::Model(:foo).table_name # => :foo
Sequel::Model(:foo___bar).table_name # => :bar
640 641 642 |
# File 'lib/sequel/model/base.rb', line 640 def table_name dataset.first_source_alias end |
#unrestrict_primary_key ⇒ Object
Allow the setting of the primary key(s) when using the mass assignment methods. Using this method can open up security issues, be very careful before using it.
Artist.set(:id=>1) # Error
Artist.unrestrict_primary_key
Artist.set(:id=>1) # No Error
650 651 652 653 |
# File 'lib/sequel/model/base.rb', line 650 def unrestrict_primary_key clear_setter_methods_cache @restrict_primary_key = false end |