Method: Sequel::Model::ClassMethods#call
- Defined in:
- lib/sequel/model/base.rb
#call(values) ⇒ Object
Initializes a model instance as an existing record. This constructor is used by Sequel to initialize model instances when fetching records. Requires that values be a hash where all keys are symbols. It probably should not be used by external code.
117 118 119 120 121 |
# File 'lib/sequel/model/base.rb', line 117 def call(values) o = allocate o.instance_variable_set(:@values, values) o end |