Module: Sequel::Model::DatasetMethods

Defined in:
lib/sequel/model/base.rb

Overview

Dataset methods are methods that the model class extends its dataset with in the call to set_dataset.

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#model ⇒ Object

The model class associated with this dataset

Artist.dataset.model # => Artist


2050
2051
2052
# File 'lib/sequel/model/base.rb', line 2050

def model
  @model
end

Instance Method Details

#[](*args) ⇒ Object

Assume if a single integer is given that it is a lookup by primary key, and call with_pk with the argument.

Artist.dataset[1] # SELECT * FROM artists WHERE (id = 1) LIMIT 1


2056
2057
2058
2059
2060
2061
2062
# File 'lib/sequel/model/base.rb', line 2056

def [](*args)
  if args.length == 1 && (i = args.at(0)) && i.is_a?(Integer)
    with_pk(i)
  else
    super
  end
end

#destroy ⇒ Object

Destroy each row in the dataset by instantiating it and then calling destroy on the resulting model object. This isn't as fast as deleting the dataset, which does a single SQL call, but this runs any destroy hooks on each object in the dataset.

Artist.dataset.destroy
# DELETE FROM artists WHERE (id = 1)
# DELETE FROM artists WHERE (id = 2)
# ...


2073
2074
2075
2076
# File 'lib/sequel/model/base.rb', line 2073

def destroy
  pr = proc{all{|r| r.destroy}.length}
  model.use_transactions ? @db.transaction(:server=>opts[:server], &pr) : pr.call
end

#graph(table, *args, &block) ⇒ Object

Allow Sequel::Model classes to be used as dataset arguments when graphing:

Artist.graph(Album, :artist_id=>id)
# SELECT artists.id, artists.name, albums.id AS albums_id, albums.artist_id, albums.name AS albums_name
# FROM artists LEFT OUTER JOIN albums ON (albums.artist_id = artists.id)


2083
2084
2085
2086
2087
2088
2089
# File 'lib/sequel/model/base.rb', line 2083

def graph(table, *args, &block)
  if table.is_a?(Class) && table < Sequel::Model
    super(table.dataset, *args, &block)
  else
    super
  end
end

#insert_sql(*values) ⇒ Object

Handle Sequel::Model instances when inserting, using the model instance's values for the insert, unless the model instance can be used directly in SQL.

Album.insert(Album.load(:name=>'A'))
# INSERT INTO albums (name) VALUES ('A')


2097
2098
2099
2100
2101
2102
2103
# File 'lib/sequel/model/base.rb', line 2097

def insert_sql(*values)
  if values.size == 1 && (v = values.at(0)).is_a?(Sequel::Model) && !v.respond_to?(:sql_literal_append)
    super(v.to_hash)
  else
    super
  end
end

#join_table(type, table, *args, &block) ⇒ Object

Allow Sequel::Model classes to be used as table name arguments in dataset join methods:

Artist.join(Album, :artist_id=>id)
# SELECT * FROM artists INNER JOIN albums ON (albums.artist_id = artists.id)


2110
2111
2112
2113
2114
2115
2116
2117
2118
2119
2120
# File 'lib/sequel/model/base.rb', line 2110

def join_table(type, table, *args, &block)
  if table.is_a?(Class) && table < Sequel::Model
    if table.dataset.simple_select_all?
      super(type, table.table_name, *args, &block)
    else
      super(type, table.dataset, *args, &block)
    end
  else
    super
  end
end

#last(*a, &block) ⇒ Object

If there is no order already defined on this dataset, order it by the primary key and call last.

Album.last
# SELECT * FROM albums ORDER BY id DESC LIMIT 1


2127
2128
2129
2130
2131
2132
2133
# File 'lib/sequel/model/base.rb', line 2127

def last(*a, &block)
  if opts[:order].nil? && model && (pk = model.primary_key)
    order(*pk).last(*a, &block)
  else
    super
  end
end

#paged_each(*a, &block) ⇒ Object

If there is no order already defined on this dataset, order it by the primary key and call paged_each.

Album.paged_each{|row| ...}
# SELECT * FROM albums ORDER BY id LIMIT 1000 OFFSET 0
# SELECT * FROM albums ORDER BY id LIMIT 1000 OFFSET 1000
# SELECT * FROM albums ORDER BY id LIMIT 1000 OFFSET 2000
# ...


2143
2144
2145
2146
2147
2148
2149
# File 'lib/sequel/model/base.rb', line 2143

def paged_each(*a, &block)
  if opts[:order].nil? && model && (pk = model.primary_key)
    order(*pk).paged_each(*a, &block)
  else
    super
  end
end

#to_hash(key_column = nil, value_column = nil) ⇒ Object

This allows you to call to_hash without any arguments, which will result in a hash with the primary key value being the key and the model object being the value.

Artist.dataset.to_hash # SELECT * FROM artists
# => {1=>#<Artist {:id=>1, ...}>,
#     2=>#<Artist {:id=>2, ...}>,
#     ...}


2159
2160
2161
2162
2163
2164
2165
2166
# File 'lib/sequel/model/base.rb', line 2159

def to_hash(key_column=nil, value_column=nil)
  if key_column
    super
  else
    raise(Sequel::Error, "No primary key for model") unless model && (pk = model.primary_key)
    super(pk, value_column) 
  end
end

#with_pk(pk) ⇒ Object

Given a primary key value, return the first record in the dataset with that primary key value. If no records matches, returns nil.

# Single primary key
Artist.dataset.with_pk(1) # SELECT * FROM artists WHERE (id = 1) LIMIT 1

# Composite primary key
Artist.dataset.with_pk([1, 2]) # SELECT * FROM artists
                             # WHERE ((id1 = 1) AND (id2 = 2)) LIMIT 1


2177
2178
2179
# File 'lib/sequel/model/base.rb', line 2177

def with_pk(pk)
  first(model.qualified_primary_key_hash(pk))
end

#with_pk!(pk) ⇒ Object

Same as with_pk, but raises NoMatchingRow instead of returning nil if no row matches.



2183
2184
2185
# File 'lib/sequel/model/base.rb', line 2183

def with_pk!(pk)
  with_pk(pk) || raise(NoMatchingRow)
end