Module: Sequel::Model::InstanceMethods

Defined in:
lib/sequel/model/base.rb,
lib/sequel/plugins/prepared_statements.rb

Overview

Sequel::Model instance methods that implement basic model functionality.

  • All of the methods in HOOKS and AROUND_HOOKS create instance methods that are called by Sequel when the appropriate action occurs. For example, when destroying a model object, Sequel will call around_destroy, which will call before_destroy, do the destroy, and then call after_destroy.
  • The following instance_methods all call the class method of the same name: columns, db, primary_key, db_schema.
  • All of the methods in BOOLEAN_SETTINGS create attr_writers allowing you to set values for the attribute. It also creates instance getters returning the value of the setting. If the value has not yet been set, it gets the default value from the class by calling the class method of the same name.

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#valuesObject (readonly) Also known as: to_hash

The hash of attribute values. Keys are symbols with the names of the underlying database columns.

Artist.new(:name=>'Bob').values # => {:name=>'Bob'}
Artist[1].values # => {:id=>1, :name=>'Jim', ...}


1126
1127
1128
# File 'lib/sequel/model/base.rb', line 1126

def values
  @values
end

Instance Method Details

#==(obj) ⇒ Object

Alias of eql?



1179
1180
1181
# File 'lib/sequel/model/base.rb', line 1179

def ==(obj)
  eql?(obj)
end

#===(obj) ⇒ Object

If pk is not nil, true only if the objects have the same class and pk. If pk is nil, false.

Artist[1] === Artist[1] # true
Artist.new === Artist.new # false
Artist[1].set(:name=>'Bob') == Artist[1] # => true


1189
1190
1191
# File 'lib/sequel/model/base.rb', line 1189

def ===(obj)
  pk.nil? ? false : (obj.class == model) && (obj.pk == pk)
end

#[](column) ⇒ Object

Returns value of the column's attribute.

Artist[1][:id] #=> 1


1154
1155
1156
# File 'lib/sequel/model/base.rb', line 1154

def [](column)
  @values[column]
end

#[]=(column, value) ⇒ Object

Sets the value for the given column. If typecasting is enabled for this object, typecast the value based on the column's type. If this is a new record or the typecasted value isn't the same as the current value for the column, mark the column as changed.

a = Artist.new
a[:name] = 'Bob'
a.values #=> {:name=>'Bob'}


1166
1167
1168
1169
1170
1171
1172
1173
1174
1175
1176
# File 'lib/sequel/model/base.rb', line 1166

def []=(column, value)
  # If it is new, it doesn't have a value yet, so we should
  # definitely set the new value.
  # If the column isn't in @values, we can't assume it is
  # NULL in the database, so assume it has changed.
  v = typecast_value(column, value)
  vals = @values
  if new? || !vals.include?(column) || v != (c = vals[column]) || v.class != c.class
    change_column_value(column, v)
  end
end

#autoincrementing_primary_keyObject

The autoincrementing primary key for this model object. Should be overridden if you have a composite primary key with one part of it being autoincrementing.



1204
1205
1206
# File 'lib/sequel/model/base.rb', line 1204

def autoincrementing_primary_key
  primary_key
end

#changed_columnsObject

The columns that have been updated. This isn't completely accurate, as it could contain columns whose values have not changed.

a = Artist[1]
a.changed_columns # => []
a.name = 'Bob'
a.changed_columns # => [:name]


1215
1216
1217
# File 'lib/sequel/model/base.rb', line 1215

def changed_columns
  @changed_columns ||= []
end

#cloneObject

Similar to Model#dup, but copies frozen status to returned object if current object is frozen.



1221
1222
1223
1224
1225
# File 'lib/sequel/model/base.rb', line 1221

def clone
  o = dup
  o.freeze if frozen?
  o
end

#deleteObject

Deletes and returns self. Does not run destroy hooks. Look into using destroy instead.

Artist[1].delete # DELETE FROM artists WHERE (id = 1)
# => #<Artist {:id=>1, ...}>

Raises:



1232
1233
1234
1235
1236
# File 'lib/sequel/model/base.rb', line 1232

def delete
  raise Sequel::Error, "can't delete frozen object" if frozen?
  _delete
  self
end

#destroy(opts = OPTS) ⇒ Object

Like delete but runs hooks before and after delete. If before_destroy returns false, returns false without deleting the object from the database. Otherwise, deletes the item from the database and returns self. Uses a transaction if use_transactions is true or if the :transaction option is given and true.

Artist[1].destroy # BEGIN; DELETE FROM artists WHERE (id = 1); COMMIT;
# => #<Artist {:id=>1, ...}>

Raises:



1247
1248
1249
1250
# File 'lib/sequel/model/base.rb', line 1247

def destroy(opts = OPTS)
  raise Sequel::Error, "can't destroy frozen object" if frozen?
  checked_save_failure(opts){checked_transaction(opts){_destroy(opts)}}
end

#dupObject

Produce a shallow copy of the object, similar to Object#dup.



1253
1254
1255
1256
1257
1258
1259
1260
1261
1262
# File 'lib/sequel/model/base.rb', line 1253

def dup
  s = self
  super.instance_eval do
    @values = s.values.dup
    @changed_columns = s.changed_columns.dup
    @errors = s.errors.dup
    @this = s.this.dup if !new? && model.primary_key
    self
  end
end

#each(&block) ⇒ Object

Iterates through all of the current values using each.

Album.each{|k, v| puts "#{k} => #v"}

id => 1

name => 'Bob'



1269
1270
1271
# File 'lib/sequel/model/base.rb', line 1269

def each(&block)
  @values.each(&block)
end

#eql?(obj) ⇒ Boolean

Compares model instances by values.

Artist[1] == Artist[1] # => true
Artist.new == Artist.new # => true
Artist[1].set(:name=>'Bob') == Artist[1] # => false

Returns:

  • (Boolean)


1278
1279
1280
# File 'lib/sequel/model/base.rb', line 1278

def eql?(obj)
  (obj.class == model) && (obj.values == @values)
end

#errorsObject

Returns the validation errors associated with this object. See Errors.



1284
1285
1286
# File 'lib/sequel/model/base.rb', line 1284

def errors
  @errors ||= errors_class.new
end

#exists?Boolean

Returns true when current instance exists, false otherwise. Generally an object that isn't new will exist unless it has been deleted. Uses a database query to check for existence, unless the model object is new, in which case this is always false.

Artist[1].exists? # SELECT 1 FROM artists WHERE (id = 1)
# => true
Artist.new.exists?
# => false

Returns:

  • (Boolean)


1298
1299
1300
# File 'lib/sequel/model/base.rb', line 1298

def exists?
  new? ? false : !this.get(SQL::AliasedExpression.new(1, :one)).nil?
end

#extend(mod) ⇒ Object

Ignore the model's setter method cache when this instances extends a module, as the module may contain setter methods.



1304
1305
1306
1307
# File 'lib/sequel/model/base.rb', line 1304

def extend(mod)
  @singleton_setter_added = true
  super
end

#freezeObject

Freeze the object in such a way that it is still usable but not modifiable. Once an object is frozen, you cannot modify it's values, changed_columns, errors, or dataset.



1312
1313
1314
1315
1316
1317
1318
1319
1320
# File 'lib/sequel/model/base.rb', line 1312

def freeze
  values.freeze
  changed_columns.freeze
  errors
  validate
  errors.freeze
  this.freeze if !new? && model.primary_key
  super
end

#hashObject

Value that should be unique for objects with the same class and pk (if pk is not nil), or the same class and values (if pk is nil).

Artist[1].hash == Artist[1].hash # true
Artist[1].set(:name=>'Bob').hash == Artist[1].hash # true
Artist.new.hash == Artist.new.hash # true
Artist.new(:name=>'Bob').hash == Artist.new.hash # false


1329
1330
1331
1332
1333
1334
1335
1336
1337
1338
# File 'lib/sequel/model/base.rb', line 1329

def hash
  case primary_key
  when Array
    [model, !pk.all? ? @values : pk].hash
  when Symbol
    [model, pk.nil? ? @values : pk].hash
  else
    [model, @values].hash
  end
end

#idObject

Returns value for the :id attribute, even if the primary key is not id. To get the primary key value, use pk.

Artist[1].id # => 1


1344
1345
1346
# File 'lib/sequel/model/base.rb', line 1344

def id
  @values[:id]
end

#initialize(values = {}) {|_self| ... } ⇒ Object

Creates new instance and passes the given values to set. If a block is given, yield the instance to the block unless from_db is true.

Arguments:

values

should be a hash to pass to set.

from_db

only for backwards compatibility, forget it exists.

Artist.new(:name=>'Bob')

Artist.new do |a|
a.name = 'Bob'
end

Yields:

  • (_self)

Yield Parameters:



1142
1143
1144
1145
1146
1147
1148
1149
# File 'lib/sequel/model/base.rb', line 1142

def initialize(values = {})
  @values = {}
  @new = true
  @modified = true
  initialize_set(values)
  changed_columns.clear 
  yield self if block_given?
end

#inspectObject

Returns a string representation of the model instance including the class name and values.



1350
1351
1352
# File 'lib/sequel/model/base.rb', line 1350

def inspect
  "#<#{model.name} @values=#{inspect_values}>"
end

#keysObject

Returns the keys in values. May not include all column names.

Artist.new.keys # => []
Artist.new(:name=>'Bob').keys # => [:name]
Artist[1].keys # => [:id, :name]


1359
1360
1361
# File 'lib/sequel/model/base.rb', line 1359

def keys
  @values.keys
end

#lock!Object

Refresh this record using for_update unless this is a new record. Returns self. This can be used to make sure no other process is updating the record at the same time.

a = Artist[1]
Artist.db.transaction do
a.lock!
a.update(...)
end


1372
1373
1374
1375
# File 'lib/sequel/model/base.rb', line 1372

def lock!
  _refresh(this.for_update) unless new?
  self
end

#marshallable!Object

Remove elements of the model object that make marshalling fail. Returns self.

a = Artist[1]
a.marshallable!
Marshal.dump(a)


1382
1383
1384
1385
# File 'lib/sequel/model/base.rb', line 1382

def marshallable!
  @this = nil
  self
end

#modified!(column = nil) ⇒ Object

Explicitly mark the object as modified, so +save_changes+/+update+ will run callbacks even if no columns have changed.

a = Artist[1]
a.save_changes # No callbacks run, as no changes
a.modified!
a.save_changes # Callbacks run, even though no changes made

If a column is given, specifically marked that column as modified, so that +save_changes+/+update+ will include that column in the update. This should be used if you plan on mutating the column value instead of assigning a new column value:

a.modified!(:name)
a.name.gsub!(/[aeou]/, 'i')


1402
1403
1404
1405
1406
1407
# File 'lib/sequel/model/base.rb', line 1402

def modified!(column=nil)
  if column && !changed_columns.include?(column)
    changed_columns << column
  end
  @modified = true
end

#modified?(column = nil) ⇒ Boolean

Whether this object has been modified since last saved, used by save_changes to determine whether changes should be saved. New values are always considered modified.

a = Artist[1]
a.modified? # => false
a.set(:name=>'Jim')
a.modified? # => true

If a column is given, specifically check if the given column has been modified:

a.modified?(:num_albums) # => false
a.num_albums = 10
a.modified?(:num_albums) # => true

Returns:

  • (Boolean)


1424
1425
1426
1427
1428
1429
1430
# File 'lib/sequel/model/base.rb', line 1424

def modified?(column=nil)
  if column
    changed_columns.include?(column)
  else
    @modified || !changed_columns.empty?
  end
end

#new?Boolean

Returns true if the current instance represents a new record.

Artist.new.new? # => true
Artist[1].new? # => false

Returns:

  • (Boolean)


1436
1437
1438
# File 'lib/sequel/model/base.rb', line 1436

def new?
  defined?(@new) ? @new : (@new = false)
end

#pkObject

Returns the primary key value identifying the model instance. Raises an Error if this model does not have a primary key. If the model has a composite primary key, returns an array of values.

Artist[1].pk # => 1
Artist[[1, 2]].pk # => [1, 2]

Raises:



1446
1447
1448
1449
1450
1451
1452
1453
1454
# File 'lib/sequel/model/base.rb', line 1446

def pk
  raise(Error, "No primary key is associated with this model") unless key = primary_key
  if key.is_a?(Array)
    vals = @values
    key.map{|k| vals[k]}
  else
    @values[key]
  end
end

#pk_hashObject

Returns a hash mapping the receivers primary key column(s) to their values.

Artist[1].pk_hash # => {:id=>1}
Artist[[1, 2]].pk_hash # => {:id1=>1, :id2=>2}


1460
1461
1462
# File 'lib/sequel/model/base.rb', line 1460

def pk_hash
  model.primary_key_hash(pk)
end

#refreshObject

Reloads attributes from database and returns self. Also clears all changed_columns information. Raises an Error if the record no longer exists in the database.

a = Artist[1]
a.name = 'Jim'
a.refresh
a.name # => 'Bob'

Raises:



1472
1473
1474
1475
1476
# File 'lib/sequel/model/base.rb', line 1472

def refresh
  raise Sequel::Error, "can't refresh frozen object" if frozen?
  _refresh(this)
  self
end

#reloadObject

Alias of refresh, but not aliased directly to make overriding in a plugin easier.



1479
1480
1481
# File 'lib/sequel/model/base.rb', line 1479

def reload
  refresh
end

#save(opts = OPTS) ⇒ Object

Creates or updates the record, after making sure the record is valid and before hooks execute successfully. Fails if:

  • the record is not valid, or
  • before_save returns false, or
  • the record is new and before_create returns false, or
  • the record is not new and before_update returns false.

If save fails and either raise_on_save_failure or the :raise_on_failure option is true, it raises ValidationFailed or HookFailed. Otherwise it returns nil.

If it succeeds, it returns self.

You can provide an optional list of columns to update, in which case it only updates those columns, or a options hash.

Takes the following options:

:changed :: save all changed columns, instead of all columns or the columns given :columns :: array of specific columns that should be saved. :raise_on_failure :: set to true or false to override the current raise_on_save_failure setting :server :: set the server/shard on the object before saving, and use that server/shard in any transaction. :transaction :: set to true or false to override the current use_transactions setting :validate :: set to false to skip validation

Raises:



1511
1512
1513
1514
1515
1516
1517
1518
1519
1520
1521
# File 'lib/sequel/model/base.rb', line 1511

def save(opts=OPTS)
  raise Sequel::Error, "can't save frozen object" if frozen?
  set_server(opts[:server]) if opts[:server] 
  if opts[:validate] != false
    unless checked_save_failure(opts){_valid?(true, opts)}
      raise(ValidationFailed.new(self)) if raise_on_failure?(opts)
      return
    end
  end
  checked_save_failure(opts){checked_transaction(opts){_save(opts)}}
end

#save_changes(opts = OPTS) ⇒ Object

Saves only changed columns if the object has been modified. If the object has not been modified, returns nil. If unable to save, returns false unless raise_on_save_failure is true.

a = Artist[1]
a.save_changes # => nil
a.name = 'Jim'
a.save_changes # UPDATE artists SET name = 'Bob' WHERE (id = 1)
# => #<Artist {:id=>1, :name=>'Jim', ...}


1532
1533
1534
# File 'lib/sequel/model/base.rb', line 1532

def save_changes(opts=OPTS)
  save(opts.merge(:changed=>true)) || false if modified? 
end

#set(hash) ⇒ Object

Updates the instance with the supplied values with support for virtual attributes, raising an exception if a value is used that doesn't have a setter method (or ignoring it if strict_param_setting = false). Does not save the record.

artist.set(:name=>'Jim')
artist.name # => 'Jim'


1543
1544
1545
# File 'lib/sequel/model/base.rb', line 1543

def set(hash)
  set_restricted(hash, :default)
end

#set_all(hash) ⇒ Object

Set all values using the entries in the hash, ignoring any setting of allowed_columns in the model.

Artist.set_allowed_columns(:num_albums)
artist.set_all(:name=>'Jim')
artist.name # => 'Jim'


1553
1554
1555
# File 'lib/sequel/model/base.rb', line 1553

def set_all(hash)
  set_restricted(hash, :all)
end

#set_fields(hash, fields, opts = nil) ⇒ Object

For each of the fields in the given array fields, call the setter method with the value of that hash entry for the field. Returns self.

You can provide an options hash, with the following options currently respected: :missing :: Can be set to :skip to skip missing entries or :raise to raise an Error for missing entries. The default behavior is not to check for missing entries, in which case the default value is used. To be friendly with most web frameworks, the missing check will also check for the string version of the argument in the hash if given a symbol.

Examples:

artist.set_fields({:name=>'Jim'}, [:name])
artist.name # => 'Jim'

artist.set_fields({:hometown=>'LA'}, [:name])
artist.name # => nil
artist.hometown # => 'Sac'

artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:skip)
artist.name # => 'Jim'

artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:raise)
# Sequel::Error raised


1583
1584
1585
1586
1587
1588
1589
1590
1591
1592
1593
1594
1595
1596
1597
1598
1599
1600
1601
1602
1603
1604
1605
1606
1607
1608
1609
1610
1611
1612
1613
# File 'lib/sequel/model/base.rb', line 1583

def set_fields(hash, fields, opts=nil)
  opts = if opts
    model.default_set_fields_options.merge(opts)
  else
    model.default_set_fields_options
  end

  case opts[:missing]
  when :skip
    fields.each do |f|
      if hash.has_key?(f) 
        send("#{f}=", hash[f])
      elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s)
        send("#{sf}=", hash[sf])
      end
    end
  when :raise
    fields.each do |f|
      if hash.has_key?(f)
        send("#{f}=", hash[f])
      elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s)
        send("#{sf}=", hash[sf])
      else
        raise(Sequel::Error, "missing field in hash: #{f.inspect} not in #{hash.inspect}")
      end
    end
  else
    fields.each{|f| send("#{f}=", hash[f])}
  end
  self
end

#set_only(hash, *only) ⇒ Object

Set the values using the entries in the hash, only if the key is included in only. It may be a better idea to use set_fields instead of this method.

artist.set_only({:name=>'Jim'}, :name)
artist.name # => 'Jim'

artist.set_only({:hometown=>'LA'}, :name) # Raise Error


1623
1624
1625
# File 'lib/sequel/model/base.rb', line 1623

def set_only(hash, *only)
  set_restricted(hash, only.flatten)
end

#set_server(s) ⇒ Object

Set the shard that this object is tied to. Returns self.



1628
1629
1630
1631
1632
# File 'lib/sequel/model/base.rb', line 1628

def set_server(s)
  @server = s
  @this.opts[:server] = s if @this
  self
end

#singleton_method_added(meth) ⇒ Object

Clear the setter_methods cache when a method is added



1635
1636
1637
1638
# File 'lib/sequel/model/base.rb', line 1635

def singleton_method_added(meth)
  @singleton_setter_added = true if meth.to_s =~ SETTER_METHOD_REGEXP
  super
end

#thisObject

Returns (naked) dataset that should return only this instance.

Artist[1].this
# SELECT * FROM artists WHERE (id = 1) LIMIT 1


1644
1645
1646
# File 'lib/sequel/model/base.rb', line 1644

def this
  @this ||= use_server(model.instance_dataset.filter(pk_hash))
end

#update(hash) ⇒ Object

Runs #set with the passed hash and then runs save_changes.

artist.update(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)


1651
1652
1653
# File 'lib/sequel/model/base.rb', line 1651

def update(hash)
  update_restricted(hash, :default)
end

#update_all(hash) ⇒ Object

Update all values using the entries in the hash, ignoring any setting of allowed_columns in the model.

Artist.set_allowed_columns(:num_albums)
artist.update_all(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)


1660
1661
1662
# File 'lib/sequel/model/base.rb', line 1660

def update_all(hash)
  update_restricted(hash, :all)
end

#update_fields(hash, fields, opts = nil) ⇒ Object

Update the instances values by calling set_fields with the arguments, then saves any changes to the record. Returns self.

artist.update_fields({:name=>'Jim'}, [:name])
# UPDATE artists SET name = 'Jim' WHERE (id = 1)

artist.update_fields({:hometown=>'LA'}, [:name])
# UPDATE artists SET name = NULL WHERE (id = 1)


1672
1673
1674
1675
# File 'lib/sequel/model/base.rb', line 1672

def update_fields(hash, fields, opts=nil)
  set_fields(hash, fields, opts)
  save_changes
end

#update_only(hash, *only) ⇒ Object

Update the values using the entries in the hash, only if the key is included in only. It may be a better idea to use update_fields instead of this method.

artist.update_only({:name=>'Jim'}, :name)
# UPDATE artists SET name = 'Jim' WHERE (id = 1)

artist.update_only({:hometown=>'LA'}, :name) # Raise Error


1685
1686
1687
# File 'lib/sequel/model/base.rb', line 1685

def update_only(hash, *only)
  update_restricted(hash, only.flatten)
end

#use_prepared_statements_for?(type) ⇒ Boolean

Whether prepared statements should be used for the given type of query (:insert, :insert_select, :refresh, :update, or :delete). True by default, can be overridden in other plugins to disallow prepared statements for specific types of queries.

Returns:

  • (Boolean)


8
9
10
# File 'lib/sequel/plugins/prepared_statements.rb', line 8

def use_prepared_statements_for?(type)
  true
end

#valid?(opts = OPTS) ⇒ Boolean

Validates the object and returns true if no errors are reported.

artist(:name=>'Valid').valid? # => true
artist(:name=>'Invalid').valid? # => false
artist.errors.full_messages # => ['name cannot be Invalid']

Returns:

  • (Boolean)


1703
1704
1705
# File 'lib/sequel/model/base.rb', line 1703

def valid?(opts = OPTS)
  _valid?(false, opts)
end

#validateObject

Validates the object. If the object is invalid, errors should be added to the errors attribute. By default, does nothing, as all models are valid by default. See the "Model Validations" guide. for details about validation. Should not be called directly by user code, call valid? instead to check if an object is valid.



1695
1696
# File 'lib/sequel/model/base.rb', line 1695

def validate
end