Module: Sequel::Model::InstanceMethods

Defined in:
lib/sequel/model/base.rb

Overview

Sequel::Model instance methods that implement basic model functionality.

  • All of the methods in HOOKS and AROUND_HOOKS create instance methods that are called by Sequel when the appropriate action occurs. For example, when destroying a model object, Sequel will call around_destroy, which will call before_destroy, do the destroy, and then call after_destroy.

  • The following instance_methods all call the class method of the same name: columns, db, primary_key, db_schema.

  • All of the methods in BOOLEAN_SETTINGS create attr_writers allowing you to set values for the attribute. It also creates instance getters returning the value of the setting. If the value has not yet been set, it gets the default value from the class by calling the class method of the same name.

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#valuesObject (readonly) Also known as: to_hash, _insert_values

The hash of attribute values. Keys are symbols with the names of the underlying database columns. The returned hash is a reference to the receiver’s values hash, and modifying it will also modify the receiver’s values.

Artist.new(:name=>'Bob').values # => {:name=>'Bob'}
Artist[1].values # => {:id=>1, :name=>'Jim', ...}


1347
1348
1349
# File 'lib/sequel/model/base.rb', line 1347

def values
  @values
end

Instance Method Details

#==(obj) ⇒ Object

Alias of eql?



1412
1413
1414
# File 'lib/sequel/model/base.rb', line 1412

def ==(obj)
  eql?(obj)
end

#===(obj) ⇒ Object

If pk is not nil, true only if the objects have the same class and pk. If pk is nil, false.

Artist[1] === Artist[1] # true
Artist.new === Artist.new # false
Artist[1].set(:name=>'Bob') == Artist[1] # => true


1422
1423
1424
# File 'lib/sequel/model/base.rb', line 1422

def ===(obj)
  pk.nil? ? false : (obj.class == model) && (obj.pk == pk)
end

#[](column) ⇒ Object

Returns value of the column’s attribute.

Artist[1][:id] #=> 1


1387
1388
1389
# File 'lib/sequel/model/base.rb', line 1387

def [](column)
  @values[column]
end

#[]=(column, value) ⇒ Object

Sets the value for the given column. If typecasting is enabled for this object, typecast the value based on the column’s type. If this is a new record or the typecasted value isn’t the same as the current value for the column, mark the column as changed.

a = Artist.new
a[:name] = 'Bob'
a.values #=> {:name=>'Bob'}


1399
1400
1401
1402
1403
1404
1405
1406
1407
1408
1409
# File 'lib/sequel/model/base.rb', line 1399

def []=(column, value)
  # If it is new, it doesn't have a value yet, so we should
  # definitely set the new value.
  # If the column isn't in @values, we can't assume it is
  # NULL in the database, so assume it has changed.
  v = typecast_value(column, value)
  vals = @values
  if new? || !vals.include?(column) || v != (c = vals[column]) || v.class != c.class
    change_column_value(column, v)
  end
end

#autoincrementing_primary_keyObject

The autoincrementing primary key for this model object. Should be overridden if you have a composite primary key with one part of it being autoincrementing.



1437
1438
1439
# File 'lib/sequel/model/base.rb', line 1437

def autoincrementing_primary_key
  primary_key
end

#cancel_action(msg = nil) ⇒ Object

Cancel the current action. Should be called in before hooks to halt the processing of the action. If a msg argument is given and the model instance is configured to raise exceptions on failure, sets the message to use for the raised HookFailed exception.



1445
1446
1447
# File 'lib/sequel/model/base.rb', line 1445

def cancel_action(msg=nil)
  raise_hook_failure(msg)
end

#changed_columnsObject

The columns that have been updated. This isn’t completely accurate, as it could contain columns whose values have not changed.

a = Artist[1]
a.changed_columns # => []
a.name = 'Bob'
a.changed_columns # => [:name]


1456
1457
1458
# File 'lib/sequel/model/base.rb', line 1456

def changed_columns
  @changed_columns ||= []
end

#cloneObject

:nocov: Ruby 1.8 doesn’t support initialize_clone, so override clone to dup and freeze.



2336
2337
2338
2339
2340
# File 'lib/sequel/model/base.rb', line 2336

def clone
  o = dup
  o.freeze if frozen?
  o
end

#deleteObject

Deletes and returns self. Does not run destroy hooks. Look into using destroy instead.

Artist[1].delete # DELETE FROM artists WHERE (id = 1)
# => #<Artist {:id=>1, ...}>

Raises:



1465
1466
1467
1468
1469
# File 'lib/sequel/model/base.rb', line 1465

def delete
  raise Sequel::Error, "can't delete frozen object" if frozen?
  _delete
  self
end

#destroy(opts = OPTS) ⇒ Object

Like delete but runs hooks before and after delete. If before_destroy returns false, returns false without deleting the object from the database. Otherwise, deletes the item from the database and returns self. Uses a transaction if use_transactions is true or if the :transaction option is given and true.

Artist[1].destroy # BEGIN; DELETE FROM artists WHERE (id = 1); COMMIT;
# => #<Artist {:id=>1, ...}>

Raises:



1480
1481
1482
1483
# File 'lib/sequel/model/base.rb', line 1480

def destroy(opts = OPTS)
  raise Sequel::Error, "can't destroy frozen object" if frozen?
  checked_save_failure(opts){checked_transaction(opts){_destroy(opts)}}
end

#each(&block) ⇒ Object

Iterates through all of the current values using each.

Album[1].each{|k, v| puts "#{k} => #{v}"}
# id => 1
# name => 'Bob'


1490
1491
1492
# File 'lib/sequel/model/base.rb', line 1490

def each(&block)
  @values.each(&block)
end

#eql?(obj) ⇒ Boolean

Compares model instances by values.

Artist[1] == Artist[1] # => true
Artist.new == Artist.new # => true
Artist[1].set(:name=>'Bob') == Artist[1] # => false

Returns:

  • (Boolean)


1499
1500
1501
# File 'lib/sequel/model/base.rb', line 1499

def eql?(obj)
  (obj.class == model) && (obj.values == @values)
end

#errorsObject

Returns the validation errors associated with this object. See Errors.



1505
1506
1507
# File 'lib/sequel/model/base.rb', line 1505

def errors
  @errors ||= errors_class.new
end

#exists?Boolean

Returns true when current instance exists, false otherwise. Generally an object that isn’t new will exist unless it has been deleted. Uses a database query to check for existence, unless the model object is new, in which case this is always false.

Artist[1].exists? # SELECT 1 FROM artists WHERE (id = 1)
# => true
Artist.new.exists?
# => false

Returns:

  • (Boolean)


1519
1520
1521
# File 'lib/sequel/model/base.rb', line 1519

def exists?
  new? ? false : !this.get(SQL::AliasedExpression.new(1, :one)).nil?
end

#extend(mod) ⇒ Object

Ignore the model’s setter method cache when this instances extends a module, as the module may contain setter methods.



1525
1526
1527
1528
# File 'lib/sequel/model/base.rb', line 1525

def extend(mod)
  @singleton_setter_added = true
  super
end

#freezeObject

Freeze the object in such a way that it is still usable but not modifiable. Once an object is frozen, you cannot modify it’s values, changed_columns, errors, or dataset.



1533
1534
1535
1536
1537
1538
1539
1540
1541
1542
# File 'lib/sequel/model/base.rb', line 1533

def freeze
  values.freeze
  changed_columns.freeze
  unless errors.frozen?
    validate
    errors.freeze
  end
  this.freeze if !new? && model.primary_key
  super
end

#hashObject

Value that should be unique for objects with the same class and pk (if pk is not nil), or the same class and values (if pk is nil).

Artist[1].hash == Artist[1].hash # true
Artist[1].set(:name=>'Bob').hash == Artist[1].hash # true
Artist.new.hash == Artist.new.hash # true
Artist.new(:name=>'Bob').hash == Artist.new.hash # false


1551
1552
1553
1554
1555
1556
1557
1558
1559
1560
# File 'lib/sequel/model/base.rb', line 1551

def hash
  case primary_key
  when Array
    [model, !pk.all? ? @values : pk].hash
  when Symbol
    [model, pk.nil? ? @values : pk].hash
  else
    [model, @values].hash
  end
end

#idObject

Returns value for the :id attribute, even if the primary key is not id. To get the primary key value, use pk.

Artist[1].id # => 1


1566
1567
1568
# File 'lib/sequel/model/base.rb', line 1566

def id
  @values[:id]
end

#initialize(values = {}) {|_self| ... } ⇒ Object

Creates new instance and passes the given values to set. If a block is given, yield the instance to the block.

Arguments:

values

should be a hash to pass to set.

Artist.new(:name=>'Bob')

Artist.new do |a|
  a.name = 'Bob'
end

Yields:

  • (_self)

Yield Parameters:



1375
1376
1377
1378
1379
1380
1381
1382
# File 'lib/sequel/model/base.rb', line 1375

def initialize(values = {})
  @values = {}
  @new = true
  @modified = true
  initialize_set(values)
  changed_columns.clear 
  yield self if block_given?
end

#inspectObject

Returns a string representation of the model instance including the class name and values.



1572
1573
1574
# File 'lib/sequel/model/base.rb', line 1572

def inspect
  "#<#{model.name} @values=#{inspect_values}>"
end

#keysObject

Returns the keys in values. May not include all column names.

Artist.new.keys # => []
Artist.new(:name=>'Bob').keys # => [:name]
Artist[1].keys # => [:id, :name]


1581
1582
1583
# File 'lib/sequel/model/base.rb', line 1581

def keys
  @values.keys
end

#lock!(style = :update) ⇒ Object

Refresh this record using for_update (by default, or the specified style when given) unless this is a new record. Returns self. This can be used to make sure no other process is updating the record at the same time.

If style is a string, it will be used directly. You should never pass a string to this method that is derived from user input, as that can lead to SQL injection.

A symbol may be used for database independent locking behavior, but all supported symbols have separate methods (e.g. for_update).

 a = Artist[1]
 Artist.db.transaction do
   a.lock!
   a.update(:name=>'A')
 end

a = Artist[2]
Artist.db.transaction do
  a.lock!('FOR NO KEY UPDATE')
  a.update(:name=>'B')
end


1608
1609
1610
1611
# File 'lib/sequel/model/base.rb', line 1608

def lock!(style=:update)
  _refresh(this.lock_style(style)) unless new?
  self
end

#marshallable!Object

Remove elements of the model object that make marshalling fail. Returns self.

a = Artist[1]
a.marshallable!
Marshal.dump(a)


1618
1619
1620
1621
# File 'lib/sequel/model/base.rb', line 1618

def marshallable!
  @this = nil
  self
end

#modified!(column = nil) ⇒ Object

Explicitly mark the object as modified, so save_changes/update will run callbacks even if no columns have changed.

a = Artist[1]
a.save_changes # No callbacks run, as no changes
a.modified!
a.save_changes # Callbacks run, even though no changes made

If a column is given, specifically marked that column as modified, so that save_changes/update will include that column in the update. This should be used if you plan on mutating the column value instead of assigning a new column value:

a.modified!(:name)
a.name.gsub!(/[aeou]/, 'i')


1638
1639
1640
1641
1642
1643
# File 'lib/sequel/model/base.rb', line 1638

def modified!(column=nil)
  if column && !changed_columns.include?(column)
    changed_columns << column
  end
  @modified = true
end

#modified?(column = nil) ⇒ Boolean

Whether this object has been modified since last saved, used by save_changes to determine whether changes should be saved. New values are always considered modified.

a = Artist[1]
a.modified? # => false
a.set(:name=>'Jim')
a.modified? # => true

If a column is given, specifically check if the given column has been modified:

a.modified?(:num_albums) # => false
a.num_albums = 10
a.modified?(:num_albums) # => true

Returns:

  • (Boolean)


1660
1661
1662
1663
1664
1665
1666
# File 'lib/sequel/model/base.rb', line 1660

def modified?(column=nil)
  if column
    changed_columns.include?(column)
  else
    @modified || !changed_columns.empty?
  end
end

#new?Boolean

Returns true if the current instance represents a new record.

Artist.new.new? # => true
Artist[1].new? # => false

Returns:

  • (Boolean)


1672
1673
1674
# File 'lib/sequel/model/base.rb', line 1672

def new?
  defined?(@new) ? @new : (@new = false)
end

#pkObject

Returns the primary key value identifying the model instance. Raises an Error if this model does not have a primary key. If the model has a composite primary key, returns an array of values.

Artist[1].pk # => 1
Artist[[1, 2]].pk # => [1, 2]

Raises:



1682
1683
1684
1685
1686
1687
1688
1689
1690
# File 'lib/sequel/model/base.rb', line 1682

def pk
  raise(Error, "No primary key is associated with this model") unless key = primary_key
  if key.is_a?(Array)
    vals = @values
    key.map{|k| vals[k]}
  else
    @values[key]
  end
end

#pk_hashObject

Returns a hash mapping the receivers primary key column(s) to their values.

Artist[1].pk_hash # => {:id=>1}
Artist[[1, 2]].pk_hash # => {:id1=>1, :id2=>2}


1696
1697
1698
# File 'lib/sequel/model/base.rb', line 1696

def pk_hash
  model.primary_key_hash(pk)
end

#qualified_pk_hash(qualifier = model.table_name) ⇒ Object

Returns a hash mapping the receivers qualified primary key column(s) to their values.

Artist[1].qualified_pk_hash
# => {Sequel[:artists][:id]=>1}
Artist[[1, 2]].qualified_pk_hash
# => {Sequel[:artists][:id1]=>1, Sequel[:artists][:id2]=>2}


1706
1707
1708
# File 'lib/sequel/model/base.rb', line 1706

def qualified_pk_hash(qualifier=model.table_name)
  model.qualified_primary_key_hash(pk, qualifier)
end

#refreshObject

Reloads attributes from database and returns self. Also clears all changed_columns information. Raises an Error if the record no longer exists in the database.

a = Artist[1]
a.name = 'Jim'
a.refresh
a.name # => 'Bob'

Raises:



1718
1719
1720
1721
1722
# File 'lib/sequel/model/base.rb', line 1718

def refresh
  raise Sequel::Error, "can't refresh frozen object" if frozen?
  _refresh(this)
  self
end

#reloadObject

Alias of refresh, but not aliased directly to make overriding in a plugin easier.



1725
1726
1727
# File 'lib/sequel/model/base.rb', line 1725

def reload
  refresh
end

#save(opts = OPTS) ⇒ Object

Creates or updates the record, after making sure the record is valid and before hooks execute successfully. Fails if:

  • the record is not valid, or

  • before_save returns false, or

  • the record is new and before_create returns false, or

  • the record is not new and before_update returns false.

If save fails and either raise_on_save_failure or the :raise_on_failure option is true, it raises ValidationFailed or HookFailed. Otherwise it returns nil.

If it succeeds, it returns self.

You can provide an optional list of columns to update, in which case it only updates those columns, or a options hash.

Takes the following options:

:changed

save all changed columns, instead of all columns or the columns given

:columns

array of specific columns that should be saved.

:raise_on_failure

set to true or false to override the current raise_on_save_failure setting

:server

set the server/shard on the object before saving, and use that server/shard in any transaction.

:transaction

set to true or false to override the current use_transactions setting

:validate

set to false to skip validation

Raises:



1757
1758
1759
1760
1761
1762
1763
1764
1765
1766
1767
1768
# File 'lib/sequel/model/base.rb', line 1757

def save(opts=OPTS)
  raise Sequel::Error, "can't save frozen object" if frozen?
  set_server(opts[:server]) if opts[:server] 
  _before_validation
  if opts[:validate] != false # SEQUEL5: Remove if
    unless checked_save_failure(opts){_valid?(opts)}
      raise(ValidationFailed.new(self)) if raise_on_failure?(opts)
      return
    end
  end
  checked_save_failure(opts){checked_transaction(opts){_save(opts)}}
end

#save_changes(opts = OPTS) ⇒ Object

Saves only changed columns if the object has been modified. If the object has not been modified, returns nil. If unable to save, returns false unless raise_on_save_failure is true.

a = Artist[1]
a.save_changes # => nil
a.name = 'Jim'
a.save_changes # UPDATE artists SET name = 'Bob' WHERE (id = 1)
# => #<Artist {:id=>1, :name=>'Jim', ...}


1779
1780
1781
# File 'lib/sequel/model/base.rb', line 1779

def save_changes(opts=OPTS)
  save(Hash[opts].merge!(:changed=>true)) || false if modified? 
end

#set(hash) ⇒ Object

Updates the instance with the supplied values with support for virtual attributes, raising an exception if a value is used that doesn’t have a setter method (or ignoring it if strict_param_setting = false). Does not save the record.

artist.set(:name=>'Jim')
artist.name # => 'Jim'


1790
1791
1792
# File 'lib/sequel/model/base.rb', line 1790

def set(hash)
  set_restricted(hash, :default)
end

#set_all(hash) ⇒ Object

Set all values using the entries in the hash, ignoring any setting of allowed_columns in the model.

Artist.set_allowed_columns(:num_albums)
artist.set_all(:name=>'Jim')
artist.name # => 'Jim'


1800
1801
1802
1803
# File 'lib/sequel/model/base.rb', line 1800

def set_all(hash)
  Sequel::Deprecation.deprecate("Sequel::Model#set_all", "Switch to set or load the whitelist_security plugin into the model class")
  set_restricted(hash, :all)
end

#set_fields(hash, fields, opts = nil) ⇒ Object

For each of the fields in the given array fields, call the setter method with the value of that hash entry for the field. Returns self.

You can provide an options hash, with the following options currently respected:

:missing

Can be set to :skip to skip missing entries or :raise to raise an Error for missing entries. The default behavior is not to check for missing entries, in which case the default value is used. To be friendly with most web frameworks, the missing check will also check for the string version of the argument in the hash if given a symbol.

Examples:

artist.set_fields({:name=>'Jim'}, [:name])
artist.name # => 'Jim'

artist.set_fields({:hometown=>'LA'}, [:name])
artist.name # => nil
artist.hometown # => 'Sac'

artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:skip)
artist.name # => 'Jim'

artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:raise)
# Sequel::Error raised


1831
1832
1833
1834
1835
1836
1837
1838
1839
1840
1841
1842
1843
1844
1845
1846
1847
1848
1849
1850
1851
1852
1853
1854
1855
1856
1857
1858
1859
1860
1861
# File 'lib/sequel/model/base.rb', line 1831

def set_fields(hash, fields, opts=nil)
  opts = if opts
    Hash[model.default_set_fields_options].merge!(opts)
  else
    model.default_set_fields_options
  end

  case opts[:missing]
  when :skip
    fields.each do |f|
      if hash.has_key?(f) 
        set_column_value("#{f}=", hash[f])
      elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s)
        set_column_value("#{sf}=", hash[sf])
      end
    end
  when :raise
    fields.each do |f|
      if hash.has_key?(f)
        set_column_value("#{f}=", hash[f])
      elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s)
        set_column_value("#{sf}=", hash[sf])
      else
        raise(Sequel::Error, "missing field in hash: #{f.inspect} not in #{hash.inspect}")
      end
    end
  else
    fields.each{|f| set_column_value("#{f}=", hash[f])}
  end
  self
end

#set_only(hash, *only) ⇒ Object

Set the values using the entries in the hash, only if the key is included in only. It may be a better idea to use set_fields instead of this method.

artist.set_only({:name=>'Jim'}, :name)
artist.name # => 'Jim'

artist.set_only({:hometown=>'LA'}, :name) # Raise Error


1871
1872
1873
1874
# File 'lib/sequel/model/base.rb', line 1871

def set_only(hash, *only)
  Sequel::Deprecation.deprecate("Sequel::Model#set_only", "Switch to set_fields with the :missing=>:skip option or load the whitelist_security plugin into the model class")
  set_restricted(hash, only.flatten)
end

#set_server(s) ⇒ Object

Set the shard that this object is tied to. Returns self.



1877
1878
1879
1880
1881
# File 'lib/sequel/model/base.rb', line 1877

def set_server(s)
  @server = s
  @this = @this.server(s) if @this
  self
end

#singleton_method_added(meth) ⇒ Object

Clear the setter_methods cache when a method is added



1884
1885
1886
1887
# File 'lib/sequel/model/base.rb', line 1884

def singleton_method_added(meth)
  @singleton_setter_added = true if meth.to_s =~ SETTER_METHOD_REGEXP
  super
end

#thisObject

Returns (naked) dataset that should return only this instance.

Artist[1].this
# SELECT * FROM artists WHERE (id = 1) LIMIT 1

Raises:



1893
1894
1895
1896
1897
1898
1899
1900
1901
1902
1903
1904
# File 'lib/sequel/model/base.rb', line 1893

def this
  return @this if @this
  raise Error, "No dataset for model #{model}" unless ds = model.instance_dataset

  cond = if ds.joined_dataset?
    qualified_pk_hash
  else
    pk_hash
  end

  @this = use_server(ds.where(cond))
end

#update(hash) ⇒ Object

Runs #set with the passed hash and then runs save_changes.

artist.update(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)


1909
1910
1911
# File 'lib/sequel/model/base.rb', line 1909

def update(hash)
  update_restricted(hash, :default)
end

#update_all(hash) ⇒ Object

Update all values using the entries in the hash, ignoring any setting of allowed_columns in the model.

Artist.set_allowed_columns(:num_albums)
artist.update_all(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)


1918
1919
1920
1921
# File 'lib/sequel/model/base.rb', line 1918

def update_all(hash)
  Sequel::Deprecation.deprecate("Sequel::Model#update_all", "Switch to update or load the whitelist_security plugin into the model class")
  update_restricted(hash, :all)
end

#update_fields(hash, fields, opts = nil) ⇒ Object

Update the instances values by calling set_fields with the arguments, then saves any changes to the record. Returns self.

artist.update_fields({:name=>'Jim'}, [:name])
# UPDATE artists SET name = 'Jim' WHERE (id = 1)

artist.update_fields({:hometown=>'LA'}, [:name])
# UPDATE artists SET name = NULL WHERE (id = 1)


1931
1932
1933
1934
# File 'lib/sequel/model/base.rb', line 1931

def update_fields(hash, fields, opts=nil)
  set_fields(hash, fields, opts)
  save_changes
end

#update_only(hash, *only) ⇒ Object

Update the values using the entries in the hash, only if the key is included in only. It may be a better idea to use update_fields instead of this method.

artist.update_only({:name=>'Jim'}, :name)
# UPDATE artists SET name = 'Jim' WHERE (id = 1)

artist.update_only({:hometown=>'LA'}, :name) # Raise Error


1944
1945
1946
1947
# File 'lib/sequel/model/base.rb', line 1944

def update_only(hash, *only)
  Sequel::Deprecation.deprecate("Sequel::Model#update_only", "Switch to update_fields with the :missing=>:skip option or load the whitelist_security plugin into the model class")
  update_restricted(hash, only.flatten)
end

#valid?(opts = OPTS) ⇒ Boolean

Validates the object and returns true if no errors are reported.

artist.set(:name=>'Valid').valid? # => true
artist.set(:name=>'Invalid').valid? # => false
artist.errors.full_messages # => ['name cannot be Invalid']

Returns:

  • (Boolean)


1963
1964
1965
1966
1967
1968
1969
1970
# File 'lib/sequel/model/base.rb', line 1963

def valid?(opts = OPTS)
  _before_validation
  begin
    _valid?(opts)
  rescue HookFailed
    false
  end
end

#validateObject

Validates the object. If the object is invalid, errors should be added to the errors attribute. By default, does nothing, as all models are valid by default. See the “Model Validations” guide. for details about validation. Should not be called directly by user code, call valid? instead to check if an object is valid.



1955
1956
# File 'lib/sequel/model/base.rb', line 1955

def validate
end