Module: Sequel::Model::InstanceMethods
- Defined in:
- lib/sequel/model/base.rb,
lib/sequel/plugins/prepared_statements.rb
Overview
Sequel::Model instance methods that implement basic model functionality.
-
All of the methods in
HOOKSandAROUND_HOOKScreate instance methods that are called by Sequel when the appropriate action occurs. For example, when destroying a model object, Sequel will callaround_destroy, which will callbefore_destroy, do the destroy, and then callafter_destroy. -
The following instance_methods all call the class method of the same name: columns, db, primary_key, db_schema.
-
All of the methods in
BOOLEAN_SETTINGScreate attr_writers allowing you to set values for the attribute. It also creates instance getters returning the value of the setting. If the value has not yet been set, it gets the default value from the class by calling the class method of the same name.
Instance Attribute Summary collapse
-
#values ⇒ Object
(also: #to_hash)
readonly
The hash of attribute values.
Instance Method Summary collapse
-
#==(obj) ⇒ Object
Alias of eql?.
-
#===(obj) ⇒ Object
If pk is not nil, true only if the objects have the same class and pk.
-
#[](column) ⇒ Object
Returns value of the column’s attribute.
-
#[]=(column, value) ⇒ Object
Sets the value for the given column.
-
#autoincrementing_primary_key ⇒ Object
The autoincrementing primary key for this model object.
-
#cancel_action(msg = nil) ⇒ Object
Cancel the current action.
-
#changed_columns ⇒ Object
The columns that have been updated.
-
#clone ⇒ Object
:nocov: Ruby 1.8 doesn’t support initialize_clone, so override clone to dup and freeze.
-
#delete ⇒ Object
Deletes and returns
self. -
#destroy(opts = OPTS) ⇒ Object
Like delete but runs hooks before and after delete.
-
#each(&block) ⇒ Object
Iterates through all of the current values using each.
-
#eql?(obj) ⇒ Boolean
Compares model instances by values.
-
#errors ⇒ Object
Returns the validation errors associated with this object.
-
#exists? ⇒ Boolean
Returns true when current instance exists, false otherwise.
-
#extend(mod) ⇒ Object
Ignore the model’s setter method cache when this instances extends a module, as the module may contain setter methods.
-
#freeze ⇒ Object
Freeze the object in such a way that it is still usable but not modifiable.
-
#hash ⇒ Object
Value that should be unique for objects with the same class and pk (if pk is not nil), or the same class and values (if pk is nil).
-
#id ⇒ Object
Returns value for the :id attribute, even if the primary key is not id.
-
#initialize(values = {}) {|_self| ... } ⇒ Object
Creates new instance and passes the given values to set.
-
#inspect ⇒ Object
Returns a string representation of the model instance including the class name and values.
-
#keys ⇒ Object
Returns the keys in
values. -
#lock! ⇒ Object
Refresh this record using
for_updateunless this is a new record. -
#marshallable! ⇒ Object
Remove elements of the model object that make marshalling fail.
-
#modified!(column = nil) ⇒ Object
Explicitly mark the object as modified, so
save_changes/updatewill run callbacks even if no columns have changed. -
#modified?(column = nil) ⇒ Boolean
Whether this object has been modified since last saved, used by save_changes to determine whether changes should be saved.
-
#new? ⇒ Boolean
Returns true if the current instance represents a new record.
-
#pk ⇒ Object
Returns the primary key value identifying the model instance.
-
#pk_hash ⇒ Object
Returns a hash mapping the receivers primary key column(s) to their values.
-
#qualified_pk_hash(qualifier = model.table_name) ⇒ Object
Returns a hash mapping the receivers primary key column(s) to their values.
-
#refresh ⇒ Object
Reloads attributes from database and returns self.
-
#reload ⇒ Object
Alias of refresh, but not aliased directly to make overriding in a plugin easier.
-
#save(opts = OPTS) ⇒ Object
Creates or updates the record, after making sure the record is valid and before hooks execute successfully.
-
#save_changes(opts = OPTS) ⇒ Object
Saves only changed columns if the object has been modified.
-
#set(hash) ⇒ Object
Updates the instance with the supplied values with support for virtual attributes, raising an exception if a value is used that doesn’t have a setter method (or ignoring it if
strict_param_setting = false). -
#set_all(hash) ⇒ Object
Set all values using the entries in the hash, ignoring any setting of allowed_columns in the model.
-
#set_fields(hash, fields, opts = nil) ⇒ Object
For each of the fields in the given array
fields, call the setter method with the value of thathashentry for the field. -
#set_only(hash, *only) ⇒ Object
Set the values using the entries in the hash, only if the key is included in only.
-
#set_server(s) ⇒ Object
Set the shard that this object is tied to.
-
#singleton_method_added(meth) ⇒ Object
Clear the setter_methods cache when a method is added.
-
#this ⇒ Object
Returns (naked) dataset that should return only this instance.
-
#update(hash) ⇒ Object
Runs #set with the passed hash and then runs save_changes.
-
#update_all(hash) ⇒ Object
Update all values using the entries in the hash, ignoring any setting of
allowed_columnsin the model. -
#update_fields(hash, fields, opts = nil) ⇒ Object
Update the instances values by calling
set_fieldswith the arguments, then saves any changes to the record. -
#update_only(hash, *only) ⇒ Object
Update the values using the entries in the hash, only if the key is included in only.
-
#use_prepared_statements_for?(type) ⇒ Boolean
Whether prepared statements should be used for the given type of query (:insert, :insert_select, :refresh, :update, or :delete).
-
#valid?(opts = OPTS) ⇒ Boolean
Validates the object and returns true if no errors are reported.
-
#validate ⇒ Object
Validates the object.
Instance Attribute Details
#values ⇒ Object (readonly) Also known as: to_hash
The hash of attribute values. Keys are symbols with the names of the underlying database columns.
Artist.new(:name=>'Bob').values # => {:name=>'Bob'}
Artist[1].values # => {:id=>1, :name=>'Jim', ...}
1139 1140 1141 |
# File 'lib/sequel/model/base.rb', line 1139 def values @values end |
Instance Method Details
#==(obj) ⇒ Object
Alias of eql?
1204 1205 1206 |
# File 'lib/sequel/model/base.rb', line 1204 def ==(obj) eql?(obj) end |
#===(obj) ⇒ Object
If pk is not nil, true only if the objects have the same class and pk. If pk is nil, false.
Artist[1] === Artist[1] # true
Artist.new === Artist.new # false
Artist[1].set(:name=>'Bob') == Artist[1] # => true
1214 1215 1216 |
# File 'lib/sequel/model/base.rb', line 1214 def ===(obj) pk.nil? ? false : (obj.class == model) && (obj.pk == pk) end |
#[](column) ⇒ Object
Returns value of the column’s attribute.
Artist[1][:id] #=> 1
1179 1180 1181 |
# File 'lib/sequel/model/base.rb', line 1179 def [](column) @values[column] end |
#[]=(column, value) ⇒ Object
Sets the value for the given column. If typecasting is enabled for this object, typecast the value based on the column’s type. If this is a new record or the typecasted value isn’t the same as the current value for the column, mark the column as changed.
a = Artist.new
a[:name] = 'Bob'
a.values #=> {:name=>'Bob'}
1191 1192 1193 1194 1195 1196 1197 1198 1199 1200 1201 |
# File 'lib/sequel/model/base.rb', line 1191 def []=(column, value) # If it is new, it doesn't have a value yet, so we should # definitely set the new value. # If the column isn't in @values, we can't assume it is # NULL in the database, so assume it has changed. v = typecast_value(column, value) vals = @values if new? || !vals.include?(column) || v != (c = vals[column]) || v.class != c.class change_column_value(column, v) end end |
#autoincrementing_primary_key ⇒ Object
The autoincrementing primary key for this model object. Should be overridden if you have a composite primary key with one part of it being autoincrementing.
1229 1230 1231 |
# File 'lib/sequel/model/base.rb', line 1229 def autoincrementing_primary_key primary_key end |
#cancel_action(msg = nil) ⇒ Object
Cancel the current action. Should be called in before hooks to halt the processing of the action. If a msg argument is given and the model instance is configured to raise exceptions on failure, sets the message to use for the raised HookFailed exception.
1237 1238 1239 |
# File 'lib/sequel/model/base.rb', line 1237 def cancel_action(msg=nil) raise_hook_failure(msg) end |
#changed_columns ⇒ Object
The columns that have been updated. This isn’t completely accurate, as it could contain columns whose values have not changed.
a = Artist[1]
a.changed_columns # => []
a.name = 'Bob'
a.changed_columns # => [:name]
1248 1249 1250 |
# File 'lib/sequel/model/base.rb', line 1248 def changed_columns @changed_columns ||= [] end |
#clone ⇒ Object
:nocov: Ruby 1.8 doesn’t support initialize_clone, so override clone to dup and freeze.
2054 2055 2056 2057 2058 |
# File 'lib/sequel/model/base.rb', line 2054 def clone o = dup o.freeze if frozen? o end |
#delete ⇒ Object
Deletes and returns self. Does not run destroy hooks. Look into using destroy instead.
Artist[1].delete # DELETE FROM artists WHERE (id = 1)
# => #<Artist {:id=>1, ...}>
1257 1258 1259 1260 1261 |
# File 'lib/sequel/model/base.rb', line 1257 def delete raise Sequel::Error, "can't delete frozen object" if frozen? _delete self end |
#destroy(opts = OPTS) ⇒ Object
Like delete but runs hooks before and after delete. If before_destroy returns false, returns false without deleting the object from the database. Otherwise, deletes the item from the database and returns self. Uses a transaction if use_transactions is true or if the :transaction option is given and true.
Artist[1].destroy # BEGIN; DELETE FROM artists WHERE (id = 1); COMMIT;
# => #<Artist {:id=>1, ...}>
1272 1273 1274 1275 |
# File 'lib/sequel/model/base.rb', line 1272 def destroy(opts = OPTS) raise Sequel::Error, "can't destroy frozen object" if frozen? checked_save_failure(opts){checked_transaction(opts){_destroy(opts)}} end |
#each(&block) ⇒ Object
Iterates through all of the current values using each.
Album[1].each{|k, v| puts "#{k} => #{v}"}
# id => 1
# name => 'Bob'
1282 1283 1284 |
# File 'lib/sequel/model/base.rb', line 1282 def each(&block) @values.each(&block) end |
#eql?(obj) ⇒ Boolean
Compares model instances by values.
Artist[1] == Artist[1] # => true
Artist.new == Artist.new # => true
Artist[1].set(:name=>'Bob') == Artist[1] # => false
1291 1292 1293 |
# File 'lib/sequel/model/base.rb', line 1291 def eql?(obj) (obj.class == model) && (obj.values == @values) end |
#errors ⇒ Object
Returns the validation errors associated with this object. See Errors.
1297 1298 1299 |
# File 'lib/sequel/model/base.rb', line 1297 def errors @errors ||= errors_class.new end |
#exists? ⇒ Boolean
Returns true when current instance exists, false otherwise. Generally an object that isn’t new will exist unless it has been deleted. Uses a database query to check for existence, unless the model object is new, in which case this is always false.
Artist[1].exists? # SELECT 1 FROM artists WHERE (id = 1)
# => true
Artist.new.exists?
# => false
1311 1312 1313 |
# File 'lib/sequel/model/base.rb', line 1311 def exists? new? ? false : !this.get(SQL::AliasedExpression.new(1, :one)).nil? end |
#extend(mod) ⇒ Object
Ignore the model’s setter method cache when this instances extends a module, as the module may contain setter methods.
1317 1318 1319 1320 |
# File 'lib/sequel/model/base.rb', line 1317 def extend(mod) @singleton_setter_added = true super end |
#freeze ⇒ Object
Freeze the object in such a way that it is still usable but not modifiable. Once an object is frozen, you cannot modify it’s values, changed_columns, errors, or dataset.
1325 1326 1327 1328 1329 1330 1331 1332 1333 1334 |
# File 'lib/sequel/model/base.rb', line 1325 def freeze values.freeze changed_columns.freeze unless errors.frozen? validate errors.freeze end this.freeze if !new? && model.primary_key super end |
#hash ⇒ Object
Value that should be unique for objects with the same class and pk (if pk is not nil), or the same class and values (if pk is nil).
Artist[1].hash == Artist[1].hash # true
Artist[1].set(:name=>'Bob').hash == Artist[1].hash # true
Artist.new.hash == Artist.new.hash # true
Artist.new(:name=>'Bob').hash == Artist.new.hash # false
1343 1344 1345 1346 1347 1348 1349 1350 1351 1352 |
# File 'lib/sequel/model/base.rb', line 1343 def hash case primary_key when Array [model, !pk.all? ? @values : pk].hash when Symbol [model, pk.nil? ? @values : pk].hash else [model, @values].hash end end |
#id ⇒ Object
Returns value for the :id attribute, even if the primary key is not id. To get the primary key value, use pk.
Artist[1].id # => 1
1358 1359 1360 |
# File 'lib/sequel/model/base.rb', line 1358 def id @values[:id] end |
#initialize(values = {}) {|_self| ... } ⇒ Object
Creates new instance and passes the given values to set. If a block is given, yield the instance to the block.
Arguments:
- values
-
should be a hash to pass to set.
Artist.new(:name=>'Bob')
Artist.new do |a|
a.name = 'Bob'
end
1167 1168 1169 1170 1171 1172 1173 1174 |
# File 'lib/sequel/model/base.rb', line 1167 def initialize(values = {}) @values = {} @new = true @modified = true initialize_set(values) changed_columns.clear yield self if block_given? end |
#inspect ⇒ Object
Returns a string representation of the model instance including the class name and values.
1364 1365 1366 |
# File 'lib/sequel/model/base.rb', line 1364 def inspect "#<#{model.name} @values=#{inspect_values}>" end |
#keys ⇒ Object
Returns the keys in values. May not include all column names.
Artist.new.keys # => []
Artist.new(:name=>'Bob').keys # => [:name]
Artist[1].keys # => [:id, :name]
1373 1374 1375 |
# File 'lib/sequel/model/base.rb', line 1373 def keys @values.keys end |
#lock! ⇒ Object
Refresh this record using for_update unless this is a new record. Returns self. This can be used to make sure no other process is updating the record at the same time.
a = Artist[1]
Artist.db.transaction do
a.lock!
a.update(:name=>'A')
end
1386 1387 1388 1389 |
# File 'lib/sequel/model/base.rb', line 1386 def lock! _refresh(this.for_update) unless new? self end |
#marshallable! ⇒ Object
Remove elements of the model object that make marshalling fail. Returns self.
a = Artist[1]
a.marshallable!
Marshal.dump(a)
1396 1397 1398 1399 |
# File 'lib/sequel/model/base.rb', line 1396 def marshallable! @this = nil self end |
#modified!(column = nil) ⇒ Object
Explicitly mark the object as modified, so save_changes/update will run callbacks even if no columns have changed.
a = Artist[1]
a.save_changes # No callbacks run, as no changes
a.modified!
a.save_changes # Callbacks run, even though no changes made
If a column is given, specifically marked that column as modified, so that save_changes/update will include that column in the update. This should be used if you plan on mutating the column value instead of assigning a new column value:
a.modified!(:name)
a.name.gsub!(/[aeou]/, 'i')
1416 1417 1418 1419 1420 1421 |
# File 'lib/sequel/model/base.rb', line 1416 def modified!(column=nil) if column && !changed_columns.include?(column) changed_columns << column end @modified = true end |
#modified?(column = nil) ⇒ Boolean
Whether this object has been modified since last saved, used by save_changes to determine whether changes should be saved. New values are always considered modified.
a = Artist[1]
a.modified? # => false
a.set(:name=>'Jim')
a.modified? # => true
If a column is given, specifically check if the given column has been modified:
a.modified?(:num_albums) # => false
a.num_albums = 10
a.modified?(:num_albums) # => true
1438 1439 1440 1441 1442 1443 1444 |
# File 'lib/sequel/model/base.rb', line 1438 def modified?(column=nil) if column changed_columns.include?(column) else @modified || !changed_columns.empty? end end |
#new? ⇒ Boolean
Returns true if the current instance represents a new record.
Artist.new.new? # => true
Artist[1].new? # => false
1450 1451 1452 |
# File 'lib/sequel/model/base.rb', line 1450 def new? defined?(@new) ? @new : (@new = false) end |
#pk ⇒ Object
Returns the primary key value identifying the model instance. Raises an Error if this model does not have a primary key. If the model has a composite primary key, returns an array of values.
Artist[1].pk # => 1
Artist[[1, 2]].pk # => [1, 2]
1460 1461 1462 1463 1464 1465 1466 1467 1468 |
# File 'lib/sequel/model/base.rb', line 1460 def pk raise(Error, "No primary key is associated with this model") unless key = primary_key if key.is_a?(Array) vals = @values key.map{|k| vals[k]} else @values[key] end end |
#pk_hash ⇒ Object
Returns a hash mapping the receivers primary key column(s) to their values.
Artist[1].pk_hash # => {:id=>1}
Artist[[1, 2]].pk_hash # => {:id1=>1, :id2=>2}
1474 1475 1476 |
# File 'lib/sequel/model/base.rb', line 1474 def pk_hash model.primary_key_hash(pk) end |
#qualified_pk_hash(qualifier = model.table_name) ⇒ Object
Returns a hash mapping the receivers primary key column(s) to their values.
Artist[1].qualified_pk_hash
# => {Sequel.qualify(:artists, :id)=>1}
Artist[[1, 2]].qualified_pk_hash
# => {Sequel.qualify(:artists, :id1)=>1, Sequel.qualify(:artists, :id2)=>2}
1484 1485 1486 |
# File 'lib/sequel/model/base.rb', line 1484 def qualified_pk_hash(qualifier=model.table_name) model.qualified_primary_key_hash(pk, qualifier) end |
#refresh ⇒ Object
Reloads attributes from database and returns self. Also clears all changed_columns information. Raises an Error if the record no longer exists in the database.
a = Artist[1]
a.name = 'Jim'
a.refresh
a.name # => 'Bob'
1496 1497 1498 1499 1500 |
# File 'lib/sequel/model/base.rb', line 1496 def refresh raise Sequel::Error, "can't refresh frozen object" if frozen? _refresh(this) self end |
#reload ⇒ Object
Alias of refresh, but not aliased directly to make overriding in a plugin easier.
1503 1504 1505 |
# File 'lib/sequel/model/base.rb', line 1503 def reload refresh end |
#save(opts = OPTS) ⇒ Object
Creates or updates the record, after making sure the record is valid and before hooks execute successfully. Fails if:
-
the record is not valid, or
-
before_save returns false, or
-
the record is new and before_create returns false, or
-
the record is not new and before_update returns false.
If save fails and either raise_on_save_failure or the :raise_on_failure option is true, it raises ValidationFailed or HookFailed. Otherwise it returns nil.
If it succeeds, it returns self.
You can provide an optional list of columns to update, in which case it only updates those columns, or a options hash.
Takes the following options:
- :changed
-
save all changed columns, instead of all columns or the columns given
- :columns
-
array of specific columns that should be saved.
- :raise_on_failure
-
set to true or false to override the current
raise_on_save_failuresetting - :server
-
set the server/shard on the object before saving, and use that server/shard in any transaction.
- :transaction
-
set to true or false to override the current
use_transactionssetting - :validate
-
set to false to skip validation
1535 1536 1537 1538 1539 1540 1541 1542 1543 1544 1545 1546 |
# File 'lib/sequel/model/base.rb', line 1535 def save(opts=OPTS) raise Sequel::Error, "can't save frozen object" if frozen? set_server(opts[:server]) if opts[:server] _before_validation if opts[:validate] != false unless checked_save_failure(opts){_valid?(true, opts)} raise(ValidationFailed.new(self)) if raise_on_failure?(opts) return end end checked_save_failure(opts){checked_transaction(opts){_save(opts)}} end |
#save_changes(opts = OPTS) ⇒ Object
Saves only changed columns if the object has been modified. If the object has not been modified, returns nil. If unable to save, returns false unless raise_on_save_failure is true.
a = Artist[1]
a.save_changes # => nil
a.name = 'Jim'
a.save_changes # UPDATE artists SET name = 'Bob' WHERE (id = 1)
# => #<Artist {:id=>1, :name=>'Jim', ...}
1557 1558 1559 |
# File 'lib/sequel/model/base.rb', line 1557 def save_changes(opts=OPTS) save(Hash[opts].merge!(:changed=>true)) || false if modified? end |
#set(hash) ⇒ Object
Updates the instance with the supplied values with support for virtual attributes, raising an exception if a value is used that doesn’t have a setter method (or ignoring it if strict_param_setting = false). Does not save the record.
artist.set(:name=>'Jim')
artist.name # => 'Jim'
1568 1569 1570 |
# File 'lib/sequel/model/base.rb', line 1568 def set(hash) set_restricted(hash, :default) end |
#set_all(hash) ⇒ Object
Set all values using the entries in the hash, ignoring any setting of allowed_columns in the model.
Artist.set_allowed_columns(:num_albums)
artist.set_all(:name=>'Jim')
artist.name # => 'Jim'
1578 1579 1580 |
# File 'lib/sequel/model/base.rb', line 1578 def set_all(hash) set_restricted(hash, :all) end |
#set_fields(hash, fields, opts = nil) ⇒ Object
For each of the fields in the given array fields, call the setter method with the value of that hash entry for the field. Returns self.
You can provide an options hash, with the following options currently respected:
- :missing
-
Can be set to :skip to skip missing entries or :raise to raise an Error for missing entries. The default behavior is not to check for missing entries, in which case the default value is used. To be friendly with most web frameworks, the missing check will also check for the string version of the argument in the hash if given a symbol.
Examples:
artist.set_fields({:name=>'Jim'}, [:name])
artist.name # => 'Jim'
artist.set_fields({:hometown=>'LA'}, [:name])
artist.name # => nil
artist.hometown # => 'Sac'
artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:skip)
artist.name # => 'Jim'
artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:raise)
# Sequel::Error raised
1608 1609 1610 1611 1612 1613 1614 1615 1616 1617 1618 1619 1620 1621 1622 1623 1624 1625 1626 1627 1628 1629 1630 1631 1632 1633 1634 1635 1636 1637 1638 |
# File 'lib/sequel/model/base.rb', line 1608 def set_fields(hash, fields, opts=nil) opts = if opts Hash[model.].merge!(opts) else model. end case opts[:missing] when :skip fields.each do |f| if hash.has_key?(f) set_column_value("#{f}=", hash[f]) elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s) set_column_value("#{sf}=", hash[sf]) end end when :raise fields.each do |f| if hash.has_key?(f) set_column_value("#{f}=", hash[f]) elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s) set_column_value("#{sf}=", hash[sf]) else raise(Sequel::Error, "missing field in hash: #{f.inspect} not in #{hash.inspect}") end end else fields.each{|f| set_column_value("#{f}=", hash[f])} end self end |
#set_only(hash, *only) ⇒ Object
Set the values using the entries in the hash, only if the key is included in only. It may be a better idea to use set_fields instead of this method.
artist.set_only({:name=>'Jim'}, :name)
artist.name # => 'Jim'
artist.set_only({:hometown=>'LA'}, :name) # Raise Error
1648 1649 1650 |
# File 'lib/sequel/model/base.rb', line 1648 def set_only(hash, *only) set_restricted(hash, only.flatten) end |
#set_server(s) ⇒ Object
Set the shard that this object is tied to. Returns self.
1653 1654 1655 1656 1657 |
# File 'lib/sequel/model/base.rb', line 1653 def set_server(s) @server = s @this.opts[:server] = s if @this self end |
#singleton_method_added(meth) ⇒ Object
Clear the setter_methods cache when a method is added
1660 1661 1662 1663 |
# File 'lib/sequel/model/base.rb', line 1660 def singleton_method_added(meth) @singleton_setter_added = true if meth.to_s =~ SETTER_METHOD_REGEXP super end |
#this ⇒ Object
Returns (naked) dataset that should return only this instance.
Artist[1].this
# SELECT * FROM artists WHERE (id = 1) LIMIT 1
1669 1670 1671 1672 1673 1674 1675 1676 1677 1678 1679 1680 |
# File 'lib/sequel/model/base.rb', line 1669 def this return @this if @this raise Error, "No dataset for model #{model}" unless ds = model.instance_dataset cond = if ds.joined_dataset? qualified_pk_hash else pk_hash end @this = use_server(ds.where(cond)) end |
#update(hash) ⇒ Object
Runs #set with the passed hash and then runs save_changes.
artist.update(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)
1685 1686 1687 |
# File 'lib/sequel/model/base.rb', line 1685 def update(hash) update_restricted(hash, :default) end |
#update_all(hash) ⇒ Object
Update all values using the entries in the hash, ignoring any setting of allowed_columns in the model.
Artist.set_allowed_columns(:num_albums)
artist.update_all(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)
1694 1695 1696 |
# File 'lib/sequel/model/base.rb', line 1694 def update_all(hash) update_restricted(hash, :all) end |
#update_fields(hash, fields, opts = nil) ⇒ Object
Update the instances values by calling set_fields with the arguments, then saves any changes to the record. Returns self.
artist.update_fields({:name=>'Jim'}, [:name])
# UPDATE artists SET name = 'Jim' WHERE (id = 1)
artist.update_fields({:hometown=>'LA'}, [:name])
# UPDATE artists SET name = NULL WHERE (id = 1)
1706 1707 1708 1709 |
# File 'lib/sequel/model/base.rb', line 1706 def update_fields(hash, fields, opts=nil) set_fields(hash, fields, opts) save_changes end |
#update_only(hash, *only) ⇒ Object
Update the values using the entries in the hash, only if the key is included in only. It may be a better idea to use update_fields instead of this method.
artist.update_only({:name=>'Jim'}, :name)
# UPDATE artists SET name = 'Jim' WHERE (id = 1)
artist.update_only({:hometown=>'LA'}, :name) # Raise Error
1719 1720 1721 |
# File 'lib/sequel/model/base.rb', line 1719 def update_only(hash, *only) update_restricted(hash, only.flatten) end |
#use_prepared_statements_for?(type) ⇒ Boolean
Whether prepared statements should be used for the given type of query (:insert, :insert_select, :refresh, :update, or :delete). True by default, can be overridden in other plugins to disallow prepared statements for specific types of queries.
8 9 10 |
# File 'lib/sequel/plugins/prepared_statements.rb', line 8 def use_prepared_statements_for?(type) true end |
#valid?(opts = OPTS) ⇒ Boolean
Validates the object and returns true if no errors are reported.
artist(:name=>'Valid').valid? # => true
artist(:name=>'Invalid').valid? # => false
artist.errors. # => ['name cannot be Invalid']
1737 1738 1739 1740 |
# File 'lib/sequel/model/base.rb', line 1737 def valid?(opts = OPTS) _before_validation _valid?(false, opts) end |
#validate ⇒ Object
Validates the object. If the object is invalid, errors should be added to the errors attribute. By default, does nothing, as all models are valid by default. See the “Model Validations” guide. for details about validation. Should not be called directly by user code, call valid? instead to check if an object is valid.
1729 1730 |
# File 'lib/sequel/model/base.rb', line 1729 def validate end |