Module: Sequel::Model::InstanceMethods
- Defined in:
- lib/sequel/model/base.rb
Overview
Sequel::Model instance methods that implement basic model functionality.
-
All of the methods in
HOOKSandAROUND_HOOKScreate instance methods that are called by Sequel when the appropriate action occurs. For example, when destroying a model object, Sequel will callaround_destroy, which will callbefore_destroy, do the destroy, and then callafter_destroy. -
The following instance_methods all call the class method of the same name: columns, db, primary_key, db_schema.
-
All of the methods in
BOOLEAN_SETTINGScreate attr_writers allowing you to set values for the attribute. It also creates instance getters returning the value of the setting. If the value has not yet been set, it gets the default value from the class by calling the class method of the same name.
Instance Attribute Summary collapse
-
#values ⇒ Object
(also: #to_hash)
readonly
The hash of attribute values.
Instance Method Summary collapse
-
#==(obj) ⇒ Object
Alias of eql?.
-
#===(obj) ⇒ Object
If pk is not nil, true only if the objects have the same class and pk.
-
#[](column) ⇒ Object
Returns value of the column’s attribute.
-
#[]=(column, value) ⇒ Object
Sets the value for the given column.
-
#autoincrementing_primary_key ⇒ Object
The autoincrementing primary key for this model object.
-
#changed_columns ⇒ Object
The columns that have been updated.
-
#delete ⇒ Object
Deletes and returns
self. -
#destroy(opts = {}) ⇒ Object
Like delete but runs hooks before and after delete.
-
#each(&block) ⇒ Object
Iterates through all of the current values using each.
-
#eql?(obj) ⇒ Boolean
Compares model instances by values.
-
#errors ⇒ Object
Returns the validation errors associated with this object.
-
#exists? ⇒ Boolean
Returns true when current instance exists, false otherwise.
-
#extend(mod) ⇒ Object
Ignore the model’s setter method cache when this instances extends a module, as the module may contain setter methods.
-
#freeze ⇒ Object
Freeze the object in such a way that it is still usable but not modifiable.
-
#hash ⇒ Object
Value that should be unique for objects with the same class and pk (if pk is not nil), or the same class and values (if pk is nil).
-
#id ⇒ Object
Returns value for the :id attribute, even if the primary key is not id.
-
#initialize(values = {}, from_db = false) ⇒ Object
Creates new instance and passes the given values to set.
-
#inspect ⇒ Object
Returns a string representation of the model instance including the class name and values.
-
#keys ⇒ Object
Returns the keys in
values. -
#lock! ⇒ Object
Refresh this record using
for_updateunless this is a new record. -
#marshallable! ⇒ Object
Remove elements of the model object that make marshalling fail.
-
#modified!(column = nil) ⇒ Object
Explicitly mark the object as modified, so
save_changes/updatewill run callbacks even if no columns have changed. -
#modified?(column = nil) ⇒ Boolean
Whether this object has been modified since last saved, used by save_changes to determine whether changes should be saved.
-
#new? ⇒ Boolean
Returns true if the current instance represents a new record.
-
#pk ⇒ Object
Returns the primary key value identifying the model instance.
-
#pk_hash ⇒ Object
Returns a hash mapping the receivers primary key column(s) to their values.
-
#refresh ⇒ Object
Reloads attributes from database and returns self.
-
#reload ⇒ Object
Alias of refresh, but not aliased directly to make overriding in a plugin easier.
-
#save(*columns) ⇒ Object
Creates or updates the record, after making sure the record is valid and before hooks execute successfully.
-
#save_changes(opts = {}) ⇒ Object
Saves only changed columns if the object has been modified.
-
#set(hash) ⇒ Object
Updates the instance with the supplied values with support for virtual attributes, raising an exception if a value is used that doesn’t have a setter method (or ignoring it if
strict_param_setting = false). -
#set_all(hash) ⇒ Object
Set all values using the entries in the hash, ignoring any setting of allowed_columns or restricted columns in the model.
-
#set_except(hash, *except) ⇒ Object
Set all values using the entries in the hash, except for the keys given in except.
-
#set_fields(hash, fields, opts = nil) ⇒ Object
For each of the fields in the given array
fields, call the setter method with the value of thathashentry for the field. -
#set_only(hash, *only) ⇒ Object
Set the values using the entries in the hash, only if the key is included in only.
-
#set_server(s) ⇒ Object
Set the shard that this object is tied to.
-
#set_values(hash) ⇒ Object
Replace the current values with hash.
-
#singleton_method_added(meth) ⇒ Object
Clear the setter_methods cache when a method is added.
-
#this ⇒ Object
Returns (naked) dataset that should return only this instance.
-
#update(hash) ⇒ Object
Runs #set with the passed hash and then runs save_changes.
-
#update_all(hash) ⇒ Object
Update all values using the entries in the hash, ignoring any setting of
allowed_columnsorrestricted_columnsin the model. -
#update_except(hash, *except) ⇒ Object
Update all values using the entries in the hash, except for the keys given in except.
-
#update_fields(hash, fields, opts = nil) ⇒ Object
Update the instances values by calling
set_fieldswith the arguments, then saves any changes to the record. -
#update_only(hash, *only) ⇒ Object
Update the values using the entries in the hash, only if the key is included in only.
-
#valid?(opts = {}) ⇒ Boolean
Validates the object and returns true if no errors are reported.
-
#validate ⇒ Object
Validates the object.
Instance Attribute Details
#values ⇒ Object (readonly) Also known as: to_hash
The hash of attribute values. Keys are symbols with the names of the underlying database columns.
Artist.new(:name=>'Bob').values # => {:name=>'Bob'}
Artist[1].values # => {:id=>1, :name=>'Jim', ...}
931 932 933 |
# File 'lib/sequel/model/base.rb', line 931 def values @values end |
Instance Method Details
#==(obj) ⇒ Object
Alias of eql?
991 992 993 |
# File 'lib/sequel/model/base.rb', line 991 def ==(obj) eql?(obj) end |
#===(obj) ⇒ Object
If pk is not nil, true only if the objects have the same class and pk. If pk is nil, false.
Artist[1] === Artist[1] # true
Artist.new === Artist.new # false
Artist[1].set(:name=>'Bob') == Artist[1] # => true
1001 1002 1003 |
# File 'lib/sequel/model/base.rb', line 1001 def ===(obj) pk.nil? ? false : (obj.class == model) && (obj.pk == pk) end |
#[](column) ⇒ Object
Returns value of the column’s attribute.
Artist[1][:id] #=> 1
966 967 968 |
# File 'lib/sequel/model/base.rb', line 966 def [](column) @values[column] end |
#[]=(column, value) ⇒ Object
Sets the value for the given column. If typecasting is enabled for this object, typecast the value based on the column’s type. If this is a new record or the typecasted value isn’t the same as the current value for the column, mark the column as changed.
a = Artist.new
a[:name] = 'Bob'
a.values #=> {:name=>'Bob'}
978 979 980 981 982 983 984 985 986 987 988 |
# File 'lib/sequel/model/base.rb', line 978 def []=(column, value) # If it is new, it doesn't have a value yet, so we should # definitely set the new value. # If the column isn't in @values, we can't assume it is # NULL in the database, so assume it has changed. v = typecast_value(column, value) vals = @values if new? || !vals.include?(column) || v != (c = vals[column]) || v.class != c.class change_column_value(column, v) end end |
#autoincrementing_primary_key ⇒ Object
The autoincrementing primary key for this model object. Should be overridden if you have a composite primary key with one part of it being autoincrementing.
1016 1017 1018 |
# File 'lib/sequel/model/base.rb', line 1016 def autoincrementing_primary_key primary_key end |
#changed_columns ⇒ Object
The columns that have been updated. This isn’t completely accurate, as it could contain columns whose values have not changed.
a = Artist[1]
a.changed_columns # => []
a.name = 'Bob'
a.changed_columns # => [:name]
1027 1028 1029 |
# File 'lib/sequel/model/base.rb', line 1027 def changed_columns @changed_columns ||= [] end |
#delete ⇒ Object
Deletes and returns self. Does not run destroy hooks. Look into using destroy instead.
Artist[1].delete # DELETE FROM artists WHERE (id = 1)
# => #<Artist {:id=>1, ...}>
1036 1037 1038 1039 1040 |
# File 'lib/sequel/model/base.rb', line 1036 def delete raise Sequel::Error, "can't delete frozen object" if frozen? _delete self end |
#destroy(opts = {}) ⇒ Object
Like delete but runs hooks before and after delete. If before_destroy returns false, returns false without deleting the object the the database. Otherwise, deletes the item from the database and returns self. Uses a transaction if use_transactions is true or if the :transaction option is given and true.
Artist[1].destroy # BEGIN; DELETE FROM artists WHERE (id = 1); COMMIT;
# => #<Artist {:id=>1, ...}>
1051 1052 1053 1054 |
# File 'lib/sequel/model/base.rb', line 1051 def destroy(opts = {}) raise Sequel::Error, "can't destroy frozen object" if frozen? checked_save_failure(opts){checked_transaction(opts){_destroy(opts)}} end |
#each(&block) ⇒ Object
Iterates through all of the current values using each.
Album[1].each{|k, v| puts "#{k} => #{v}"}
# id => 1
# name => 'Bob'
1061 1062 1063 |
# File 'lib/sequel/model/base.rb', line 1061 def each(&block) @values.each(&block) end |
#eql?(obj) ⇒ Boolean
Compares model instances by values.
Artist[1] == Artist[1] # => true
Artist.new == Artist.new # => true
Artist[1].set(:name=>'Bob') == Artist[1] # => false
1070 1071 1072 |
# File 'lib/sequel/model/base.rb', line 1070 def eql?(obj) (obj.class == model) && (obj.values == @values) end |
#errors ⇒ Object
Returns the validation errors associated with this object. See Errors.
1076 1077 1078 |
# File 'lib/sequel/model/base.rb', line 1076 def errors @errors ||= Errors.new end |
#exists? ⇒ Boolean
Returns true when current instance exists, false otherwise. Generally an object that isn’t new will exist unless it has been deleted. Uses a database query to check for existence, unless the model object is new, in which case this is always false.
Artist[1].exists? # SELECT 1 FROM artists WHERE (id = 1)
# => true
Artist.new.exists?
# => false
1090 1091 1092 |
# File 'lib/sequel/model/base.rb', line 1090 def exists? new? ? false : !this.get(1).nil? end |
#extend(mod) ⇒ Object
Ignore the model’s setter method cache when this instances extends a module, as the module may contain setter methods.
1096 1097 1098 1099 |
# File 'lib/sequel/model/base.rb', line 1096 def extend(mod) @singleton_setter_added = true super end |
#freeze ⇒ Object
Freeze the object in such a way that it is still usable but not modifiable. Once an object is frozen, you cannot modify it’s values, changed_columns, errors, or dataset.
1104 1105 1106 1107 1108 1109 1110 1111 1112 |
# File 'lib/sequel/model/base.rb', line 1104 def freeze values.freeze changed_columns.freeze errors validate errors.freeze this.freeze if !new? && model.primary_key super end |
#hash ⇒ Object
Value that should be unique for objects with the same class and pk (if pk is not nil), or the same class and values (if pk is nil).
Artist[1].hash == Artist[1].hash # true
Artist[1].set(:name=>'Bob').hash == Artist[1].hash # true
Artist.new.hash == Artist.new.hash # true
Artist.new(:name=>'Bob').hash == Artist.new.hash # false
1121 1122 1123 1124 1125 1126 1127 1128 1129 1130 |
# File 'lib/sequel/model/base.rb', line 1121 def hash case primary_key when Array [model, !pk.all? ? @values : pk].hash when Symbol [model, pk.nil? ? @values : pk].hash else [model, @values].hash end end |
#id ⇒ Object
Returns value for the :id attribute, even if the primary key is not id. To get the primary key value, use pk.
Artist[1].id # => 1
1136 1137 1138 |
# File 'lib/sequel/model/base.rb', line 1136 def id @values[:id] end |
#initialize(values = {}, from_db = false) ⇒ Object
Creates new instance and passes the given values to set. If a block is given, yield the instance to the block unless from_db is true. This method runs the after_initialize hook after it has optionally yielded itself to the block.
Arguments:
- values
-
should be a hash to pass to set.
- from_db
-
only for backwards compatibility, forget it exists.
Artist.new(:name=>'Bob')
Artist.new do |a|
a.name = 'Bob'
end
949 950 951 952 953 954 955 956 957 958 959 960 961 |
# File 'lib/sequel/model/base.rb', line 949 def initialize(values = {}, from_db = false) if from_db set_values(values) else @values = {} @new = true @modified = true initialize_set(values) changed_columns.clear yield self if block_given? end after_initialize end |
#inspect ⇒ Object
Returns a string representation of the model instance including the class name and values.
1142 1143 1144 |
# File 'lib/sequel/model/base.rb', line 1142 def inspect "#<#{model.name} @values=#{inspect_values}>" end |
#keys ⇒ Object
Returns the keys in values. May not include all column names.
Artist.new.keys # => []
Artist.new(:name=>'Bob').keys # => [:name]
Artist[1].keys # => [:id, :name]
1151 1152 1153 |
# File 'lib/sequel/model/base.rb', line 1151 def keys @values.keys end |
#lock! ⇒ Object
Refresh this record using for_update unless this is a new record. Returns self. This can be used to make sure no other process is updating the record at the same time.
a = Artist[1]
Artist.db.transaction do
a.lock!
a.update(...)
end
1164 1165 1166 1167 |
# File 'lib/sequel/model/base.rb', line 1164 def lock! _refresh(this.for_update) unless new? self end |
#marshallable! ⇒ Object
Remove elements of the model object that make marshalling fail. Returns self.
a = Artist[1]
a.marshallable!
Marshal.dump(a)
1174 1175 1176 1177 |
# File 'lib/sequel/model/base.rb', line 1174 def marshallable! @this = nil self end |
#modified!(column = nil) ⇒ Object
Explicitly mark the object as modified, so save_changes/update will run callbacks even if no columns have changed.
a = Artist[1]
a.save_changes # No callbacks run, as no changes
a.modified!
a.save_changes # Callbacks run, even though no changes made
If a column is given, specifically marked that column as modified, so that save_changes/update will include that column in the update. This should be used if you plan on mutating the column value instead of assigning a new column value:
a.modified!(:name)
a.name.gsub!(/[aeou]/, 'i')
1194 1195 1196 1197 1198 1199 |
# File 'lib/sequel/model/base.rb', line 1194 def modified!(column=nil) if column && !changed_columns.include?(column) changed_columns << column end @modified = true end |
#modified?(column = nil) ⇒ Boolean
Whether this object has been modified since last saved, used by save_changes to determine whether changes should be saved. New values are always considered modified.
a = Artist[1]
a.modified? # => false
a.set(:name=>'Jim')
a.modified? # => true
If a column is given, specifically check if the given column has been modified:
a.modified?(:num_albums) # => false
a.num_albums = 10
a.modified?(:num_albums) # => true
1216 1217 1218 1219 1220 1221 1222 |
# File 'lib/sequel/model/base.rb', line 1216 def modified?(column=nil) if column changed_columns.include?(column) else @modified || !changed_columns.empty? end end |
#new? ⇒ Boolean
Returns true if the current instance represents a new record.
Artist.new.new? # => true
Artist[1].new? # => false
1228 1229 1230 |
# File 'lib/sequel/model/base.rb', line 1228 def new? defined?(@new) ? @new : (@new = false) end |
#pk ⇒ Object
Returns the primary key value identifying the model instance. Raises an Error if this model does not have a primary key. If the model has a composite primary key, returns an array of values.
Artist[1].pk # => 1
Artist[[1, 2]].pk # => [1, 2]
1238 1239 1240 1241 1242 1243 1244 1245 1246 |
# File 'lib/sequel/model/base.rb', line 1238 def pk raise(Error, "No primary key is associated with this model") unless key = primary_key if key.is_a?(Array) vals = @values key.map{|k| vals[k]} else @values[key] end end |
#pk_hash ⇒ Object
Returns a hash mapping the receivers primary key column(s) to their values.
Artist[1].pk_hash # => {:id=>1}
Artist[[1, 2]].pk_hash # => {:id1=>1, :id2=>2}
1252 1253 1254 |
# File 'lib/sequel/model/base.rb', line 1252 def pk_hash model.primary_key_hash(pk) end |
#refresh ⇒ Object
Reloads attributes from database and returns self. Also clears all changed_columns information. Raises an Error if the record no longer exists in the database.
a = Artist[1]
a.name = 'Jim'
a.refresh
a.name # => 'Bob'
1264 1265 1266 1267 1268 |
# File 'lib/sequel/model/base.rb', line 1264 def refresh raise Sequel::Error, "can't refresh frozen object" if frozen? _refresh(this) self end |
#reload ⇒ Object
Alias of refresh, but not aliased directly to make overriding in a plugin easier.
1271 1272 1273 |
# File 'lib/sequel/model/base.rb', line 1271 def reload refresh end |
#save(*columns) ⇒ Object
Creates or updates the record, after making sure the record is valid and before hooks execute successfully. Fails if:
-
the record is not valid, or
-
before_save returns false, or
-
the record is new and before_create returns false, or
-
the record is not new and before_update returns false.
If save fails and either raise_on_save_failure or the :raise_on_failure option is true, it raises ValidationFailed or HookFailed. Otherwise it returns nil.
If it succeeds, it returns self.
You can provide an optional list of columns to update, in which case it only updates those columns, or a options hash.
Takes the following options:
- :changed
-
save all changed columns, instead of all columns or the columns given
- :raise_on_failure
-
set to true or false to override the current
raise_on_save_failuresetting - :server
-
set the server/shard on the object before saving, and use that server/shard in any transaction.
- :transaction
-
set to true or false to override the current
use_transactionssetting - :validate
-
set to false to skip validation
1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 |
# File 'lib/sequel/model/base.rb', line 1302 def save(*columns) raise Sequel::Error, "can't save frozen object" if frozen? opts = columns.last.is_a?(Hash) ? columns.pop : {} set_server(opts[:server]) if opts[:server] if opts[:validate] != false unless checked_save_failure(opts){_valid?(true, opts)} raise(ValidationFailed.new(self)) if raise_on_failure?(opts) return end end checked_save_failure(opts){checked_transaction(opts){_save(columns, opts)}} end |
#save_changes(opts = {}) ⇒ Object
Saves only changed columns if the object has been modified. If the object has not been modified, returns nil. If unable to save, returns false unless raise_on_save_failure is true.
a = Artist[1]
a.save_changes # => nil
a.name = 'Jim'
a.save_changes # UPDATE artists SET name = 'Bob' WHERE (id = 1)
# => #<Artist {:id=>1, :name=>'Jim', ...}
1324 1325 1326 |
# File 'lib/sequel/model/base.rb', line 1324 def save_changes(opts={}) save(opts.merge(:changed=>true)) || false if modified? end |
#set(hash) ⇒ Object
Updates the instance with the supplied values with support for virtual attributes, raising an exception if a value is used that doesn’t have a setter method (or ignoring it if strict_param_setting = false). Does not save the record.
artist.set(:name=>'Jim')
artist.name # => 'Jim'
1335 1336 1337 |
# File 'lib/sequel/model/base.rb', line 1335 def set(hash) set_restricted(hash, nil, nil) end |
#set_all(hash) ⇒ Object
Set all values using the entries in the hash, ignoring any setting of allowed_columns or restricted columns in the model.
Artist.set_restricted_columns(:name)
artist.set_all(:name=>'Jim')
artist.name # => 'Jim'
1345 1346 1347 |
# File 'lib/sequel/model/base.rb', line 1345 def set_all(hash) set_restricted(hash, false, false) end |
#set_except(hash, *except) ⇒ Object
Set all values using the entries in the hash, except for the keys given in except. You should probably use set_fields or set_only instead of this method, as blacklist approaches to security are a bad idea.
artist.set_except({:name=>'Jim'}, :hometown)
artist.name # => 'Jim'
1355 1356 1357 |
# File 'lib/sequel/model/base.rb', line 1355 def set_except(hash, *except) set_restricted(hash, false, except.flatten) end |
#set_fields(hash, fields, opts = nil) ⇒ Object
For each of the fields in the given array fields, call the setter method with the value of that hash entry for the field. Returns self.
You can provide an options hash, with the following options currently respected:
- :missing
-
Can be set to :skip to skip missing entries or :raise to raise an Error for missing entries. The default behavior is not to check for missing entries, in which case the default value is used. To be friendly with most web frameworks, the missing check will also check for the string version of the argument in the hash if given a symbol.
Examples:
artist.set_fields({:name=>'Jim'}, [:name])
artist.name # => 'Jim'
artist.set_fields({:hometown=>'LA'}, [:name])
artist.name # => nil
artist.hometown # => 'Sac'
artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:skip)
artist.name # => 'Jim'
artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:raise)
# Sequel::Error raised
1385 1386 1387 1388 1389 1390 1391 1392 1393 1394 1395 1396 1397 1398 1399 1400 1401 1402 1403 1404 1405 1406 1407 1408 1409 1410 1411 1412 1413 1414 1415 |
# File 'lib/sequel/model/base.rb', line 1385 def set_fields(hash, fields, opts=nil) opts = if opts model..merge(opts) else model. end case opts[:missing] when :skip fields.each do |f| if hash.has_key?(f) send("#{f}=", hash[f]) elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s) send("#{sf}=", hash[sf]) end end when :raise fields.each do |f| if hash.has_key?(f) send("#{f}=", hash[f]) elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s) send("#{sf}=", hash[sf]) else raise(Sequel::Error, "missing field in hash: #{f.inspect} not in #{hash.inspect}") end end else fields.each{|f| send("#{f}=", hash[f])} end self end |
#set_only(hash, *only) ⇒ Object
Set the values using the entries in the hash, only if the key is included in only. It may be a better idea to use set_fields instead of this method.
artist.set_only({:name=>'Jim'}, :name)
artist.name # => 'Jim'
artist.set_only({:hometown=>'LA'}, :name) # Raise Error
1425 1426 1427 |
# File 'lib/sequel/model/base.rb', line 1425 def set_only(hash, *only) set_restricted(hash, only.flatten, false) end |
#set_server(s) ⇒ Object
Set the shard that this object is tied to. Returns self.
1430 1431 1432 1433 1434 |
# File 'lib/sequel/model/base.rb', line 1430 def set_server(s) @server = s @this.opts[:server] = s if @this self end |
#set_values(hash) ⇒ Object
Replace the current values with hash. Should definitely not be used with untrusted input, and should probably not be called directly by user code.
1439 1440 1441 |
# File 'lib/sequel/model/base.rb', line 1439 def set_values(hash) @values = hash end |
#singleton_method_added(meth) ⇒ Object
Clear the setter_methods cache when a method is added
1444 1445 1446 1447 |
# File 'lib/sequel/model/base.rb', line 1444 def singleton_method_added(meth) @singleton_setter_added = true if meth.to_s =~ SETTER_METHOD_REGEXP super end |
#this ⇒ Object
Returns (naked) dataset that should return only this instance.
Artist[1].this
# SELECT * FROM artists WHERE (id = 1) LIMIT 1
1453 1454 1455 |
# File 'lib/sequel/model/base.rb', line 1453 def this @this ||= use_server(model.instance_dataset.filter(pk_hash)) end |
#update(hash) ⇒ Object
Runs #set with the passed hash and then runs save_changes.
artist.update(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)
1460 1461 1462 |
# File 'lib/sequel/model/base.rb', line 1460 def update(hash) update_restricted(hash, nil, nil) end |
#update_all(hash) ⇒ Object
Update all values using the entries in the hash, ignoring any setting of allowed_columns or restricted_columns in the model.
Artist.set_restricted_columns(:name)
artist.update_all(:name=>'Jim') # UPDATE artists SET name = 'Jim' WHERE (id = 1)
1469 1470 1471 |
# File 'lib/sequel/model/base.rb', line 1469 def update_all(hash) update_restricted(hash, false, false) end |
#update_except(hash, *except) ⇒ Object
Update all values using the entries in the hash, except for the keys given in except. You should probably use update_fields or update_only instead of this method, as blacklist approaches to security are a bad idea.
artist.update_except({:name=>'Jim'}, :hometown) # UPDATE artists SET name = 'Jim' WHERE (id = 1)
1478 1479 1480 |
# File 'lib/sequel/model/base.rb', line 1478 def update_except(hash, *except) update_restricted(hash, false, except.flatten) end |
#update_fields(hash, fields, opts = nil) ⇒ Object
Update the instances values by calling set_fields with the arguments, then saves any changes to the record. Returns self.
artist.update_fields({:name=>'Jim'}, [:name])
# UPDATE artists SET name = 'Jim' WHERE (id = 1)
artist.update_fields({:hometown=>'LA'}, [:name])
# UPDATE artists SET name = NULL WHERE (id = 1)
1490 1491 1492 1493 |
# File 'lib/sequel/model/base.rb', line 1490 def update_fields(hash, fields, opts=nil) set_fields(hash, fields, opts) save_changes end |
#update_only(hash, *only) ⇒ Object
Update the values using the entries in the hash, only if the key is included in only. It may be a better idea to use update_fields instead of this method.
artist.update_only({:name=>'Jim'}, :name)
# UPDATE artists SET name = 'Jim' WHERE (id = 1)
artist.update_only({:hometown=>'LA'}, :name) # Raise Error
1503 1504 1505 |
# File 'lib/sequel/model/base.rb', line 1503 def update_only(hash, *only) update_restricted(hash, only.flatten, false) end |
#valid?(opts = {}) ⇒ Boolean
Validates the object and returns true if no errors are reported.
artist(:name=>'Valid').valid? # => true
artist(:name=>'Invalid').valid? # => false
artist.errors. # => ['name cannot be Invalid']
1521 1522 1523 |
# File 'lib/sequel/model/base.rb', line 1521 def valid?(opts = {}) _valid?(false, opts) end |
#validate ⇒ Object
Validates the object. If the object is invalid, errors should be added to the errors attribute. By default, does nothing, as all models are valid by default. See the “Model Validations” guide. for details about validation. Should not be called directly by user code, call valid? instead to check if an object is valid.
1513 1514 |
# File 'lib/sequel/model/base.rb', line 1513 def validate end |