Method: Sequel::Model::InstanceMethods#set_fields
- Defined in:
- lib/sequel/model/base.rb
#set_fields(hash, fields, opts = nil) ⇒ Object
For each of the fields in the given array fields, call the setter method with the value of that hash entry for the field. Returns self.
You can provide an options hash, with the following options currently respected:
- :missing
-
Can be set to :skip to skip missing entries or :raise to raise an Error for missing entries. The default behavior is not to check for missing entries, in which case the default value is used. To be friendly with most web frameworks, the missing check will also check for the string version of the argument in the hash if given a symbol.
Examples:
artist.set_fields({:name=>'Jim'}, [:name])
artist.name # => 'Jim'
artist.set_fields({:hometown=>'LA'}, [:name])
artist.name # => nil
artist.hometown # => 'Sac'
artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:skip)
artist.name # => 'Jim'
artist.name # => 'Jim'
artist.set_fields({}, [:name], :missing=>:raise)
# Sequel::Error raised
1508 1509 1510 1511 1512 1513 1514 1515 1516 1517 1518 1519 1520 1521 1522 1523 1524 1525 1526 1527 1528 1529 1530 1531 1532 1533 1534 1535 1536 1537 1538 |
# File 'lib/sequel/model/base.rb', line 1508 def set_fields(hash, fields, opts=nil) opts = if opts model..merge(opts) else model. end case opts[:missing] when :skip fields.each do |f| if hash.has_key?(f) send("#{f}=", hash[f]) elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s) send("#{sf}=", hash[sf]) end end when :raise fields.each do |f| if hash.has_key?(f) send("#{f}=", hash[f]) elsif f.is_a?(Symbol) && hash.has_key?(sf = f.to_s) send("#{sf}=", hash[sf]) else raise(Sequel::Error, "missing field in hash: #{f.inspect} not in #{hash.inspect}") end end else fields.each{|f| send("#{f}=", hash[f])} end self end |