Method: Sequel::Model::InstanceMethods#values
- Defined in:
- lib/sequel/model/base.rb
#values ⇒ Object (readonly) Also known as: to_hash, _insert_values
The hash of attribute values. Keys are symbols with the names of the underlying database columns. The returned hash is a reference to the receiver’s values hash, and modifying it will also modify the receiver’s values.
Artist.new(name: 'Bob').values # => {:name=>'Bob'}
Artist[1].values # => {:id=>1, :name=>'Jim', ...}
1062 1063 1064 |
# File 'lib/sequel/model/base.rb', line 1062 def values @values end |