Class: Sequel::MySQL::Dataset
- Includes:
- DatasetMethods, StoredProcedures
- Defined in:
- lib/sequel/adapters/mysql.rb
Overview
Dataset class for MySQL datasets accessed via the native driver.
Defined Under Namespace
Modules: CallableStatementMethods, PreparedStatementMethods, StoredProcedureMethods
Constant Summary
Constants included from DatasetMethods
Sequel::MySQL::DatasetMethods::BOOL_FALSE, Sequel::MySQL::DatasetMethods::BOOL_TRUE, Sequel::MySQL::DatasetMethods::COMMA_SEPARATOR, Sequel::MySQL::DatasetMethods::DELETE_CLAUSE_METHODS, Sequel::MySQL::DatasetMethods::INSERT_CLAUSE_METHODS, Sequel::MySQL::DatasetMethods::SELECT_CLAUSE_METHODS, Sequel::MySQL::DatasetMethods::UPDATE_CLAUSE_METHODS
Constants inherited from Dataset
Dataset::AND_SEPARATOR, Dataset::ARRAY_ACCESS_ERROR_MSG, Dataset::BOOL_FALSE, Dataset::BOOL_TRUE, Dataset::COLUMN_CHANGE_OPTS, Dataset::COLUMN_REF_RE1, Dataset::COLUMN_REF_RE2, Dataset::COLUMN_REF_RE3, Dataset::COMMA_SEPARATOR, Dataset::COUNT_FROM_SELF_OPTS, Dataset::COUNT_OF_ALL_AS_COUNT, Dataset::DATASET_ALIAS_BASE_NAME, Dataset::DELETE_CLAUSE_METHODS, Dataset::FROM_SELF_KEEP_OPTS, Dataset::GET_ERROR_MSG, Dataset::IMPORT_ERROR_MSG, Dataset::INSERT_CLAUSE_METHODS, Dataset::IS_LITERALS, Dataset::IS_OPERATORS, Dataset::MAP_ERROR_MSG, Dataset::MUTATION_METHODS, Dataset::NOTIMPL_MSG, Dataset::NULL, Dataset::N_ARITY_OPERATORS, Dataset::PREPARED_ARG_PLACEHOLDER, Dataset::QUALIFY_KEYS, Dataset::QUESTION_MARK, Dataset::SELECT_CLAUSE_METHODS, Dataset::SQL_WITH, Dataset::STANDARD_TIMESTAMP_FORMAT, Dataset::STOCK_COUNT_OPTS, Dataset::TIMESTAMP_FORMAT, Dataset::TWO_ARITY_OPERATORS, Dataset::UPDATE_CLAUSE_METHODS, Dataset::WILDCARD, Dataset::WITH_SUPPORTED
Instance Attribute Summary
Attributes inherited from Dataset
#db, #identifier_input_method, #identifier_output_method, #opts, #quote_identifiers, #row_proc
Instance Method Summary collapse
-
#call(type, bind_arguments = {}, values = nil) ⇒ Object
MySQL is different in that it supports prepared statements but not bound variables outside of prepared statements.
-
#delete ⇒ Object
Delete rows matching this dataset.
-
#fetch_rows(sql, &block) ⇒ Object
Yield all rows matching this dataset.
-
#graph ⇒ Object
Don’t allow graphing a dataset that splits multiple statements.
-
#insert(*values) ⇒ Object
Insert a new value into this dataset.
-
#prepare(type, name = nil, values = nil) ⇒ Object
Store the given type of prepared statement in the associated database with the given name.
-
#replace(*args) ⇒ Object
Replace (update or insert) the matching row.
-
#split_multiple_result_sets ⇒ Object
Makes each yield arrays of rows, with each array containing the rows for a given result set.
-
#update(values = {}) ⇒ Object
Update the matching rows.
Methods included from DatasetMethods
#complex_expression_sql, #full_text_search, #full_text_sql, #having, #insert_ignore, #join_table, #join_type_sql, #multi_insert_sql, #on_duplicate_key_update, #quoted_identifier, #replace_sql, #supports_distinct_on?, #supports_intersect_except?, #supports_timestamp_usecs?
Methods inherited from Dataset
#<<, #[], #[]=, #add_graph_aliases, #aliased_expression_sql, #all, #and, #array_sql, #as, #avg, #case_expression_sql, #cast_sql, clause_methods, #clone, #column_all_sql, #columns, #columns!, #complex_expression_sql, #constant_sql, #count, def_mutation_method, #def_mutation_method, #delete_sql, #distinct, #each, #each_page, #empty?, #except, #exclude, #exists, #filter, #first, #first_source_alias, #from, #from_self, #function_sql, #get, #grep, #group, #group_and_count, #having, #import, #initialize, #insert_multiple, #insert_sql, #inspect, #intersect, #interval, #invert, #join_clause_sql, #join_on_clause_sql, #join_table, #join_using_clause_sql, #last, #limit, #literal, #map, #max, #min, #multi_insert, #multi_insert_sql, #naked, #or, #order, #order_more, #ordered_expression_sql, #paginate, #placeholder_literal_string_sql, #print, #qualified_identifier_sql, #qualify, #qualify_to, #qualify_to_first_source, #query, #quote_identifier, #quote_identifiers?, #quote_schema_table, #quoted_identifier, #range, #requires_sql_standard_datetimes?, #reverse_order, #schema_and_table, #select, #select_all, #select_more, #select_sql, #server, #set, #set_defaults, #set_graph_aliases, #set_overrides, #single_record, #single_value, #sql, #subscript_sql, #sum, #supports_cte?, #supports_distinct_on?, #supports_intersect_except?, #supports_intersect_except_all?, #supports_is_true?, #supports_timestamp_timezones?, #supports_timestamp_usecs?, #supports_window_functions?, #to_csv, #to_hash, #truncate, #truncate_sql, #unfiltered, #ungraphed, #ungrouped, #union, #unlimited, #unordered, #update_sql, #where, #window_function_sql, #window_sql, #with, #with_recursive, #with_sql
Methods included from Sequel::Metaprogramming
Constructor Details
This class inherits a constructor from Sequel::Dataset
Instance Method Details
#call(type, bind_arguments = {}, values = nil) ⇒ Object
MySQL is different in that it supports prepared statements but not bound variables outside of prepared statements. The default implementation breaks the use of subselects in prepared statements, so extend the temporary prepared statement that this creates with a module that fixes it.
305 306 307 308 309 |
# File 'lib/sequel/adapters/mysql.rb', line 305 def call(type, bind_arguments={}, values=nil) ps = to_prepared_statement(type, values) ps.extend(CallableStatementMethods) ps.call(bind_arguments) end |
#delete ⇒ Object
Delete rows matching this dataset
312 313 314 |
# File 'lib/sequel/adapters/mysql.rb', line 312 def delete execute_dui(delete_sql){|c| c.affected_rows} end |
#fetch_rows(sql, &block) ⇒ Object
Yield all rows matching this dataset. If the dataset is set to split multiple statements, yield arrays of hashes one per statement instead of yielding results for all statements as hashes.
319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 |
# File 'lib/sequel/adapters/mysql.rb', line 319 def fetch_rows(sql, &block) execute(sql) do |r| i = -1 cols = r.fetch_fields.map{|f| [output_identifier(f.name), MYSQL_TYPES[f.type], i+=1]} @columns = cols.map{|c| c.first} if opts[:split_multiple_result_sets] s = [] yield_rows(r, cols){|h| s << h} yield s else yield_rows(r, cols, &block) end end self end |
#graph ⇒ Object
Don’t allow graphing a dataset that splits multiple statements
336 337 338 339 |
# File 'lib/sequel/adapters/mysql.rb', line 336 def graph(*) raise(Error, "Can't graph a dataset that splits multiple result sets") if opts[:split_multiple_result_sets] super end |
#insert(*values) ⇒ Object
Insert a new value into this dataset
342 343 344 |
# File 'lib/sequel/adapters/mysql.rb', line 342 def insert(*values) execute_dui(insert_sql(*values)){|c| c.insert_id} end |
#prepare(type, name = nil, values = nil) ⇒ Object
Store the given type of prepared statement in the associated database with the given name.
348 349 350 351 352 353 354 355 356 |
# File 'lib/sequel/adapters/mysql.rb', line 348 def prepare(type, name=nil, values=nil) ps = to_prepared_statement(type, values) ps.extend(PreparedStatementMethods) if name ps.prepared_statement_name = name db.prepared_statements[name] = ps end ps end |
#replace(*args) ⇒ Object
Replace (update or insert) the matching row.
359 360 361 |
# File 'lib/sequel/adapters/mysql.rb', line 359 def replace(*args) execute_dui(replace_sql(*args)){|c| c.insert_id} end |
#split_multiple_result_sets ⇒ Object
Makes each yield arrays of rows, with each array containing the rows for a given result set. Does not work with graphing. So you can submit SQL with multiple statements and easily determine which statement returned which results.
Modifies the row_proc of the returned dataset so that it still works as expected (running on the hashes instead of on the arrays of hashes). If you modify the row_proc afterward, note that it will receive an array of hashes instead of a hash.
372 373 374 375 376 377 |
# File 'lib/sequel/adapters/mysql.rb', line 372 def split_multiple_result_sets raise(Error, "Can't split multiple statements on a graphed dataset") if opts[:graph] ds = clone(:split_multiple_result_sets=>true) ds.row_proc = proc{|x| x.map{|h| row_proc.call(h)}} if row_proc ds end |
#update(values = {}) ⇒ Object
Update the matching rows.
380 381 382 |
# File 'lib/sequel/adapters/mysql.rb', line 380 def update(values={}) execute_dui(update_sql(values)){|c| c.affected_rows} end |