Module: Sequel::MySQL::DatasetMethods

Includes:
Dataset::Replace
Included in:
DataObjects::MySQL::Dataset, Dataset, Sequel::Mysql2::Dataset, Swift::MySQL::Dataset
Defined in:
lib/sequel/adapters/shared/mysql.rb

Overview

Dataset methods shared by datasets that use MySQL databases.

Constant Summary collapse

BOOL_TRUE =
'1'.freeze
BOOL_FALSE =
'0'.freeze
COMMA_SEPARATOR =
', '.freeze
FOR_SHARE =
' LOCK IN SHARE MODE'.freeze
SQL_CALC_FOUND_ROWS =
' SQL_CALC_FOUND_ROWS'.freeze
APOS =
Dataset::APOS
APOS_RE =
Dataset::APOS_RE
DOUBLE_APOS =
Dataset::DOUBLE_APOS
SPACE =
Dataset::SPACE
PAREN_OPEN =
Dataset::PAREN_OPEN
PAREN_CLOSE =
Dataset::PAREN_CLOSE
NOT_SPACE =
Dataset::NOT_SPACE
FROM =
Dataset::FROM
COMMA =
Dataset::COMMA
LIMIT =
Dataset::LIMIT
GROUP_BY =
Dataset::GROUP_BY
ESCAPE =
Dataset::ESCAPE
BACKSLASH =
Dataset::BACKSLASH
REGEXP =
'REGEXP'.freeze
LIKE =
'LIKE'.freeze
BINARY =
'BINARY '.freeze
CONCAT =
"CONCAT".freeze
CAST_BITCOMP_OPEN =
"CAST(~".freeze
CAST_BITCOMP_CLOSE =
" AS SIGNED INTEGER)".freeze
STRAIGHT_JOIN =
'STRAIGHT_JOIN'.freeze
NATURAL_LEFT_JOIN =
'NATURAL LEFT JOIN'.freeze
BACKTICK =
'`'.freeze
BACKTICK_RE =
/`/.freeze
DOUBLE_BACKTICK =
'``'.freeze
EMPTY_COLUMNS =
" ()".freeze
EMPTY_VALUES =
" VALUES ()".freeze
IGNORE =
" IGNORE".freeze
ON_DUPLICATE_KEY_UPDATE =
" ON DUPLICATE KEY UPDATE ".freeze
EQ_VALUES =
'=VALUES('.freeze
EQ =
'='.freeze
WITH_ROLLUP =
' WITH ROLLUP'.freeze
MATCH_AGAINST =
["(MATCH ".freeze, " AGAINST (".freeze, "))".freeze].freeze
MATCH_AGAINST_BOOLEAN =
["(MATCH ".freeze, " AGAINST (".freeze, " IN BOOLEAN MODE))".freeze].freeze
EXPLAIN =
'EXPLAIN '.freeze
EXPLAIN_EXTENDED =
'EXPLAIN EXTENDED '.freeze
BACKSLASH_RE =
/\\/.freeze
QUAD_BACKSLASH =
"\\\\\\\\".freeze
BLOB_START =
"0x".freeze
EMPTY_BLOB =
"''".freeze
HSTAR =
"H*".freeze
CURRENT_TIMESTAMP_56 =
'CURRENT_TIMESTAMP(6)'.freeze
ONLY_OFFSET =

Comes directly from MySQL’s documentation, used for queries with limits without offsets

",18446744073709551615".freeze

Constants included from Dataset::Replace

Dataset::Replace::INSERT, Dataset::Replace::REPLACE

Instance Method Summary collapse

Methods included from Dataset::Replace

#multi_replace, #replace, #replace_sql, #supports_replace?

Instance Method Details

#calc_found_rowsObject

Sets up the select methods to use SQL_CALC_FOUND_ROWS option.

dataset.calc_found_rows.limit(10)
# SELECT SQL_CALC_FOUND_ROWS * FROM table LIMIT 10


668
669
670
# File 'lib/sequel/adapters/shared/mysql.rb', line 668

def calc_found_rows
  clone(:calc_found_rows => true)
end

#complex_expression_sql_append(sql, op, args) ⇒ Object

MySQL specific syntax for LIKE/REGEXP searches, as well as string concatenation.



608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
# File 'lib/sequel/adapters/shared/mysql.rb', line 608

def complex_expression_sql_append(sql, op, args)
  case op
  when :IN, :"NOT IN"
    ds = args.at(1)
    if ds.is_a?(Sequel::Dataset) && ds.opts[:limit]
      super(sql, op, [args.at(0), ds.from_self])
    else
      super
    end
  when :~, :'!~', :'~*', :'!~*', :LIKE, :'NOT LIKE', :ILIKE, :'NOT ILIKE'
    sql << PAREN_OPEN
    literal_append(sql, args.at(0))
    sql << SPACE
    sql << 'NOT ' if [:'NOT LIKE', :'NOT ILIKE', :'!~', :'!~*'].include?(op)
    sql << ([:~, :'!~', :'~*', :'!~*'].include?(op) ? REGEXP : LIKE)
    sql << SPACE
    sql << BINARY if [:~, :'!~', :LIKE, :'NOT LIKE'].include?(op)
    literal_append(sql, args.at(1))
    if [:LIKE, :'NOT LIKE', :ILIKE, :'NOT ILIKE'].include?(op)
      sql << ESCAPE
      literal_append(sql, BACKSLASH)
    end
    sql << PAREN_CLOSE
  when :'||'
    if args.length > 1
      sql << CONCAT
      array_sql_append(sql, args)
    else
      literal_append(sql, args.at(0))
    end
  when :'B~'
    sql << CAST_BITCOMP_OPEN
    literal_append(sql, args.at(0))
    sql << CAST_BITCOMP_CLOSE
  else
    super
  end
end

#constant_sql_append(sql, constant) ⇒ Object

MySQL’s CURRENT_TIMESTAMP does not use fractional seconds, even if the database itself supports fractional seconds. If MySQL 5.6.4+ is being used, use a value that will return fractional seconds.



651
652
653
654
655
656
657
# File 'lib/sequel/adapters/shared/mysql.rb', line 651

def constant_sql_append(sql, constant)
  if constant == :CURRENT_TIMESTAMP && supports_timestamp_usecs?
    sql << CURRENT_TIMESTAMP_56
  else
    super
  end
end

#distinct(*args) ⇒ Object

Use GROUP BY instead of DISTINCT ON if arguments are provided.



660
661
662
# File 'lib/sequel/adapters/shared/mysql.rb', line 660

def distinct(*args)
  args.empty? ? super : group(*args)
end

#explain(opts = OPTS) ⇒ Object

Return the results of an EXPLAIN query as a string. Options:

:extended

Use EXPLAIN EXPTENDED instead of EXPLAIN if true.



674
675
676
677
678
679
680
681
# File 'lib/sequel/adapters/shared/mysql.rb', line 674

def explain(opts=OPTS)
  # Load the PrettyTable class, needed for explain output
  Sequel.extension(:_pretty_table) unless defined?(Sequel::PrettyTable)

  ds = db.send(:metadata_dataset).with_sql((opts[:extended] ? EXPLAIN_EXTENDED : EXPLAIN) + select_sql).naked
  rows = ds.all
  Sequel::PrettyTable.string(rows, ds.columns)
end

#for_shareObject

Return a cloned dataset which will use LOCK IN SHARE MODE to lock returned rows.



684
685
686
# File 'lib/sequel/adapters/shared/mysql.rb', line 684

def for_share
  lock_style(:share)
end

#full_text_search(cols, terms, opts = OPTS) ⇒ Object

Adds full text filter



689
690
691
# File 'lib/sequel/adapters/shared/mysql.rb', line 689

def full_text_search(cols, terms, opts = OPTS)
  filter(full_text_sql(cols, terms, opts))
end

#full_text_sql(cols, terms, opts = OPTS) ⇒ Object

MySQL specific full text search syntax.



694
695
696
697
# File 'lib/sequel/adapters/shared/mysql.rb', line 694

def full_text_sql(cols, terms, opts = OPTS)
  terms = terms.join(' ') if terms.is_a?(Array)
  SQL::PlaceholderLiteralString.new((opts[:boolean] ? MATCH_AGAINST_BOOLEAN : MATCH_AGAINST), [Array(cols), terms])
end

#insert_ignoreObject

Sets up the insert methods to use INSERT IGNORE. Useful if you have a unique key and want to just skip inserting rows that violate the unique key restriction.

dataset.insert_ignore.multi_insert(
 [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
)
# INSERT IGNORE INTO tablename (name, value) VALUES (a, 1), (b, 2)


728
729
730
# File 'lib/sequel/adapters/shared/mysql.rb', line 728

def insert_ignore
  clone(:insert_ignore=>true)
end

#join_table(type, table, expr = nil, opts = OPTS, &block) ⇒ Object

Transforms an CROSS JOIN to an INNER JOIN if the expr is not nil. Raises an error on use of :full_outer type, since MySQL doesn’t support it.

Raises:



701
702
703
704
705
# File 'lib/sequel/adapters/shared/mysql.rb', line 701

def join_table(type, table, expr=nil, opts=OPTS, &block)
  type = :inner if (type == :cross) && !expr.nil?
  raise(Sequel::Error, "MySQL doesn't support FULL OUTER JOIN") if type == :full_outer
  super(type, table, expr, opts, &block)
end

#join_type_sql(join_type) ⇒ Object

Transforms :natural_inner to NATURAL LEFT JOIN and straight to STRAIGHT_JOIN.



709
710
711
712
713
714
715
716
717
718
# File 'lib/sequel/adapters/shared/mysql.rb', line 709

def join_type_sql(join_type)
  case join_type
  when :straight
    STRAIGHT_JOIN
  when :natural_inner
    NATURAL_LEFT_JOIN
  else
    super
  end
end

#on_duplicate_key_update(*args) ⇒ Object

Sets up the insert methods to use ON DUPLICATE KEY UPDATE If you pass no arguments, ALL fields will be updated with the new values. If you pass the fields you want then ONLY those field will be updated. If you pass a hash you can customize the values (for example, to increment a numeric field).

Useful if you have a unique key and want to update inserting rows that violate the unique key restriction.

dataset.on_duplicate_key_update.multi_insert(
 [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
)
# INSERT INTO tablename (name, value) VALUES (a, 1), (b, 2)
# ON DUPLICATE KEY UPDATE name=VALUES(name), value=VALUES(value)

dataset.on_duplicate_key_update(:value).multi_insert(
  [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
)
# INSERT INTO tablename (name, value) VALUES (a, 1), (b, 2)
# ON DUPLICATE KEY UPDATE value=VALUES(value)

dataset.on_duplicate_key_update(
  :value => Sequel.lit('value + VALUES(value)')
).multi_insert(
  [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
)
# INSERT INTO tablename (name, value) VALUES (a, 1), (b, 2)
# ON DUPLICATE KEY UPDATE value=value + VALUES(value)


761
762
763
# File 'lib/sequel/adapters/shared/mysql.rb', line 761

def on_duplicate_key_update(*args)
  clone(:on_duplicate_key_update => args)
end

#quoted_identifier_append(sql, c) ⇒ Object

MySQL uses the nonstandard ‘ (backtick) for quoting identifiers.



766
767
768
# File 'lib/sequel/adapters/shared/mysql.rb', line 766

def quoted_identifier_append(sql, c)
  sql << BACKTICK << c.to_s.gsub(BACKTICK_RE, DOUBLE_BACKTICK) << BACKTICK
end

#supports_derived_column_lists?Boolean

MySQL does not support derived column lists

Returns:

  • (Boolean)


771
772
773
# File 'lib/sequel/adapters/shared/mysql.rb', line 771

def supports_derived_column_lists?
  false
end

#supports_distinct_on?Boolean

MySQL can emulate DISTINCT ON with its non-standard GROUP BY implementation, though the rows returned cannot be made deterministic through ordering.

Returns:

  • (Boolean)


777
778
779
# File 'lib/sequel/adapters/shared/mysql.rb', line 777

def supports_distinct_on?
  true
end

#supports_group_rollup?Boolean

MySQL supports GROUP BY WITH ROLLUP (but not CUBE)

Returns:

  • (Boolean)


782
783
784
# File 'lib/sequel/adapters/shared/mysql.rb', line 782

def supports_group_rollup?
  true
end

#supports_intersect_except?Boolean

MySQL does not support INTERSECT or EXCEPT

Returns:

  • (Boolean)


787
788
789
# File 'lib/sequel/adapters/shared/mysql.rb', line 787

def supports_intersect_except?
  false
end

#supports_limits_in_correlated_subqueries?Boolean

MySQL does not support limits in correlated subqueries (or any subqueries that use IN).

Returns:

  • (Boolean)


792
793
794
# File 'lib/sequel/adapters/shared/mysql.rb', line 792

def supports_limits_in_correlated_subqueries?
  false
end

#supports_modifying_joins?Boolean

MySQL supports modifying joined datasets

Returns:

  • (Boolean)


797
798
799
# File 'lib/sequel/adapters/shared/mysql.rb', line 797

def supports_modifying_joins?
  true
end

#supports_ordered_distinct_on?Boolean

MySQL’s DISTINCT ON emulation using GROUP BY does not respect the queries ORDER BY clause.

Returns:

  • (Boolean)


803
804
805
# File 'lib/sequel/adapters/shared/mysql.rb', line 803

def supports_ordered_distinct_on?
  false
end

#supports_regexp?Boolean

MySQL supports pattern matching via regular expressions

Returns:

  • (Boolean)


808
809
810
# File 'lib/sequel/adapters/shared/mysql.rb', line 808

def supports_regexp?
  true
end

#supports_timestamp_usecs?Boolean

MySQL does support fractional timestamps in literal timestamps, but it ignores them. Also, using them seems to cause problems on 1.9. Since they are ignored anyway, not using them is probably best.

Returns:

  • (Boolean)


815
816
817
# File 'lib/sequel/adapters/shared/mysql.rb', line 815

def supports_timestamp_usecs?
  db.supports_timestamp_usecs?
end

#update_ignoreObject

Sets up the update methods to use UPDATE IGNORE. Useful if you have a unique key and want to just skip updating rows that violate the unique key restriction.

dataset.update_ignore.update({:name => 'a', :value => 1})
# UPDATE IGNORE tablename SET name = 'a', value = 1


825
826
827
# File 'lib/sequel/adapters/shared/mysql.rb', line 825

def update_ignore
  clone(:update_ignore=>true)
end