Class: Sequel::Mysql2::Database
- Includes:
- Sequel::MySQL::DatabaseMethods, Sequel::MySQL::MysqlMysql2::DatabaseMethods, Sequel::MySQL::PreparedStatements::DatabaseMethods
- Defined in:
- lib/sequel/adapters/mysql2.rb
Constant Summary
Constants included from Sequel::MySQL::MysqlMysql2::DatabaseMethods
Sequel::MySQL::MysqlMysql2::DatabaseMethods::MYSQL_DATABASE_DISCONNECT_ERRORS
Constants included from Sequel::MySQL::DatabaseMethods
Sequel::MySQL::DatabaseMethods::CAST_TYPES, Sequel::MySQL::DatabaseMethods::COLUMN_DEFINITION_ORDER
Instance Attribute Summary collapse
-
#convert_tinyint_to_bool ⇒ Object
Whether to convert tinyint columns to bool for this database.
Attributes included from Sequel::MySQL::DatabaseMethods
#default_charset, #default_collate, #default_engine
Instance Method Summary collapse
-
#connect(server) ⇒ Object
Connect to the database.
- #execute_dui(sql, opts = OPTS) ⇒ Object
- #execute_insert(sql, opts = OPTS) ⇒ Object
- #freeze ⇒ Object
-
#server_version(_server = nil) ⇒ Object
Return the version of the MySQL server to which we are connecting.
Methods included from Sequel::MySQL::MysqlMysql2::DatabaseMethods
Methods included from Sequel::MySQL::DatabaseMethods
#cast_type_literal, #commit_prepared_transaction, #database_type, #foreign_key_list, #global_index_namespace?, #indexes, #mariadb?, #rename_tables, #rollback_prepared_transaction, #supports_create_table_if_not_exists?, #supports_generated_columns?, #supports_prepared_transactions?, #supports_savepoints?, #supports_savepoints_in_prepared_transactions?, #supports_timestamp_usecs?, #supports_transaction_isolation_levels?, #tables, #views, #with_advisory_lock
Instance Attribute Details
#convert_tinyint_to_bool ⇒ Object
Whether to convert tinyint columns to bool for this database
23 24 25 |
# File 'lib/sequel/adapters/mysql2.rb', line 23 def @convert_tinyint_to_bool end |
Instance Method Details
#connect(server) ⇒ Object
Connect to the database. In addition to the usual database options, the following options have effect:
:auto_is_null :: Set to true to use MySQL default behavior of having a filter for an autoincrement column equals NULL to return the last inserted row. :charset :: Same as :encoding (:encoding takes precendence) :encoding :: Set all the related character sets for this connection (connection, client, database, server, and results).
The options hash is also passed to mysql2, and can include mysql2 options such as :local_infile.
37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 |
# File 'lib/sequel/adapters/mysql2.rb', line 37 def connect(server) opts = server_opts(server) opts[:username] ||= opts.delete(:user) opts[:flags] ||= 0 opts[:flags] |= ::Mysql2::Client::FOUND_ROWS if ::Mysql2::Client.const_defined?(:FOUND_ROWS) opts[:encoding] ||= opts[:charset] conn = ::Mysql2::Client.new(opts) conn..merge!(:symbolize_keys=>true, :cache_rows=>false) if NativePreparedStatements conn.instance_variable_set(:@sequel_default_query_options, conn..dup) end sqls = mysql_connection_setting_sqls # Set encoding a slightly different way after connecting, # in case the READ_DEFAULT_GROUP overrode the provided encoding. # Doesn't work across implicit reconnects, but Sequel doesn't turn on # that feature. if encoding = opts[:encoding] sqls.unshift("SET NAMES #{conn.escape(encoding.to_s)}") end sqls.each{|sql| log_connection_yield(sql, conn){conn.query(sql)}} add_prepared_statements_cache(conn) conn end |
#execute_dui(sql, opts = OPTS) ⇒ Object
66 67 68 |
# File 'lib/sequel/adapters/mysql2.rb', line 66 def execute_dui(sql, opts=OPTS) execute(sql, opts){|c| return c.affected_rows} end |
#execute_insert(sql, opts = OPTS) ⇒ Object
70 71 72 |
# File 'lib/sequel/adapters/mysql2.rb', line 70 def execute_insert(sql, opts=OPTS) execute(sql, opts){|c| return c.last_id} end |
#freeze ⇒ Object
74 75 76 77 |
# File 'lib/sequel/adapters/mysql2.rb', line 74 def freeze server_version super end |
#server_version(_server = nil) ⇒ Object
Return the version of the MySQL server to which we are connecting.
80 81 82 |
# File 'lib/sequel/adapters/mysql2.rb', line 80 def server_version(_server=nil) @server_version ||= super() end |