Module: Sequel::Plugins::ClassTableInheritance::ClassMethods

Defined in:
lib/sequel/plugins/class_table_inheritance.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#cti_instance_datasetObject (readonly)

The dataset that table instance datasets are based on. Used for database modifications



228
229
230
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 228

def cti_instance_dataset
  @cti_instance_dataset
end

#cti_modelsObject (readonly)

An array of each model in the inheritance hierarchy that uses an backed by a new table.



213
214
215
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 213

def cti_models
  @cti_models
end

#cti_table_columnsObject (readonly)

An array of column symbols for the backing database table, giving the columns to update in each backing database table.



224
225
226
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 224

def cti_table_columns
  @cti_table_columns
end

#cti_table_mapObject (readonly)

A hash with class name symbol keys and table name symbol values. Specified with the :table_map option to the plugin, and used if the implicit naming is incorrect.



238
239
240
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 238

def cti_table_map
  @cti_table_map
end

#cti_tablesObject (readonly)

An array of table symbols that back this model. The first is cti_base_model table symbol, and the last is the current model table symbol.



233
234
235
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 233

def cti_tables
  @cti_tables
end

Instance Method Details

#cti_base_modelObject

The parent/root/base model for this class table inheritance hierarchy. This is the only model in the hierarchy that loads the class_table_inheritance plugin. For backwards compatibility.



218
219
220
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 218

def cti_base_model
  @cti_models.first
end

#cti_columnsObject

Hash with table name symbol keys and arrays of column symbol values, giving the columns to update in each backing database table. For backwards compatibility.



243
244
245
246
247
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 243

def cti_columns
  h = {}
  cti_models.each { |m| h[m.cti_table_name] = m.cti_table_columns }
  h
end

#cti_keyObject

Alias to sti_key, for backwards compatibility.



250
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 250

def cti_key; sti_key; end

#cti_model_mapObject

Alias to sti_model_map, for backwards compatibility.



253
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 253

def cti_model_map; sti_model_map; end

#cti_table_nameObject

The name of the most recently joined table.



338
339
340
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 338

def cti_table_name
  cti_tables ? cti_tables.last : dataset.first_source_alias
end

#freezeObject

Freeze CTI information when freezing model class.



256
257
258
259
260
261
262
263
264
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 256

def freeze
  @cti_models.freeze
  @cti_tables.freeze
  @cti_instance_dataset.freeze
  @cti_table_columns.freeze
  @cti_table_map.freeze

  super
end

#inherited(subclass) ⇒ Object



268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 268

def inherited(subclass)
  ds = sti_dataset

  # Prevent inherited in model/base.rb from setting the dataset
  subclass.instance_eval { @dataset = nil }

  super

  # Set table if this is a class table inheritance
  table = nil
  columns = nil
  if (n = subclass.name) && !n.empty?
    if table = cti_table_map[n.to_sym]
      columns = db.from(table).columns
    else
      table = subclass.implicit_table_name
      columns = check_non_connection_error{db.from(table).columns}
      table = nil if !columns || columns.empty?
    end
  end
  table = nil if table && (table == cti_table_name)

  return unless table

  pk = primary_key
  subclass.instance_eval do
    if cti_tables.length == 1
      ds = ds.select(*self.columns.map{|cc| Sequel.qualify(cti_table_name, Sequel.identifier(cc))})
    end
    cols = columns - [pk]
    unless (cols & ds.columns).empty?
      Sequel::Deprecation.deprecate('Using class_table_inheritance with duplicate column names in subclass tables (other than the primary key column)', 'Make sure all tables used have unique column names, or implement support for handling duplicate column names in the class_table_inheritance plugin')
    end
    sel_app = cols.map{|cc| Sequel.qualify(table, Sequel.identifier(cc))}
    @sti_dataset = ds = ds.join(table, pk=>pk).select_append(*sel_app)

    if @cti_alias
      ds = ds.from_self(:alias=>@cti_alias)
    end

    set_dataset(ds)
    set_columns(self.columns)
    @dataset = @dataset.with_row_proc(lambda{|r| subclass.sti_load(r)})
    cols.each{|a| define_lazy_attribute_getter(a, :dataset=>dataset, :table=>@cti_alias||table)}

    @cti_models += [self]
    @cti_tables += [table]
    @cti_table_columns = columns
    @cti_instance_dataset = db.from(table)

    cti_tables.reverse_each do |ct|
      db.schema(ct).each{|sk,v| db_schema[sk] = v}
    end
  end
end

#sti_class_from_key(key) ⇒ Object



342
343
344
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 342

def sti_class_from_key(key)
  sti_class(sti_model_map[key])
end

#table_nameObject

The table name for the current model class’s main table.



325
326
327
328
329
330
331
332
333
334
335
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 325

def table_name
  if cti_tables
    if @cti_alias
      @cti_alias
    else
      cti_tables.last
    end
  else
    super
  end
end