Module: Sequel::Plugins::ClassTableInheritance::InstanceMethods
- Defined in:
- lib/sequel/plugins/class_table_inheritance.rb
Instance Method Summary collapse
-
#delete ⇒ Object
Delete the row from all backing tables, starting from the most recent table and going through all superclasses.
-
#use_prepared_statements_for?(type) ⇒ Boolean
Don’t allow use of prepared statements.
Instance Method Details
#delete ⇒ Object
Delete the row from all backing tables, starting from the most recent table and going through all superclasses.
362 363 364 365 366 367 368 |
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 362 def delete raise Sequel::Error, "can't delete frozen object" if frozen? model.cti_models.reverse_each do |m| cti_this(m).delete end self end |
#use_prepared_statements_for?(type) ⇒ Boolean
Don’t allow use of prepared statements.
371 372 373 |
# File 'lib/sequel/plugins/class_table_inheritance.rb', line 371 def use_prepared_statements_for?(type) false end |