Module: Sequel::Plugins::SplitValues::InstanceMethods

Defined in:
lib/sequel/plugins/split_values.rb

Instance Method Summary collapse

Instance Method Details

#[](k) ⇒ Object

If there isn’t an entry in the values hash, but there is a noncolumn_values hash, look in that hash for the value.



49
50
51
52
53
54
55
# File 'lib/sequel/plugins/split_values.rb', line 49

def [](k)
  if  (res = super).nil?
    @noncolumn_values[k] if !@values.has_key?(k) && @noncolumn_values
  else
    res
  end
end

#split_noncolumn_valuesObject

Check all entries in the values hash. If any of the keys are not columns, move the entry into the noncolumn_values hash.



59
60
61
62
63
64
65
66
# File 'lib/sequel/plugins/split_values.rb', line 59

def split_noncolumn_values
  @values.keys.each do |k|
    unless columns.include?(k)
      (@noncolumn_values ||= {})[k] = @values.delete(k)
    end
  end
  self
end