Class: Sequel::Postgres::AutoParameterizeInArray::TreatStringListAsUntypedArray::ParameterizedUntypedPGArray
- Inherits:
-
SQL::Wrapper
- Object
- SQL::Expression
- SQL::GenericExpression
- SQL::Wrapper
- Sequel::Postgres::AutoParameterizeInArray::TreatStringListAsUntypedArray::ParameterizedUntypedPGArray
- Defined in:
- lib/sequel/extensions/pg_auto_parameterize_in_array.rb
Overview
Wrapper for untyped PGArray values that will be parameterized directly into the query. This should only be used in cases where you know the value should be added as a query parameter.
Instance Attribute Summary
Attributes inherited from SQL::Wrapper
Instance Method Summary collapse
Methods inherited from SQL::Wrapper
Methods included from SQL::IsDistinctFrom::Methods
Methods included from SQLite::JSONOpMethods
#sqlite_json_op, #sqlite_jsonb_op
Methods included from HStoreOpMethods
Methods included from RangeOpMethods
Methods included from Sequel::Postgres::ArrayOpMethods
Methods included from JSONOpMethods
Methods included from InetOpMethods
Methods included from PGRowOp::ExpressionMethods
Methods included from SQL::SubscriptMethods
Methods included from SQL::StringMethods
#escaped_ilike, #escaped_like, #ilike, #like
Methods included from SQL::PatternMatchMethods
Methods included from SQL::OrderMethods
Methods included from SQL::NumericMethods
Methods included from SQL::ComplexExpressionMethods
#extract, #sql_boolean, #sql_number, #sql_string
Methods included from SQL::CastMethods
#cast, #cast_numeric, #cast_string
Methods included from SQL::BooleanMethods
Methods included from SQL::AliasMethods
Methods inherited from SQL::Expression
#==, attr_reader, #clone, #eql?, #hash, inherited, #inspect
Constructor Details
This class inherits a constructor from Sequel::SQL::Wrapper
Instance Method Details
#to_s_append(ds, sql) ⇒ Object
60 61 62 |
# File 'lib/sequel/extensions/pg_auto_parameterize_in_array.rb', line 60 def to_s_append(ds, sql) sql.add_arg(@value) end |