Class: Sequel::Postgres::Database
- Includes:
- DatabaseMethods
- Defined in:
- lib/sequel/adapters/postgres.rb
Overview
Database class for PostgreSQL databases used with Sequel and the pg, postgres, or postgres-pr driver.
Constant Summary collapse
- INFINITE_TIMESTAMP_STRINGS =
['infinity'.freeze, '-infinity'.freeze].freeze
- INFINITE_DATETIME_VALUES =
([PLUS_INFINITY, MINUS_INFINITY] + INFINITE_TIMESTAMP_STRINGS).freeze
- DatasetClass =
self
Constants included from DatabaseMethods
Sequel::Postgres::DatabaseMethods::EXCLUDE_SCHEMAS, Sequel::Postgres::DatabaseMethods::FOREIGN_KEY_LIST_ON_DELETE_MAP, Sequel::Postgres::DatabaseMethods::PREPARED_ARG_PLACEHOLDER, Sequel::Postgres::DatabaseMethods::RE_CURRVAL_ERROR, Sequel::Postgres::DatabaseMethods::SELECT_CUSTOM_SEQUENCE_SQL, Sequel::Postgres::DatabaseMethods::SELECT_PK_SQL, Sequel::Postgres::DatabaseMethods::SELECT_SERIAL_SEQUENCE_SQL, Sequel::Postgres::DatabaseMethods::SYSTEM_TABLE_REGEXP
Constants inherited from Database
Database::ADAPTERS, Database::AUTOINCREMENT, Database::COLUMN_DEFINITION_ORDER, Database::COMMA_SEPARATOR, Database::DEFAULT_JOIN_TABLE_COLUMN_OPTIONS, Database::EXTENSIONS, Database::MSSQL_DEFAULT_RE, Database::MYSQL_TIMESTAMP_RE, Database::NOT_NULL, Database::NULL, Database::POSTGRES_DEFAULT_RE, Database::PRIMARY_KEY, Database::SQL_BEGIN, Database::SQL_COMMIT, Database::SQL_RELEASE_SAVEPOINT, Database::SQL_ROLLBACK, Database::SQL_ROLLBACK_TO_SAVEPOINT, Database::SQL_SAVEPOINT, Database::STRING_DEFAULT_RE, Database::TEMPORARY, Database::TRANSACTION_BEGIN, Database::TRANSACTION_COMMIT, Database::TRANSACTION_ISOLATION_LEVELS, Database::TRANSACTION_ROLLBACK, Database::UNDERSCORE, Database::UNIQUE, Database::UNSIGNED
Instance Attribute Summary collapse
-
#conversion_procs ⇒ Object
readonly
A hash of conversion procs, keyed by type integer (oid) and having callable values for the conversion proc for that type.
-
#convert_infinite_timestamps ⇒ Object
Whether infinite timestamps should be converted on retrieval.
Attributes inherited from Database
#cache_schema, #dataset_class, #default_schema, #log_warn_duration, #loggers, #opts, #pool, #prepared_statements, #sql_log_level, #timezone, #transaction_isolation_level
Instance Method Summary collapse
-
#bound_variable_arg(arg, conn) ⇒ Object
Convert given argument so that it can be used directly by pg.
-
#connect(server) ⇒ Object
Connects to the database.
-
#copy_table(table, opts = {}) ⇒ Object
copy_tableuses PostgreSQL’sCOPYSQL statement to return formatted results directly to the caller. -
#execute(sql, opts = {}, &block) ⇒ Object
Execute the given SQL with the given args on an available connection.
-
#initialize(*args) ⇒ Database
constructor
Add the primary_keys and primary_key_sequences instance variables, so we can get the correct return values for inserted rows.
-
#listen(channels, opts = {}, &block) ⇒ Object
Listens on the given channel (or multiple channels if channel is an array), waiting for notifications.
-
#reset_conversion_procs ⇒ Object
Reset the database’s conversion procs, requires a server query if there any named types.
-
#to_application_timestamp(value) ⇒ Object
If convert_infinite_timestamps is true and the value is infinite, return an appropriate value based on the convert_infinite_timestamps setting.
Methods included from DatabaseMethods
#commit_prepared_transaction, #create_function, #create_language, #create_schema, #create_trigger, #database_type, #drop_function, #drop_language, #drop_schema, #drop_trigger, #foreign_key_list, #indexes, #locks, #notify, #primary_key, #primary_key_sequence, #reset_primary_key_sequence, #rollback_prepared_transaction, #serial_primary_key_options, #server_version, #supports_create_table_if_not_exists?, #supports_drop_table_if_exists?, #supports_prepared_transactions?, #supports_savepoints?, #supports_transaction_isolation_levels?, #supports_transactional_ddl?, #tables, #type_supported?, #views
Methods inherited from Database
#<<, #[], adapter_class, adapter_scheme, #adapter_scheme, #add_column, #add_index, #add_servers, #after_commit, #after_rollback, #alter_table, #alter_table_generator, #call, #cast_type_literal, connect, #create_join_table, #create_or_replace_view, #create_table, #create_table!, #create_table?, #create_table_generator, #create_view, #database_type, #dataset, #disconnect, #drop_column, #drop_index, #drop_join_table, #drop_table, #drop_table?, #drop_view, #dump_foreign_key_migration, #dump_indexes_migration, #dump_schema_cache, #dump_schema_cache?, #dump_schema_migration, #dump_table_schema, #each_server, #execute_ddl, #execute_dui, #execute_insert, #extend_datasets, #extension, #fetch, #foreign_key_list, #from, #from_application_timestamp, #get, #global_index_namespace?, #identifier_input_method, identifier_input_method, #identifier_input_method=, identifier_input_method=, #identifier_output_method, identifier_output_method, #identifier_output_method=, identifier_output_method=, #in_transaction?, #indexes, #inspect, #literal, #load_schema_cache, #load_schema_cache?, #log_exception, #log_info, #log_yield, #logger=, #prepared_statement, #query, #quote_identifiers=, quote_identifiers=, #quote_identifiers?, register_extension, #remove_servers, #rename_column, #rename_table, #run, #schema, #select, #serial_primary_key_options, #servers, #set_column_default, #set_column_type, #set_prepared_statement, single_threaded=, #single_threaded?, #supports_create_table_if_not_exists?, #supports_drop_table_if_exists?, #supports_prepared_transactions?, #supports_savepoints?, #supports_savepoints_in_prepared_transactions?, #supports_transaction_isolation_levels?, #supports_transactional_ddl?, #synchronize, #table_exists?, #tables, #test_connection, #transaction, #typecast_value, #uri, #url, #views
Methods included from Metaprogramming
Constructor Details
#initialize(*args) ⇒ Database
Add the primary_keys and primary_key_sequences instance variables, so we can get the correct return values for inserted rows.
172 173 174 175 176 177 |
# File 'lib/sequel/adapters/postgres.rb', line 172 def initialize(*args) super = false @primary_keys = {} @primary_key_sequences = {} end |
Instance Attribute Details
#conversion_procs ⇒ Object (readonly)
A hash of conversion procs, keyed by type integer (oid) and having callable values for the conversion proc for that type.
162 163 164 |
# File 'lib/sequel/adapters/postgres.rb', line 162 def conversion_procs @conversion_procs end |
#convert_infinite_timestamps ⇒ Object
Whether infinite timestamps should be converted on retrieval. By default, no conversion is done, so an error is raised if you attempt to retrieve an infinite timestamp. You can set this to :nil to convert to nil, :string to leave as a string, or :float to convert to an infinite float.
168 169 170 |
# File 'lib/sequel/adapters/postgres.rb', line 168 def end |
Instance Method Details
#bound_variable_arg(arg, conn) ⇒ Object
Convert given argument so that it can be used directly by pg. Currently, pg doesn’t handle fractional seconds in Time/DateTime or blobs with “0”, and it won’t ever handle Sequel::SQLTime values correctly. Only public for use by the adapter, shouldn’t be used by external code.
183 184 185 186 187 188 189 190 191 192 193 194 |
# File 'lib/sequel/adapters/postgres.rb', line 183 def bound_variable_arg(arg, conn) case arg when Sequel::SQL::Blob conn.escape_bytea(arg) when Sequel::SQLTime literal(arg) when DateTime, Time literal(arg) else arg end end |
#connect(server) ⇒ Object
Connects to the database. In addition to the standard database options, using the :encoding or :charset option changes the client encoding for the connection, :connect_timeout is a connection timeout in seconds, and :sslmode sets whether postgres’s sslmode. :connect_timeout and :ssl_mode are only supported if the pg driver is used.
202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 |
# File 'lib/sequel/adapters/postgres.rb', line 202 def connect(server) opts = server_opts(server) conn = if SEQUEL_POSTGRES_USES_PG connection_params = { :host => opts[:host], :port => opts[:port] || 5432, :dbname => opts[:database], :user => opts[:user], :password => opts[:password], :connect_timeout => opts[:connect_timeout] || 20, :sslmode => opts[:sslmode] }.delete_if { |key, value| blank_object?(value) } Adapter.connect(connection_params) else Adapter.connect( (opts[:host] unless blank_object?(opts[:host])), opts[:port] || 5432, nil, '', opts[:database], opts[:user], opts[:password] ) end if encoding = opts[:encoding] || opts[:charset] if conn.respond_to?(:set_client_encoding) conn.set_client_encoding(encoding) else conn.async_exec("set client_encoding to '#{encoding}'") end end conn.instance_variable_set(:@db, self) conn.instance_variable_set(:@prepared_statements, {}) if SEQUEL_POSTGRES_USES_PG connection_configuration_sqls.each{|sql| conn.execute(sql)} @conversion_procs ||= get_conversion_procs(conn) conn end |
#copy_table(table, opts = {}) ⇒ Object
copy_table uses PostgreSQL’s COPY SQL statement to return formatted results directly to the caller. This method is only supported if pg is the underlying ruby driver. This method should only be called if you want results returned to the client. If you are using COPY FROM or COPY TO with a filename, you should just use run instead of this method. This method does not currently support COPY FROM STDIN, but that may be supported in the future.
The table argument supports the following types:
- String
-
Uses the first argument directly as literal SQL. If you are using a version of PostgreSQL before 9.0, you will probably want to use a string if you are using any options at all, as the syntax Sequel uses for options is only compatible with PostgreSQL 9.0+.
- Dataset
-
Uses a query instead of a table name when copying.
- other
-
Uses a table name (usually a symbol) when copying.
The following options are respected:
- :format
-
The format to use. text is the default, so this should be :csv or :binary.
- :options
-
An options SQL string to use, which should contain comma separated options.
- :server
-
The server on which to run the query.
If a block is provided, the method continually yields to the block, one yield per row. If a block is not provided, a single string is returned with all of the data.
271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 |
# File 'lib/sequel/adapters/postgres.rb', line 271 def copy_table(table, opts={}) sql = if table.is_a?(String) sql = table else if opts[:options] || opts[:format] = " (" << "FORMAT #{opts[:format]}" if opts[:format] << "#{', ' if opts[:format]}#{opts[:options]}" if opts[:options] << ')' end table = if table.is_a?(::Sequel::Dataset) "(#{table.sql})" else literal(table) end sql = "COPY #{table} TO STDOUT#{options}" end synchronize(opts[:server]) do |conn| conn.execute(sql) begin if block_given? while buf = conn.get_copy_data yield buf end nil else b = '' b << buf while buf = conn.get_copy_data b end ensure raise DatabaseDisconnectError, "disconnecting as a partial COPY may leave the connection in an unusable state" if buf end end end |
#execute(sql, opts = {}, &block) ⇒ Object
Execute the given SQL with the given args on an available connection.
240 241 242 |
# File 'lib/sequel/adapters/postgres.rb', line 240 def execute(sql, opts={}, &block) synchronize(opts[:server]){|conn| check_database_errors{_execute(conn, sql, opts, &block)}} end |
#listen(channels, opts = {}, &block) ⇒ Object
Listens on the given channel (or multiple channels if channel is an array), waiting for notifications. After a notification is received, or the timeout has passed, stops listening to the channel. Options:
- :after_listen
-
An object that responds to
callthat is called with the underlying connection after the LISTEN statement is sent, but before the connection starts waiting for notifications. - :loop
-
Whether to continually wait for notifications, instead of just waiting for a single notification. If this option is given, a block must be provided. If this object responds to call, it is called with the underlying connection after each notification is received (after the block is called). If a :timeout option is used, and a callable object is given, the object will also be called if the timeout expires. If :loop is used and you want to stop listening, you can either break from inside the block given to #listen, or you can throw :stop from inside the :loop object’s call method or the block.
- :server
-
The server on which to listen, if the sharding support is being used.
- :timeout
-
How long to wait for a notification, in seconds (can provide a float value for fractional seconds). If not given or nil, waits indefinitely.
This method is only supported if pg is used as the underlying ruby driver. It returns the channel the notification was sent to (as a string), unless :loop was used, in which case it returns nil. If a block is given, it is yielded 3 arguments:
-
the channel the notification was sent to (as a string)
-
the backend pid of the notifier (as an integer),
-
and the payload of the notification (as a string or nil).
328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 |
# File 'lib/sequel/adapters/postgres.rb', line 328 def listen(channels, opts={}, &block) check_database_errors do synchronize(opts[:server]) do |conn| begin channels = Array(channels) channels.each{|channel| conn.execute("LISTEN #{channel}")} opts[:after_listen].call(conn) if opts[:after_listen] timeout = opts[:timeout] ? [opts[:timeout]] : [] if l = opts[:loop] raise Error, 'calling #listen with :loop requires a block' unless block loop_call = l.respond_to?(:call) catch(:stop) do loop do conn.wait_for_notify(*timeout, &block) l.call(conn) if loop_call end end nil else conn.wait_for_notify(*timeout, &block) end ensure conn.execute("UNLISTEN *") end end end end |
#reset_conversion_procs ⇒ Object
Reset the database’s conversion procs, requires a server query if there any named types.
359 360 361 |
# File 'lib/sequel/adapters/postgres.rb', line 359 def reset_conversion_procs synchronize{|conn| @conversion_procs = get_conversion_procs(conn)} end |
#to_application_timestamp(value) ⇒ Object
If convert_infinite_timestamps is true and the value is infinite, return an appropriate value based on the convert_infinite_timestamps setting.
365 366 367 368 369 370 371 372 373 374 375 376 |
# File 'lib/sequel/adapters/postgres.rb', line 365 def (value) if c = case value when *INFINITE_TIMESTAMP_STRINGS (value) else super end else super end end |