Module: Sequel::Postgres::DatabaseMethods
- Includes:
- UnmodifiedIdentifiers::DatabaseMethods
- Included in:
- JDBC::Postgres::DatabaseMethods, Database
- Defined in:
- lib/sequel/adapters/shared/postgres.rb
Constant Summary collapse
- PREPARED_ARG_PLACEHOLDER =
LiteralString.new('$').freeze
- FOREIGN_KEY_LIST_ON_DELETE_MAP =
{'a'=>:no_action, 'r'=>:restrict, 'c'=>:cascade, 'n'=>:set_null, 'd'=>:set_default}.freeze
- ON_COMMIT =
{:drop => 'DROP', :delete_rows => 'DELETE ROWS', :preserve_rows => 'PRESERVE ROWS'}.freeze
- SELECT_CUSTOM_SEQUENCE_SQL =
SQL fragment for custom sequences (ones not created by serial primary key), Returning the schema and literal form of the sequence name, by parsing the column defaults table.
(<<-end_sql SELECT name.nspname AS "schema", CASE WHEN split_part(def.adsrc, '''', 2) ~ '.' THEN substr(split_part(def.adsrc, '''', 2), strpos(split_part(def.adsrc, '''', 2), '.')+1) ELSE split_part(def.adsrc, '''', 2) END AS "sequence" FROM pg_class t JOIN pg_namespace name ON (t.relnamespace = name.oid) JOIN pg_attribute attr ON (t.oid = attrelid) JOIN pg_attrdef def ON (adrelid = attrelid AND adnum = attnum) JOIN pg_constraint cons ON (conrelid = adrelid AND adnum = conkey[1]) WHERE cons.contype = 'p' AND def.adsrc ~* 'nextval' end_sql ).strip.gsub(/\s+/, ' ').freeze
- SELECT_PK_SQL =
SQL fragment for determining primary key column for the given table. Only returns the first primary key if the table has a composite primary key.
(<<-end_sql SELECT pg_attribute.attname AS pk FROM pg_class, pg_attribute, pg_index, pg_namespace WHERE pg_class.oid = pg_attribute.attrelid AND pg_class.relnamespace = pg_namespace.oid AND pg_class.oid = pg_index.indrelid AND pg_index.indkey[0] = pg_attribute.attnum AND pg_index.indisprimary = 't' end_sql ).strip.gsub(/\s+/, ' ').freeze
- SELECT_SERIAL_SEQUENCE_SQL =
SQL fragment for getting sequence associated with table’s primary key, assuming it was a serial primary key column.
(<<-end_sql SELECT name.nspname AS "schema", seq.relname AS "sequence" FROM pg_class seq, pg_attribute attr, pg_depend dep, pg_namespace name, pg_constraint cons, pg_class t WHERE seq.oid = dep.objid AND seq.relnamespace = name.oid AND seq.relkind = 'S' AND attr.attrelid = dep.refobjid AND attr.attnum = dep.refobjsubid AND attr.attrelid = cons.conrelid AND attr.attnum = cons.conkey[1] AND attr.attrelid = t.oid AND cons.contype = 'p' end_sql ).strip.gsub(/\s+/, ' ').freeze
Instance Attribute Summary collapse
-
#conversion_procs ⇒ Object
readonly
A hash of conversion procs, keyed by type integer (oid) and having callable values for the conversion proc for that type.
Instance Method Summary collapse
-
#add_conversion_proc(oid, callable = Proc.new) ⇒ Object
Set a conversion proc for the given oid.
-
#add_named_conversion_proc(name, &block) ⇒ Object
Add a conversion proc for a named type, using the given block.
- #commit_prepared_transaction(transaction_id, opts = OPTS) ⇒ Object
-
#create_function(name, definition, opts = OPTS) ⇒ Object
Creates the function in the database.
-
#create_language(name, opts = OPTS) ⇒ Object
Create the procedural language in the database.
-
#create_schema(name, opts = OPTS) ⇒ Object
Create a schema in the database.
-
#create_trigger(table, name, function, opts = OPTS) ⇒ Object
Create a trigger in the database.
- #database_type ⇒ Object
-
#do(code, opts = OPTS) ⇒ Object
Use PostgreSQL’s DO syntax to execute an anonymous code block.
-
#drop_function(name, opts = OPTS) ⇒ Object
Drops the function from the database.
-
#drop_language(name, opts = OPTS) ⇒ Object
Drops a procedural language from the database.
-
#drop_schema(name, opts = OPTS) ⇒ Object
Drops a schema from the database.
-
#drop_trigger(table, name, opts = OPTS) ⇒ Object
Drops a trigger from the database.
-
#foreign_key_list(table, opts = OPTS) ⇒ Object
Return full foreign key information using the pg system tables, including :name, :on_delete, :on_update, and :deferrable entries in the hashes.
- #freeze ⇒ Object
-
#indexes(table, opts = OPTS) ⇒ Object
Use the pg_* system tables to determine indexes on a table.
-
#locks ⇒ Object
Dataset containing all current database locks.
-
#notify(channel, opts = OPTS) ⇒ Object
Notifies the given channel.
-
#primary_key(table, opts = OPTS) ⇒ Object
Return primary key for the given table.
-
#primary_key_sequence(table, opts = OPTS) ⇒ Object
Return the sequence providing the default for the primary key for the given table.
-
#refresh_view(name, opts = OPTS) ⇒ Object
Refresh the materialized view with the given name.
-
#reset_primary_key_sequence(table) ⇒ Object
Reset the primary key sequence for the given table, basing it on the maximum current value of the table’s primary key.
- #rollback_prepared_transaction(transaction_id, opts = OPTS) ⇒ Object
-
#serial_primary_key_options ⇒ Object
PostgreSQL uses SERIAL psuedo-type instead of AUTOINCREMENT for managing incrementing primary keys.
-
#server_version(server = nil) ⇒ Object
The version of the PostgreSQL server, used for determining capability.
-
#supports_create_table_if_not_exists? ⇒ Boolean
PostgreSQL supports CREATE TABLE IF NOT EXISTS on 9.1+.
-
#supports_deferrable_constraints? ⇒ Boolean
PostgreSQL 9.0+ supports some types of deferrable constraints beyond foreign key constraints.
-
#supports_deferrable_foreign_key_constraints? ⇒ Boolean
PostgreSQL supports deferrable foreign key constraints.
-
#supports_drop_table_if_exists? ⇒ Boolean
PostgreSQL supports DROP TABLE IF EXISTS.
-
#supports_partial_indexes? ⇒ Boolean
PostgreSQL supports partial indexes.
-
#supports_prepared_transactions? ⇒ Boolean
PostgreSQL supports prepared transactions (two-phase commit) if max_prepared_transactions is greater than 0.
-
#supports_savepoints? ⇒ Boolean
PostgreSQL supports savepoints.
-
#supports_transaction_isolation_levels? ⇒ Boolean
PostgreSQL supports transaction isolation levels.
-
#supports_transactional_ddl? ⇒ Boolean
PostgreSQL supports transaction DDL statements.
-
#supports_trigger_conditions? ⇒ Boolean
PostgreSQL 9.0+ supports trigger conditions.
-
#tables(opts = OPTS, &block) ⇒ Object
Array of symbols specifying table names in the current database.
-
#type_supported?(type) ⇒ Boolean
Check whether the given type name string/symbol (e.g. :hstore) is supported by the database.
-
#values(v) ⇒ Object
Creates a dataset that uses the VALUES clause:.
-
#views(opts = OPTS) ⇒ Object
Array of symbols specifying view names in the current database.
Instance Attribute Details
#conversion_procs ⇒ Object (readonly)
A hash of conversion procs, keyed by type integer (oid) and having callable values for the conversion proc for that type.
194 195 196 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 194 def conversion_procs @conversion_procs end |
Instance Method Details
#add_conversion_proc(oid, callable = Proc.new) ⇒ Object
Set a conversion proc for the given oid. The callable can be passed either as a argument or a block.
198 199 200 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 198 def add_conversion_proc(oid, callable=Proc.new) conversion_procs[oid] = callable end |
#add_named_conversion_proc(name, &block) ⇒ Object
Add a conversion proc for a named type, using the given block. This should be used for types without fixed OIDs, which includes all types that are not included in a default PostgreSQL installation.
205 206 207 208 209 210 211 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 205 def add_named_conversion_proc(name, &block) name = name.to_s if name.is_a?(Symbol) unless oid = from(:pg_type).where(:typtype=>['b', 'e'], :typname=>name.to_s).get(:oid) raise Error, "No matching type in pg_type for #{name.inspect}" end add_conversion_proc(oid, block) end |
#commit_prepared_transaction(transaction_id, opts = OPTS) ⇒ Object
213 214 215 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 213 def commit_prepared_transaction(transaction_id, opts=OPTS) run("COMMIT PREPARED #{literal(transaction_id)}", opts) end |
#create_function(name, definition, opts = OPTS) ⇒ Object
Creates the function in the database. Arguments:
- name
-
name of the function to create
- definition
-
string definition of the function, or object file for a dynamically loaded C function.
- opts
-
options hash:
- :args
-
function arguments, can be either a symbol or string specifying a type or an array of 1-3 elements:
- 1
-
argument data type
- 2
-
argument name
- 3
-
argument mode (e.g. in, out, inout)
- :behavior
-
Should be IMMUTABLE, STABLE, or VOLATILE. PostgreSQL assumes VOLATILE by default.
- :cost
-
The estimated cost of the function, used by the query planner.
- :language
-
The language the function uses. SQL is the default.
- :link_symbol
-
For a dynamically loaded see function, the function’s link symbol if different from the definition argument.
- :returns
-
The data type returned by the function. If you are using OUT or INOUT argument modes, this is ignored. Otherwise, if this is not specified, void is used by default to specify the function is not supposed to return a value.
- :rows
-
The estimated number of rows the function will return. Only use if the function returns SETOF something.
- :security_definer
-
Makes the privileges of the function the same as the privileges of the user who defined the function instead of the privileges of the user who runs the function. There are security implications when doing this, see the PostgreSQL documentation.
- :set
-
Configuration variables to set while the function is being run, can be a hash or an array of two pairs. search_path is often used here if :security_definer is used.
- :strict
-
Makes the function return NULL when any argument is NULL.
237 238 239 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 237 def create_function(name, definition, opts=OPTS) self << create_function_sql(name, definition, opts) end |
#create_language(name, opts = OPTS) ⇒ Object
Create the procedural language in the database. Arguments:
- name
-
Name of the procedural language (e.g. plpgsql)
- opts
-
options hash:
- :handler
-
The name of a previously registered function used as a call handler for this language.
- :replace
-
Replace the installed language if it already exists (on PostgreSQL 9.0+).
- :trusted
-
Marks the language being created as trusted, allowing unprivileged users to create functions using this language.
- :validator
-
The name of previously registered function used as a validator of functions defined in this language.
248 249 250 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 248 def create_language(name, opts=OPTS) self << create_language_sql(name, opts) end |
#create_schema(name, opts = OPTS) ⇒ Object
Create a schema in the database. Arguments:
- name
-
Name of the schema (e.g. admin)
- opts
-
options hash:
- :if_not_exists
-
Don’t raise an error if the schema already exists (PostgreSQL 9.3+)
- :owner
-
The owner to set for the schema (defaults to current user if not specified)
257 258 259 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 257 def create_schema(name, opts=OPTS) self << create_schema_sql(name, opts) end |
#create_trigger(table, name, function, opts = OPTS) ⇒ Object
Create a trigger in the database. Arguments:
- table
-
the table on which this trigger operates
- name
-
the name of this trigger
- function
-
the function to call for this trigger, which should return type trigger.
- opts
-
options hash:
- :after
-
Calls the trigger after execution instead of before.
- :args
-
An argument or array of arguments to pass to the function.
- :each_row
-
Calls the trigger for each row instead of for each statement.
- :events
-
Can be :insert, :update, :delete, or an array of any of those. Calls the trigger whenever that type of statement is used. By default, the trigger is called for insert, update, or delete.
- :when
-
A filter to use for the trigger
272 273 274 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 272 def create_trigger(table, name, function, opts=OPTS) self << create_trigger_sql(table, name, function, opts) end |
#database_type ⇒ Object
276 277 278 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 276 def database_type :postgres end |
#do(code, opts = OPTS) ⇒ Object
Use PostgreSQL’s DO syntax to execute an anonymous code block. The code should be the literal code string to use in the underlying procedural language. Options:
- :language
-
The procedural language the code is written in. The PostgreSQL default is plpgsql. Can be specified as a string or a symbol.
285 286 287 288 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 285 def do(code, opts=OPTS) language = opts[:language] run "DO #{"LANGUAGE #{literal(language.to_s)} " if language}#{literal(code)}" end |
#drop_function(name, opts = OPTS) ⇒ Object
Drops the function from the database. Arguments:
- name
-
name of the function to drop
- opts
-
options hash:
- :args
-
The arguments for the function. See create_function_sql.
- :cascade
-
Drop other objects depending on this function.
- :if_exists
-
Don’t raise an error if the function doesn’t exist.
296 297 298 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 296 def drop_function(name, opts=OPTS) self << drop_function_sql(name, opts) end |
#drop_language(name, opts = OPTS) ⇒ Object
Drops a procedural language from the database. Arguments:
- name
-
name of the procedural language to drop
- opts
-
options hash:
- :cascade
-
Drop other objects depending on this function.
- :if_exists
-
Don’t raise an error if the function doesn’t exist.
305 306 307 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 305 def drop_language(name, opts=OPTS) self << drop_language_sql(name, opts) end |
#drop_schema(name, opts = OPTS) ⇒ Object
Drops a schema from the database. Arguments:
- name
-
name of the schema to drop
- opts
-
options hash:
- :cascade
-
Drop all objects in this schema.
- :if_exists
-
Don’t raise an error if the schema doesn’t exist.
314 315 316 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 314 def drop_schema(name, opts=OPTS) self << drop_schema_sql(name, opts) end |
#drop_trigger(table, name, opts = OPTS) ⇒ Object
Drops a trigger from the database. Arguments:
- table
-
table from which to drop the trigger
- name
-
name of the trigger to drop
- opts
-
options hash:
- :cascade
-
Drop other objects depending on this function.
- :if_exists
-
Don’t raise an error if the function doesn’t exist.
324 325 326 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 324 def drop_trigger(table, name, opts=OPTS) self << drop_trigger_sql(table, name, opts) end |
#foreign_key_list(table, opts = OPTS) ⇒ Object
Return full foreign key information using the pg system tables, including :name, :on_delete, :on_update, and :deferrable entries in the hashes.
330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 330 def foreign_key_list(table, opts=OPTS) m = output_identifier_meth schema, _ = opts.fetch(:schema, schema_and_table(table)) oid = regclass_oid(table) if server_version >= 90500 cpos = Sequel.expr{array_position(co[:conkey], att[:attnum])} rpos = Sequel.expr{array_position(co[:confkey], att2[:attnum])} else range = 0...32 cpos = Sequel.expr{SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(co[:conkey], [x]), x]}, 32, att[:attnum])} rpos = Sequel.expr{SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(co[:confkey], [x]), x]}, 32, att2[:attnum])} end ds = . from{pg_constraint.as(:co)}. join(Sequel[:pg_class].as(:cl), :oid=>:conrelid). join(Sequel[:pg_attribute].as(:att), :attrelid=>:oid, :attnum=>SQL::Function.new(:ANY, Sequel[:co][:conkey])). join(Sequel[:pg_class].as(:cl2), :oid=>Sequel[:co][:confrelid]). join(Sequel[:pg_attribute].as(:att2), :attrelid=>:oid, :attnum=>SQL::Function.new(:ANY, Sequel[:co][:confkey])). order{[co[:conname], cpos]}. where{{ cl[:relkind]=>'r', co[:contype]=>'f', cl[:oid]=>oid, cpos=>rpos }}. select{[ co[:conname].as(:name), att[:attname].as(:column), co[:confupdtype].as(:on_update), co[:confdeltype].as(:on_delete), cl2[:relname].as(:table), att2[:attname].as(:refcolumn), SQL::BooleanExpression.new(:AND, co[:condeferrable], co[:condeferred]).as(:deferrable) ]} # If a schema is given, we only search in that schema, and the returned :table # entry is schema qualified as well. if schema ds = ds.join(Sequel[:pg_namespace].as(:nsp2), :oid=>Sequel[:cl2][:relnamespace]). select_append{nsp2[:nspname].as(:schema)} end h = {} fklod_map = FOREIGN_KEY_LIST_ON_DELETE_MAP ds.each do |row| if r = h[row[:name]] r[:columns] << m.call(row[:column]) r[:key] << m.call(row[:refcolumn]) else h[row[:name]] = { :name=>m.call(row[:name]), :columns=>[m.call(row[:column])], :key=>[m.call(row[:refcolumn])], :on_update=>fklod_map[row[:on_update]], :on_delete=>fklod_map[row[:on_delete]], :deferrable=>row[:deferrable], :table=>schema ? SQL::QualifiedIdentifier.new(m.call(row[:schema]), m.call(row[:table])) : m.call(row[:table]) } end end h.values end |
#freeze ⇒ Object
397 398 399 400 401 402 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 397 def freeze server_version supports_prepared_transactions? @conversion_procs.freeze super end |
#indexes(table, opts = OPTS) ⇒ Object
Use the pg_* system tables to determine indexes on a table
405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 405 def indexes(table, opts=OPTS) m = output_identifier_meth oid = regclass_oid(table, opts) if server_version >= 90500 order = [Sequel[:indc][:relname], Sequel.function(:array_position, Sequel[:ind][:indkey], Sequel[:att][:attnum])] else range = 0...32 order = [Sequel[:indc][:relname], SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(Sequel[:ind][:indkey], [x]), x]}, 32, Sequel[:att][:attnum])] attnums = range.map{|x| SQL::Subscript.new(Sequel[:ind][:indkey], [x])} unless server_version >= 80100 end attnums ||= SQL::Function.new(:ANY, Sequel[:ind][:indkey]) ds = . from{pg_class.as(:tab)}. join(Sequel[:pg_index].as(:ind), :indrelid=>:oid). join(Sequel[:pg_class].as(:indc), :oid=>:indexrelid). join(Sequel[:pg_attribute].as(:att), :attrelid=>Sequel[:tab][:oid], :attnum=>attnums). left_join(Sequel[:pg_constraint].as(:con), :conname=>Sequel[:indc][:relname]). where{{ indc[:relkind]=>'i', ind[:indisprimary]=>false, :indexprs=>nil, :indpred=>nil, :indisvalid=>true, tab[:oid]=>oid}}. order(*order). select{[indc[:relname].as(:name), ind[:indisunique].as(:unique), att[:attname].as(:column), con[:condeferrable].as(:deferrable)]} ds = ds.where(:indisready=>true, :indcheckxmin=>false) if server_version >= 80300 indexes = {} ds.each do |r| i = indexes[m.call(r[:name])] ||= {:columns=>[], :unique=>r[:unique], :deferrable=>r[:deferrable]} i[:columns] << m.call(r[:column]) end indexes end |
#locks ⇒ Object
Dataset containing all current database locks
446 447 448 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 446 def locks dataset.from(:pg_class).join(:pg_locks, :relation=>:relfilenode).select{[pg_class[:relname], Sequel::SQL::ColumnAll.new(:pg_locks)]} end |
#notify(channel, opts = OPTS) ⇒ Object
Notifies the given channel. See the PostgreSQL NOTIFY documentation. Options:
- :payload
-
The payload string to use for the NOTIFY statement. Only supported in PostgreSQL 9.0+.
- :server
-
The server to which to send the NOTIFY statement, if the sharding support is being used.
456 457 458 459 460 461 462 463 464 465 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 456 def notify(channel, opts=OPTS) sql = String.new sql << "NOTIFY " dataset.send(:identifier_append, sql, channel) if payload = opts[:payload] sql << ", " dataset.literal_append(sql, payload.to_s) end execute_ddl(sql, opts) end |
#primary_key(table, opts = OPTS) ⇒ Object
Return primary key for the given table.
468 469 470 471 472 473 474 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 468 def primary_key(table, opts=OPTS) quoted_table = quote_schema_table(table) Sequel.synchronize{return @primary_keys[quoted_table] if @primary_keys.has_key?(quoted_table)} sql = "#{SELECT_PK_SQL} AND pg_class.oid = #{literal(regclass_oid(table, opts))}" value = fetch(sql).single_value Sequel.synchronize{@primary_keys[quoted_table] = value} end |
#primary_key_sequence(table, opts = OPTS) ⇒ Object
Return the sequence providing the default for the primary key for the given table.
477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 477 def primary_key_sequence(table, opts=OPTS) quoted_table = quote_schema_table(table) Sequel.synchronize{return @primary_key_sequences[quoted_table] if @primary_key_sequences.has_key?(quoted_table)} sql = "#{SELECT_SERIAL_SEQUENCE_SQL} AND t.oid = #{literal(regclass_oid(table, opts))}" if pks = fetch(sql).single_record value = literal(SQL::QualifiedIdentifier.new(pks[:schema], pks[:sequence])) Sequel.synchronize{@primary_key_sequences[quoted_table] = value} else sql = "#{SELECT_CUSTOM_SEQUENCE_SQL} AND t.oid = #{literal(regclass_oid(table, opts))}" if pks = fetch(sql).single_record value = literal(SQL::QualifiedIdentifier.new(pks[:schema], LiteralString.new(pks[:sequence]))) Sequel.synchronize{@primary_key_sequences[quoted_table] = value} end end end |
#refresh_view(name, opts = OPTS) ⇒ Object
Refresh the materialized view with the given name.
DB.refresh_view(:items_view)
# REFRESH MATERIALIZED VIEW items_view
DB.refresh_view(:items_view, :concurrently=>true)
# REFRESH MATERIALIZED VIEW CONCURRENTLY items_view
499 500 501 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 499 def refresh_view(name, opts=OPTS) run "REFRESH MATERIALIZED VIEW#{' CONCURRENTLY' if opts[:concurrently]} #{quote_schema_table(name)}" end |
#reset_primary_key_sequence(table) ⇒ Object
Reset the primary key sequence for the given table, basing it on the maximum current value of the table’s primary key.
505 506 507 508 509 510 511 512 513 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 505 def reset_primary_key_sequence(table) return unless seq = primary_key_sequence(table) pk = SQL::Identifier.new(primary_key(table)) db = self seq_ds = db.from(LiteralString.new(seq)) s, t = schema_and_table(table) table = Sequel.qualify(s, t) if s get{setval(seq, db[table].select{coalesce(max(pk)+seq_ds.select{:increment_by}, seq_ds.select(:min_value))}, false)} end |
#rollback_prepared_transaction(transaction_id, opts = OPTS) ⇒ Object
515 516 517 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 515 def rollback_prepared_transaction(transaction_id, opts=OPTS) run("ROLLBACK PREPARED #{literal(transaction_id)}", opts) end |
#serial_primary_key_options ⇒ Object
PostgreSQL uses SERIAL psuedo-type instead of AUTOINCREMENT for managing incrementing primary keys.
521 522 523 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 521 def {:primary_key => true, :serial => true, :type=>Integer} end |
#server_version(server = nil) ⇒ Object
The version of the PostgreSQL server, used for determining capability.
526 527 528 529 530 531 532 533 534 535 536 537 538 539 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 526 def server_version(server=nil) return @server_version if @server_version @server_version = synchronize(server) do |conn| (conn.server_version rescue nil) if conn.respond_to?(:server_version) end unless @server_version @server_version = if m = /PostgreSQL (\d+)\.(\d+)(?:(?:rc\d+)|\.(\d+))?/.match(fetch('SELECT version()').single_value) (m[1].to_i * 10000) + (m[2].to_i * 100) + m[3].to_i else 0 end end @server_version end |
#supports_create_table_if_not_exists? ⇒ Boolean
PostgreSQL supports CREATE TABLE IF NOT EXISTS on 9.1+
542 543 544 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 542 def supports_create_table_if_not_exists? server_version >= 90100 end |
#supports_deferrable_constraints? ⇒ Boolean
PostgreSQL 9.0+ supports some types of deferrable constraints beyond foreign key constraints.
547 548 549 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 547 def supports_deferrable_constraints? server_version >= 90000 end |
#supports_deferrable_foreign_key_constraints? ⇒ Boolean
PostgreSQL supports deferrable foreign key constraints.
552 553 554 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 552 def supports_deferrable_foreign_key_constraints? true end |
#supports_drop_table_if_exists? ⇒ Boolean
PostgreSQL supports DROP TABLE IF EXISTS
557 558 559 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 557 def supports_drop_table_if_exists? true end |
#supports_partial_indexes? ⇒ Boolean
PostgreSQL supports partial indexes.
562 563 564 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 562 def supports_partial_indexes? true end |
#supports_prepared_transactions? ⇒ Boolean
PostgreSQL supports prepared transactions (two-phase commit) if max_prepared_transactions is greater than 0.
573 574 575 576 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 573 def supports_prepared_transactions? return @supports_prepared_transactions if defined?(@supports_prepared_transactions) @supports_prepared_transactions = self['SHOW max_prepared_transactions'].get.to_i > 0 end |
#supports_savepoints? ⇒ Boolean
PostgreSQL supports savepoints
579 580 581 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 579 def supports_savepoints? true end |
#supports_transaction_isolation_levels? ⇒ Boolean
PostgreSQL supports transaction isolation levels
584 585 586 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 584 def supports_transaction_isolation_levels? true end |
#supports_transactional_ddl? ⇒ Boolean
PostgreSQL supports transaction DDL statements.
589 590 591 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 589 def supports_transactional_ddl? true end |
#supports_trigger_conditions? ⇒ Boolean
PostgreSQL 9.0+ supports trigger conditions.
567 568 569 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 567 def supports_trigger_conditions? server_version >= 90000 end |
#tables(opts = OPTS, &block) ⇒ Object
Array of symbols specifying table names in the current database. The dataset used is yielded to the block if one is provided, otherwise, an array of symbols of table names is returned.
Options:
- :qualify
-
Return the tables as Sequel::SQL::QualifiedIdentifier instances, using the schema the table is located in as the qualifier.
- :schema
-
The schema to search
- :server
-
The server to use
602 603 604 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 602 def tables(opts=OPTS, &block) pg_class_relname('r', opts, &block) end |
#type_supported?(type) ⇒ Boolean
Check whether the given type name string/symbol (e.g. :hstore) is supported by the database.
608 609 610 611 612 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 608 def type_supported?(type) Sequel.synchronize{return @supported_types[type] if @supported_types.has_key?(type)} supported = from(:pg_type).where(:typtype=>'b', :typname=>type.to_s).count > 0 Sequel.synchronize{return @supported_types[type] = supported} end |
#values(v) ⇒ Object
Creates a dataset that uses the VALUES clause:
DB.values([[1, 2], [3, 4]])
# VALUES ((1, 2), (3, 4))
DB.values([[1, 2], [3, 4]]).order(:column2).limit(1, 1)
# VALUES ((1, 2), (3, 4)) ORDER BY column2 LIMIT 1 OFFSET 1
621 622 623 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 621 def values(v) @default_dataset.clone(:values=>v) end |
#views(opts = OPTS) ⇒ Object
Array of symbols specifying view names in the current database.
Options:
- :materialized
-
Return materialized views
- :qualify
-
Return the views as Sequel::SQL::QualifiedIdentifier instances, using the schema the view is located in as the qualifier.
- :schema
-
The schema to search
- :server
-
The server to use
633 634 635 636 |
# File 'lib/sequel/adapters/shared/postgres.rb', line 633 def views(opts=OPTS) relkind = opts[:materialized] ? 'm' : 'v' pg_class_relname(relkind, opts) end |