Module: Sequel::Postgres::DatabaseMethods

Overview

Methods shared by Database instances that connect to PostgreSQL.

Constant Summary collapse

EXCLUDE_SCHEMAS =
/pg_*|information_schema/i
PREPARED_ARG_PLACEHOLDER =
LiteralString.new('$').freeze
RE_CURRVAL_ERROR =
/currval of sequence "(.*)" is not yet defined in this session|relation "(.*)" does not exist/.freeze
SYSTEM_TABLE_REGEXP =
/^pg|sql/.freeze
FOREIGN_KEY_LIST_ON_DELETE_MAP =
{'a'.freeze=>:no_action, 'r'.freeze=>:restrict, 'c'.freeze=>:cascade, 'n'.freeze=>:set_null, 'd'.freeze=>:set_default}.freeze
SELECT_CUSTOM_SEQUENCE_SQL =

SQL fragment for custom sequences (ones not created by serial primary key), Returning the schema and literal form of the sequence name, by parsing the column defaults table.

("  SELECT name.nspname AS \"schema\",\n      CASE\n      WHEN split_part(def.adsrc, '''', 2) ~ '.' THEN\n        substr(split_part(def.adsrc, '''', 2),\n               strpos(split_part(def.adsrc, '''', 2), '.')+1)\n      ELSE split_part(def.adsrc, '''', 2)\n    END AS \"sequence\"\n  FROM pg_class t\n  JOIN pg_namespace  name ON (t.relnamespace = name.oid)\n  JOIN pg_attribute  attr ON (t.oid = attrelid)\n  JOIN pg_attrdef    def  ON (adrelid = attrelid AND adnum = attnum)\n  JOIN pg_constraint cons ON (conrelid = adrelid AND adnum = conkey[1])\n  WHERE cons.contype = 'p'\n    AND def.adsrc ~* 'nextval'\n"
).strip.gsub(/\s+/, ' ').freeze
SELECT_PK_SQL =

SQL fragment for determining primary key column for the given table. Only returns the first primary key if the table has a composite primary key.

("  SELECT pg_attribute.attname AS pk\n  FROM pg_class, pg_attribute, pg_index, pg_namespace\n  WHERE pg_class.oid = pg_attribute.attrelid\n    AND pg_class.relnamespace  = pg_namespace.oid\n    AND pg_class.oid = pg_index.indrelid\n    AND pg_index.indkey[0] = pg_attribute.attnum\n    AND pg_index.indisprimary = 't'\n"
).strip.gsub(/\s+/, ' ').freeze
SELECT_SERIAL_SEQUENCE_SQL =

SQL fragment for getting sequence associated with table’s primary key, assuming it was a serial primary key column.

("  SELECT  name.nspname AS \"schema\", seq.relname AS \"sequence\"\n  FROM pg_class seq, pg_attribute attr, pg_depend dep,\n    pg_namespace name, pg_constraint cons\n  WHERE seq.oid = dep.objid\n    AND seq.relnamespace  = name.oid\n    AND seq.relkind = 'S'\n    AND attr.attrelid = dep.refobjid\n    AND attr.attnum = dep.refobjsubid\n    AND attr.attrelid = cons.conrelid\n    AND attr.attnum = cons.conkey[1]\n    AND cons.contype = 'p'\n"
).strip.gsub(/\s+/, ' ').freeze

Instance Method Summary collapse

Instance Method Details

#commit_prepared_transaction(transaction_id) ⇒ Object

Commit an existing prepared transaction with the given transaction identifier string.



151
152
153
# File 'lib/sequel/adapters/shared/postgres.rb', line 151

def commit_prepared_transaction(transaction_id)
  run("COMMIT PREPARED #{literal(transaction_id)}")
end

#create_function(name, definition, opts = {}) ⇒ Object

Creates the function in the database. Arguments:

  • name : name of the function to create

  • definition : string definition of the function, or object file for a dynamically loaded C function.

  • opts : options hash:

    • :args : function arguments, can be either a symbol or string specifying a type or an array of 1-3 elements:

      • element 1 : argument data type

      • element 2 : argument name

      • element 3 : argument mode (e.g. in, out, inout)

    • :behavior : Should be IMMUTABLE, STABLE, or VOLATILE. PostgreSQL assumes VOLATILE by default.

    • :cost : The estimated cost of the function, used by the query planner.

    • :language : The language the function uses. SQL is the default.

    • :link_symbol : For a dynamically loaded see function, the function’s link symbol if different from the definition argument.

    • :returns : The data type returned by the function. If you are using OUT or INOUT argument modes, this is ignored. Otherwise, if this is not specified, void is used by default to specify the function is not supposed to return a value.

    • :rows : The estimated number of rows the function will return. Only use if the function returns SETOF something.

    • :security_definer : Makes the privileges of the function the same as the privileges of the user who defined the function instead of the privileges of the user who runs the function. There are security implications when doing this, see the PostgreSQL documentation.

    • :set : Configuration variables to set while the function is being run, can be a hash or an array of two pairs. search_path is often used here if :security_definer is used.

    • :strict : Makes the function return NULL when any argument is NULL.



175
176
177
# File 'lib/sequel/adapters/shared/postgres.rb', line 175

def create_function(name, definition, opts={})
  self << create_function_sql(name, definition, opts)
end

#create_language(name, opts = {}) ⇒ Object

Create the procedural language in the database. Arguments:

  • name : Name of the procedural language (e.g. plpgsql)

  • opts : options hash:

    • :handler : The name of a previously registered function used as a call handler for this language.

    • :replace: Replace the installed language if it already exists (on PostgreSQL 9.0+).

    • :trusted : Marks the language being created as trusted, allowing unprivileged users to create functions using this language.

    • :validator : The name of previously registered function used as a validator of functions defined in this language.



186
187
188
# File 'lib/sequel/adapters/shared/postgres.rb', line 186

def create_language(name, opts={})
  self << create_language_sql(name, opts)
end

#create_schema(name) ⇒ Object

Create a schema in the database. Arguments:

  • name : Name of the schema (e.g. admin)



192
193
194
# File 'lib/sequel/adapters/shared/postgres.rb', line 192

def create_schema(name)
  self << create_schema_sql(name)
end

#create_trigger(table, name, function, opts = {}) ⇒ Object

Create a trigger in the database. Arguments:

  • table : the table on which this trigger operates

  • name : the name of this trigger

  • function : the function to call for this trigger, which should return type trigger.

  • opts : options hash:

    • :after : Calls the trigger after execution instead of before.

    • :args : An argument or array of arguments to pass to the function.

    • :each_row : Calls the trigger for each row instead of for each statement.

    • :events : Can be :insert, :update, :delete, or an array of any of those. Calls the trigger whenever that type of statement is used. By default, the trigger is called for insert, update, or delete.



206
207
208
# File 'lib/sequel/adapters/shared/postgres.rb', line 206

def create_trigger(table, name, function, opts={})
  self << create_trigger_sql(table, name, function, opts)
end

#database_typeObject

PostgreSQL uses the :postgres database type.



211
212
213
# File 'lib/sequel/adapters/shared/postgres.rb', line 211

def database_type
  :postgres
end

#drop_function(name, opts = {}) ⇒ Object

Drops the function from the database. Arguments:

  • name : name of the function to drop

  • opts : options hash:

    • :args : The arguments for the function. See create_function_sql.

    • :cascade : Drop other objects depending on this function.

    • :if_exists : Don’t raise an error if the function doesn’t exist.



221
222
223
# File 'lib/sequel/adapters/shared/postgres.rb', line 221

def drop_function(name, opts={})
  self << drop_function_sql(name, opts)
end

#drop_language(name, opts = {}) ⇒ Object

Drops a procedural language from the database. Arguments:

  • name : name of the procedural language to drop

  • opts : options hash:

    • :cascade : Drop other objects depending on this function.

    • :if_exists : Don’t raise an error if the function doesn’t exist.



230
231
232
# File 'lib/sequel/adapters/shared/postgres.rb', line 230

def drop_language(name, opts={})
  self << drop_language_sql(name, opts)
end

#drop_schema(name, opts = {}) ⇒ Object

Drops a schema from the database. Arguments:

  • name : name of the schema to drop

  • opts : options hash:

    • :cascade : Drop all objects in this schema.

    • :if_exists : Don’t raise an error if the schema doesn’t exist.



239
240
241
# File 'lib/sequel/adapters/shared/postgres.rb', line 239

def drop_schema(name, opts={})
  self << drop_schema_sql(name, opts)
end

#drop_trigger(table, name, opts = {}) ⇒ Object

Drops a trigger from the database. Arguments:

  • table : table from which to drop the trigger

  • name : name of the trigger to drop

  • opts : options hash:

    • :cascade : Drop other objects depending on this function.

    • :if_exists : Don’t raise an error if the function doesn’t exist.



249
250
251
# File 'lib/sequel/adapters/shared/postgres.rb', line 249

def drop_trigger(table, name, opts={})
  self << drop_trigger_sql(table, name, opts)
end

#foreign_key_list(table, opts = {}) ⇒ Object

Return full foreign key information using the pg system tables, including :name, :on_delete, :on_update, and :deferrable entries in the hashes.



255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
# File 'lib/sequel/adapters/shared/postgres.rb', line 255

def foreign_key_list(table, opts={})
  m = output_identifier_meth
  im = input_identifier_meth
  schema, table = schema_and_table(table)
  range = 0...32

  base_ds = .
    where(:cl__relkind=>'r', :co__contype=>'f', :cl__relname=>im.call(table)).
    from(:pg_constraint___co).
    join(:pg_class___cl, :oid=>:conrelid)

  # We split the parsing into two separate queries, which are merged manually later.
  # This is because PostgreSQL stores both the referencing and referenced columns in
  # arrays, and I don't know a simple way to not create a cross product, as PostgreSQL
  # doesn't appear to have a function that takes an array and element and gives you
  # the index of that element in the array.

  ds = base_ds.
    join(:pg_attribute___att, :attrelid=>:oid, :attnum=>SQL::Function.new(:ANY, :co__conkey)).
    order(:co__conname, SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(:co__conkey, [x]), x]}, 32, :att__attnum)).
    select(:co__conname___name, :att__attname___column, :co__confupdtype___on_update, :co__confdeltype___on_delete,
           SQL::BooleanExpression.new(:AND, :co__condeferrable, :co__condeferred).as(:deferrable))

  ref_ds = base_ds.
    join(:pg_class___cl2, :oid=>:co__confrelid).
    join(:pg_attribute___att2, :attrelid=>:oid, :attnum=>SQL::Function.new(:ANY, :co__confkey)).
    order(:co__conname, SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(:co__conkey, [x]), x]}, 32, :att2__attnum)).
    select(:co__conname___name, :cl2__relname___table, :att2__attname___refcolumn)

  # If a schema is given, we only search in that schema, and the returned :table
  # entry is schema qualified as well.
  if schema
    ds = ds.join(:pg_namespace___nsp, :oid=>:cl__relnamespace).
      where(:nsp__nspname=>im.call(schema))
    ref_ds = ref_ds.join(:pg_namespace___nsp2, :oid=>:cl2__relnamespace).
      select_more(:nsp2__nspname___schema)
  end

  h = {}
  fklod_map = FOREIGN_KEY_LIST_ON_DELETE_MAP 
  ds.each do |row|
    if r = h[row[:name]]
      r[:columns] << m.call(row[:column])
    else
      h[row[:name]] = {:name=>m.call(row[:name]), :columns=>[m.call(row[:column])], :on_update=>fklod_map[row[:on_update]], :on_delete=>fklod_map[row[:on_delete]], :deferrable=>row[:deferrable]}
    end
  end
  ref_ds.each do |row|
    r = h[row[:name]]
    r[:table] ||= schema ? SQL::QualifiedIdentifier.new(m.call(row[:schema]), m.call(row[:table])) : m.call(row[:table])
    r[:key] ||= []
    r[:key] << m.call(row[:refcolumn])
  end
  h.values
end

#indexes(table, opts = {}) ⇒ Object

Use the pg_* system tables to determine indexes on a table



312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
# File 'lib/sequel/adapters/shared/postgres.rb', line 312

def indexes(table, opts={})
  m = output_identifier_meth
  im = input_identifier_meth
  schema, table = schema_and_table(table)
  range = 0...32
  attnums = server_version >= 80100 ? SQL::Function.new(:ANY, :ind__indkey) : range.map{|x| SQL::Subscript.new(:ind__indkey, [x])}
  ds = .
    from(:pg_class___tab).
    join(:pg_index___ind, :indrelid=>:oid, im.call(table)=>:relname).
    join(:pg_class___indc, :oid=>:indexrelid).
    join(:pg_attribute___att, :attrelid=>:tab__oid, :attnum=>attnums).
    filter(:indc__relkind=>'i', :ind__indisprimary=>false, :indexprs=>nil, :indpred=>nil, :indisvalid=>true).
    order(:indc__relname, SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(:ind__indkey, [x]), x]}, 32, :att__attnum)).
    select(:indc__relname___name, :ind__indisunique___unique, :att__attname___column)

  ds.join!(:pg_namespace___nsp, :oid=>:tab__relnamespace, :nspname=>schema.to_s) if schema
  ds.filter!(:indisready=>true, :indcheckxmin=>false) if server_version >= 80300

  indexes = {}
  ds.each do |r|
    i = indexes[m.call(r[:name])] ||= {:columns=>[], :unique=>r[:unique]}
    i[:columns] << m.call(r[:column])
  end
  indexes
end

#locksObject

Dataset containing all current database locks



339
340
341
# File 'lib/sequel/adapters/shared/postgres.rb', line 339

def locks
  dataset.from(:pg_class).join(:pg_locks, :relation=>:relfilenode).select(:pg_class__relname, Sequel::SQL::ColumnAll.new(:pg_locks))
end

#notify(channel, opts = {}) ⇒ Object

Notifies the given channel. See the PostgreSQL NOTIFY documentation. Options:

:payload

The payload string to use for the NOTIFY statement. Only supported in PostgreSQL 9.0+.

:server

The server to which to send the NOTIFY statement, if the sharding support is being used.



349
350
351
# File 'lib/sequel/adapters/shared/postgres.rb', line 349

def notify(channel, opts={})
  execute_ddl("NOTIFY #{channel}#{", #{literal(opts[:payload].to_s)}" if opts[:payload]}", opts)
end

#primary_key(table, opts = {}) ⇒ Object

Return primary key for the given table.



354
355
356
357
358
359
360
361
362
# File 'lib/sequel/adapters/shared/postgres.rb', line 354

def primary_key(table, opts={})
  quoted_table = quote_schema_table(table)
  @primary_keys.fetch(quoted_table) do
    schema, table = schema_and_table(table)
    sql = "#{SELECT_PK_SQL} AND pg_class.relname = #{literal(table)}"
    sql << "AND pg_namespace.nspname = #{literal(schema)}" if schema
    @primary_keys[quoted_table] = fetch(sql).single_value
  end
end

#primary_key_sequence(table, opts = {}) ⇒ Object

Return the sequence providing the default for the primary key for the given table.



365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
# File 'lib/sequel/adapters/shared/postgres.rb', line 365

def primary_key_sequence(table, opts={})
  quoted_table = quote_schema_table(table)
  @primary_key_sequences.fetch(quoted_table) do
    schema, table = schema_and_table(table)
    table = literal(table)
    sql = "#{SELECT_SERIAL_SEQUENCE_SQL} AND seq.relname = #{table}"
    sql << " AND name.nspname = #{literal(schema)}" if schema
    unless pks = fetch(sql).single_record
      sql = "#{SELECT_CUSTOM_SEQUENCE_SQL} AND t.relname = #{table}"
      sql << " AND name.nspname = #{literal(schema)}" if schema
      pks = fetch(sql).single_record
    end

    @primary_key_sequences[quoted_table] = if pks
      literal(SQL::QualifiedIdentifier.new(pks[:schema], LiteralString.new(pks[:sequence])))
    end
  end
end

#reset_primary_key_sequence(table) ⇒ Object

Reset the primary key sequence for the given table, baseing it on the maximum current value of the table’s primary key.



386
387
388
389
390
391
392
# File 'lib/sequel/adapters/shared/postgres.rb', line 386

def reset_primary_key_sequence(table)
  return unless seq = primary_key_sequence(table)
  pk = SQL::Identifier.new(primary_key(table))
  db = self
  seq_ds = db.from(LiteralString.new(seq))
  get{setval(seq, db[table].select{coalesce(max(pk)+seq_ds.select{:increment_by}, seq_ds.select(:min_value))}, false)}
end

#rollback_prepared_transaction(transaction_id) ⇒ Object

Rollback an existing prepared transaction with the given transaction identifier string.



396
397
398
# File 'lib/sequel/adapters/shared/postgres.rb', line 396

def rollback_prepared_transaction(transaction_id)
  run("ROLLBACK PREPARED #{literal(transaction_id)}")
end

#serial_primary_key_optionsObject

PostgreSQL uses SERIAL psuedo-type instead of AUTOINCREMENT for managing incrementing primary keys.



402
403
404
# File 'lib/sequel/adapters/shared/postgres.rb', line 402

def serial_primary_key_options
  {:primary_key => true, :serial => true, :type=>Integer}
end

#server_version(server = nil) ⇒ Object

The version of the PostgreSQL server, used for determining capability.



407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
# File 'lib/sequel/adapters/shared/postgres.rb', line 407

def server_version(server=nil)
  return @server_version if @server_version
  @server_version = synchronize(server) do |conn|
    (conn.server_version rescue nil) if conn.respond_to?(:server_version)
  end
  unless @server_version
    @server_version = if m = /PostgreSQL (\d+)\.(\d+)(?:(?:rc\d+)|\.(\d+))?/.match(fetch('SELECT version()').single_value)
      (m[1].to_i * 10000) + (m[2].to_i * 100) + m[3].to_i
    else
      0
    end
  end
  warn 'Sequel no longer supports PostgreSQL <8.2, some things may not work' if @server_version < 80200
  @server_version
end

#supports_create_table_if_not_exists?Boolean

PostgreSQL supports CREATE TABLE IF NOT EXISTS on 9.1+

Returns:

  • (Boolean)


424
425
426
# File 'lib/sequel/adapters/shared/postgres.rb', line 424

def supports_create_table_if_not_exists?
  server_version >= 90100
end

#supports_drop_table_if_exists?Boolean

PostgreSQL supports DROP TABLE IF EXISTS

Returns:

  • (Boolean)


429
430
431
# File 'lib/sequel/adapters/shared/postgres.rb', line 429

def supports_drop_table_if_exists?
  true
end

#supports_prepared_transactions?Boolean

PostgreSQL supports prepared transactions (two-phase commit) if max_prepared_transactions is greater than 0.

Returns:

  • (Boolean)


435
436
437
438
# File 'lib/sequel/adapters/shared/postgres.rb', line 435

def supports_prepared_transactions?
  return @supports_prepared_transactions if defined?(@supports_prepared_transactions)
  @supports_prepared_transactions = self['SHOW max_prepared_transactions'].get.to_i > 0
end

#supports_savepoints?Boolean

PostgreSQL supports savepoints

Returns:

  • (Boolean)


441
442
443
# File 'lib/sequel/adapters/shared/postgres.rb', line 441

def supports_savepoints?
  true
end

#supports_transaction_isolation_levels?Boolean

PostgreSQL supports transaction isolation levels

Returns:

  • (Boolean)


446
447
448
# File 'lib/sequel/adapters/shared/postgres.rb', line 446

def supports_transaction_isolation_levels?
  true
end

#supports_transactional_ddl?Boolean

PostgreSQL supports transaction DDL statements.

Returns:

  • (Boolean)


451
452
453
# File 'lib/sequel/adapters/shared/postgres.rb', line 451

def supports_transactional_ddl?
  true
end

#tables(opts = {}, &block) ⇒ Object

Array of symbols specifying table names in the current database. The dataset used is yielded to the block if one is provided, otherwise, an array of symbols of table names is returned.

Options:

  • :schema - The schema to search (default_schema by default)

  • :server - The server to use



462
463
464
# File 'lib/sequel/adapters/shared/postgres.rb', line 462

def tables(opts={}, &block)
  pg_class_relname('r', opts, &block)
end

#type_supported?(type) ⇒ Boolean

Check whether the given type name string/symbol (e.g. :hstore) is supported by the database.

Returns:

  • (Boolean)


468
469
470
471
# File 'lib/sequel/adapters/shared/postgres.rb', line 468

def type_supported?(type)
  @supported_types ||= {}
  @supported_types.fetch(type){@supported_types[type] = (from(:pg_type).filter(:typtype=>'b', :typname=>type.to_s).count > 0)}
end

#views(opts = {}) ⇒ Object

Array of symbols specifying view names in the current database.

Options:

  • :schema - The schema to search (default_schema by default)

  • :server - The server to use



478
479
480
# File 'lib/sequel/adapters/shared/postgres.rb', line 478

def views(opts={})
  pg_class_relname('v', opts)
end