Module: Sequel::Postgres::DatasetMethods

Includes:
UnmodifiedIdentifiers::DatasetMethods
Included in:
JDBC::Postgres::Dataset, Dataset
Defined in:
lib/sequel/adapters/shared/postgres.rb

Constant Summary collapse

NULL =
LiteralString.new('NULL').freeze
LOCK_MODES =
['ACCESS SHARE', 'ROW SHARE', 'ROW EXCLUSIVE', 'SHARE UPDATE EXCLUSIVE', 'SHARE', 'SHARE ROW EXCLUSIVE', 'EXCLUSIVE', 'ACCESS EXCLUSIVE'].each(&:freeze).freeze

Instance Method Summary collapse

Instance Method Details

#analyzeObject

Return the results of an EXPLAIN ANALYZE query as a string



1555
1556
1557
# File 'lib/sequel/adapters/shared/postgres.rb', line 1555

def analyze
  explain(:analyze=>true)
end

#complex_expression_sql_append(sql, op, args) ⇒ Object

Handle converting the ruby xor operator (^) into the PostgreSQL xor operator (#), and use the ILIKE and NOT ILIKE operators.



1562
1563
1564
1565
1566
1567
1568
1569
1570
1571
1572
1573
1574
1575
1576
1577
1578
1579
1580
1581
1582
1583
# File 'lib/sequel/adapters/shared/postgres.rb', line 1562

def complex_expression_sql_append(sql, op, args)
  case op
  when :^
    j = ' # '
    c = false
    args.each do |a|
      sql << j if c
      literal_append(sql, a)
      c ||= true
    end
  when :ILIKE, :'NOT ILIKE'
    sql << '('
    literal_append(sql, args[0])
    sql << ' ' << op.to_s << ' '
    literal_append(sql, args[1])
    sql << " ESCAPE "
    literal_append(sql, "\\")
    sql << ')'
  else
    super
  end
end

#disable_insert_returningObject

Disables automatic use of INSERT ... RETURNING. You can still use returning manually to force the use of RETURNING when inserting.

This is designed for cases where INSERT RETURNING cannot be used, such as when you are using partitioning with trigger functions or conditional rules, or when you are using a PostgreSQL version less than 8.2, or a PostgreSQL derivative that does not support returning.

Note that when this method is used, insert will not return the primary key of the inserted row, you will have to get the primary key of the inserted row before inserting via nextval, or after inserting via currval or lastval (making sure to use the same database connection for currval or lastval).



1599
1600
1601
# File 'lib/sequel/adapters/shared/postgres.rb', line 1599

def disable_insert_returning
  clone(:disable_insert_returning=>true)
end

#explain(opts = OPTS) ⇒ Object

Return the results of an EXPLAIN query as a string



1604
1605
1606
# File 'lib/sequel/adapters/shared/postgres.rb', line 1604

def explain(opts=OPTS)
  with_sql((opts[:analyze] ? 'EXPLAIN ANALYZE ' : 'EXPLAIN ') + select_sql).map(:'QUERY PLAN').join("\r\n")
end

#for_shareObject

Return a cloned dataset which will use FOR SHARE to lock returned rows.



1609
1610
1611
# File 'lib/sequel/adapters/shared/postgres.rb', line 1609

def for_share
  lock_style(:share)
end

#full_text_search(cols, terms, opts = OPTS) ⇒ Object

Run a full text search on PostgreSQL. By default, searching for the inclusion of any of the terms in any of the cols.

Options: :headline :: Append a expression to the selected columns aliased to headline that contains an extract of the matched text. :language :: The language to use for the search (default: 'simple') :plain :: Whether a plain search should be used (default: false). In this case, terms should be a single string, and it will do a search where cols contains all of the words in terms. This ignores search operators in terms. :phrase :: Similar to :plain, but also adding an ILIKE filter to ensure that returned rows also include the exact phrase used. :rank :: Set to true to order by the rank, so that closer matches are returned first. :to_tsquery :: Can be set to :plain or :phrase to specify the function to use to convert the terms to a ts_query. :tsquery :: Specifies the terms argument is already a valid SQL expression returning a tsquery, and can be used directly in the query. :tsvector :: Specifies the cols argument is already a valid SQL expression returning a tsvector, and can be used directly in the query.



1632
1633
1634
1635
1636
1637
1638
1639
1640
1641
1642
1643
1644
1645
1646
1647
1648
1649
1650
1651
1652
1653
1654
1655
1656
1657
1658
1659
1660
1661
1662
1663
1664
1665
1666
1667
1668
1669
# File 'lib/sequel/adapters/shared/postgres.rb', line 1632

def full_text_search(cols, terms, opts = OPTS)
  lang = Sequel.cast(opts[:language] || 'simple', :regconfig)

  unless opts[:tsvector]
    phrase_cols = full_text_string_join(cols)
    cols = Sequel.function(:to_tsvector, lang, phrase_cols)
  end

  unless opts[:tsquery]
    phrase_terms = terms.is_a?(Array) ? terms.join(' | ') : terms

    query_func = case to_tsquery = opts[:to_tsquery]
    when :phrase, :plain
      :"#{to_tsquery}to_tsquery"
    else
      (opts[:phrase] || opts[:plain]) ? :plainto_tsquery : :to_tsquery
    end

    terms = Sequel.function(query_func, lang, phrase_terms)
  end

  ds = where(Sequel.lit(["", " @@ ", ""], cols, terms))

  if opts[:phrase]
    raise Error, "can't use :phrase with either :tsvector or :tsquery arguments to full_text_search together" if opts[:tsvector] || opts[:tsquery]
    ds = ds.grep(phrase_cols, "%#{escape_like(phrase_terms)}%", :case_insensitive=>true)
  end

  if opts[:rank]
    ds = ds.reverse{ts_rank_cd(cols, terms)}
  end

  if opts[:headline]
    ds = ds.select_append{ts_headline(lang, phrase_cols, terms).as(:headline)}
  end

  ds
end

#insert(*values) ⇒ Object

Insert given values into the database.



1672
1673
1674
1675
1676
1677
1678
1679
1680
1681
1682
1683
1684
1685
1686
# File 'lib/sequel/adapters/shared/postgres.rb', line 1672

def insert(*values)
  if @opts[:returning]
    # Already know which columns to return, let the standard code handle it
    super
  elsif @opts[:sql] || @opts[:disable_insert_returning]
    # Raw SQL used or RETURNING disabled, just use the default behavior
    # and return nil since sequence is not known.
    super
    nil
  else
    # Force the use of RETURNING with the primary key value,
    # unless it has been disabled.
    returning(insert_pk).insert(*values){|r| return r.values.first}
  end
end

#insert_conflict(opts = OPTS) ⇒ Object

Handle uniqueness violations when inserting, by updating the conflicting row, using ON CONFLICT. With no options, uses ON CONFLICT DO NOTHING. Options: :conflict_where :: The index filter, when using a partial index to determine uniqueness. :constraint :: An explicit constraint name, has precendence over :target. :target :: The column name or expression to handle uniqueness violations on. :update :: A hash of columns and values to set. Uses ON CONFLICT DO UPDATE. :update_where :: A WHERE condition to use for the update.

Examples:

DB[:table].insert_conflict.insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT DO NOTHING

DB[:table].insert_conflict(constraint: :table_a_uidx).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT ON CONSTRAINT table_a_uidx DO NOTHING

DB[:table].insert_conflict(target: :a).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT (a) DO NOTHING

DB[:table].insert_conflict(target: :a, conflict_where: {c: true}).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT (a) WHERE (c IS TRUE) DO NOTHING

DB[:table].insert_conflict(target: :a, update: {b: Sequel[:excluded][:b]}).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT (a) DO UPDATE SET b = excluded.b

DB[:table].insert_conflict(constraint: :table_a_uidx,
update: {b: Sequel[:excluded][:b]}, update_where: {Sequel[:table][:status_id] => 1}).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT ON CONSTRAINT table_a_uidx
# DO UPDATE SET b = excluded.b WHERE (table.status_id = 1)


1723
1724
1725
# File 'lib/sequel/adapters/shared/postgres.rb', line 1723

def insert_conflict(opts=OPTS)
  clone(:insert_conflict => opts)
end

#insert_ignoreObject

Ignore uniqueness/exclusion violations when inserting, using ON CONFLICT DO NOTHING. Exists mostly for compatibility to MySQL's insert_ignore. Example:

DB[:table].insert_ignore.insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT DO NOTHING


1733
1734
1735
# File 'lib/sequel/adapters/shared/postgres.rb', line 1733

def insert_ignore
  insert_conflict
end

#insert_select(*values) ⇒ Object

Insert a record, returning the record inserted, using RETURNING. Always returns nil without running an INSERT statement if disable_insert_returning is used. If the query runs but returns no values, returns false.



1740
1741
1742
1743
1744
# File 'lib/sequel/adapters/shared/postgres.rb', line 1740

def insert_select(*values)
  return unless supports_insert_select?
  # Handle case where query does not return a row
  server?(:default).with_sql_first(insert_select_sql(*values)) || false
end

#insert_select_sql(*values) ⇒ Object

The SQL to use for an insert_select, adds a RETURNING clause to the insert unless the RETURNING clause is already present.



1748
1749
1750
1751
# File 'lib/sequel/adapters/shared/postgres.rb', line 1748

def insert_select_sql(*values)
  ds = opts[:returning] ? self : returning
  ds.insert_sql(*values)
end

#join_table(type, table, expr = nil, options = OPTS, &block) ⇒ Object

Support SQL::AliasedExpression as expr to setup a USING join with a table alias for the USING columns.



1755
1756
1757
1758
1759
1760
# File 'lib/sequel/adapters/shared/postgres.rb', line 1755

def join_table(type, table, expr=nil, options=OPTS, &block)
  if expr.is_a?(SQL::AliasedExpression) && expr.expression.is_a?(Array) && !expr.expression.empty? && expr.expression.all?
    options = options.merge(:join_using=>true)
  end
  super
end

#lock(mode, opts = OPTS) ⇒ Object

Locks all tables in the dataset's FROM clause (but not in JOINs) with the specified mode (e.g. 'EXCLUSIVE'). If a block is given, starts a new transaction, locks the table, and yields. If a block is not given, just locks the tables. Note that PostgreSQL will probably raise an error if you lock the table outside of an existing transaction. Returns nil.



1767
1768
1769
1770
1771
1772
1773
1774
1775
1776
1777
1778
1779
1780
1781
# File 'lib/sequel/adapters/shared/postgres.rb', line 1767

def lock(mode, opts=OPTS)
  if defined?(yield) # perform locking inside a transaction and yield to block
    @db.transaction(opts){lock(mode, opts); yield}
  else
    sql = 'LOCK TABLE '.dup
    source_list_append(sql, @opts[:from])
    mode = mode.to_s.upcase.strip
    unless LOCK_MODES.include?(mode)
      raise Error, "Unsupported lock mode: #{mode}"
    end
    sql << " IN #{mode} MODE"
    @db.execute(sql, opts)
  end
  nil
end

#merge_do_nothing_when_matched(&block) ⇒ Object

Return a dataset with a WHEN MATCHED THEN DO NOTHING clause added to the MERGE statement. If a block is passed, treat it as a virtual row and use it as additional conditions for the match.

merge_do_nothing_when_matched
# WHEN MATCHED THEN DO NOTHING

merge_do_nothing_when_matched{a > 30}
# WHEN MATCHED AND (a > 30) THEN DO NOTHING


1792
1793
1794
# File 'lib/sequel/adapters/shared/postgres.rb', line 1792

def merge_do_nothing_when_matched(&block)
  _merge_when(:type=>:matched, &block)
end

#merge_do_nothing_when_not_matched(&block) ⇒ Object

Return a dataset with a WHEN NOT MATCHED THEN DO NOTHING clause added to the MERGE statement. If a block is passed, treat it as a virtual row and use it as additional conditions for the match.

merge_do_nothing_when_not_matched
# WHEN NOT MATCHED THEN DO NOTHING

merge_do_nothing_when_not_matched{a > 30}
# WHEN NOT MATCHED AND (a > 30) THEN DO NOTHING


1805
1806
1807
# File 'lib/sequel/adapters/shared/postgres.rb', line 1805

def merge_do_nothing_when_not_matched(&block)
  _merge_when(:type=>:not_matched, &block)
end

#merge_insert(*values, &block) ⇒ Object

Support OVERRIDING USER|SYSTEM VALUE for MERGE INSERT.



1810
1811
1812
1813
1814
1815
1816
# File 'lib/sequel/adapters/shared/postgres.rb', line 1810

def merge_insert(*values, &block)
  h = {:type=>:insert, :values=>values}
  if override = @opts[:override]
    h[:override] = insert_override_sql(String.new)
  end
  _merge_when(h, &block)
end

#overriding_system_valueObject

Use OVERRIDING USER VALUE for INSERT statements, so that identity columns always use the user supplied value, and an error is not raised for identity columns that are GENERATED ALWAYS.



1821
1822
1823
# File 'lib/sequel/adapters/shared/postgres.rb', line 1821

def overriding_system_value
  clone(:override=>:system)
end

#overriding_user_valueObject

Use OVERRIDING USER VALUE for INSERT statements, so that identity columns always use the sequence value instead of the user supplied value.



1827
1828
1829
# File 'lib/sequel/adapters/shared/postgres.rb', line 1827

def overriding_user_value
  clone(:override=>:user)
end

#supports_cte?(type = :select) ⇒ Boolean

Returns:

  • (Boolean)


1831
1832
1833
1834
1835
1836
1837
# File 'lib/sequel/adapters/shared/postgres.rb', line 1831

def supports_cte?(type=:select)
  if type == :select
    server_version >= 80400
  else
    server_version >= 90100
  end
end

#supports_cte_in_subqueries?Boolean

PostgreSQL supports using the WITH clause in subqueries if it supports using WITH at all (i.e. on PostgreSQL 8.4+).

Returns:

  • (Boolean)


1841
1842
1843
# File 'lib/sequel/adapters/shared/postgres.rb', line 1841

def supports_cte_in_subqueries?
  supports_cte?
end

#supports_distinct_on?Boolean

DISTINCT ON is a PostgreSQL extension

Returns:

  • (Boolean)


1846
1847
1848
# File 'lib/sequel/adapters/shared/postgres.rb', line 1846

def supports_distinct_on?
  true
end

#supports_group_cube?Boolean

PostgreSQL 9.5+ supports GROUP CUBE

Returns:

  • (Boolean)


1851
1852
1853
# File 'lib/sequel/adapters/shared/postgres.rb', line 1851

def supports_group_cube?
  server_version >= 90500
end

#supports_group_rollup?Boolean

PostgreSQL 9.5+ supports GROUP ROLLUP

Returns:

  • (Boolean)


1856
1857
1858
# File 'lib/sequel/adapters/shared/postgres.rb', line 1856

def supports_group_rollup?
  server_version >= 90500
end

#supports_grouping_sets?Boolean

PostgreSQL 9.5+ supports GROUPING SETS

Returns:

  • (Boolean)


1861
1862
1863
# File 'lib/sequel/adapters/shared/postgres.rb', line 1861

def supports_grouping_sets?
  server_version >= 90500
end

#supports_insert_conflict?Boolean

PostgreSQL 9.5+ supports the ON CONFLICT clause to INSERT.

Returns:

  • (Boolean)


1871
1872
1873
# File 'lib/sequel/adapters/shared/postgres.rb', line 1871

def supports_insert_conflict?
  server_version >= 90500
end

#supports_insert_select?Boolean

True unless insert returning has been disabled for this dataset.

Returns:

  • (Boolean)


1866
1867
1868
# File 'lib/sequel/adapters/shared/postgres.rb', line 1866

def supports_insert_select?
  !@opts[:disable_insert_returning]
end

#supports_lateral_subqueries?Boolean

PostgreSQL 9.3+ supports lateral subqueries

Returns:

  • (Boolean)


1876
1877
1878
# File 'lib/sequel/adapters/shared/postgres.rb', line 1876

def supports_lateral_subqueries?
  server_version >= 90300
end

#supports_merge?Boolean

PostgreSQL 15+ supports MERGE.

Returns:

  • (Boolean)


1886
1887
1888
# File 'lib/sequel/adapters/shared/postgres.rb', line 1886

def supports_merge?
  server_version >= 150000
end

#supports_modifying_joins?Boolean

PostgreSQL supports modifying joined datasets

Returns:

  • (Boolean)


1881
1882
1883
# File 'lib/sequel/adapters/shared/postgres.rb', line 1881

def supports_modifying_joins?
  true
end

#supports_nowait?Boolean

PostgreSQL supports NOWAIT.

Returns:

  • (Boolean)


1891
1892
1893
# File 'lib/sequel/adapters/shared/postgres.rb', line 1891

def supports_nowait?
  true
end

#supports_regexp?Boolean

PostgreSQL supports pattern matching via regular expressions

Returns:

  • (Boolean)


1901
1902
1903
# File 'lib/sequel/adapters/shared/postgres.rb', line 1901

def supports_regexp?
  true
end

#supports_returning?(type) ⇒ Boolean

Returning is always supported.

Returns:

  • (Boolean)


1896
1897
1898
# File 'lib/sequel/adapters/shared/postgres.rb', line 1896

def supports_returning?(type)
  true
end

#supports_skip_locked?Boolean

PostgreSQL 9.5+ supports SKIP LOCKED.

Returns:

  • (Boolean)


1906
1907
1908
# File 'lib/sequel/adapters/shared/postgres.rb', line 1906

def supports_skip_locked?
  server_version >= 90500
end

#supports_timestamp_timezones?Boolean

PostgreSQL supports timezones in literal timestamps

Returns:

  • (Boolean)


1911
1912
1913
# File 'lib/sequel/adapters/shared/postgres.rb', line 1911

def supports_timestamp_timezones?
  true
end

#supports_window_clause?Boolean

PostgreSQL 8.4+ supports WINDOW clause.

Returns:

  • (Boolean)


1916
1917
1918
# File 'lib/sequel/adapters/shared/postgres.rb', line 1916

def supports_window_clause?
  server_version >= 80400
end

#supports_window_function_frame_option?(option) ⇒ Boolean

Base support added in 8.4, offset supported added in 9.0, GROUPS and EXCLUDE support added in 11.0.

Returns:

  • (Boolean)


1927
1928
1929
1930
1931
1932
1933
1934
1935
1936
1937
1938
# File 'lib/sequel/adapters/shared/postgres.rb', line 1927

def supports_window_function_frame_option?(option)
  case option
  when :rows, :range
    true
  when :offset
    server_version >= 90000
  when :groups, :exclude
    server_version >= 110000
  else
    false
  end
end

#supports_window_functions?Boolean

PostgreSQL 8.4+ supports window functions

Returns:

  • (Boolean)


1921
1922
1923
# File 'lib/sequel/adapters/shared/postgres.rb', line 1921

def supports_window_functions?
  server_version >= 80400
end

#truncate(opts = OPTS) ⇒ Object

Truncates the dataset. Returns nil.

Options: :cascade :: whether to use the CASCADE option, useful when truncating tables with foreign keys. :only :: truncate using ONLY, so child tables are unaffected :restart :: use RESTART IDENTITY to restart any related sequences

:only and :restart only work correctly on PostgreSQL 8.4+.

Usage: DB.truncate

TRUNCATE TABLE "table"

DB[:table].truncate(cascade: true, only: true, restart: true)
# TRUNCATE TABLE ONLY "table" RESTART IDENTITY CASCADE


1956
1957
1958
1959
1960
1961
1962
# File 'lib/sequel/adapters/shared/postgres.rb', line 1956

def truncate(opts = OPTS)
  if opts.empty?
    super()
  else
    clone(:truncate_opts=>opts).truncate
  end
end

#with_tiesObject

Use WITH TIES when limiting the result set to also include additional rules that have the same results for the order column as the final row. Requires PostgreSQL 13.



1967
1968
1969
# File 'lib/sequel/adapters/shared/postgres.rb', line 1967

def with_ties
  clone(:limit_with_ties=>true)
end