Class: Sequel::SQL::BooleanExpression

Inherits:
ComplexExpression show all
Includes:
BooleanMethods
Defined in:
lib/sequel/sql.rb

Overview

Subclass of ComplexExpression where the expression results in a boolean value in SQL.

Constant Summary

Constants inherited from ComplexExpression

ComplexExpression::ASSOCIATIVE_OPERATORS, ComplexExpression::BITWISE_OPERATORS, ComplexExpression::BOOLEAN_OPERATOR_METHODS, ComplexExpression::CONSTANT_INVERSIONS, ComplexExpression::CUSTOM_EXPRESSIONS, ComplexExpression::EQUALITY_OPERATORS, ComplexExpression::INEQUALITY_OPERATORS, ComplexExpression::IN_OPERATORS, ComplexExpression::IS_OPERATORS, ComplexExpression::LIKE_OPERATORS, ComplexExpression::MATHEMATICAL_OPERATORS, ComplexExpression::N_ARITY_OPERATORS, ComplexExpression::ONE_ARITY_OPERATORS, ComplexExpression::OPERTATOR_INVERSIONS, ComplexExpression::REGEXP_OPERATORS, ComplexExpression::TWO_ARITY_OPERATORS

Instance Attribute Summary

Attributes inherited from ComplexExpression

#args, #op

Class Method Summary collapse

Instance Method Summary collapse

Methods included from BooleanMethods

#~

Methods inherited from ComplexExpression

#initialize, #sql_number, #sql_string

Methods included from SubscriptMethods

#sql_subscript

Methods included from PatternMatchMethods

#!~, #=~

Methods included from OrderMethods

#asc, #desc

Methods included from CastMethods

#cast, #cast_numeric, #cast_string

Methods included from AliasMethods

#as

Methods inherited from Expression

#==, attr_reader, #clone, #eql?, #hash, inherited, #inspect

Constructor Details

This class inherits a constructor from Sequel::SQL::ComplexExpression

Class Method Details

.from_value_pairs(pairs, op = :AND, negate = false) ⇒ Object

Take pairs of values (e.g. a hash or array of two element arrays) and converts it to a BooleanExpression. The operator and args used depends on the case of the right (2nd) argument:

0..10

left >= 0 AND left <= 10

[1,2]

left IN (1,2)

nil

left IS NULL

true

left IS TRUE

false

left IS FALSE

/as/

left ~ 'as'

:blah :: left = blah

'blah'

left = 'blah'

If multiple arguments are given, they are joined with the op given (AND by default, OR possible). If negate is set to true, all subexpressions are inverted before used. Therefore, the following expressions are equivalent:

~from_value_pairs(hash)
from_value_pairs(hash, :OR, true)


1067
1068
1069
1070
1071
# File 'lib/sequel/sql.rb', line 1067

def self.from_value_pairs(pairs, op=:AND, negate=false)
  pairs = pairs.map{|l,r| from_value_pair(l, r)}
  pairs.map!{|ce| invert(ce)} if negate
  pairs.length == 1 ? pairs[0] : new(op, *pairs)
end

.invert(ce) ⇒ Object

Invert the expression, if possible. If the expression cannot be inverted, raise an error. An inverted expression should match everything that the uninverted expression did not match, and vice-versa, except for possible issues with SQL NULL (i.e. 1 == NULL is NULL and 1 != NULL is also NULL).

BooleanExpression.invert(:a) # NOT "a"


1108
1109
1110
1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
1124
# File 'lib/sequel/sql.rb', line 1108

def self.invert(ce)
  case ce
  when BooleanExpression
    case op = ce.op
    when :AND, :OR
      BooleanExpression.new(OPERTATOR_INVERSIONS[op], *ce.args.map{|a| BooleanExpression.invert(a)})
    else
      BooleanExpression.new(OPERTATOR_INVERSIONS[op], *ce.args.dup)
    end
  when StringExpression, NumericExpression
    raise(Sequel::Error, "cannot invert #{ce.inspect}")
  when Constant
    CONSTANT_INVERSIONS[ce] || raise(Sequel::Error, "cannot invert #{ce.inspect}")
  else
    BooleanExpression.new(:NOT, ce)
  end
end

Instance Method Details

#&(ce) ⇒ Object

Always use an AND operator for & on BooleanExpressions



1127
1128
1129
# File 'lib/sequel/sql.rb', line 1127

def &(ce)
  BooleanExpression.new(:AND, self, ce)
end

#sql_booleanObject

Return self instead of creating a new object to save on memory.



1137
1138
1139
# File 'lib/sequel/sql.rb', line 1137

def sql_boolean
  self
end

#|(ce) ⇒ Object

Always use an OR operator for | on BooleanExpressions



1132
1133
1134
# File 'lib/sequel/sql.rb', line 1132

def |(ce)
  BooleanExpression.new(:OR, self, ce)
end