Class: Sequel::SQL::OrderedExpression
- Inherits:
-
Expression
- Object
- Expression
- Sequel::SQL::OrderedExpression
- Defined in:
- lib/sequel/sql.rb
Overview
Represents a column/expression to order the result set by.
Instance Attribute Summary collapse
-
#descending ⇒ Object
readonly
Whether the expression should order the result set in a descending manner.
-
#expression ⇒ Object
readonly
The expression to order the result set by.
Instance Method Summary collapse
-
#initialize(expression, descending = true) ⇒ OrderedExpression
constructor
Set the expression and descending attributes to the given values.
Methods inherited from Expression
Constructor Details
#initialize(expression, descending = true) ⇒ OrderedExpression
Set the expression and descending attributes to the given values.
668 669 670 |
# File 'lib/sequel/sql.rb', line 668 def initialize(expression, descending = true) @expression, @descending = expression, descending end |
Instance Attribute Details
#descending ⇒ Object (readonly)
Whether the expression should order the result set in a descending manner
665 666 667 |
# File 'lib/sequel/sql.rb', line 665 def descending @descending end |
#expression ⇒ Object (readonly)
The expression to order the result set by.
662 663 664 |
# File 'lib/sequel/sql.rb', line 662 def expression @expression end |