Class: Sequel::SQL::Window
- Inherits:
-
Expression
- Object
- Expression
- Sequel::SQL::Window
- Defined in:
- lib/sequel/sql.rb
Overview
A Window is part of a window function specifying the window over which the function operates. It is separated from the WindowFunction class because it also can be used separately on some databases.
Instance Attribute Summary collapse
-
#opts ⇒ Object
readonly
The options for this window.
Instance Method Summary collapse
-
#initialize(opts = {}) ⇒ Window
constructor
Set the options to the options given.
Methods inherited from Expression
#==, attr_reader, #eql?, #hash, inherited, #inspect, #lit, #sql_literal
Constructor Details
#initialize(opts = {}) ⇒ Window
Set the options to the options given
1587 1588 1589 |
# File 'lib/sequel/sql.rb', line 1587 def initialize(opts={}) @opts = opts end |
Instance Attribute Details
#opts ⇒ Object (readonly)
The options for this window. Options currently supported:
- :frame
-
if specified, should be :all, :rows, or a String that is used literally. :all always operates over all rows in the partition, while :rows excludes the current row’s later peers. The default is to include all previous rows in the partition up to the current row’s last peer.
- :order
-
order on the column(s) given
- :partition
-
partition/group on the column(s) given
- :window
-
base results on a previously specified named window
1584 1585 1586 |
# File 'lib/sequel/sql.rb', line 1584 def opts @opts end |