Method: Sequel::SQL::Window#opts
- Defined in:
- lib/sequel/sql.rb
#opts ⇒ Object (readonly)
The options for this window. Options currently supported:
- :frame
-
if specified, should be :all, :rows, or a String that is used literally. :all always operates over all rows in the partition, while :rows excludes the current row’s later peers. The default is to include all previous rows in the partition up to the current row’s last peer.
- :order
-
order on the column(s) given
- :partition
-
partition/group on the column(s) given
- :window
-
base results on a previously specified named window
1522 1523 1524 |
# File 'lib/sequel/sql.rb', line 1522 def opts @opts end |