Module: Sequel::SQLite::DatasetMethods
- Includes:
- Dataset::Replace, UnmodifiedIdentifiers::DatasetMethods
- Included in:
- Amalgalite::Dataset, Dataset
- Defined in:
- lib/sequel/adapters/shared/sqlite.rb
Constant Summary collapse
- INSERT_CONFLICT_RESOLUTIONS =
The allowed values for insert_conflict
%w'ROLLBACK ABORT FAIL IGNORE REPLACE'.each(&:freeze).freeze
- CONSTANT_MAP =
{:CURRENT_DATE=>"date(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIMESTAMP=>"datetime(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIME=>"time(CURRENT_TIMESTAMP, 'localtime')".freeze}.freeze
- EXTRACT_MAP =
{:year=>"'%Y'", :month=>"'%m'", :day=>"'%d'", :hour=>"'%H'", :minute=>"'%M'", :second=>"'%f'"}.freeze
Instance Method Summary collapse
- #cast_sql_append(sql, expr, type) ⇒ Object
-
#complex_expression_sql_append(sql, op, args) ⇒ Object
SQLite doesn't support a NOT LIKE b, you need to use NOT (a LIKE b).
-
#constant_sql_append(sql, constant) ⇒ Object
SQLite has CURRENT_TIMESTAMP and related constants in UTC instead of in localtime, so convert those constants to local time.
-
#delete(&block) ⇒ Object
SQLite performs a TRUNCATE style DELETE if no filter is specified.
-
#empty? ⇒ Boolean
Always return false when using VALUES.
-
#explain(opts = nil) ⇒ Object
Return a string specifying a query explanation for a SELECT of the current dataset.
-
#having(*cond) ⇒ Object
HAVING requires GROUP BY on SQLite.
-
#insert_conflict(opts = :ignore) ⇒ Object
Handle uniqueness violations when inserting, by using a specified resolution algorithm.
-
#insert_ignore ⇒ Object
Ignore uniqueness/exclusion violations when inserting, using INSERT OR IGNORE.
-
#insert_select(*values) ⇒ Object
Support insert select for associations, so that the model code can use returning instead of a separate query.
-
#insert_select_sql(*values) ⇒ Object
The SQL to use for an insert_select, adds a RETURNING clause to the insert unless the RETURNING clause is already present.
-
#quoted_identifier_append(sql, c) ⇒ Object
SQLite uses the nonstandard ` (backtick) for quoting identifiers.
-
#returning(*values) ⇒ Object
Automatically add aliases to RETURNING values to work around SQLite bug.
-
#select(*cols) ⇒ Object
When a qualified column is selected on SQLite and the qualifier is a subselect, the column name used is the full qualified name (including the qualifier) instead of just the column name.
-
#supports_cte?(type = :select) ⇒ Boolean
SQLite 3.8.3+ supports common table expressions.
-
#supports_cte_in_subqueries? ⇒ Boolean
SQLite supports CTEs in subqueries if it supports CTEs.
-
#supports_deleting_joins? ⇒ Boolean
SQLite does not support deleting from a joined dataset.
-
#supports_derived_column_lists? ⇒ Boolean
SQLite does not support table aliases with column aliases.
-
#supports_intersect_except_all? ⇒ Boolean
SQLite does not support INTERSECT ALL or EXCEPT ALL.
-
#supports_is_true? ⇒ Boolean
SQLite does not support IS TRUE.
-
#supports_modifying_joins? ⇒ Boolean
SQLite 3.33.0 supports modifying joined datasets.
-
#supports_multiple_column_in? ⇒ Boolean
SQLite does not support multiple columns for the IN/NOT IN operators.
-
#supports_returning?(_) ⇒ Boolean
SQLite 3.35.0 supports RETURNING on INSERT/UPDATE/DELETE.
-
#supports_timestamp_timezones? ⇒ Boolean
SQLite supports timezones in literal timestamps, since it stores them as text.
-
#supports_where_true? ⇒ Boolean
SQLite cannot use WHERE 't'.
-
#supports_window_clause? ⇒ Boolean
SQLite 3.28+ supports the WINDOW clause.
-
#supports_window_function_frame_option?(option) ⇒ Boolean
SQLite 3.28.0+ supports all window frame options that Sequel supports.
-
#supports_window_functions? ⇒ Boolean
SQLite 3.25+ supports window functions.
Instance Method Details
#cast_sql_append(sql, expr, type) ⇒ Object
598 599 600 601 602 603 604 605 606 607 608 609 610 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 598 def cast_sql_append(sql, expr, type) if type == Time or type == DateTime sql << "datetime(" literal_append(sql, expr) sql << ')' elsif type == Date sql << "date(" literal_append(sql, expr) sql << ')' else super end end |
#complex_expression_sql_append(sql, op, args) ⇒ Object
SQLite doesn't support a NOT LIKE b, you need to use NOT (a LIKE b). It doesn't support xor, power, or the extract function natively, so those have to be emulated.
614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 614 def complex_expression_sql_append(sql, op, args) case op when :"NOT LIKE", :"NOT ILIKE" sql << 'NOT ' complex_expression_sql_append(sql, (op == :"NOT ILIKE" ? :ILIKE : :LIKE), args) when :^ complex_expression_arg_pairs_append(sql, args){|a, b| Sequel.lit(["((~(", " & ", ")) & (", " | ", "))"], a, b, a, b)} when :** unless (exp = args[1]).is_a?(Integer) raise(Sequel::Error, "can only emulate exponentiation on SQLite if exponent is an integer, given #{exp.inspect}") end case exp when 0 sql << '1' else sql << '(' arg = args[0] if exp < 0 invert = true exp = exp.abs sql << '(1.0 / (' end (exp - 1).times do literal_append(sql, arg) sql << " * " end literal_append(sql, arg) sql << ')' if invert sql << "))" end end when :extract part = args[0] raise(Sequel::Error, "unsupported extract argument: #{part.inspect}") unless format = EXTRACT_MAP[part] sql << "CAST(strftime(" << format << ', ' literal_append(sql, args[1]) sql << ') AS ' << (part == :second ? 'NUMERIC' : 'INTEGER') << ')' else super end end |
#constant_sql_append(sql, constant) ⇒ Object
SQLite has CURRENT_TIMESTAMP and related constants in UTC instead of in localtime, so convert those constants to local time.
659 660 661 662 663 664 665 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 659 def constant_sql_append(sql, constant) if (c = CONSTANT_MAP[constant]) && !db. sql << c else super end end |
#delete(&block) ⇒ Object
SQLite performs a TRUNCATE style DELETE if no filter is specified. Since we want to always return the count of records, add a condition that is always true and then delete.
670 671 672 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 670 def delete(&block) @opts[:where] ? super : where(1=>1).delete(&block) end |
#empty? ⇒ Boolean
Always return false when using VALUES
675 676 677 678 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 675 def empty? return false if @opts[:values] super end |
#explain(opts = nil) ⇒ Object
Return a string specifying a query explanation for a SELECT of the current dataset. Options: :query_plan :: Use EXPLAIN QUERY PLAN instead of EXPLAIN if true.
683 684 685 686 687 688 689 690 691 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 683 def explain(opts=nil) # Load the PrettyTable class, needed for explain output Sequel.extension(:_pretty_table) unless defined?(Sequel::PrettyTable) keyword = (opts && opts[:query_plan]) ? "EXPLAIN QUERY PLAN" : "EXPLAIN" ds = db.send(:metadata_dataset).clone(:sql=>"#{keyword} #{select_sql}".freeze) rows = ds.all Sequel::PrettyTable.string(rows, ds.columns) end |
#having(*cond) ⇒ Object
HAVING requires GROUP BY on SQLite
694 695 696 697 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 694 def having(*cond) raise(InvalidOperation, "Can only specify a HAVING clause on a grouped dataset") if !@opts[:group] && db.sqlite_version < 33900 super end |
#insert_conflict(opts = :ignore) ⇒ Object
Handle uniqueness violations when inserting, by using a specified resolution algorithm. With no options, uses INSERT OR REPLACE. SQLite supports the following conflict resolution algoriths: ROLLBACK, ABORT, FAIL, IGNORE and REPLACE.
On SQLite 3.24.0+, you can pass a hash to use an ON CONFLICT clause. With out :update option, uses ON CONFLICT DO NOTHING. Options:
:conflict_where :: The index filter, when using a partial index to determine uniqueness. :target :: The column name or expression to handle uniqueness violations on. :update :: A hash of columns and values to set. Uses ON CONFLICT DO UPDATE. :update_where :: A WHERE condition to use for the update.
Examples:
DB[:table].insert_conflict.insert(a: 1, b: 2)
# INSERT OR IGNORE INTO TABLE (a, b) VALUES (1, 2)
DB[:table].insert_conflict(:replace).insert(a: 1, b: 2)
# INSERT OR REPLACE INTO TABLE (a, b) VALUES (1, 2)
DB[:table].insert_conflict({}).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT DO NOTHING
DB[:table].insert_conflict(target: :a).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT (a) DO NOTHING
DB[:table].insert_conflict(target: :a, conflict_where: {c: true}).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT (a) WHERE (c IS TRUE) DO NOTHING
DB[:table].insert_conflict(target: :a, update: {b: Sequel[:excluded][:b]}).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT (a) DO UPDATE SET b = excluded.b
DB[:table].insert_conflict(target: :a,
update: {b: Sequel[:excluded][:b]}, update_where: {Sequel[:table][:status_id] => 1}).insert(a: 1, b: 2)
# INSERT INTO TABLE (a, b) VALUES (1, 2)
# ON CONFLICT (a) DO UPDATE SET b = excluded.b WHERE (table.status_id = 1)
772 773 774 775 776 777 778 779 780 781 782 783 784 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 772 def insert_conflict(opts = :ignore) case opts when Symbol, String unless INSERT_CONFLICT_RESOLUTIONS.include?(opts.to_s.upcase) raise Error, "Invalid symbol or string passed to Dataset#insert_conflict: #{opts.inspect}. The allowed values are: :rollback, :abort, :fail, :ignore, or :replace" end clone(:insert_conflict => opts) when Hash clone(:insert_on_conflict => opts) else raise Error, "Invalid value passed to Dataset#insert_conflict: #{opts.inspect}, should use a symbol or a hash" end end |
#insert_ignore ⇒ Object
Ignore uniqueness/exclusion violations when inserting, using INSERT OR IGNORE. Exists mostly for compatibility to MySQL's insert_ignore. Example:
DB[:table].insert_ignore.insert(a: 1, b: 2)
# INSERT OR IGNORE INTO TABLE (a, b) VALUES (1, 2)
791 792 793 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 791 def insert_ignore insert_conflict(:ignore) end |
#insert_select(*values) ⇒ Object
Support insert select for associations, so that the model code can use returning instead of a separate query.
701 702 703 704 705 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 701 def insert_select(*values) return unless supports_insert_select? # Handle case where query does not return a row server?(:default).with_sql_first(insert_select_sql(*values)) || false end |
#insert_select_sql(*values) ⇒ Object
The SQL to use for an insert_select, adds a RETURNING clause to the insert unless the RETURNING clause is already present.
709 710 711 712 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 709 def insert_select_sql(*values) ds = opts[:returning] ? self : returning ds.insert_sql(*values) end |
#quoted_identifier_append(sql, c) ⇒ Object
SQLite uses the nonstandard ` (backtick) for quoting identifiers.
715 716 717 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 715 def quoted_identifier_append(sql, c) sql << '`' << c.to_s.gsub('`', '``') << '`' end |
#returning(*values) ⇒ Object
Automatically add aliases to RETURNING values to work around SQLite bug.
796 797 798 799 800 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 796 def returning(*values) return super if values.empty? raise Error, "RETURNING is not supported on #{db.database_type}" unless supports_returning?(:insert) clone(:returning=>_returning_values(values).freeze) end |
#select(*cols) ⇒ Object
When a qualified column is selected on SQLite and the qualifier is a subselect, the column name used is the full qualified name (including the qualifier) instead of just the column name. To get correct column names, you must use an alias.
723 724 725 726 727 728 729 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 723 def select(*cols) if ((f = @opts[:from]) && f.any?{|t| t.is_a?(Dataset) || (t.is_a?(SQL::AliasedExpression) && t.expression.is_a?(Dataset))}) || ((j = @opts[:join]) && j.any?{|t| t.table.is_a?(Dataset)}) super(*cols.map{|c| alias_qualified_column(c)}) else super end end |
#supports_cte?(type = :select) ⇒ Boolean
SQLite 3.8.3+ supports common table expressions.
803 804 805 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 803 def supports_cte?(type=:select) db.sqlite_version >= 30803 end |
#supports_cte_in_subqueries? ⇒ Boolean
SQLite supports CTEs in subqueries if it supports CTEs.
808 809 810 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 808 def supports_cte_in_subqueries? supports_cte? end |
#supports_deleting_joins? ⇒ Boolean
SQLite does not support deleting from a joined dataset
818 819 820 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 818 def supports_deleting_joins? false end |
#supports_derived_column_lists? ⇒ Boolean
SQLite does not support table aliases with column aliases
813 814 815 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 813 def supports_derived_column_lists? false end |
#supports_intersect_except_all? ⇒ Boolean
SQLite does not support INTERSECT ALL or EXCEPT ALL
823 824 825 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 823 def supports_intersect_except_all? false end |
#supports_is_true? ⇒ Boolean
SQLite does not support IS TRUE
828 829 830 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 828 def supports_is_true? false end |
#supports_modifying_joins? ⇒ Boolean
SQLite 3.33.0 supports modifying joined datasets
833 834 835 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 833 def db.sqlite_version >= 33300 end |
#supports_multiple_column_in? ⇒ Boolean
SQLite does not support multiple columns for the IN/NOT IN operators
838 839 840 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 838 def supports_multiple_column_in? false end |
#supports_returning?(_) ⇒ Boolean
SQLite 3.35.0 supports RETURNING on INSERT/UPDATE/DELETE.
843 844 845 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 843 def supports_returning?(_) db.sqlite_version >= 33500 end |
#supports_timestamp_timezones? ⇒ Boolean
SQLite supports timezones in literal timestamps, since it stores them as text. But using timezones in timestamps breaks SQLite datetime functions, so we allow the user to override the default per database.
850 851 852 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 850 def db. end |
#supports_where_true? ⇒ Boolean
SQLite cannot use WHERE 't'.
855 856 857 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 855 def supports_where_true? false end |
#supports_window_clause? ⇒ Boolean
SQLite 3.28+ supports the WINDOW clause.
860 861 862 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 860 def supports_window_clause? db.sqlite_version >= 32800 end |
#supports_window_function_frame_option?(option) ⇒ Boolean
SQLite 3.28.0+ supports all window frame options that Sequel supports
873 874 875 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 873 def supports_window_function_frame_option?(option) db.sqlite_version >= 32800 ? true : super end |
#supports_window_functions? ⇒ Boolean
SQLite 3.25+ supports window functions. However, support is only enabled on SQLite 3.26.0+ because internal Sequel usage of window functions to implement eager loading of limited associations triggers an SQLite crash bug in versions 3.25.0-3.25.3.
868 869 870 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 868 def supports_window_functions? db.sqlite_version >= 32600 end |