Module: Sequel::SQLite::DatasetMethods
- Includes:
- Dataset::Replace
- Included in:
- Amalgalite::Dataset, Dataset, Sequel::Swift::SQLite::Dataset
- Defined in:
- lib/sequel/adapters/shared/sqlite.rb
Overview
Instance methods for datasets that connect to an SQLite database
Constant Summary collapse
- SELECT_CLAUSE_METHODS =
Dataset.clause_methods(:select, %w'select distinct columns from join where group having compounds order limit')
- CONSTANT_MAP =
{:CURRENT_DATE=>"date(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIMESTAMP=>"datetime(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIME=>"time(CURRENT_TIMESTAMP, 'localtime')".freeze}
- EMULATED_FUNCTION_MAP =
{:char_length=>'length'.freeze}
- EXTRACT_MAP =
{:year=>"'%Y'", :month=>"'%m'", :day=>"'%d'", :hour=>"'%H'", :minute=>"'%M'", :second=>"'%f'"}
- NOT_SPACE =
Dataset::NOT_SPACE
- COMMA =
Dataset::COMMA
- PAREN_CLOSE =
Dataset::PAREN_CLOSE
- AS =
Dataset::AS
- APOS =
Dataset::APOS
- EXTRACT_OPEN =
"CAST(strftime(".freeze
- EXTRACT_CLOSE =
') AS '.freeze
- NUMERIC =
'NUMERIC'.freeze
- INTEGER =
'INTEGER'.freeze
- BACKTICK =
'`'.freeze
- BACKTICK_RE =
/`/.freeze
- DOUBLE_BACKTICK =
'``'.freeze
- BLOB_START =
"X'".freeze
- HSTAR =
"H*".freeze
- DATE_OPEN =
"date(".freeze
- DATETIME_OPEN =
"datetime(".freeze
Instance Method Summary collapse
- #cast_sql_append(sql, expr, type) ⇒ Object
-
#complex_expression_sql_append(sql, op, args) ⇒ Object
SQLite doesn’t support a NOT LIKE b, you need to use NOT (a LIKE b).
-
#constant_sql_append(sql, constant) ⇒ Object
SQLite has CURRENT_TIMESTAMP and related constants in UTC instead of in localtime, so convert those constants to local time.
-
#delete ⇒ Object
SQLite performs a TRUNCATE style DELETE if no filter is specified.
-
#explain(opts = nil) ⇒ Object
Return an array of strings specifying a query explanation for a SELECT of the current dataset.
-
#having(*cond) ⇒ Object
HAVING requires GROUP BY on SQLite.
-
#quoted_identifier_append(sql, c) ⇒ Object
SQLite uses the nonstandard ‘ (backtick) for quoting identifiers.
-
#select(*cols) ⇒ Object
When a qualified column is selected on SQLite and the qualifier is a subselect, the column name used is the full qualified name (including the qualifier) instead of just the column name.
-
#supports_intersect_except_all? ⇒ Boolean
SQLite does not support INTERSECT ALL or EXCEPT ALL.
-
#supports_is_true? ⇒ Boolean
SQLite does not support IS TRUE.
-
#supports_multiple_column_in? ⇒ Boolean
SQLite does not support multiple columns for the IN/NOT IN operators.
-
#supports_timestamp_timezones? ⇒ Boolean
SQLite supports timezones in literal timestamps, since it stores them as text.
-
#supports_where_true? ⇒ Boolean
SQLite cannot use WHERE ‘t’.
Instance Method Details
#cast_sql_append(sql, expr, type) ⇒ Object
501 502 503 504 505 506 507 508 509 510 511 512 513 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 501 def cast_sql_append(sql, expr, type) if type == Time or type == DateTime sql << DATETIME_OPEN literal_append(sql, expr) sql << PAREN_CLOSE elsif type == Date sql << DATE_OPEN literal_append(sql, expr) sql << PAREN_CLOSE else super end end |
#complex_expression_sql_append(sql, op, args) ⇒ Object
SQLite doesn’t support a NOT LIKE b, you need to use NOT (a LIKE b). It doesn’t support xor or the extract function natively, so those have to be emulated.
517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 517 def complex_expression_sql_append(sql, op, args) case op when :"NOT LIKE", :"NOT ILIKE" sql << NOT_SPACE complex_expression_sql_append(sql, (op == :"NOT ILIKE" ? :ILIKE : :LIKE), args) when :^ sql << complex_expression_arg_pairs(args) do |a, b| a = literal(a) b = literal(b) "((~(#{a} & #{b})) & (#{a} | #{b}))" end when :extract part = args.at(0) raise(Sequel::Error, "unsupported extract argument: #{part.inspect}") unless format = EXTRACT_MAP[part] sql << EXTRACT_OPEN << format << COMMA literal_append(sql, args.at(1)) sql << EXTRACT_CLOSE << (part == :second ? NUMERIC : INTEGER) << PAREN_CLOSE else super end end |
#constant_sql_append(sql, constant) ⇒ Object
SQLite has CURRENT_TIMESTAMP and related constants in UTC instead of in localtime, so convert those constants to local time.
541 542 543 544 545 546 547 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 541 def constant_sql_append(sql, constant) if c = CONSTANT_MAP[constant] sql << c else super end end |
#delete ⇒ Object
SQLite performs a TRUNCATE style DELETE if no filter is specified. Since we want to always return the count of records, add a condition that is always true and then delete.
552 553 554 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 552 def delete @opts[:where] ? super : where(1=>1).delete end |
#explain(opts = nil) ⇒ Object
Return an array of strings specifying a query explanation for a SELECT of the current dataset. Currently, the options are ignore, but it accepts options to be compatible with other adapters.
559 560 561 562 563 564 565 566 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 559 def explain(opts=nil) # Load the PrettyTable class, needed for explain output Sequel.extension(:_pretty_table) unless defined?(Sequel::PrettyTable) ds = db.send(:metadata_dataset).clone(:sql=>"EXPLAIN #{select_sql}") rows = ds.all Sequel::PrettyTable.string(rows, ds.columns) end |
#having(*cond) ⇒ Object
HAVING requires GROUP BY on SQLite
569 570 571 572 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 569 def having(*cond) raise(InvalidOperation, "Can only specify a HAVING clause on a grouped dataset") unless @opts[:group] super end |
#quoted_identifier_append(sql, c) ⇒ Object
SQLite uses the nonstandard ‘ (backtick) for quoting identifiers.
575 576 577 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 575 def quoted_identifier_append(sql, c) sql << BACKTICK << c.to_s.gsub(BACKTICK_RE, DOUBLE_BACKTICK) << BACKTICK end |
#select(*cols) ⇒ Object
When a qualified column is selected on SQLite and the qualifier is a subselect, the column name used is the full qualified name (including the qualifier) instead of just the column name. To get correct column names, you must use an alias.
583 584 585 586 587 588 589 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 583 def select(*cols) if ((f = @opts[:from]) && f.any?{|t| t.is_a?(Dataset) || (t.is_a?(SQL::AliasedExpression) && t.expression.is_a?(Dataset))}) || ((j = @opts[:join]) && j.any?{|t| t.table.is_a?(Dataset)}) super(*cols.map{|c| alias_qualified_column(c)}) else super end end |
#supports_intersect_except_all? ⇒ Boolean
SQLite does not support INTERSECT ALL or EXCEPT ALL
592 593 594 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 592 def supports_intersect_except_all? false end |
#supports_is_true? ⇒ Boolean
SQLite does not support IS TRUE
597 598 599 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 597 def supports_is_true? false end |
#supports_multiple_column_in? ⇒ Boolean
SQLite does not support multiple columns for the IN/NOT IN operators
602 603 604 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 602 def supports_multiple_column_in? false end |
#supports_timestamp_timezones? ⇒ Boolean
SQLite supports timezones in literal timestamps, since it stores them as text. But using timezones in timestamps breaks SQLite datetime functions, so we allow the user to override the default per database.
609 610 611 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 609 def db. end |
#supports_where_true? ⇒ Boolean
SQLite cannot use WHERE ‘t’.
614 615 616 |
# File 'lib/sequel/adapters/shared/sqlite.rb', line 614 def supports_where_true? false end |