Module: Sequel::SQLite::DatasetMethods

Includes:
Dataset::Replace
Included in:
Amalgalite::Dataset, Dataset, Sequel::Swift::SQLite::Dataset
Defined in:
lib/sequel/adapters/shared/sqlite.rb

Overview

Instance methods for datasets that connect to an SQLite database

Constant Summary collapse

CONSTANT_MAP =
{:CURRENT_DATE=>"date(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIMESTAMP=>"datetime(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIME=>"time(CURRENT_TIMESTAMP, 'localtime')".freeze}
EMULATED_FUNCTION_MAP =
{:char_length=>'length'.freeze}
EXTRACT_MAP =
{:year=>"'%Y'", :month=>"'%m'", :day=>"'%d'", :hour=>"'%H'", :minute=>"'%M'", :second=>"'%f'"}
NOT_SPACE =
Dataset::NOT_SPACE
COMMA =
Dataset::COMMA
PAREN_CLOSE =
Dataset::PAREN_CLOSE
AS =
Dataset::AS
APOS =
Dataset::APOS
EXTRACT_OPEN =
"CAST(strftime(".freeze
EXTRACT_CLOSE =
') AS '.freeze
NUMERIC =
'NUMERIC'.freeze
INTEGER =
'INTEGER'.freeze
BACKTICK =
'`'.freeze
BACKTICK_RE =
/`/.freeze
DOUBLE_BACKTICK =
'``'.freeze
BLOB_START =
"X'".freeze
HSTAR =
"H*".freeze
DATE_OPEN =
"date(".freeze
DATETIME_OPEN =
"datetime(".freeze
ONLY_OFFSET =
" LIMIT -1 OFFSET ".freeze
OR =
" OR ".freeze
SELECT_VALUES =
"VALUES ".freeze

Instance Method Summary collapse

Instance Method Details

#cast_sql_append(sql, expr, type) ⇒ Object



535
536
537
538
539
540
541
542
543
544
545
546
547
# File 'lib/sequel/adapters/shared/sqlite.rb', line 535

def cast_sql_append(sql, expr, type)
  if type == Time or type == DateTime
    sql << DATETIME_OPEN
    literal_append(sql, expr)
    sql << PAREN_CLOSE
  elsif type == Date
    sql << DATE_OPEN
    literal_append(sql, expr)
    sql << PAREN_CLOSE
  else
    super
  end
end

#complex_expression_sql_append(sql, op, args) ⇒ Object

SQLite doesn’t support a NOT LIKE b, you need to use NOT (a LIKE b). It doesn’t support xor or the extract function natively, so those have to be emulated.



551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
# File 'lib/sequel/adapters/shared/sqlite.rb', line 551

def complex_expression_sql_append(sql, op, args)
  case op
  when :"NOT LIKE", :"NOT ILIKE"
    sql << NOT_SPACE
    complex_expression_sql_append(sql, (op == :"NOT ILIKE" ? :ILIKE : :LIKE), args)
  when :^
    complex_expression_arg_pairs_append(sql, args){|a, b| Sequel.lit(["((~(", " & ", ")) & (", " | ", "))"], a, b, a, b)}
  when :extract
    part = args.at(0)
    raise(Sequel::Error, "unsupported extract argument: #{part.inspect}") unless format = EXTRACT_MAP[part]
    sql << EXTRACT_OPEN << format << COMMA
    literal_append(sql, args.at(1))
    sql << EXTRACT_CLOSE << (part == :second ? NUMERIC : INTEGER) << PAREN_CLOSE
  else
    super
  end
end

#constant_sql_append(sql, constant) ⇒ Object

SQLite has CURRENT_TIMESTAMP and related constants in UTC instead of in localtime, so convert those constants to local time.



571
572
573
574
575
576
577
# File 'lib/sequel/adapters/shared/sqlite.rb', line 571

def constant_sql_append(sql, constant)
  if c = CONSTANT_MAP[constant]
    sql << c
  else
    super
  end
end

#deleteObject

SQLite performs a TRUNCATE style DELETE if no filter is specified. Since we want to always return the count of records, add a condition that is always true and then delete.



582
583
584
# File 'lib/sequel/adapters/shared/sqlite.rb', line 582

def delete
  @opts[:where] ? super : where(1=>1).delete
end

#explain(opts = nil) ⇒ Object

Return an array of strings specifying a query explanation for a SELECT of the current dataset. Currently, the options are ignore, but it accepts options to be compatible with other adapters.



589
590
591
592
593
594
595
596
# File 'lib/sequel/adapters/shared/sqlite.rb', line 589

def explain(opts=nil)
  # Load the PrettyTable class, needed for explain output
  Sequel.extension(:_pretty_table) unless defined?(Sequel::PrettyTable)

  ds = db.send(:metadata_dataset).clone(:sql=>"EXPLAIN #{select_sql}")
  rows = ds.all
  Sequel::PrettyTable.string(rows, ds.columns)
end

#having(*cond) ⇒ Object

HAVING requires GROUP BY on SQLite

Raises:



599
600
601
602
# File 'lib/sequel/adapters/shared/sqlite.rb', line 599

def having(*cond)
  raise(InvalidOperation, "Can only specify a HAVING clause on a grouped dataset") unless @opts[:group]
  super
end

#insert_conflict(resolution = :ignore) ⇒ Object

Handle uniqueness violations when inserting, by using a specified resolution algorithm. With no options, uses INSERT OR REPLACE. SQLite supports the following conflict resolution algoriths: ROLLBACK, ABORT, FAIL, IGNORE and REPLACE.

Examples:

DB[:table].insert_conflict.insert(:a=>1, :b=>2)
# INSERT OR IGNORE INTO TABLE (a, b) VALUES (1, 2)

DB[:table].insert_conflict(:replace).insert(:a=>1, :b=>2)
# INSERT OR REPLACE INTO TABLE (a, b) VALUES (1, 2)


633
634
635
# File 'lib/sequel/adapters/shared/sqlite.rb', line 633

def insert_conflict(resolution = :ignore)
  clone(:insert_conflict => resolution)
end

#insert_ignoreObject

Ignore uniqueness/exclusion violations when inserting, using INSERT OR IGNORE. Exists mostly for compatibility to MySQL’s insert_ignore. Example:

DB[:table].insert_ignore.insert(:a=>1, :b=>2)
# INSERT OR IGNORE INTO TABLE (a, b) VALUES (1, 2)


642
643
644
# File 'lib/sequel/adapters/shared/sqlite.rb', line 642

def insert_ignore
  insert_conflict(:ignore)
end

#quoted_identifier_append(sql, c) ⇒ Object

SQLite uses the nonstandard ‘ (backtick) for quoting identifiers.



605
606
607
# File 'lib/sequel/adapters/shared/sqlite.rb', line 605

def quoted_identifier_append(sql, c)
  sql << BACKTICK << c.to_s.gsub(BACKTICK_RE, DOUBLE_BACKTICK) << BACKTICK
end

#select(*cols) ⇒ Object

When a qualified column is selected on SQLite and the qualifier is a subselect, the column name used is the full qualified name (including the qualifier) instead of just the column name. To get correct column names, you must use an alias.



613
614
615
616
617
618
619
# File 'lib/sequel/adapters/shared/sqlite.rb', line 613

def select(*cols)
  if ((f = @opts[:from]) && f.any?{|t| t.is_a?(Dataset) || (t.is_a?(SQL::AliasedExpression) && t.expression.is_a?(Dataset))}) || ((j = @opts[:join]) && j.any?{|t| t.table.is_a?(Dataset)})
    super(*cols.map{|c| alias_qualified_column(c)})
  else
    super
  end
end

#supports_cte?(type = :select) ⇒ Boolean

SQLite 3.8.3+ supports common table expressions.

Returns:

  • (Boolean)


647
648
649
# File 'lib/sequel/adapters/shared/sqlite.rb', line 647

def supports_cte?(type=:select)
  db.sqlite_version >= 30803
end

#supports_derived_column_lists?Boolean

SQLite does not support table aliases with column aliases

Returns:

  • (Boolean)


652
653
654
# File 'lib/sequel/adapters/shared/sqlite.rb', line 652

def supports_derived_column_lists?
  false
end

#supports_intersect_except_all?Boolean

SQLite does not support INTERSECT ALL or EXCEPT ALL

Returns:

  • (Boolean)


657
658
659
# File 'lib/sequel/adapters/shared/sqlite.rb', line 657

def supports_intersect_except_all?
  false
end

#supports_is_true?Boolean

SQLite does not support IS TRUE

Returns:

  • (Boolean)


662
663
664
# File 'lib/sequel/adapters/shared/sqlite.rb', line 662

def supports_is_true?
  false
end

#supports_multiple_column_in?Boolean

SQLite does not support multiple columns for the IN/NOT IN operators

Returns:

  • (Boolean)


667
668
669
# File 'lib/sequel/adapters/shared/sqlite.rb', line 667

def supports_multiple_column_in?
  false
end

#supports_timestamp_timezones?Boolean

SQLite supports timezones in literal timestamps, since it stores them as text. But using timezones in timestamps breaks SQLite datetime functions, so we allow the user to override the default per database.

Returns:

  • (Boolean)


674
675
676
# File 'lib/sequel/adapters/shared/sqlite.rb', line 674

def supports_timestamp_timezones?
  db.use_timestamp_timezones?
end

#supports_where_true?Boolean

SQLite cannot use WHERE ‘t’.

Returns:

  • (Boolean)


679
680
681
# File 'lib/sequel/adapters/shared/sqlite.rb', line 679

def supports_where_true?
  false
end