Class: Sequel::Schema::AlterTableGenerator
- Defined in:
- lib/sequel/database/schema_generator.rb
Overview
Schema::AlterTableGenerator is an internal class that the user is not expected to instantiate directly. Instances are created by Database#alter_table. It is used to specify table alteration parameters. It takes a Database object and a block of operations to perform on the table, and gives the Database an array of table altering operations, which the database uses to alter a table’s description.
For more information on Sequel’s support for schema modification, see the “Migrations and Schema Modification” guide.
Direct Known Subclasses
Instance Attribute Summary collapse
-
#operations ⇒ Object
readonly
An array of DDL operations to perform.
Instance Method Summary collapse
-
#add_column(name, type, opts = {}) ⇒ Object
Add a column with the given name, type, and opts to the DDL for the table.
-
#add_constraint(name, *args, &block) ⇒ Object
Add a constraint with the given name and args to the DDL for the table.
-
#add_foreign_key(name, table, opts = {}) ⇒ Object
Add a foreign key with the given name and referencing the given table to the DDL for the table.
-
#add_full_text_index(columns, opts = {}) ⇒ Object
Add a full text index on the given columns to the DDL for the table.
-
#add_index(columns, opts = {}) ⇒ Object
Add an index on the given columns to the DDL for the table.
-
#add_primary_key(name, opts = {}) ⇒ Object
Add a primary key to the DDL for the table.
-
#add_spatial_index(columns, opts = {}) ⇒ Object
Add a spatial index on the given columns to the DDL for the table.
-
#add_unique_constraint(columns, opts = {}) ⇒ Object
Add a unique constraint to the given column(s).
-
#drop_column(name, opts = {}) ⇒ Object
Remove a column from the DDL for the table.
-
#drop_constraint(name, opts = {}) ⇒ Object
Remove a constraint from the DDL for the table.
-
#drop_index(columns, options = {}) ⇒ Object
Remove an index from the DDL for the table.
-
#initialize(db, &block) ⇒ AlterTableGenerator
constructor
Set the Database object to which to apply the DDL, and evaluate the block in the context of this object.
-
#rename_column(name, new_name, opts = {}) ⇒ Object
Modify a column’s name in the DDL for the table.
-
#set_column_allow_null(name, allow_null) ⇒ Object
Modify a column’s NOT NULL constraint.
-
#set_column_default(name, default) ⇒ Object
Modify a column’s default value in the DDL for the table.
-
#set_column_type(name, type, opts = {}) ⇒ Object
Modify a column’s type in the DDL for the table.
Constructor Details
#initialize(db, &block) ⇒ AlterTableGenerator
Set the Database object to which to apply the DDL, and evaluate the block in the context of this object.
277 278 279 280 281 |
# File 'lib/sequel/database/schema_generator.rb', line 277 def initialize(db, &block) @db = db @operations = [] instance_eval(&block) if block end |
Instance Attribute Details
#operations ⇒ Object (readonly)
An array of DDL operations to perform
273 274 275 |
# File 'lib/sequel/database/schema_generator.rb', line 273 def operations @operations end |
Instance Method Details
#add_column(name, type, opts = {}) ⇒ Object
Add a column with the given name, type, and opts to the DDL for the table. See CreateTableGenerator#column for the available options.
add_column(:name, String) # ADD COLUMN name varchar(255)
287 288 289 |
# File 'lib/sequel/database/schema_generator.rb', line 287 def add_column(name, type, opts = {}) @operations << {:op => :add_column, :name => name, :type => type}.merge(opts) end |
#add_constraint(name, *args, &block) ⇒ Object
Add a constraint with the given name and args to the DDL for the table. See CreateTableGenerator#constraint.
add_constraint(:valid_name, :name.like('A%'))
# ADD CONSTRAINT valid_name CHECK (name LIKE 'A%')
296 297 298 |
# File 'lib/sequel/database/schema_generator.rb', line 296 def add_constraint(name, *args, &block) @operations << {:op => :add_constraint, :name => name, :type => :check, :check => block || args} end |
#add_foreign_key(name, table, opts = {}) ⇒ Object
Add a foreign key with the given name and referencing the given table to the DDL for the table. See CreateTableGenerator#column for the available options.
You can also pass an array of column names for creating composite foreign keys. In this case, it will assume the columns exist and will only add the constraint. You can provide a :name option to name the constraint.
NOTE: If you need to add a foreign key constraint to a single existing column use the composite key syntax even if it is only one column.
add_foreign_key(:artist_id, :table) # ADD COLUMN artist_id integer REFERENCES table
add_foreign_key([:name], :table) # ADD FOREIGN KEY (name) REFERENCES table
PostgreSQL specific options:
- :not_valid
-
Set to true to add the constraint with the NOT VALID syntax. This makes it so that future inserts must respect referential integrity, but allows the constraint to be added even if existing column values reference rows that do not exist. After all the existing data has been cleaned up, validate_constraint can be used to mark the constraint as valid. Note that this option only makes sense when using an array of columns.
330 331 332 333 |
# File 'lib/sequel/database/schema_generator.rb', line 330 def add_foreign_key(name, table, opts = {}) return add_composite_foreign_key(name, table, opts) if name.is_a?(Array) add_column(name, Integer, {:table=>table}.merge(opts)) end |
#add_full_text_index(columns, opts = {}) ⇒ Object
Add a full text index on the given columns to the DDL for the table. See CreateTableGenerator#index for available options.
337 338 339 |
# File 'lib/sequel/database/schema_generator.rb', line 337 def add_full_text_index(columns, opts = {}) add_index(columns, {:type=>:full_text}.merge(opts)) end |
#add_index(columns, opts = {}) ⇒ Object
Add an index on the given columns to the DDL for the table. See CreateTableGenerator#index for available options.
add_index(:artist_id) # CREATE INDEX table_artist_id_index ON table (artist_id)
345 346 347 |
# File 'lib/sequel/database/schema_generator.rb', line 345 def add_index(columns, opts = {}) @operations << {:op => :add_index, :columns => Array(columns)}.merge(opts) end |
#add_primary_key(name, opts = {}) ⇒ Object
Add a primary key to the DDL for the table. See CreateTableGenerator#column for the available options. Like add_foreign_key, if you specify the column name as an array, it just creates a constraint:
add_primary_key(:id) # ADD COLUMN id serial PRIMARY KEY
add_primary_key([:artist_id, :name]) # ADD PRIMARY KEY (artist_id, name)
355 356 357 358 359 |
# File 'lib/sequel/database/schema_generator.rb', line 355 def add_primary_key(name, opts = {}) return add_composite_primary_key(name, opts) if name.is_a?(Array) opts = @db..merge(opts) add_column(name, opts.delete(:type), opts) end |
#add_spatial_index(columns, opts = {}) ⇒ Object
Add a spatial index on the given columns to the DDL for the table. See CreateTableGenerator#index for available options.
363 364 365 |
# File 'lib/sequel/database/schema_generator.rb', line 363 def add_spatial_index(columns, opts = {}) add_index(columns, {:type=>:spatial}.merge(opts)) end |
#add_unique_constraint(columns, opts = {}) ⇒ Object
Add a unique constraint to the given column(s)
add_unique_constraint(:name) # ADD UNIQUE (name)
add_unique_constraint(:name, :name=>:unique_name) # ADD CONSTRAINT unique_name UNIQUE (name)
304 305 306 |
# File 'lib/sequel/database/schema_generator.rb', line 304 def add_unique_constraint(columns, opts = {}) @operations << {:op => :add_constraint, :type => :unique, :columns => Array(columns)}.merge(opts) end |
#drop_column(name, opts = {}) ⇒ Object
Remove a column from the DDL for the table.
drop_column(:artist_id) # DROP COLUMN artist_id
drop_column(:artist_id, :cascade=>true) # DROP COLUMN artist_id CASCADE
371 372 373 |
# File 'lib/sequel/database/schema_generator.rb', line 371 def drop_column(name, opts={}) @operations << {:op => :drop_column, :name => name}.merge(opts) end |
#drop_constraint(name, opts = {}) ⇒ Object
Remove a constraint from the DDL for the table. MySQL/SQLite specific options:
- :type
-
Set the type of constraint to drop, either :primary_key, :foreign_key, or :unique.
drop_constraint(:unique_name) # DROP CONSTRAINT unique_name
drop_constraint(:unique_name, :cascade=>true) # DROP CONSTRAINT unique_name CASCADE
382 383 384 |
# File 'lib/sequel/database/schema_generator.rb', line 382 def drop_constraint(name, opts={}) @operations << {:op => :drop_constraint, :name => name}.merge(opts) end |
#drop_index(columns, options = {}) ⇒ Object
Remove an index from the DDL for the table. General options:
- :name
-
The name of the index to drop. If not given, uses the same name that would be used by add_index with the same columns.
PostgreSQL specific options:
- :cascade
-
Cascade the index drop to dependent objects.
- :concurrently
-
Drop the index using CONCURRENTLY, which doesn’t block operations on the table. Supported in PostgreSQL 9.2+.
- :if_exists
-
Only drop the index if it already exists.
drop_index(:artist_id) # DROP INDEX table_artist_id_index
drop_index([:a, :b]) # DROP INDEX table_a_b_index
drop_index([:a, :b], :name=>:foo) # DROP INDEX foo
401 402 403 |
# File 'lib/sequel/database/schema_generator.rb', line 401 def drop_index(columns, ={}) @operations << {:op => :drop_index, :columns => Array(columns)}.merge() end |
#rename_column(name, new_name, opts = {}) ⇒ Object
Modify a column’s name in the DDL for the table.
rename_column(:name, :artist_name) # RENAME COLUMN name TO artist_name
408 409 410 |
# File 'lib/sequel/database/schema_generator.rb', line 408 def rename_column(name, new_name, opts = {}) @operations << {:op => :rename_column, :name => name, :new_name => new_name}.merge(opts) end |
#set_column_allow_null(name, allow_null) ⇒ Object
Modify a column’s NOT NULL constraint.
set_column_allow_null(:artist_name, false) # ALTER COLUMN artist_name SET NOT NULL
433 434 435 |
# File 'lib/sequel/database/schema_generator.rb', line 433 def set_column_allow_null(name, allow_null) @operations << {:op => :set_column_null, :name => name, :null => allow_null} end |
#set_column_default(name, default) ⇒ Object
Modify a column’s default value in the DDL for the table.
set_column_default(:artist_name, 'a') # ALTER COLUMN artist_name SET DEFAULT 'a'
415 416 417 |
# File 'lib/sequel/database/schema_generator.rb', line 415 def set_column_default(name, default) @operations << {:op => :set_column_default, :name => name, :default => default} end |
#set_column_type(name, type, opts = {}) ⇒ Object
Modify a column’s type in the DDL for the table.
set_column_type(:artist_name, 'char(10)') # ALTER COLUMN artist_name TYPE char(10)
PostgreSQL specific options:
- :using
-
Add a USING clause that specifies how to convert existing values to new values.
426 427 428 |
# File 'lib/sequel/database/schema_generator.rb', line 426 def set_column_type(name, type, opts={}) @operations << {:op => :set_column_type, :name => name, :type => type}.merge(opts) end |