Class: Sequel::TimestampMigrator

Inherits:
Migrator show all
Defined in:
lib/sequel/extensions/migration.rb

Overview

The migrator used if any migration file version appears to be a timestamp. Stores filenames of migration files, and can figure out which migrations have not been applied and apply them, even if earlier migrations are added after later migrations. If you plan to do that, the responsibility is on you to make sure the migrations don’t conflict. Part of the migration extension.

Constant Summary collapse

DEFAULT_SCHEMA_COLUMN =
:filename
DEFAULT_SCHEMA_TABLE =
:schema_migrations
Error =
Migrator::Error

Constants inherited from Migrator

Migrator::MIGRATION_FILE_PATTERN, Migrator::MIGRATION_SPLITTER, Migrator::MINIMUM_TIMESTAMP

Instance Attribute Summary collapse

Attributes inherited from Migrator

#column, #db, #directory, #ds, #files, #table, #target

Instance Method Summary collapse

Methods inherited from Migrator

apply, check_current, is_current?, migrator_class, run

Constructor Details

#initialize(db, directory, opts = OPTS) ⇒ TimestampMigrator

Set up all state for the migrator instance



683
684
685
686
687
688
# File 'lib/sequel/extensions/migration.rb', line 683

def initialize(db, directory, opts=OPTS)
  super
  @target = opts[:target]
  @applied_migrations = get_applied_migrations
  @migration_tuples = get_migration_tuples
end

Instance Attribute Details

#applied_migrationsObject (readonly)

Array of strings of applied migration filenames



677
678
679
# File 'lib/sequel/extensions/migration.rb', line 677

def applied_migrations
  @applied_migrations
end

#migration_tuplesObject (readonly)

Get tuples of migrations, filenames, and actions for each migration



680
681
682
# File 'lib/sequel/extensions/migration.rb', line 680

def migration_tuples
  @migration_tuples
end

Instance Method Details

#is_current?Boolean

The timestamp migrator is current if there are no migrations to apply in either direction.

Returns:

  • (Boolean)


692
693
694
# File 'lib/sequel/extensions/migration.rb', line 692

def is_current?
  migration_tuples.empty?
end

#runObject

Apply all migration tuples on the database



697
698
699
700
701
702
703
704
705
706
707
708
709
# File 'lib/sequel/extensions/migration.rb', line 697

def run
  migration_tuples.each do |m, f, direction|
    t = Time.now
    db.log_info("Begin applying migration #{f}, direction: #{direction}")
    checked_transaction(m) do
      m.apply(db, direction)
      fi = f.downcase
      direction == :up ? ds.insert(column=>fi) : ds.where(column=>fi).delete
    end
    db.log_info("Finished applying migration #{f}, direction: #{direction}, took #{sprintf('%0.6f', Time.now - t)} seconds")
  end
  nil
end